
/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including software, documents, or other related items (the
   "Software"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM)  API Implementation License. By
   obtaining, using and/or copying this Software, you agree that you have
   read, understand, and will comply with the following terms and
   conditions of the MIT OKI(TM) API Implementation License:

   Permission to use, copy, modify, and distribute this Software and its
   documentation, with or without modification, for any purpose and
   without fee or royalty is hereby granted, provided that you include
   the following on ALL copies of the Software or portions thereof,
   including modifications or derivatives, that you make:

    *  The full text of the MIT OKI(TM) API Implementation License in a
       location viewable to users of the redistributed or derivative
       work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed or
       derivative Software:
       "Copyright (c) 2002 Massachusetts Institute of Technology
       All Rights Reserved."

    *  Notice of any changes or modifications to the MIT OKI(TM)
       Software, including the date the changes were made. Any modified
       software must be distributed in such as manner as to avoid any
       confusion with the original MIT OKI(TM) Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Software without
   specific, written prior permission. Title to copyright in the Software
   and any associated documentation will at all times remain with the
   copyright holders.

   The export of software employing encryption technology may require a
   specific license from the United States Government. It is the
   responsibility of any person or organization contemplating export to
   obtain such a license before exporting this Software.
*/

/*
 * $Source: /cvs/oki/tech/contrib/mit/apps/examples/service/filing/demo1/FileDemo1.java,v $
 */

/**
 *  Sample pseudo-application demonstrating Filing API using
 *  "local file system" reference implementation
 *
 * Licensed under the {@link org.okip.service.ApiImplementationLicenseMIT MIT OKI(TM) API Implementation License}.
 *
 * @version $Revision: 1.4 $ / $Date: 2002/04/29 20:35:06 $
 */
public class FileDemo1 {

  /**
   * Method main
   *
   *
   * @param args
   *
   */
  public static void main(String[] args) {
    try {

      // get factory for implmentation
      org.okip.service.filing.api.Factory fileFactory =
        (org.okip.service.filing.api.Factory) org.okip.service.shared.api.FactoryManager.getDefaultFactory(
          "org.okip.service.filing.api", null);

      // List all CabinetFactories available
      System.out.println("Cabinet Factories available: "
                         + fileFactory.getAvailableCabinetFactories(null));

      // Show available properties
      System.out.println("Properties available: "
                         + fileFactory.getAvailableProperties());

      // Get all CabinetFactories with the desired properties
      // want something with readwrite

      java.util.Map desiredProperties = new java.util.Hashtable();

      desiredProperties.put(
        new org.okip.service.filing.api.CapabilityType("MIT", "readwrite"),
        new Boolean(true));
      desiredProperties.put(
        new org.okip.service.filing.api.CapabilityType("MIT", "readwrite"),
        new Boolean(true));
      desiredProperties.put(
        new org.okip.service.filing.api.CapabilityType("MIT", "readwrite"),
        new Boolean(true));
      desiredProperties.put(
        new org.okip.service.filing.api.CapabilityType("MIT", "loseOnly"),
        new Boolean(false));

      org.okip.service.filing.api.CabinetFactory cabinetFactories[] =
        fileFactory.getCabinetFactory(desiredProperties);

      System.out.println("got " + cabinetFactories.length + " cabinetFactor"
                         + ((cabinetFactories.length > 1)
                            ? "ies"
                            : "y"));

      if (cabinetFactories.length > 0) {
        org.okip.service.filing.api.CabinetFactory myCabinetFactory =
          cabinetFactories[0];

        // now we have a cabinet factory to play with.

        System.out.println("roots of " + myCabinetFactory + " are "
                           + myCabinetFactory.listRoots());
      }

      // Now demonstrate getting a cabinetFactory by Type
      java.util.List allCabinetFactories =
        fileFactory.getAvailableCabinetFactories(null);

      System.out.println("\nthe types of all available cabinetFactories :"
                         + allCabinetFactories);

      if (cabinetFactories != null) {
        org.okip.service.shared.api.Type           cabinetType           =
          (org.okip.service.shared.api.Type) allCabinetFactories.get(0);
        org.okip.service.filing.api.CabinetFactory anotherCabinetFactory =
          fileFactory.getCabinetFactoryByType(cabinetType);

        System.out.println("got cabinet type " + cabinetType + " which is "
                           + anotherCabinetFactory);

        // now we have a cabinet factory to play with.
        System.out.println("roots of this are "
                           + anotherCabinetFactory.listRoots());

        org.okip.service.filing.api.Cabinet c    =
          anotherCabinetFactory.getCabinet(
            anotherCabinetFactory.idFromString("/tmp/"));

        java.util.Iterator                  iter = c.entries();

        System.out.println("The cabinet with id '/tmp/' has entries:");

        while (iter.hasNext()) {
          org.okip.service.filing.api.CabinetEntry ce =
            (org.okip.service.filing.api.CabinetEntry) iter.next();

          System.out.println((ce.isByteStore()
                              ? " ByteStore "
                              : " Cabinet   ") + "name: " + ce.getName());

          if (ce.isByteStore()) {
            System.out.println(
              "   Mime Type:"
              + ((org.okip.service.filing.api.ByteStore) ce).getMimeType());

            org.okip.service.filing.api.OkiInputStream is    =
              ((org.okip.service.filing.api.ByteStore) ce).getOkiInputStream();

            byte[]                                  b     = new byte[100];

            int                                     nRead = is.read(b);

            if (nRead >= 0) {
              System.out.println("   Contents: \n'" + new String(b, 0, nRead)
                                 + "'\n\n");
            }
          }
        }

        c    = anotherCabinetFactory.getCabinet(
          anotherCabinetFactory.idFromString("/var/tmp/"));
        iter = c.entries();

        while (iter.hasNext()) {
          org.okip.service.filing.api.CabinetEntry ce =
            (org.okip.service.filing.api.CabinetEntry) iter.next();

          System.out.println((ce.isByteStore()
                              ? " ByteStore "
                              : " Cabinet   ") + "name: " + ce.getName());

          if (ce.isByteStore()) {
            System.out.println(
              "   Mime Type:"
              + ((org.okip.service.filing.api.ByteStore) ce).getMimeType());

            org.okip.service.filing.api.OkiInputStream is    =
              ((org.okip.service.filing.api.ByteStore) ce).getOkiInputStream();

            byte[]                                  b     = new byte[100];

            int                                     nRead = is.read(b);

            if (nRead >= 0) {
              System.out.println("   Contents: \n'" + new String(b, 0, nRead)
                                 + "'\n\n");
            }
          }
        }

        try {
          System.out.println("try creating byte store:"
                             + c.createByteStore("console.log"));
        }
        catch (Exception e) {
          System.err.println("error:" + e);
        }

        try {
          System.out.println("try creating another byte store");

          org.okip.service.filing.api.ByteStore bs =
            c.createByteStore("foobar");

          System.out.println("got:" + bs);

          org.okip.service.filing.api.OkiOutputStream bsos = bs.getOkiOutputStream();

          bsos.write("this is a test\n".getBytes());

          System.out.println();
          System.out.println("Parent:" + c.getParent());

          org.okip.service.filing.api.Cabinet c2 = c.getParent();

          System.out.println("Parent of that parent:" + c2.getParent()
                             + " is it the root? :"
                             + c2.getParent().isRootCabinet());

          c2 = c2.getParent();

          System.out.println("Root:" + c.getRootCabinet());
          System.out.println("Root is root?:"
                             + c.getRootCabinet().isRootCabinet());
        }
        catch (Exception e) {
          System.err.println("error:" + e);
        }
      }
    }
    catch (Exception e) {
      System.err.println("oops: " + e);
      e.printStackTrace();
    }
  }
}

