//
//  CabSyncViewer.java
//  S.Fraize July 2002
//
// TODO:
//      package stuff up (cabdemo? demo2?)
//      create a package.html w/directions
//      add dual view (remove tree for the moment) -- each window titled with cabinet
//      add polling & auto-sync
//      add back tree view & recursive directory sync

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import org.okip.service.filing.api.*;

public class CabSyncViewer extends JFrame
    implements ActionListener, Runnable
{
    JTree jtree;
    JTable jtable;
    SyncList syncList;
    
    class SyncListTreeNode extends DefaultMutableTreeNode
    {
        // a tree node capable of creating a data-model based
        // on it's recalled object.
        
        // a TreeNode, initialzed with some object that can generate a
        // list of items that can be fed to a given factory to
        // produduce an AbstractTableModel of some kind.  Could use
        // this genericall with CabViewer if we used a
        // TableModelFactory and an additional name factory to be used
        // in toString() (to get a displayable name from the user
        // object); OR, if it was passed in as an argument.
        
	AbstractTableModel dataModel;
	String displayName;
	
	SyncListTreeNode(Object userObject)
	{
	    super(userObject);
	}
        
        public void setDisplayName(String name)
        {
            this.displayName = name;
        }

        private SyncList getSyncList()
        {
            return (SyncList) getUserObject();
        }

	AbstractTableModel getDataModel()
	{
	    if (this.dataModel == null) {
                debug("SyncListTreeNode: creating TableModel for " + getSyncList());
                this.dataModel =  new SyncListTableModel(getSyncList());
            }
	    return this.dataModel;
	}
	
	/*
	 * toString is very functional here -- it's
	 * used to produce the text that the tree widget displays.
	 */
	public String toString()
	{
	    if (displayName != null)
		return displayName;
	    try {
                // won't work once we're recursing...
		return getUserObject().toString();
	    } catch (Exception e) {
		return getUserObject().getClass().toString();
	    }
	}
    }

    class SyncListTableModel extends AbstractTableModel
                                     implements ChangeListener
    {
        SyncList syncList;
	ArrayList entries;
	SyncListTreeNode treeNode;//not used at moment (not recursing)
	
        SyncListTableModel(SyncList sl)
	{
	    this(sl, null);
	}
        SyncListTableModel(SyncList sl, SyncListTreeNode node)
	{
	    this.treeNode = node;
	    this.entries = new ArrayList();
	    this.refreshModelFromSyncList(sl, true);
	}

        private void setSyncList(SyncList sl)
        {
            if (syncList != sl) {
                if (syncList != null)
                    syncList.removeChangeListener(this);
                syncList = sl;
                syncList.addChangeListener(this);
            }
        }

        public void stateChanged(ChangeEvent e)
        {
            debug("SyncListTableModel:stateChanged: " + e);
            refreshModel();
        }
        
        final int COL_NAME = 0;
        final int COL_SIZE = 1;
        final int COL_MODTIME = 2;
        final int COL_STATUS = 3;

        TableColumnModel columnModel;
        public TableColumnModel getTableColumnModel()
        {
            if (columnModel != null)
                return columnModel;

            columnModel = new DefaultTableColumnModel();

            addTableColumn(COL_NAME, "Entry Name", 1000);
            addTableColumn(COL_SIZE, "Size", 350);
            addTableColumn(COL_MODTIME, "Modified", 850);
            addTableColumn(COL_STATUS, "Status", 400);
            //addTableColumn(COL_OWNER, "Owner", 500);

            return columnModel;
        }
        
	public Class getColumnClass(int col) {
	    if (col == COL_NAME) return SyncEntry.class;
	    if (col == COL_SIZE) return Integer.class;	        // this will cause align-right
	    if (col == COL_MODTIME) return Date.class;
 	    return super.getColumnClass(col);
	}

        int columnCount = 0;
        public void addTableColumn(int index, String name, int width)
        {
            TableColumn tc = new TableColumn(index, width);
            tc.setHeaderValue(name);
            columnCount++;
            columnModel.addColumn(tc);
        }
        
        public int 	getColumnCount() { return columnCount; }
        public int 	getRowCount() { return entries.size(); }
        public Object 	getValueAt(int row, int col)
	{
	    SyncEntry se = (SyncEntry) entries.get(row);
	    return getColumnData(se, col);
	}
	
	private Object getColumnData(SyncEntry se, int col)
	{
	    boolean isBS = se.isByteStore();
	    boolean isCAB = se.isCabinet();
	    ByteStore bs = isBS ? se.getByteStore() : null;
	    Object data = null;
            CabinetEntry ce = se.getCabinetEntry();

	    try {
		if (col ==  COL_NAME) {
                    data = se;
                } else if (col == COL_SIZE) {
                    // TODO: deleting a local entry while the viewer is up
                    // will cause the modified time to be rendered as
                    // <invalid lastModified time> until a refresh is done --
                    // if we get any exceptions here, maybe we should
                    // initate a single refresh attempt (w/looping protection).
		    if (isBS)
                        data = new Long(bs.length());
                    else
                        data = "";
                } else if (col == COL_MODTIME) {
                    if (isBS)
                        data = new Date(bs.getLastModifiedTime());
                    else
                        data = "";
                } else if (col == COL_STATUS)
                    data = se.getStatusName();
	    } catch (Exception e) {
		errout(e.toString());
		data = "<" + e.getMessage() + ">";
	    }
	    return data == null ? "-" : data;
	}

	public void refreshModel()
        {
            refreshModelFromSyncList(this.syncList, false);
        }
        
	public void refreshModelFromSyncList(SyncList sl)
        {
            refreshModelFromSyncList(sl, false);
        }
            
	private void refreshModelFromSyncList(SyncList sl, boolean buildTree)
	{
            setSyncList(sl);
	    this.entries.clear();

            Enumeration e = this.syncList.elements();
            while (e.hasMoreElements())
            {
                SyncEntry se = (SyncEntry) e.nextElement();
		//CabinetEntry ce = se.getLocalCabinet();
		//debug("SyncListTableModel adding " + ce);
		entries.add(se);

		// TODO: need to allow for updating the tree model also --
		// will need to clear & reset children.
                // Also: do we really want to do this here?
		//if (buildTree && se.isCabinet() && treeNode != null) {
                //  this.treeNode.add(new SyncListTreeNode(se));
		    //this.treeNode.add(new SyncListTreeNode((Cabinet)ce));
		//}
	    }
            
            fireTableDataChanged();
	}
    };
        
    //------------------------------------------------------------------
    // CabSyncViewer
    //------------------------------------------------------------------

    public CabSyncViewer(SyncList syncList)
    {
        super("OKI Cabinet Synchronizer");
        debug("new on SyncList " + syncList);

        Cabinet cabinet = syncList.getLocalCabinet();
        this.syncList = syncList;
        this.setDefaultCloseOperation(EXIT_ON_CLOSE);
        this.setSize(new Dimension(800,640));
        this.setLocation(100,100);
        this.addMenus();
        this.setVisible(true);// show something right away
        
        SyncListTreeNode rootNode;
	rootNode = new SyncListTreeNode(syncList);
        rootNode.setDisplayName(cabinet.getPath());
        
	
        // rootNode.getDataModel() will invoke 1st level directory cache
        SyncListTableModel dataModel = (SyncListTableModel) rootNode.getDataModel();
        this.jtree = new JTree(rootNode, true);
        //        this.tableModel = dataModel;

        addJTreeListeners();
        
        TableColumnModel tableColumnModel = dataModel.getTableColumnModel();
        tableColumnModel.setColumnMargin(10);// this makes for ugly breaks in line-selection indicator tho...
        this.jtable = new JTable(dataModel, tableColumnModel);
        this.jtable.setGridColor(Color.lightGray);

        addTableRenderers();
        
        JSplitPane splitPane = new JSplitPane();
        JScrollPane scrollPane = new JScrollPane(this.jtable);
        splitPane.setResizeWeight(0.25); // 25% space to the left component
        splitPane.setContinuousLayout(true);
        splitPane.setLeftComponent(jtree);
        splitPane.setOneTouchExpandable(true);
        splitPane.setRightComponent(scrollPane);

        // layout all the application components

        Container pane = getContentPane();
        pane.setLayout(new BorderLayout());
        pane.add(getToolBar(), BorderLayout.NORTH);
        pane.add(splitPane, BorderLayout.CENTER);

        this.constructing = false;
    }

    protected void addJTreeListeners()
    {
	jtree.addTreeSelectionListener
	    (new TreeSelectionListener() {
		    public void valueChanged(TreeSelectionEvent e)
		    {
			SyncListTreeNode cabNode = (SyncListTreeNode) jtree.getLastSelectedPathComponent();
			if (cabNode == null) return;
			debug("tree node selected: " + cabNode);
			jtable.setModel(cabNode.getDataModel());
		    }
		});

        // could put these on the the class instead of inline
        jtree.addTreeWillExpandListener
           (new TreeWillExpandListener() {
                   public void treeWillExpand(TreeExpansionEvent e)
                   {
                        TreePath path = e.getPath();
                        debug("tree expanding at path: " + path);
                        SyncListTreeNode cabNode = (SyncListTreeNode) path.getLastPathComponent();
			if (cabNode == null) return;
			debug("tree expanding at node: " + cabNode);
                        jtree.setSelectionPath(path);
                        jtable.setModel(cabNode.getDataModel());
                   }
                   public void treeWillCollapse(TreeExpansionEvent e) {}
               });

    }
                   
    static final Color darkGreen = new Color(0, 128, 0);
    protected void addTableRenderers()
    {
        // Set some renderers (defined in-line) for the CabinetEntry & Date classes.
        // Could also set these up by column instead of class in SyncListTableModel.createTableColumnModel();
        // It would probably be cleaner to do it there.

        jtable.setDefaultRenderer(SyncEntry.class, new DefaultTableCellRenderer()
            { // in-line class definition

                // An example custom renderer -- if the CabinetEntry is unwriteable, make it red,
                // if a directory (Cabinet vs. ByteStore) make it blue and add a "/", if
                // emacs backup file, gray it out.
                public Component getTableCellRendererComponent(JTable table, Object value,
                                                               boolean isSelected, boolean hasFocus, int row, int column)
                 {
                     try{
                     if (value instanceof SyncEntry) {
                         SyncEntry se = (SyncEntry) value;
                         String label = se.getName();
                         if (se.isCabinet())
                             label += "/";

                         switch (se.getStatus()) {
                         case SyncList.STATUS_NEW_REMOTE:
                         case SyncList.STATUS_CHANGED_REMOTE:
                             setForeground(Color.red);
                             break;
                         case SyncList.STATUS_NEW_LOCAL:
                         case SyncList.STATUS_CHANGED_LOCAL:
                             setForeground(darkGreen);
                             break;
                         case SyncList.STATUS_CONFLICT:
                             setForeground(Color.orange);
                             break;
                         default:
                             setForeground(Color.black);
                         }
                         
                         value = label;
                     }
                     } catch (Exception e) {
                         errout(e);
                     }
                     return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                 }
            });
        
        jtable.setDefaultRenderer(Date.class, new DefaultTableCellRenderer()
            { // in-line class definition
                DateFormat dateFormatter = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss");//fix: this ignores locale
                {
                    setHorizontalAlignment(SwingConstants.RIGHT);
                }
                public Component getTableCellRendererComponent(JTable table, Object value,
                                                               boolean isSelected, boolean hasFocus, int row, int column)
                {
                    if (value instanceof Date) {
                        Date d = (Date) value;
                        value = dateFormatter.format(d);
                    } // else, it may be an empty string -- no date info available
                    return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                }
            });
    }
        
    
    private boolean constructing = true;
    private Font font = new Font("serif", Font.ITALIC+Font.BOLD, 48);
    public void paint(Graphics g)
    {
        super.paint(g);
        if (constructing) {
            g.setColor(Color.lightGray);
            g.setFont(font);
            g.drawString("OKI FILING DEMO", getWidth()/4, getHeight()/2);
        }
    }
    protected void _processEvent(AWTEvent e)
    {
        debug(e.toString());
	super.processEvent(e);
    }
    
    
    // Declarations for menus
    static JMenuBar mainMenuBar = null;
    static JMenu fileMenu = null;
    
    private static JMenuBar getMainMenuBar()
    {
        if (mainMenuBar == null)
            mainMenuBar = new JMenuBar();
        return mainMenuBar;
    }
    private static JMenu getFileMenu()
    {
        if (fileMenu == null)
            fileMenu = new JMenu("File");
        return fileMenu;
    }
        
    JToolBar toolBar;
    JButton tbClose = new JButton("Close");
    JButton tbRefresh = new JButton("Refresh");
    JButton tbSync = new JButton("Synchronize");
    JButton tbCancel = new JButton("Cancel");
    //    AbstractButton tbPoll = new JRadioButton("AutoRefresh");
    AbstractButton tbAutoSync = new JRadioButton("AutoSync");
    private JToolBar getToolBar()
    {
        if (toolBar != null)
            return toolBar;

        toolBar = new JToolBar();
        //toolBar.setBorderPainted(false);
        toolBar.setFloatable(false);
        
        addToolButton(tbClose);
        addToolButton(tbRefresh);
        addToolButton(tbSync);
        addToolButton(tbCancel);

        toolBar.addSeparator();
        toolBar.addSeparator();
        toolBar.addSeparator();
        //        addToolButton(tbPoll);
        //        toolBar.addSeparator();
        addToolButton(tbAutoSync);

        tbCancel.setEnabled(false);
        return toolBar;
    }

    private void addToolButton(AbstractButton b)
    {
        toolBar.addSeparator();
        toolBar.add(b);
        b.addActionListener(this);
        //b.setDefaultCapable(false);
    }
    
    JButton btnSync = null;//new JButton("Synchronize");
    public void addMenus() {
        addFileMenuItems();
        addEditMenuItems();
        //mainMenuBar.add(new JLabel("     Hello World    ", JLabel.RIGHT));
        //mainMenuBar.add(new JLabel("(message)"));
        if (btnSync != null) {
            mainMenuBar.add(btnSync);
            btnSync.addActionListener(this);
            btnSync.setDefaultCapable(false);//drawn border gets messed up in apple LAF
        }
        setJMenuBar(mainMenuBar);
    }
    
    protected JMenuItem miNew;
    protected JMenuItem miOpen;
    protected JMenuItem miClose;
    protected JMenuItem miSave;
    protected JMenuItem miSaveAs;
	
    /*static*/ final JMenu editMenu = new JMenu("Edit");
    // Whoa: making a JMenu static forces the LAF to be
    // native, and not subject to a runtime change!
    // This probably applies to any swing component made static.
    protected JMenuItem miUndo;
    protected JMenuItem miCut;
    protected JMenuItem miCopy;
    protected JMenuItem miPaste;
    protected JMenuItem miClear;
    protected JMenuItem miSelectAll;
	
    public void addFileMenuItems() {
        getFileMenu();
        miNew = new JMenuItem ("New");
        miNew.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.Event.META_MASK));
        fileMenu.add(miNew).setEnabled(false);
        miNew.addActionListener(this);

        miOpen = new JMenuItem ("Open...");
        miOpen.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.Event.META_MASK));
        fileMenu.add(miOpen).setEnabled(false);
        miOpen.addActionListener(this);
		
        miClose = new JMenuItem ("Close");
        miClose.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_W, java.awt.Event.META_MASK));
        fileMenu.add(miClose).setEnabled(true);
        miClose.addActionListener(this);
		
        miSave = new JMenuItem ("Save");
        miSave.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.Event.META_MASK));
        fileMenu.add(miSave).setEnabled(false);
        miSave.addActionListener(this);
		
        miSaveAs = new JMenuItem ("Save As...");
        fileMenu.add(miSaveAs).setEnabled(false);
        miSaveAs.addActionListener(this);
		
        getMainMenuBar().add(fileMenu);
    }
	
	
    public void addEditMenuItems() {
        miUndo = new JMenuItem("Undo");
        miUndo.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Z, java.awt.Event.META_MASK));
        editMenu.add(miUndo).setEnabled(false);
        miUndo.addActionListener(this);
        editMenu.addSeparator();
 
        miCut = new JMenuItem("Cut");
        miCut.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, java.awt.Event.META_MASK));
        editMenu.add(miCut).setEnabled(false);
        miCut.addActionListener(this);

        miCopy = new JMenuItem("Copy");
        miCopy.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.Event.META_MASK));
        editMenu.add(miCopy).setEnabled(false);
        miCopy.addActionListener(this);

        miPaste = new JMenuItem("Paste");
        miPaste.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_V, java.awt.Event.META_MASK));
        editMenu.add(miPaste).setEnabled(false);
        miPaste.addActionListener(this);

        miClear = new JMenuItem("Clear");
        editMenu.add(miClear).setEnabled(true);
        miClear.addActionListener(this);
        editMenu.addSeparator();

        miSelectAll = new JMenuItem("Select All");
        miSelectAll.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_A, java.awt.Event.META_MASK));
        editMenu.add(miSelectAll).setEnabled(true);
        miSelectAll.addActionListener(this);

        getMainMenuBar().add(editMenu);
    }
	

    // ActionListener interface (for menus)
    public void actionPerformed(ActionEvent newEvent) {
        if (newEvent.getActionCommand().equals(miNew.getActionCommand())) doNew();
        else if (newEvent.getActionCommand().equals(miOpen.getActionCommand())) doOpen();
        else if (newEvent.getActionCommand().equals(miClose.getActionCommand())) doClose();
        else if (newEvent.getActionCommand().equals(miSave.getActionCommand())) doSave();
        else if (newEvent.getActionCommand().equals(miSaveAs.getActionCommand())) doSaveAs();
        else if (newEvent.getActionCommand().equals(miUndo.getActionCommand())) doUndo();
        else if (newEvent.getActionCommand().equals(miCut.getActionCommand())) doCut();
        else if (newEvent.getActionCommand().equals(miCopy.getActionCommand())) doCopy();
        else if (newEvent.getActionCommand().equals(miPaste.getActionCommand())) doPaste();
        else if (newEvent.getActionCommand().equals(miClear.getActionCommand())) doClear();
        else if (newEvent.getActionCommand().equals(miSelectAll.getActionCommand())) doSelectAll();
        else if (newEvent.getActionCommand().equals("Synchronize")) doAsyncSync();
        else if (newEvent.getActionCommand().equals("Refresh")) doRefresh();
        else if (newEvent.getActionCommand().equals("Cancel")) doCancel();

        else
            debug("actionPerformed: " + newEvent.toString());
    }

    public void run()
    {
        debug("ASYNC sync started");
        doSync();
        tbSync.setEnabled(true);
        tbCancel.setEnabled(false);
        debug("ASYNC sync complete");
    }

    public synchronized void doAsyncSync()
    {
        tbSync.setEnabled(false);
        tbCancel.setEnabled(true);
        new Thread(this).start();
    }

    public void doSync()
    {
        debug("synchronizing");
        syncList.doSynchronize(A_TAKE_ACTION, A_VERBOSE, A_DEBUG);
    }
    
    public void doCancel()
    {
        debug("doCancel");
        // unimplemented -- need SyncList update listeners for interruption
    }
    public void doRefresh()
    {
        debug("refreshing");
        syncList.computeListStatus(A_DEBUG);
        // the SyncListTableModel has set itself up
        // as a listener for changes to the syncList,
        // and will get a notification if anything changes.
    }


    public void doNew() {}
    public void doOpen() { debug("doOpen"); }
    public void doClose() { debug("doClose"); System.exit(0); }
    public void doSave() {}
    public void doSaveAs() {}
    public void doUndo() {}
    public void doCut() { debug("cut row " + jtable.getSelectedRow()); }
    public void doCopy() {}
    public void doPaste() {}
    public void doClear() { jtable.clearSelection(); }
    public void doSelectAll() { jtable.selectAll(); }


    
    private static void out(String s) { System.out.print(s); }
    private static void outln(String s) { System.out.println(s); }
    private static void errout(String s) { System.err.println("Error: " + s); }
    private static void errout(Exception e)
    {
        errout(e.toString());
        e.printStackTrace();
    }
    static void debug(String s) { if (A_DEBUG) System.out.println("CabSyncViewer: " + s); }
    static boolean A_DEBUG = false;
    static boolean A_VERBOSE = false;
    static boolean A_TAKE_ACTION = true;
    public static void main(String args[])
    {
        CabUtil.parseCommandLine(args);
        for (int i = 0; i < args.length; i++) {
            String a = args[i];
            if (a.equals("-debug")) {
                A_DEBUG = true;
                A_VERBOSE = true;
            } else if (a.equals("-verbose"))
                A_VERBOSE = true;
            else if (a.equals("-n"))
                 A_TAKE_ACTION = !A_TAKE_ACTION;
        }
        
        Cabinet cabinet0 = CabUtil.getCabinetFromDirectory("./c/l");       //local
        Cabinet cabinet1 = CabUtil.getCabinetFromDirectory("./c/r");       //remote
        //        CabUtil.printCabinet(cabinet0);
        //        CabUtil.printCabinet(cabinet1);

        debug("Syncing local " + cabinet0 + " with remote " + cabinet1);
        SyncList syncList = new SyncList(cabinet0, cabinet1);
        syncList.computeListStatus();

        //CabSyncViewer cv = new CabSyncViewer(cabinet0);
        CabSyncViewer cv = new CabSyncViewer(syncList);

        cv.setVisible(true);
    }
}
