import org.okip.service.filing.api.*;

import java.util.*;
import java.text.*;

public class SyncEntry
{
    private static DateFormat dateFormatter = new SimpleDateFormat("yyyy/MM/dd hh:mm:ss");

    //-----------------------------------------------------------------------------
    // the following values are determined from the local configuration file
    // .cabsync (that's too unwieldy a name...)

    //boolean ENTRY_IGNORE = false;           // user requests ignore this particular entry
    long lastSync = 0;                      //
    boolean isPlaceHolder = false;

    private final String STATUS_NAMES[] = {
        "UNKNOWN  ",
        "CURRENT  ",
        "NEWLOCAL ",
        "CHGLOCAL ",
        "NEWREMOTE",
        "CHGREMOTE",
        "CONFLICT ",
    };
    private final String STATUS_CODES[] = { "UN", "CU", "NL", "CL", "NR", "CR", "CO", };

    int syncStatus = SyncList.STATUS_UNKNOWN;

    // in all cases except where syncStatys == STATUS_NEW_REMOTE,
    // cabEntry is in a LOCAL cabinet.  Otherwise, it's temporarily
    // pointed at the remote cabinet entry
    private CabinetEntry localEntry;

    public SyncEntry(CabinetEntry ce)
    {
        setCabinetEntry(ce);
    }

    public boolean isPlaceHolder()
    {
        return isPlaceHolder;
    }

    public void setIsPlaceHolder(boolean tv)
    {
        isPlaceHolder = tv;
    }
    
    public long getLastSyncTime()
    {
        return this.lastSync;
    }
        
    public void setLastSyncTime(long date)
    {
        this.lastSync = date;
    }
        
    protected void setStatus(int s)
    {
        // throw into conflic state if setting changed remote on
        // top of changed local...
        this.syncStatus = s;
        // will want to notify any gui listeners at this point
    }
    public int getStatus()
    {
        return this.syncStatus;
    }
        
    public void setCabinetEntry(CabinetEntry ce)
    {
        this.localEntry = ce;
    }
        
    public CabinetEntry getCabinetEntry()
    {
        return localEntry;
    }

    public ByteStore getByteStore()
    {
        // manage the cast exception / the problem of SyncEntries
        // not really being bytestores...
        return (ByteStore) getCabinetEntry();
    }

    public String toString()
    {
        if (getStatus() == SyncList.STATUS_NEW_REMOTE)
            // meaning, our CabinetEntry is a temporary bogus pointer to the remote cabinet
            return "SyncEntry<" + descriptionString() + "\t" + localEntry + ">";
        else
            return "SyncEntry(" + descriptionString() + "\t" + localEntry + ")";
    }

    public String descriptionString()
    {
        CabinetEntry ce = getCabinetEntry();
        String s = new String();
        long length = -1;
	    
        s += "'" + ce.getName() + "'\t";
        s += getStatusName();
        s += ce.isByteStore() ? " BS  " : " CAB ";
        s += ce.isCabinet() ? 'd' : '-';
        s += ce.canRead() ? 'r' : '-';
        s += ce.canWrite() ? 'w' : '-';

        String lastMtimeStr = "-----------------";
        String lastSyncTimeStr = "-----------------";
        try {
            if (ce.isByteStore()) {
                ByteStore bs = ((ByteStore)ce);
                length = bs.length();
                lastMtimeStr = dateFormatter.format(new Date(bs.getLastModifiedTime()));
                lastSyncTimeStr = dateFormatter.format(new Date(getLastSyncTime()));
            }
        } catch (Exception e) {
            lastMtimeStr = e.toString();
            e.printStackTrace();
        }

            
        s += " " + lastMtimeStr + " / " + lastSyncTimeStr;
        if (length >= 0)
            s += " len=" + length;
        return s;
    }


    public String getStatusName()
    {
        return STATUS_NAMES[syncStatus];
    }

    // support for the CabinetEntry interface
    // boy, wouldn't delegation be nice in java...
    public boolean isCabinet() { return localEntry.isCabinet(); }
    public boolean isByteStore() { return localEntry.isByteStore(); }
    public String getName() { return localEntry.getName(); }
    public String getPath() { return localEntry.getPath(); }
    public boolean canRead() { return localEntry.canRead(); }
    public boolean canWrite() { return localEntry.canWrite(); }
    public Cabinet getParent() throws FilingIOException, FilingPermissionDeniedException
    { return localEntry.getParent(); }

}
//-----------------------------------------------------------------------------
// END OF SyncEntry
//-----------------------------------------------------------------------------

