public class CShape
{
    private java.awt.image.BufferedImage bimg;
    private String id = "0";
    private java.awt.Font idFont = new java.awt.Font("Helvetica", java.awt.Font.BOLD, 24);

    //could move some of this to a local-dependent resource file

    public static final int CIRCLE = 0;
    public static final int SQUARE = 1;
    public static final int TRIANGLE = 2;
    public static final int DIAMOND = 3;
    public static final int PARRALLELOGRAM = 4;
    public static final int PENTAGON = 5;
    public static final int INVERTED_TRIANGLE = 6;
    public static final int TRAPEZOID = 7;
    
    public static final int THIN_EDGE = 0;
    public static final float THIN_EDGE_VALUE = 2.0f;
    public static final int THICK_EDGE = 1;
    public static final float THICK_EDGE_VALUE = 8.0f;
    
    public static final int BLUE = 0;
    public static final java.awt.Color BLUE_VALUE = java.awt.Color.blue;
    public static final int CYAN = 1;
    public static final java.awt.Color CYAN_VALUE = java.awt.Color.cyan;
    public static final int GREEN = 2;
    public static final java.awt.Color GREEN_VALUE = java.awt.Color.green;
    public static final int MAGENTA = 3;
    public static final java.awt.Color MAGENTA_VALUE = java.awt.Color.magenta;
    public static final int ORANGE = 4;
    public static final java.awt.Color ORANGE_VALUE = java.awt.Color.orange;
    public static final int PINK = 5;
    public static final java.awt.Color PINK_VALUE = java.awt.Color.pink;
    public static final int RED = 6;
    public static final java.awt.Color RED_VALUE = java.awt.Color.red;
    
    public static int DEFAULT_SHAPE = CIRCLE;
    public static int DEFAULT_COLOR = BLUE;
    public static int DEFAULT_EDGE = THIN_EDGE;
    
    public static String CIRCLE_STRING = "Circle";
    public static String SQUARE_STRING = "Square";
    public static String TRIANGLE_STRING = "Triangle";
    public static String DIAMOND_STRING = "Diamond";
    public static String PARRALLELOGRAM_STRING = "Parrallelogram";
    public static String PENTAGON_STRING = "Pentagon";
    public static String INVERTED_TRIANGLE_STRING = "Inverted_Triangle";
    public static String TRAPEZOID_STRING = "Trapezoid";
    
    public static String BLUE_STRING = "Blue";
    public static String CYAN_STRING = "Cyan";
    public static String GREEN_STRING = "Green";
    public static String MAGENTA_STRING = "Magenta";
    public static String ORANGE_STRING = "Orange";
    public static String PINK_STRING = "Pink";
    public static String RED_STRING = "Red";
    
    public static String THIN_STRING = "Thin";
    public static String THICK_STRING = "Thick";
    
    private int geometricShape = DEFAULT_SHAPE;
    private int fillColor = DEFAULT_COLOR;
    private int edgeThickness = DEFAULT_EDGE; 
    
    private String[] shapeStrings = {CIRCLE_STRING, SQUARE_STRING, TRIANGLE_STRING, 
    DIAMOND_STRING, PARRALLELOGRAM_STRING,
    PENTAGON_STRING, INVERTED_TRIANGLE_STRING, TRAPEZOID_STRING};
    private String[] colorStrings = {BLUE_STRING, CYAN_STRING, GREEN_STRING, MAGENTA_STRING,
    ORANGE_STRING, PINK_STRING, RED_STRING};
    private String[] edgeStrings = {THIN_STRING, THICK_STRING};
    
    public CShape()
    {
    }
    
    public CShape (int shape, int color, int thickness)  
    {
        geometricShape = shape;
        fillColor = color;
        edgeThickness = thickness;
    }

    public CShape (String s, int shape, int color, int thickness)  
    {
        id = s;
        geometricShape = shape;
        fillColor = color;
        edgeThickness = thickness;
    }

    public CShape (String idString, String shapeString, String colorString, String edgeString) 
    //needs to be more efficient and trap invalid data
    {
        id = idString;

        if (shapeString.equals(CIRCLE_STRING)) geometricShape = CIRCLE;
        if (shapeString.equals(SQUARE_STRING)) geometricShape = SQUARE;
        if (shapeString.equals(TRIANGLE_STRING)) geometricShape = TRIANGLE;
        if (shapeString.equals(DIAMOND_STRING)) geometricShape = DIAMOND;
        if (shapeString.equals(PARRALLELOGRAM_STRING)) geometricShape = PARRALLELOGRAM;
        if (shapeString.equals(PENTAGON_STRING)) geometricShape = PENTAGON;
        if (shapeString.equals(INVERTED_TRIANGLE_STRING)) geometricShape = INVERTED_TRIANGLE;
        if (shapeString.equals(TRAPEZOID_STRING)) geometricShape = TRAPEZOID;
        
        if (colorString.equals(BLUE_STRING)) fillColor = BLUE;
        if (colorString.equals(CYAN_STRING)) fillColor = CYAN;
        if (colorString.equals(GREEN_STRING)) fillColor = GREEN;
        if (colorString.equals(MAGENTA_STRING)) fillColor = MAGENTA;
        if (colorString.equals(ORANGE_STRING)) fillColor = ORANGE;
        if (colorString.equals(PINK_STRING)) fillColor = PINK;
        if (colorString.equals(RED_STRING)) fillColor = RED;

        if (edgeString.equals(THIN_STRING)) edgeThickness = THIN_EDGE;
        if (edgeString.equals(THICK_STRING)) edgeThickness = THICK_EDGE;
    }

    public int getShape()
    {        
        return geometricShape;
    }
    
    public String getShapeString()
    {
        return shapeStrings[geometricShape];
    }
        
    public void setShape(int s)
    {
        geometricShape = s;
    }
    
    public int getColor()
    {
        return fillColor;
    }
    
    public String getColorString()
    {
        return colorStrings[fillColor];
    }
    
    public void setColor(int c)
    {
        fillColor = c;
    }
    
    public int getEdge()
    {
        return this.edgeThickness;
    }
    
    public String getEdgeString()
    {
        return edgeStrings[edgeThickness];
    }
    
    public void setEdge(int t)
    {
        edgeThickness = t;
    }
    
    public void setIdString(String s) //only called for root and new objects
    {
        id = s;
    }
    
    public String getIdString()
    {
        return id;
    }
    
    public String toString()
    {
        return (id+":"+shapeStrings[geometricShape]+":"+colorStrings[fillColor]+":"+edgeStrings[edgeThickness]);
    }

    // draw a shape with a non-zero id, if any
    public void draw (java.awt.Graphics2D g2, float x, float y, float w, float h)
    {        
        if (id.equals("-1")) return;
        
        java.awt.Shape currentShape = null;
        switch (geometricShape)
        {
            case CIRCLE:
                currentShape = new java.awt.geom.Ellipse2D.Float(x,y,w,h);
                break;
            case SQUARE:
                currentShape = new java.awt.geom.Rectangle2D.Float(x,y,w,h);
                break;
            case TRIANGLE:
                java.awt.geom.GeneralPath triangle = new java.awt.geom.GeneralPath();
                triangle.moveTo(x+w/2,y);
                triangle.lineTo(x+w,y+h);
                triangle.lineTo(x,y+h);
                triangle.closePath();
                currentShape = triangle;
                break;
            case DIAMOND:
                java.awt.geom.GeneralPath diamond = new java.awt.geom.GeneralPath();
                diamond.moveTo(x+w/2,y);
                diamond.lineTo(x+w,y+h/2);
                diamond.lineTo(x+w/2,y+h);
                diamond.lineTo(x,y+h/2);
                diamond.closePath();
                currentShape = diamond;
                break;
            case PARRALLELOGRAM:
                java.awt.geom.GeneralPath parrallelogram = new java.awt.geom.GeneralPath();
                parrallelogram.moveTo(x+w/2,y);
                parrallelogram.lineTo(x+w,y);
                parrallelogram.lineTo(x+w/2,y+h);
                parrallelogram.lineTo(x,y+h);
                parrallelogram.closePath();
                currentShape = parrallelogram;
                break;
            case PENTAGON:
                java.awt.geom.GeneralPath pentagon = new java.awt.geom.GeneralPath();
                pentagon.moveTo(x+w/2,y);
                pentagon.lineTo(x+w,y+h/2);
                pentagon.lineTo(x+w,y+h);
                pentagon.lineTo(x,y+h);
                pentagon.lineTo(x,y+h/2);
                pentagon.closePath();
                currentShape = pentagon;
                break;
            case INVERTED_TRIANGLE:
                java.awt.geom.GeneralPath itriangle = new java.awt.geom.GeneralPath();
                itriangle.moveTo(x,y);
                itriangle.lineTo(x+w,y);
                itriangle.lineTo(x+w/2,y+h);
                itriangle.closePath();
                currentShape = itriangle;
                break;
            case TRAPEZOID:
                java.awt.geom.GeneralPath trapezoid = new java.awt.geom.GeneralPath();
                trapezoid.moveTo(x+w/4,y);
                trapezoid.lineTo(x+3*w/4,y);
                trapezoid.lineTo(x+w,y+h);
                trapezoid.lineTo(x,y+h);
                trapezoid.closePath();
                currentShape = trapezoid;
                break;
            default:
                break;
        }

        java.awt.Color currentColor = null;
        switch (fillColor)
        {
            case BLUE:
                currentColor = BLUE_VALUE;
                break;
            case CYAN:
                currentColor = CYAN_VALUE;
                break;
            case GREEN:
                currentColor = GREEN_VALUE;
                break;
            case MAGENTA:
                currentColor = MAGENTA_VALUE;
                break;
            case ORANGE:
                currentColor = ORANGE_VALUE;
                break;
            case PINK:
                currentColor = PINK_VALUE;
                break;
            case RED:
                currentColor = RED_VALUE;
                break;
            default:
                break;
        }
        
        switch (edgeThickness)
        {
            case THIN_EDGE: 
                g2.setStroke(new java.awt.BasicStroke(THIN_EDGE_VALUE));
                break;
            case THICK_EDGE:
                g2.setStroke(new java.awt.BasicStroke(THICK_EDGE_VALUE));
                break;
            default:
                break;
        }
        
        g2.setColor(java.awt.Color.black);
        g2.setFont(idFont);
        g2.drawString(id,x + 50,y + 32); 			
        g2.draw(currentShape);
        g2.setColor(currentColor);
        g2.fill(currentShape);
    }
}