public class ColorPropertyEditor extends javax.swing.JFrame 
implements javax.swing.event.ListSelectionListener
{
    private String[] colorStrings = {CShape.BLUE_STRING, CShape.CYAN_STRING, 
        CShape.GREEN_STRING,CShape.MAGENTA_STRING, 
        CShape.ORANGE_STRING, CShape.PINK_STRING, 
        CShape.RED_STRING};
    private static javax.swing.JTabbedPane tabbedPane = new javax.swing.JTabbedPane();
    private javax.swing.JList colorList;
    private javax.swing.DefaultListSelectionModel colorListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
    private ColorPropertyEditorRenderer colorRenderer = new ColorPropertyEditorRenderer();
    private int minWidth = 290;
    private int minHeight = 470;
    private javax.swing.JTree colorTree;
    private javax.swing.tree.DefaultTreeModel colorTreeModel = null;
    private javax.swing.tree.DefaultMutableTreeNode currentChild = null;
    private javax.swing.JFrame containingFrame = null;
 
    public ColorPropertyEditor(javax.swing.JTree tree, javax.swing.tree.DefaultTreeModel model)
    {
        super("Color Editor");
        colorTree = tree;
        colorTreeModel = model;
        setSize(minWidth,minHeight);
                
        colorList = new javax.swing.JList(colorStrings);
        colorListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        colorList.setSelectionModel(colorListSelectionModel);
        colorList.addListSelectionListener(this);
        colorListSelectionModel.addSelectionInterval(0,0);
        colorList.setFixedCellHeight(50);
        colorList.setCellRenderer(colorRenderer);
        colorList.setVisible(true);
        tabbedPane.addTab("Color", colorList);
        tabbedPane.setVisible(true);

        getContentPane().add(tabbedPane);

        //handle close gesture
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent ev) { 
                ApplySelections();
            }
        });        
    }

    private void ApplySelections()
    {
        // this code gets the last selection.  Maybe we want to do something else
        javax.swing.tree.TreePath path = colorTree.getLeadSelectionPath();
        if (path == null) return; // nothing selected
        javax.swing.tree.DefaultMutableTreeNode selectedNode = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
        CShape cshape = (CShape)selectedNode.getUserObject();
        cshape.setColor(colorList.getSelectedIndex());

        PropagateProperties(selectedNode,colorList.getSelectedIndex());
    }
             
    private void PropagateProperties(javax.swing.tree.DefaultMutableTreeNode parent, int color)
    {
        javax.swing.tree.DefaultMutableTreeNode root = 
            (javax.swing.tree.DefaultMutableTreeNode)colorTreeModel.getRoot();
        // for every node, find its most senior property parent and propagate properties back down
        java.util.Enumeration e = parent.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();

            CShape cshape = (CShape)nextNode.getUserObject();
            cshape.setColor(color);
        }
    }    
        
    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
    }

    public void setBounds(java.awt.Rectangle r)
    {
        r.width = minWidth;
        r.height = minHeight;
        super.setBounds(r);
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        super.setBounds(500, 5, minWidth, minHeight);
    }
    
    public void setColor (int s)
    {
        colorListSelectionModel.addSelectionInterval(0,s);
    }

    public int getColor()
    {
        return colorList.getSelectedIndex();
    }    

    public void setShape (int s)
    {
        colorRenderer.setShape(s);
    }

    public void setEdge (int e)
    {
        colorRenderer.setEdge(e);
    }
}