public class EdgePropertyEditor extends javax.swing.JFrame 
implements javax.swing.event.ListSelectionListener
{
    private String[] edgeStrings = {CShape.THIN_STRING, CShape.THICK_STRING};
    private static javax.swing.JTabbedPane tabbedPane = new javax.swing.JTabbedPane();
    private javax.swing.JList edgeList;
    private javax.swing.DefaultListSelectionModel edgeListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
    private EdgePropertyEditorRenderer edgeRenderer = new EdgePropertyEditorRenderer();
    private int minWidth = 290;
    private int minHeight = 470;
    private javax.swing.JTree edgeTree;
    private javax.swing.tree.DefaultTreeModel edgeTreeModel = null;
    private javax.swing.tree.DefaultMutableTreeNode currentChild = null;
    private javax.swing.JFrame containingFrame = null;
 
    public EdgePropertyEditor(javax.swing.JTree tree, javax.swing.tree.DefaultTreeModel model)
    {
        super("Edge Editor");
        edgeTree = tree;
        edgeTreeModel = model;
        setSize(minWidth,minHeight);
                
        edgeList = new javax.swing.JList(edgeStrings);
        edgeListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        edgeList.setSelectionModel(edgeListSelectionModel);
        edgeList.addListSelectionListener(this);
        edgeListSelectionModel.addSelectionInterval(0,0);
        edgeList.setFixedCellHeight(50);
        edgeList.setCellRenderer(edgeRenderer);
        edgeList.setVisible(true);
        tabbedPane.addTab("Edge", edgeList);
        tabbedPane.setVisible(true);

        getContentPane().add(tabbedPane);

        //handle close gesture
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent ev) { 
                ApplySelections();
            }
        });        
    }

    private void ApplySelections()
    {
        // this code gets the last selection.  Maybe we want to do something else
        javax.swing.tree.TreePath path = edgeTree.getLeadSelectionPath();
        if (path == null) return; // nothing selected
        javax.swing.tree.DefaultMutableTreeNode selectedNode = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
        CShape cshape = (CShape)selectedNode.getUserObject();
        cshape.setEdge(edgeList.getSelectedIndex());

        PropagateProperties(selectedNode,edgeList.getSelectedIndex());
    }
             
    private void PropagateProperties(javax.swing.tree.DefaultMutableTreeNode parent, int edge)
    {
        javax.swing.tree.DefaultMutableTreeNode root = 
            (javax.swing.tree.DefaultMutableTreeNode)edgeTreeModel.getRoot();
        // for every node, find its most senior property parent and propagate properties back down
        java.util.Enumeration e = parent.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();

            CShape cshape = (CShape)nextNode.getUserObject();
            cshape.setEdge(edge);
        }
    }    
        
    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
    }

    public void setBounds(java.awt.Rectangle r)
    {
        r.width = minWidth;
        r.height = minHeight;
        super.setBounds(r);
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        super.setBounds(500, 5, minWidth, minHeight);
    }
    
    public void setEdge (int s)
    {
        edgeListSelectionModel.addSelectionInterval(0,s);
    }

    public int getEdge()
    {
        return edgeList.getSelectedIndex();
    }    

    public void setShape (int s)
    {
        edgeRenderer.setShape(s);
    }

    public void setColor (int c)
    {
        edgeRenderer.setColor(c);
    }
}