public class HierarchyDemo extends javax.swing.JFrame 
implements java.awt.event.MouseListener, 
           java.awt.event.KeyListener, 
           javax.swing.event.ListSelectionListener
{
    private static javax.swing.JTabbedPane tabbedPane = new javax.swing.JTabbedPane();

    private javax.swing.JTree shapeTree = null;
    private javax.swing.JTree colorTree = null;
    private javax.swing.JTree edgeTree = null;
    
    private javax.swing.tree.DefaultTreeModel shapeHierarchy = null;
    private javax.swing.tree.DefaultTreeModel colorHierarchy = null;
    private javax.swing.tree.DefaultTreeModel edgeHierarchy = null;
    
    private ShapePropertyEditor shapePropertyEditor = null;
    private ColorPropertyEditor colorPropertyEditor = null;
    private EdgePropertyEditor edgePropertyEditor = null;
    
    private String[] nodeStrings = {"1","2","3","4","5","6","7"};
    private javax.swing.JList nodeList;
    private javax.swing.DefaultListSelectionModel nodeListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
    
    private final static HierarchyDemo demo = new HierarchyDemo();
    private HierarchyStore hstore = new HierarchyStore();
        
    public HierarchyDemo()
    {
        super ("Hierarchy API Demo version 0.1");
    }
    
    private void init ()
    {
        shapeHierarchy = hstore.getShapeHierarchy();
        shapeTree = new javax.swing.JTree(shapeHierarchy);
        shapeTree.setCellRenderer(new CShapeTreeCellRenderer());
        shapeTree.setRowHeight(50);
        shapeTree.setSelectionRow(1);
        shapeTree.setVisible(true);
        shapePropertyEditor = new ShapePropertyEditor(shapeTree,shapeHierarchy);
        
        colorHierarchy = hstore.getColorHierarchy();
        colorTree = new javax.swing.JTree(colorHierarchy);
        colorTree.setCellRenderer(new CShapeTreeCellRenderer());
        colorTree.setRowHeight(50);
        colorTree.setSelectionRow(1);
        colorTree.setVisible(true);
        colorPropertyEditor = new ColorPropertyEditor(colorTree,colorHierarchy);

        edgeHierarchy = hstore.getEdgeHierarchy();
        edgeTree = new javax.swing.JTree(edgeHierarchy);
        edgeTree.setCellRenderer(new CShapeTreeCellRenderer());
        edgeTree.setRowHeight(50);
        edgeTree.setSelectionRow(1);
        edgeTree.setVisible(true);
        edgePropertyEditor = new EdgePropertyEditor(edgeTree,edgeHierarchy);
 
        ExpandTree(shapeTree);
        ExpandTree(colorTree);
        ExpandTree(edgeTree);
        
        nodeList = new javax.swing.JList(hstore.getNodes());
        nodeListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        nodeList.setSelectionModel(nodeListSelectionModel);
        nodeListSelectionModel.addSelectionInterval(0,0);
        nodeList.setFixedCellHeight(50);
        nodeList.setCellRenderer(new NodeRenderer());
        nodeList.setVisible(true);

        tabbedPane.addTab("Nodes", nodeList);
        tabbedPane.addTab("Shape",shapeTree);
        tabbedPane.addTab("Color",colorTree);
        tabbedPane.addTab("Edge",edgeTree);
        tabbedPane.setVisible(true);

        shapeTree.addMouseListener(this);                
        shapeTree.addKeyListener(this);  
        colorTree.addMouseListener(this);                
        colorTree.addKeyListener(this);    
        edgeTree.addMouseListener(this);                
        edgeTree.addKeyListener(this);    
        nodeList.addListSelectionListener(this);  

        getContentPane().add(tabbedPane);
    }

    private void ExpandTree(javax.swing.JTree tree)
    {
        //expand entire tree, kludge, but could find a better way
        int currentRowCount = 0;
        int previousRowCount = -1;
        while (previousRowCount != currentRowCount)
        {
            previousRowCount = tree.getRowCount();
            for (int r = 0; r < previousRowCount; r++) tree.expandRow(r);
            currentRowCount = tree.getRowCount();
        }
    }

    private void EditProperties(java.awt.Component component)
    {
        // this code gets the last selection.  Maybe we want to do something else
        // before showing the editor, set initial values based on those of the selection
        javax.swing.tree.TreePath path = shapeTree.getLeadSelectionPath();
        if (path == null) return; // nothing selected
        javax.swing.tree.DefaultMutableTreeNode selectedNode = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
        CShape cshape = (CShape)selectedNode.getUserObject();

        if (component == shapeTree)
        {
            shapePropertyEditor.setShape(cshape.getShape());
            shapePropertyEditor.setColor(cshape.getColor());
            shapePropertyEditor.setEdge(cshape.getEdge());
            shapePropertyEditor.setVisible(true);
        }
        else if (component == colorTree)
        {
            colorPropertyEditor.setShape(cshape.getShape());
            colorPropertyEditor.setColor(cshape.getColor());
            colorPropertyEditor.setEdge(cshape.getEdge());
            colorPropertyEditor.setVisible(true);
        }
        else if (component == edgeTree) 
        {
            edgePropertyEditor.setShape(cshape.getShape());
            edgePropertyEditor.setColor(cshape.getColor());
            edgePropertyEditor.setEdge(cshape.getEdge());
            edgePropertyEditor.setVisible(true);
        }
    }

    public void mouseClicked(java.awt.event.MouseEvent me)
    {
        if (me.isControlDown()) 
        {
            EditProperties(me.getComponent());
        }
    }

    public void mousePressed(java.awt.event.MouseEvent me){}
    public void mouseReleased(java.awt.event.MouseEvent me){}
    public void mouseEntered(java.awt.event.MouseEvent me){}
    public void mouseExited(java.awt.event.MouseEvent me){}    
    public void keyTyped(java.awt.event.KeyEvent ke){}    
    public void keyPressed(java.awt.event.KeyEvent ke){}
    public void keyReleased(java.awt.event.KeyEvent ke){}
    
    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
        int index = nodeList.getSelectedIndex();
        CShape target = hstore.getNode(index);
        
        UpdateSelectionInTrees(shapeTree, shapeHierarchy, target);
        UpdateSelectionInTrees(colorTree, colorHierarchy, target);
        UpdateSelectionInTrees(edgeTree, edgeHierarchy, target);
    }
            
    // find a target CShape in a tree model and select that node in a tree
    private void UpdateSelectionInTrees(javax.swing.JTree tree,
                                        javax.swing.tree.DefaultTreeModel model,
                                        CShape target)
    {
        javax.swing.tree.DefaultMutableTreeNode root = 
            (javax.swing.tree.DefaultMutableTreeNode)model.getRoot();
        java.util.Enumeration e = root.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode node = 
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();
            CShape cshape = (CShape)node.getUserObject();
            if (cshape == target)
            {
                javax.swing.tree.TreePath p = new javax.swing.tree.TreePath(model.getPathToRoot(node));
                tree.setSelectionPath(p);
            }
        }
    }

    public static void main(String args[]) 
    {
        demo.addWindowListener(
            new java.awt.event.WindowAdapter() 
            {
                public void windowClosing(java.awt.event.WindowEvent ev) 
                {
                    System.exit(0);
                }
            }
        );
        demo.init();
        demo.setSize(450,500);
        demo.setVisible(true);
    }
}
