public class HierarchyGraph extends javax.swing.JScrollPane
{
    private java.awt.image.BufferedImage bimg = null;
    private javax.swing.tree.DefaultTreeModel hierarchy = null;
    private float columnWidthPerRoot;
    private float rowHeight;
    private int[][] parentsForRootAtLevel;
    private java.util.Vector rootNodesVector = new java.util.Vector();
    private int numLevels;
    private int numRoots;
    private int width = 1000;
    private int height = 600;
    
    public HierarchyGraph(javax.swing.tree.DefaultTreeModel model)
    {
        super();
        hierarchy = model;
        ScaleGraph(width,height);
    }
    
    private void ScaleGraph(int w, int h)
    {
        javax.swing.tree.DefaultMutableTreeNode nextNode = null;
  
        javax.swing.tree.DefaultMutableTreeNode root = (javax.swing.tree.DefaultMutableTreeNode)hierarchy.getRoot();
        numRoots = root.getChildCount();
        
        columnWidthPerRoot = w / (numRoots + 1);
        numLevels = root.getDepth();
        rowHeight = h / (numLevels + 1);
        int level;
        
        parentsForRootAtLevel = new int[numRoots][numLevels];
        for (int i=0; i<numRoots; i++)
        {
            for (int j=0; j<numLevels; j++)
            {
                parentsForRootAtLevel[i][j]=0;
            }
        }

        java.util.Enumeration e = root.breadthFirstEnumeration();
        // skip root which has no meaning in hirarchy
        nextNode = (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();  
        while (e.hasMoreElements())
        {
            nextNode = (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();
            level = nextNode.getLevel();
            // if this is one level below the root, it is a hierarchy root, 
            // store this away for later comparisons   
            if (level == 1)
            {
                rootNodesVector.add(nextNode);
            }
            else
            {
                // find the root child for whom this is a parent
                javax.swing.tree.TreeNode[] pathToRoot = hierarchy.getPathToRoot(nextNode);
                int pathLength = pathToRoot.length-2;
                javax.swing.tree.DefaultMutableTreeNode n = (javax.swing.tree.DefaultMutableTreeNode)pathToRoot[1];
                int numRoot = rootNodesVector.indexOf(n);
System.out.println(nextNode + " is parent of " + n + " which is root number " + numRoot);
System.out.println("level is " + level);
                parentsForRootAtLevel[numRoot][level-1] += 1;
            }
        }
    }
    
    private void draw (java.awt.Graphics2D g2, int w, int h)
    {
        float xValue;
        float yValue;
        DemoShape currentShape;
        CShape cshape;
        int rootNum = 0;
        int level;

        int[][] numParentForRootAtLevel = new int[numRoots][numLevels];
        for (int i=0; i<numRoots; i++)
        {
            for (int j=0; j<numLevels; j++)
            {
                numParentForRootAtLevel[i][j]=0;
            }
        }

        javax.swing.tree.DefaultMutableTreeNode nextNode = null;
        javax.swing.tree.DefaultMutableTreeNode root = (javax.swing.tree.DefaultMutableTreeNode)hierarchy.getRoot();
        
        java.util.Enumeration e = root.breadthFirstEnumeration();
        // skip root which has no meaning in hirarchy
        nextNode = (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();  
        while (e.hasMoreElements())
        {
            nextNode = (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();
            level = nextNode.getLevel();
            // for hierarchy roots
            if (level == 1)
            {
                xValue = columnWidthPerRoot * ++rootNum;
                yValue = h - rowHeight;

                currentShape = (DemoShape)nextNode.getUserObject();
                cshape = new CShape(currentShape.getShape(),currentShape.getColor(),currentShape.getEdge());
                cshape.setIdString(currentShape.getIdString());
//System.out.println("x is " + xValue);
//System.out.println("y is " + yValue);
                cshape.draw(g2, xValue, yValue, 40, 40);

            }
            // for greater levels, match this to a root
            else
            {   
                // find the root child for whom this is a parent
                javax.swing.tree.TreeNode[] pathToRoot = hierarchy.getPathToRoot(nextNode);
                int pathLength = pathToRoot.length-2;
                javax.swing.tree.DefaultMutableTreeNode n = (javax.swing.tree.DefaultMutableTreeNode)pathToRoot[1];
                rootNum = rootNodesVector.indexOf(n);

                numParentForRootAtLevel[rootNum][level-1] += 1;
System.out.println("level is " + level);
System.out.println("root number is " + rootNum);
System.out.println("numParentForRootAtLevel " + numParentForRootAtLevel[rootNum][level-1]);
System.out.println("parentForRootAtLevel " + parentsForRootAtLevel[rootNum][level-1]);

                float fraction = (float)numParentForRootAtLevel[rootNum][level-1] / 									 (float)parentsForRootAtLevel[rootNum][level-1];
                System.out.println(fraction);
                xValue = columnWidthPerRoot * (fraction + rootNum);
                yValue = h - (rowHeight * level);

                currentShape = (DemoShape)nextNode.getUserObject();
                cshape = new CShape(currentShape.getShape(),currentShape.getColor(),currentShape.getEdge());
                cshape.setIdString(currentShape.getIdString());
System.out.println("x is " + xValue);
System.out.println("y is " + yValue);
                cshape.draw(g2, xValue, yValue, 40, 40);
            }
        }
    }
    
    public void paint (java.awt.Graphics g)
    {
        java.awt.Graphics2D g2 = null;
        int w = width;
        int h = height;
        if (bimg == null || bimg.getWidth() != w || bimg.getHeight() != h) {
            bimg = (java.awt.image.BufferedImage) createImage(w, h);
        } 
        g2 = bimg.createGraphics();
        g2.setBackground(getBackground());
        g2.clearRect(0, 0, w, h);
        g2.setRenderingHint(java.awt.RenderingHints.KEY_ANTIALIASING,
                            java.awt.RenderingHints.VALUE_ANTIALIAS_ON);
        draw(g2,width,height);
        g2.dispose();
        g.drawImage(bimg, 0, 0, this);
    }
}