public class ShapePropertyEditor extends javax.swing.JFrame 
implements javax.swing.event.ListSelectionListener
{
    private String[] shapeStrings = {DemoShape.CIRCLE_STRING, DemoShape.SQUARE_STRING, 
        DemoShape.TRIANGLE_STRING,DemoShape.DIAMOND_STRING, 
        DemoShape.PARRALLELOGRAM_STRING, DemoShape.PENTAGON_STRING, 
        DemoShape.INVERTED_TRIANGLE_STRING,
        DemoShape.TRAPEZOID_STRING};
    private static javax.swing.JTabbedPane tabbedPane = new javax.swing.JTabbedPane();
    private javax.swing.JList shapeList;
    private javax.swing.DefaultListSelectionModel shapeListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
    private ShapePropertyEditorRenderer shapeRenderer = new ShapePropertyEditorRenderer();
    private int minWidth = 290;
    private int minHeight = 470;
    private javax.swing.JTree shapeTree;
    private javax.swing.tree.DefaultTreeModel shapeTreeModel = null;
    private javax.swing.tree.DefaultMutableTreeNode currentChild = null;
    private javax.swing.JFrame containingFrame = null;
 
    public ShapePropertyEditor(javax.swing.JTree tree, javax.swing.tree.DefaultTreeModel model)
    {
        super("Shape Editor");
        shapeTree = tree;
        shapeTreeModel = model;
        setSize(minWidth,minHeight);
                
        shapeList = new javax.swing.JList(shapeStrings);
        shapeListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        shapeList.setSelectionModel(shapeListSelectionModel);
        shapeList.addListSelectionListener(this);
        shapeListSelectionModel.addSelectionInterval(0,0);
        shapeList.setFixedCellHeight(50);
        shapeList.setCellRenderer(shapeRenderer);
        shapeList.setVisible(true);
        tabbedPane.addTab("Shape", shapeList);
        tabbedPane.setVisible(true);

        getContentPane().add(tabbedPane);

        //handle close gesture
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent ev) { 
                ApplySelections();
            }
        });        
    }

    private void ApplySelections()
    {
        // this code gets the last selection.  Maybe we want to do something else
        javax.swing.tree.TreePath path = shapeTree.getLeadSelectionPath();
        if (path == null) return; // nothing selected
        javax.swing.tree.DefaultMutableTreeNode selectedNode = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
        CShape cshape = (CShape)selectedNode.getUserObject();
        cshape.setShape(shapeList.getSelectedIndex());

        PropagateProperties(selectedNode,shapeList.getSelectedIndex());
    }
             
    private void PropagateProperties(javax.swing.tree.DefaultMutableTreeNode parent, int shape)
    {
        javax.swing.tree.DefaultMutableTreeNode root = 
            (javax.swing.tree.DefaultMutableTreeNode)shapeTreeModel.getRoot();
        // for every node, find its most senior property parent and propagate properties back down
        java.util.Enumeration e = parent.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();

            CShape nextShape = (CShape)nextNode.getUserObject();
            nextShape.setShape(shape);
        }
    }    
        
    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
    }

    public void setBounds(java.awt.Rectangle r)
    {
        r.width = minWidth;
        r.height = minHeight;
        super.setBounds(r);
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        super.setBounds(500, 5, minWidth, minHeight);
    }
    
    public void setShape (int s)
    {
        shapeListSelectionModel.addSelectionInterval(0,s);
    }

    public int getShape()
    {
        return shapeList.getSelectedIndex();
    }    

    public void setColor (int c)
    {
        shapeRenderer.setColor(c);
    }

    public void setEdge (int e)
    {
        shapeRenderer.setEdge(e);
    }
}