public class BuilderDemo extends javax.swing.JFrame 
implements java.awt.event.ActionListener
{
    private static HierarchyBuilder builder = new HierarchyBuilder();
    
    private String fileMenuName;
    private String[] fileMenuItems;
    private char[] fileMenuShortCuts;
    private javax.swing.JMenu fileMenu;

    private String nodeMenuName;
    private javax.swing.JMenu nodeMenu;
    private javax.swing.JCheckBoxMenuItem menuItemNodeIdVisible;
    private javax.swing.JCheckBoxMenuItem menuItemNodeDescriptionVisible;
    private javax.swing.JCheckBoxMenuItem menuItemNodeKeyVisible;
    private javax.swing.JCheckBoxMenuItem menuItemDrawWithProperties;
    
    private String buildMenuName;
    private String[] buildMenuItems;
    private char[] buildMenuShortCuts;
    private javax.swing.JMenu buildMenu;
    
    private static javax.swing.JMenuBar menuBar;
    
    private static javax.swing.JLabel inStreamLabel = new javax.swing.JLabel();
    private static javax.swing.JLabel outStreamLabel = new javax.swing.JLabel();
    private static javax.swing.JLabel inHierarchyLabel = new javax.swing.JLabel();
    private static javax.swing.JLabel outHierarchyLabel = new javax.swing.JLabel();
    private static boolean dataModified = false;
    
    private final static BuilderDemo demo = new BuilderDemo();
    private static java.util.Properties propertiesFile = new java.util.Properties();
    private static String inStream = null;
    private static String outStream = null;
    private static String inHierarchyFilename = null;
    private static String outHierarchyFilename = null;
    
    private boolean showObjectProperties = true;
    
    public BuilderDemo()
    {
        super ("Builder Demo");
    }
    
    private void initMenus ()
    {
        fileMenuName = propertiesFile.getProperty("file_menu_name");
        fileMenuItems = new String[] {
            propertiesFile.getProperty("file_menu_item_1"),
            propertiesFile.getProperty("file_menu_item_2"),
            propertiesFile.getProperty("file_menu_item_3"),
            propertiesFile.getProperty("file_menu_item_4")
        };
        
        fileMenuShortCuts = new char[] {
            propertiesFile.getProperty("file_menu_shortcut_1").charAt(0),
            propertiesFile.getProperty("file_menu_shortcut_2").charAt(0),
            propertiesFile.getProperty("file_menu_shortcut_3").charAt(0),
            propertiesFile.getProperty("file_menu_shortcut_4").charAt(0)
        };
        
        nodeMenuName = propertiesFile.getProperty("node_menu_name");

        buildMenuName = propertiesFile.getProperty("build_menu_name");
        buildMenuItems = new String[] {
            propertiesFile.getProperty("build_menu_item_1"),
            propertiesFile.getProperty("build_menu_item_2"),
            propertiesFile.getProperty("build_menu_item_3"),
            propertiesFile.getProperty("build_menu_item_4"),
            propertiesFile.getProperty("build_menu_item_5")
        };
        
        buildMenuShortCuts = new char[] {
            propertiesFile.getProperty("build_menu_shortcut_1").charAt(0),
            propertiesFile.getProperty("build_menu_shortcut_2").charAt(0),
            propertiesFile.getProperty("build_menu_shortcut_3").charAt(0),
            propertiesFile.getProperty("build_menu_shortcut_4").charAt(0),
            propertiesFile.getProperty("build_menu_shortcut_5").charAt(0)
        };
        
        fileMenu = new javax.swing.JMenu(fileMenuName);
        nodeMenu = new javax.swing.JMenu(nodeMenuName);    
        buildMenu = new javax.swing.JMenu(buildMenuName);
        menuBar = new javax.swing.JMenuBar();

        //mnemonics and accelerators for each menu
        for (int i=0; i < fileMenuItems.length; i++)
        {
            javax.swing.JMenuItem menuItem = new javax.swing.JMenuItem(fileMenuItems[i], fileMenuShortCuts[i]);
            menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(fileMenuShortCuts[i],
            java.awt.Event.CTRL_MASK, false));
            menuItem.addActionListener(this);
            fileMenu.add(menuItem);
        }
        menuBar.add(fileMenu);
        
        menuItemNodeIdVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_1"));
        menuItemNodeIdVisible.addActionListener(this);
        menuItemNodeIdVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_node_id_visible_state")).booleanValue());
        
        menuItemNodeDescriptionVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_2"));
        menuItemNodeDescriptionVisible.addActionListener(this);
        menuItemNodeDescriptionVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_node_description_state")).booleanValue());
        
        menuItemNodeKeyVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_3"));
        menuItemNodeKeyVisible.addActionListener(this);
        menuItemNodeKeyVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_node_key_visible_state")).booleanValue());
        
        menuItemDrawWithProperties = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_4"));
        menuItemDrawWithProperties.addActionListener(this);
        menuItemDrawWithProperties.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_draw_with_properties_state")).booleanValue());

        nodeMenu.add(menuItemNodeIdVisible);
        nodeMenu.add(menuItemNodeDescriptionVisible);
        nodeMenu.add(menuItemNodeKeyVisible);
        nodeMenu.add(menuItemDrawWithProperties);
        menuBar.add(nodeMenu);

        //mnemonics and accelerators for each menu
        for (int i=0; i < buildMenuItems.length; i++)
        {
            javax.swing.JMenuItem menuItem = new javax.swing.JMenuItem(buildMenuItems[i], buildMenuShortCuts[i]);
            menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(buildMenuShortCuts[i],
            java.awt.Event.CTRL_MASK, false));
            menuItem.addActionListener(this);
            buildMenu.add(menuItem);
        }                
        menuBar.add(buildMenu);
    }
    
    private void init()
    {
        if (inStream == null)
        {
            builder = new HierarchyBuilder();
        }
        else
        {
            builder = new HierarchyBuilder(inStream,inHierarchyFilename);
        }
        builder.setVisible(true);
        getContentPane().add(builder);
    }
    
    private void configure()
    {
        //load some configuration values
        CShape cshape = new CShape();

        cshape.setShowId(menuItemNodeIdVisible.getState());
        cshape.setShowKey(menuItemNodeKeyVisible.getState());
        cshape.setShowNodeDescription(menuItemNodeDescriptionVisible.getState());
        cshape.setShowObjectProperties(menuItemDrawWithProperties.getState());

        cshape.setKeyFont(
            propertiesFile.getProperty("key_font_name"),
            new java.lang.Integer(propertiesFile.getProperty("key_font_style")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("key_font_size")).intValue());
        cshape.setIdFont(
            propertiesFile.getProperty("id_font_name"),
            new java.lang.Integer(propertiesFile.getProperty("id_font_style")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("id_font_size")).intValue());
        cshape.setDescriptionFont(
            propertiesFile.getProperty("description_font_name"),
            new java.lang.Integer(propertiesFile.getProperty("description_font_style")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("description_font_size")).intValue());
        setSize(
            new java.lang.Integer(propertiesFile.getProperty("application_frame_width")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("application_frame_height")).intValue());

        cshape.setTextDrawingConstants(
            new java.lang.Integer(propertiesFile.getProperty("draw_id_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_id_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_key_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_key_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line1_length")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line1_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line1_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2_length")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2a_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2a_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2b_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2b_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3_length")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3a_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3a_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3b_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3b_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3c_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3c_y_offset")).intValue());
    }

    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        String command = ae.getActionCommand();
        if (propertiesFile.getProperty("file_menu_item_1").equals(command)) newBuilder();
        if (propertiesFile.getProperty("file_menu_item_2").equals(command)) open(); 
        if (propertiesFile.getProperty("file_menu_item_3").equals(command)) save(); 
        if (propertiesFile.getProperty("file_menu_item_4").equals(command)) exit();
        if (propertiesFile.getProperty("node_menu_item_1").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowId(menuItemNodeIdVisible.getState());
        }
        if (propertiesFile.getProperty("node_menu_item_2").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowNodeDescription(menuItemNodeDescriptionVisible.getState());
        }
        if (propertiesFile.getProperty("node_menu_item_3").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowKey(menuItemNodeKeyVisible.getState());
        }
        if (propertiesFile.getProperty("node_menu_item_4").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowObjectProperties(menuItemDrawWithProperties.getState());
        }
        if (propertiesFile.getProperty("build_menu_item_1").equals(command)) builder.createRootNode(); 
        if (propertiesFile.getProperty("build_menu_item_2").equals(command)) builder.addChild(); 
        if (propertiesFile.getProperty("build_menu_item_3").equals(command)) builder.addParent();
        if (propertiesFile.getProperty("build_menu_item_4").equals(command)) builder.remove();
        if (propertiesFile.getProperty("build_menu_item_5").equals(command)) builder.removeParent();
    }
    
    private void cleanStart()
    {
        if (builder != null)
        {	
            builder.setVisible(false);
            builder = null;
        }
        setDataModified(false);
        getContentPane().removeAll();
        initStreamInfo();
    }
    
    private void newBuilder()
    {
        save();
        cleanStart();
        inStream = null;
        init();
        paintComponents(getGraphics());
    }
    
    private void open()
    {
        javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
        chooser.setMultiSelectionEnabled(false);
        String extensions[] = {"store"};
        chooser.addChoosableFileFilter(new FileFilterExt(extensions, "Data (*.store)"));
        int option = chooser.showOpenDialog(this);
        if (option == javax.swing.JFileChooser.APPROVE_OPTION) 
        {
            if (chooser.getSelectedFile() != null)
            {
                inStream = chooser.getSelectedFile().getName();
                extensions[0] = "hierarchy";
                chooser.addChoosableFileFilter(new FileFilterExt(extensions, "Hierarchy (*.hierarchy)"));
                option = chooser.showOpenDialog(this);
                if (option == javax.swing.JFileChooser.APPROVE_OPTION) 
                {
                    if (chooser.getSelectedFile() != null)
                    {
                        inHierarchyFilename = chooser.getSelectedFile().getName();
                        cleanStart();
                        init();
                        setVisible(true);
                    }
                }
            }
        }
    }
        
    private void save()
    {
        try
        {
            if (builder.isModified())
            {
                if (outStream == null)
                {
                    javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
                    chooser.setMultiSelectionEnabled(false);
                    chooser.setFileSelectionMode(javax.swing.JFileChooser.FILES_ONLY);
                    String extensions[] = {"store"};
                    chooser.addChoosableFileFilter(new FileFilterExt(extensions, "Data (*.store)"));
                    int option = chooser.showSaveDialog(this);
                    if (option == javax.swing.JFileChooser.APPROVE_OPTION) 
                    {
                        if (chooser.getSelectedFile() != null)
                        {
                            outStream = chooser.getSelectedFile().getName();
                            extensions[0] = "hierarchy";
                            chooser.addChoosableFileFilter(new FileFilterExt
                                (extensions, "Hierarchies (*.hierarchy)"));
                            option = chooser.showSaveDialog(this);
                            if (option == javax.swing.JFileChooser.APPROVE_OPTION) 
                            {
                                if (chooser.getSelectedFile() != null)
                                {
                                    outHierarchyFilename = chooser.getSelectedFile().getName();
                                }
                            }
                        }
                    }
                }
                if ((outStream != null) && (outHierarchyFilename != null))
                {
                    builder.save(outStream,outHierarchyFilename);
                    setDataModified(false);
                }
            }
        }
        catch (Exception e)
        {
            System.out.println("Exception writing to "+outStream);
        }
    }
            
    private void exit()
    {
        save();
        System.exit(0);
    }
                   
    private void setDataModified(boolean b)
    {
        dataModified = b;
        updateHierarchyStreamInfo();
    }
    
    private void updateHierarchyStreamInfo()
    {
        String rNull = propertiesFile.getProperty("read_from_null_file_prompt");
        String r = propertiesFile.getProperty("read_from_file_prompt");
        String wNull = propertiesFile.getProperty("write_to_null_file_prompt");
        String w = propertiesFile.getProperty("write_to_file_prompt");
        
        String rhNull = propertiesFile.getProperty("read_from_null_hierarchy_prompt");
        String rh = propertiesFile.getProperty("read_from_hierarchy_prompt");
        String whNull = propertiesFile.getProperty("write_to_null_hierarchy_prompt");
        String wh = propertiesFile.getProperty("write_to_hierarchy_prompt");

        inStreamLabel.setText((inStream == null) ? rNull : r + ((dataModified) ? "*" : "") + inStream);
        outStreamLabel.setText((outStream == null) ? wNull : w + outStream);
        inHierarchyLabel.setText((inHierarchyFilename == null) ? rhNull : rh + ((dataModified) ? "*" : "") + inHierarchyFilename);
        outHierarchyLabel.setText((outHierarchyFilename == null) ? whNull : wh + outHierarchyFilename);
    }
    
    private void initStreamInfo()
    {
        updateHierarchyStreamInfo();
        javax.swing.JPanel p = new javax.swing.JPanel(new java.awt.GridLayout(2,2));
        p.add(inStreamLabel);
        p.add(outStreamLabel);
        p.add(inHierarchyLabel);
        p.add(outHierarchyLabel);
        getContentPane().add(p,java.awt.BorderLayout.NORTH);
    }
    
    public static void main(String args[]) 
    {
        demo.addWindowListener(
            new java.awt.event.WindowAdapter() 
            {
                public void windowClosing(java.awt.event.WindowEvent ev) 
                {
                    if (dataModified) demo.save();
                    System.exit(0);
                }
            }
        );
        try
        {
            propertiesFile.load(new java.io.FileInputStream("BuilderDemo.properties"));
        }
        catch (Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }
        demo.initMenus();
        demo.setJMenuBar(menuBar);
        demo.configure();
        
        if (args.length >= 1) 
        {
            inStream = args[0];
            if (args.length == 2)
            {
                inHierarchyFilename = args[1];
            }
        }
        demo.initStreamInfo();
        demo.init();
        demo.setVisible(true);
    }
}
