public class HierarchyBuilder extends javax.swing.JSplitPane 
{
    private DataStore store = new DataStore();
    
    private javax.swing.JTree tree = null;
    private javax.swing.tree.DefaultTreeModel hierarchy = null;
    private javax.swing.tree.DefaultMutableTreeNode root = null;
        
    private Integer nextId = new Integer(1);
    private javax.swing.JList nodeList;
    private javax.swing.DefaultListModel nodeListModel = new javax.swing.DefaultListModel();
    private javax.swing.DefaultListSelectionModel nodeListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
        
    private java.util.Properties propertiesFile = new java.util.Properties();
    private java.util.Vector idsVector = new java.util.Vector();
    private static boolean dataModified = false;
    private String mostRecentlyUsedKey;
        
    private boolean showErrorsWithDialog;
    private boolean showErrorsWithSystemOut;
    
    private String inHierarchy = null;
    private String outHierarchy = null;

    // set aside for platform-sepcific issues
    private int os;
    private static int WINDOWS = 0;
    private static int MAC = 1;    
    
/*
    For create new root node:
    
    If "new object" is selected in node list, a new object is created, its key is set to a random value, the
    object is added to the database (store), and to the hierarhcy (tree panel). If one of the already loaded
    objects is selected (if any) it is added to the tree as a new root node.  The same node can appear more than
    once as a root node.  Selection changes to the created node.
    
    For add child:
    
    This adds a new or exisitng node as a child to the currently selected hierarchy node.  Some node in the hierarchy
    must be selected (to add a child as a root, use create root node).  The same node can appear more than once in
    the hierarchy but not more than once in a given path to a root.  Selection changes to the added node.
    
    For add parent:
    
    This adds a new or exisitng node as a parent to the currently selected hierarchy node.  Some node in the
    hierarchy must be selected.  The same node can appear more than once in the hierarchy but not more than once
    in a given path to a root.  Selection changes to the added node.
    
    For remove node and its children:
    
    This removes the selected hierarchy node and its children.  
    
    For remove parent:
    
    This removes the parent of the selected hierarchy node and its children.  The selected node cannot be a root
    node (as it has no parent).  
    
*/
    public HierarchyBuilder()
    {
        super ();
        init();
    }
    
    public HierarchyBuilder(String dataStore, String hierarchy)
    {
        super ();
        init();
        store.load(dataStore);
        CShape nodes[] = store.getNodes();
        for (int i=0; i < nodes.length; i++)
        {
            nodeListModel.addElement("key: " + nodes[i].getKey() + " id: "+ nodes[i].getIdString());
        }
        inHierarchy = hierarchy;
        outHierarchy = hierarchy;
        loadHierarchy(inHierarchy);    
    }
    
    private void init()
    {
        try
        {
            propertiesFile.load(new java.io.FileInputStream("HierarchyBuilder.properties"));
        }
        catch (Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }

        String osName = System.getProperty("os.name");
        os = (osName.startsWith("Mac")) ? MAC : WINDOWS;

        // create a real root node for the tree model representing the hierarchy -- an Id of -1 means don't draw
        root = new javax.swing.tree.DefaultMutableTreeNode();
        CShape cshape = new CShape();
        cshape.setIdString("-1");
        root.setUserObject(cshape);
        hierarchy = new javax.swing.tree.DefaultTreeModel(root);

        tree = new javax.swing.JTree(hierarchy);
        tree.setCellRenderer(new CShapeTreeCellRenderer());
        tree.setCellEditor(new CShapeTreeCellTextEditor());
        tree.setEditable(true);
        tree.setRowHeight(50);
        tree.setSelectionRow(0);
        tree.setVisible(true);
        
        nodeList = new javax.swing.JList(nodeListModel);
        nodeListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        nodeList.setSelectionModel(nodeListSelectionModel);
        nodeListSelectionModel.addSelectionInterval(0,0);

        // start the list of nodes with a placeholder to signify a new node 
        cshape = new CShape();
        cshape.setIdString(propertiesFile.getProperty("new_object_tag"));
        nodeListModel.addElement(propertiesFile.getProperty("new_object_tag"));
        nodeList.setVisible(true);

        setOrientation(HORIZONTAL_SPLIT);
        setLeftComponent(nodeList);
        setRightComponent(tree);
        setVisible(true);

        showErrorsWithDialog = 
            new java.lang.Boolean(propertiesFile.getProperty("showErrorsWithDialog")).booleanValue();
        showErrorsWithSystemOut = 
            new java.lang.Boolean(propertiesFile.getProperty("showErrorsWithSystemOut")).booleanValue();
    }

    private void expandTree()
    {
        setDataModified(true);
        
        // expand entire tree, kludge, but could find a better way
        int currentRowCount = 0;
        int previousRowCount = -1;
        while (previousRowCount != currentRowCount)
        {
            previousRowCount = tree.getRowCount();
            for (int r = 0; r < previousRowCount; r++) tree.expandRow(r);
            currentRowCount = tree.getRowCount();
        }
        
        // select most recently used node, or nothing
        boolean found = false;
        java.util.Enumeration e = root.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();
            CShape cshape = (CShape)nextNode.getUserObject();
            if (cshape.getKey().equals(mostRecentlyUsedKey))
            {
                javax.swing.tree.TreePath path = 
                    new javax.swing.tree.TreePath(hierarchy.getPathToRoot(nextNode));
                tree.setSelectionPath(path);
                found = true;
                break;
            }
        }
        if (!found) tree.clearSelection();        
    }
    
    // If the first node in the list is selected, create a new node with a new id and add it to the store;
    // if another node was selected, use it.  In either case return the associated object.
    private CShape getSelectedBuildNodeObject()
    {
        int index = nodeList.getSelectedIndex();
        CShape cshape = null;
        if (index == 0)
        {
            cshape = new CShape();
            // advance until we have a new id
            while (idsVector.contains(nextId.toString())) nextId = new Integer(nextId.intValue() + 1);
            cshape.setIdString(nextId.toString());
            idsVector.addElement(nextId.toString());
            store.addNode(cshape);
            nodeListModel.addElement("key: " + cshape.getKey() + " id: "+ cshape.getIdString());
        }
        else
        {
            cshape = store.getNode(index - 1);
        }
        mostRecentlyUsedKey = cshape.getKey(); 
        return cshape;
    }
    
    // test if a node already is in a branch of the hierarchy
    private boolean isSelectedAlreadyInPath(javax.swing.tree.TreePath path)
    {
        String searchKey = getSelectedBuildNodeObject().getKey();

        javax.swing.tree.DefaultMutableTreeNode branchRoot =     
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(1);
        java.util.Enumeration e = branchRoot.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();
            String nextKey = ((CShape)nextNode.getUserObject()).getKey();
            if (searchKey.equals(nextKey))
            {
                return true;
            }
        }
        return false;
    }
        
    public void createRootNode()
    {
        javax.swing.tree.DefaultMutableTreeNode newNode = new javax.swing.tree.DefaultMutableTreeNode();
        newNode.setUserObject(getSelectedBuildNodeObject());
        hierarchy.insertNodeInto(newNode,root,0);
        expandTree();
    }
    
    public void addChild()
    {
        javax.swing.tree.TreePath path = tree.getLeadSelectionPath();
        // nothing selected?
        if (path == null) return; 
        // don't add a child to the real root, create a root node instead
        if (path.getPathCount() == 1)
        {
            showErrorMessage("add_error1");
            return;
        }
        // don't allow the same node to appear twice in a branch
        if (isSelectedAlreadyInPath(path))
        {
            showErrorMessage("add_error2");
            return;
        }
        
        javax.swing.tree.DefaultMutableTreeNode parent = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
            
        javax.swing.tree.DefaultMutableTreeNode child = new javax.swing.tree.DefaultMutableTreeNode();

        child.setUserObject(getSelectedBuildNodeObject());
        hierarchy.insertNodeInto(child,parent,0);
        expandTree();
    }
    
    public void addParent()
    {
        javax.swing.tree.TreePath path = tree.getLeadSelectionPath();
        // nothing selected?
        if (path == null) return;
        //can't add parent of root 
        if (tree.getLeadSelectionRow() == 0) 
        {
            showErrorMessage("add_error3");
            return;  
        }
        // don't allow the same node to appear twice in a branch
        if (isSelectedAlreadyInPath(path))
        {
            showErrorMessage("add_error2");
            return;
        }

        javax.swing.tree.DefaultMutableTreeNode oldParent = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-2);
        javax.swing.tree.DefaultMutableTreeNode newChild = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
            
        hierarchy.removeNodeFromParent(newChild);
        
        javax.swing.tree.DefaultMutableTreeNode newParent = new javax.swing.tree.DefaultMutableTreeNode();
        newParent.setUserObject(getSelectedBuildNodeObject());

        hierarchy.insertNodeInto(newChild,newParent,0);
        hierarchy.insertNodeInto(newParent,oldParent,0);
        expandTree();
    }

    public void remove()
    {
        javax.swing.tree.TreePath path = tree.getLeadSelectionPath();
        if (path == null) return; // nothing selected
        //can't remove real root
        if (tree.getLeadSelectionRow() == 0)
        {
            showErrorMessage("remove_error2");
            return;  
        }
        javax.swing.tree.DefaultMutableTreeNode selectedNode = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
        
        CShape cshape = (CShape)selectedNode.getUserObject();
        String id = cshape.getIdString();
        hierarchy.removeNodeFromParent(selectedNode);
        
        expandTree();
    }
    
    public void removeParent()
    {
        javax.swing.tree.TreePath path = tree.getLeadSelectionPath();
        // nothing selected?
        if (path == null) return;
        // can't remove real root
        if (tree.getLeadSelectionRow() == 0)
        {
            showErrorMessage("remove_error2");
            return;  
        }
        // can't remove a root node
        if (path.getPathCount() < 3) 
        {
            showErrorMessage("remove_error1");
            return;
        }
         
        javax.swing.tree.DefaultMutableTreeNode newParent = 
                (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-3);
        javax.swing.tree.DefaultMutableTreeNode oldParent = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-2);
        javax.swing.tree.DefaultMutableTreeNode child = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
            
        hierarchy.removeNodeFromParent(oldParent);
        hierarchy.insertNodeInto(child,newParent,0);

        expandTree();
    }
           
    private void setDataModified(boolean b)
    {
        dataModified = b;
    }
    
    private void repaintFix()
    {
        if (os == WINDOWS) 
        {
            invalidate();
        }
    }
    
    public boolean isModified()
    {
        return dataModified;
    }
    
    public void save(String dataStore, String hierarchyStore)
    {
        store.save(dataStore);	    	  // save data
        saveHierarchy(hierarchyStore);    // save hierarchy
    }
    
    // this will be obsolete when Hierarchy API is implemented
    private void saveHierarchy(String file)
    {
        java.util.Properties properties = new java.util.Properties();
        
        // save hierarchy of nodes, skipping the real root
        javax.swing.tree.DefaultMutableTreeNode root = 
            (javax.swing.tree.DefaultMutableTreeNode)hierarchy.getRoot();    
        java.util.Enumeration e = root.preorderEnumeration();
        int i = 0;
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();
                
            // key of child
            CShape childCShape = (CShape)nextNode.getUserObject();
            String childKey = childCShape.getKey();
            
            // key of parent
            javax.swing.tree.DefaultMutableTreeNode parent;
            if (nextNode.isRoot())
            {
                parent = root;
            }
            else
            {	    
            	parent = (javax.swing.tree.DefaultMutableTreeNode)nextNode.getParent();
            }
            CShape parentCShape = (CShape)parent.getUserObject();
            String parentKey = parentCShape.getKey();
            
            // data for future insertNodeInto operation
            String link = childKey + "_" + parentKey;
            properties.setProperty( (i==0) ? "root" : "link_"+i , (i==0) ? childKey : link ); 
            i++;
        }
        
        try
        {
            properties.store(new java.io.FileOutputStream(file),"");
        }
        catch (Exception ex)
        {
            System.out.println(ex);
        }
    }
    
    private void loadHierarchy(String file)
    {
        java.util.Properties properties = new java.util.Properties();
        if (file != null)
        {
            try
            {
                properties.load(new java.io.FileInputStream(file));
            }
            catch (Exception e)
            {
                System.out.println(e);
                return;
            }
        }
        // working variables
        String rootKey = properties.getProperty("root");
        java.util.Vector keysVector = new java.util.Vector();
        java.util.Vector nodesVector = new java.util.Vector();
        javax.swing.tree.DefaultMutableTreeNode childNode;
        javax.swing.tree.DefaultMutableTreeNode parentNode;
        CShape cshape;
        String link;
        String childKey;
        String parentKey;
        int j = 1;
        
        // create parallel tree node and key lists 
        CShape cshapes[] = store.getNodes();
        for (int i=0; i < cshapes.length; i++)
        {
            keysVector.addElement(cshapes[i].getKey());
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                new javax.swing.tree.DefaultMutableTreeNode();
            nextNode.setUserObject(cshapes[i]);
            nodesVector.addElement(nextNode);
            
            idsVector.addElement(cshapes[i].getIdString());
        }
        keysVector.addElement(rootKey);
        nodesVector.addElement(root);

        // For each link, extract the child and parent keys.
        // For each key, if it is new, create a tree node for it and assign as the user object
        // what has already been loaded.  If the key is a duplicate, just use the already constructed node
        while ((link = properties.getProperty("link_"+j)) != null)
        {
            childKey = link.substring(0,link.indexOf("_"));
            parentKey = link.substring(link.indexOf("_")+1);
            
            childNode = (javax.swing.tree.DefaultMutableTreeNode)nodesVector.elementAt(
                keysVector.indexOf(childKey));
            parentNode = (javax.swing.tree.DefaultMutableTreeNode)nodesVector.elementAt(
                keysVector.indexOf(parentKey));
            hierarchy.insertNodeInto(childNode,parentNode,0);
            j++;
        }
        expandTree();
    }
    
    // this indirection allows alternate models for error message notification
    // performance could be improved by loading this all only once   
    private void showErrorMessage(String propertyFileTag)
    {
        if (showErrorsWithDialog)
        {
            javax.swing.JOptionPane.showMessageDialog(
                this,
                propertiesFile.getProperty(propertyFileTag),
                "",
                javax.swing.JOptionPane.ERROR_MESSAGE
            );
        }
        
        if (showErrorsWithSystemOut)
        {
            System.out.println(propertiesFile.getProperty(propertyFileTag));
        }
    }
}
