public class InheritanceDemo extends javax.swing.JFrame 
implements java.awt.event.ActionListener
{
    private static JTreeViewer tabbedPane = null;
    
    private String fileMenuName;
    private String[] fileMenuItems;
    private char[] fileMenuShortCuts;
    private String editMenuName;
    private String[] editMenuItems;
    private char[] editMenuShortCuts;
    private String nodeMenuName;

    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenu editMenu;
    private javax.swing.JMenu nodeMenu;
    private javax.swing.JCheckBoxMenuItem menuItemNodeIdVisible;
    private javax.swing.JCheckBoxMenuItem menuItemNodeDescriptionVisible;
    private javax.swing.JCheckBoxMenuItem menuItemNodeKeyVisible;
    private javax.swing.JCheckBoxMenuItem menuItemPropertyDescriptionVisible;
    
    private static javax.swing.JMenuBar menuBar;
    
    private static javax.swing.JLabel inStreamLabel = new javax.swing.JLabel();
    private static javax.swing.JLabel outStreamLabel = new javax.swing.JLabel();
    private static boolean dataModified = false;
    
    private final static InheritanceDemo demo = new InheritanceDemo();
    private static java.util.Properties propertiesFile = new java.util.Properties();
    private static String inStream = null;
    private static String outStream = null;
    private static String shapeHierarchyFilename = null;
    private static String colorHierarchyFilename = null;
    private static String edgeHierarchyFilename = null;
    
    public InheritanceDemo()
    {
        super ("Inheritance Demo");
    }
    
    private void initMenus ()
    {
        fileMenuName = propertiesFile.getProperty("file_menu_name");
        fileMenuItems = new String[] {
            propertiesFile.getProperty("file_menu_item_1"),
            propertiesFile.getProperty("file_menu_item_2")
        };
        
        fileMenuShortCuts = new char[] {
            propertiesFile.getProperty("file_menu_shortcut_1").charAt(0),
            propertiesFile.getProperty("file_menu_shortcut_2").charAt(0)
        };
        
        editMenuName = propertiesFile.getProperty("edit_menu_name");
        editMenuItems = new String[] {
            propertiesFile.getProperty("edit_menu_item_1")
        };
        editMenuShortCuts = new char[] {
            propertiesFile.getProperty("edit_menu_shortcut_1").charAt(0)
        };

        nodeMenuName = propertiesFile.getProperty("node_menu_name");

        fileMenu = new javax.swing.JMenu(fileMenuName);
        editMenu = new javax.swing.JMenu(editMenuName);
        nodeMenu = new javax.swing.JMenu(nodeMenuName);    
        menuBar = new javax.swing.JMenuBar();

        //mnemonics and accelerators for each menu
        for (int i=0; i < fileMenuItems.length; i++)
        {
            javax.swing.JMenuItem menuItem = new javax.swing.JMenuItem(fileMenuItems[i], fileMenuShortCuts[i]);
            menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(fileMenuShortCuts[i],
            java.awt.Event.CTRL_MASK, false));
            menuItem.addActionListener(this);
            fileMenu.add(menuItem);
        }
        menuBar.add(fileMenu);
        
        for (int i=0; i < editMenuItems.length; i++)
        {
            javax.swing.JMenuItem menuItem = new javax.swing.JMenuItem(editMenuItems[i], editMenuShortCuts[i]);
            menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(editMenuShortCuts[i],
            java.awt.Event.CTRL_MASK, false));
            menuItem.addActionListener(this);
            editMenu.add(menuItem);
        }
        menuBar.add(editMenu);

        menuItemNodeIdVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_1"));
        menuItemNodeIdVisible.addActionListener(this);
        menuItemNodeIdVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_node_id_visible_state")).booleanValue());
        
        menuItemNodeDescriptionVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_2"));
        menuItemNodeDescriptionVisible.addActionListener(this);
        menuItemNodeDescriptionVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_node_description_state")).booleanValue());
        
        menuItemNodeKeyVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_3"));
        menuItemNodeKeyVisible.addActionListener(this);
        menuItemNodeKeyVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_node_key_visible_state")).booleanValue());
        
        menuItemPropertyDescriptionVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_4"));
        menuItemPropertyDescriptionVisible.addActionListener(this);
        menuItemPropertyDescriptionVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_property_description_visible_state")).booleanValue());
        
        nodeMenu.add(menuItemNodeIdVisible);
        nodeMenu.add(menuItemNodeDescriptionVisible);
        nodeMenu.add(menuItemNodeKeyVisible);
        nodeMenu.add(menuItemPropertyDescriptionVisible);
        menuBar.add(nodeMenu);
    }
    
    private void init()
    {
        tabbedPane = new JTreeViewer(inStream,shapeHierarchyFilename,
            colorHierarchyFilename,edgeHierarchyFilename);
        tabbedPane.setVisible(true);
        getContentPane().add(tabbedPane);
    }
    
    private void configure()
    {

        //load some configuration values
        CShape cshape = new CShape();

        cshape.setShowId(menuItemNodeIdVisible.getState());
        cshape.setShowKey(menuItemNodeKeyVisible.getState());
        cshape.setShowNodeDescription(menuItemNodeDescriptionVisible.getState());
        cshape.setShowPropertyDescription(menuItemPropertyDescriptionVisible.getState());

        cshape.setKeyFont(
            propertiesFile.getProperty("key_font_name"),
            new java.lang.Integer(propertiesFile.getProperty("key_font_style")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("key_font_size")).intValue());
        cshape.setIdFont(
            propertiesFile.getProperty("id_font_name"),
            new java.lang.Integer(propertiesFile.getProperty("id_font_style")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("id_font_size")).intValue());
        cshape.setDescriptionFont(
            propertiesFile.getProperty("description_font_name"),
            new java.lang.Integer(propertiesFile.getProperty("description_font_style")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("description_font_size")).intValue());
        setSize(
            new java.lang.Integer(propertiesFile.getProperty("application_frame_width")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("application_frame_height")).intValue());

        cshape.setTextDrawingConstants(
            new java.lang.Integer(propertiesFile.getProperty("draw_id_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_id_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_key_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_key_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line1_length")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line1_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line1_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2_length")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2a_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2a_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2b_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line2b_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3_length")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3a_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3a_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3b_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3b_y_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3c_x_offset")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("draw_description_line3c_y_offset")).intValue());
    }

    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        String command = ae.getActionCommand();
        if (propertiesFile.getProperty("file_menu_item_1").equals(command)) save();
        if (propertiesFile.getProperty("file_menu_item_2").equals(command)) exit(); 
        if (propertiesFile.getProperty("edit_menu_item_1").equals(command)) 
        {
            tabbedPane.showEditor();
        }
        if (propertiesFile.getProperty("node_menu_item_1").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowId(menuItemNodeIdVisible.getState());
        }
        if (propertiesFile.getProperty("node_menu_item_2").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowNodeDescription(menuItemNodeDescriptionVisible.getState());
        }
        if (propertiesFile.getProperty("node_menu_item_3").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowKey(menuItemNodeKeyVisible.getState());
        }
        if (propertiesFile.getProperty("node_menu_item_4").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowPropertyDescription(menuItemPropertyDescriptionVisible.getState());
        }
        tabbedPane.paintComponents();
    }
    
    private void cleanStart()
    {
        if (tabbedPane != null)
        {
            tabbedPane.setVisible(false);
            tabbedPane = null;
        }
        setDataModified(false);
        getContentPane().removeAll();
        initStreamInfo();
        pack();
    }
    
    private void newTabbedPane()
    {
        save();
        cleanStart();
        inStream = null;
        init();
    }
    
    private void save()
    {
       try
        {
            if ((tabbedPane != null) && (tabbedPane.isModified()))
            {
                if (outStream == null)
                {
                    javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
                    chooser.setMultiSelectionEnabled(false);
                    chooser.setFileSelectionMode(javax.swing.JFileChooser.FILES_ONLY);
                    String extensions[] = {"store"};
                    chooser.addChoosableFileFilter(new FileFilterExt(extensions, "Data (*.store)"));
                    int option = chooser.showSaveDialog(this);
                    if (option == javax.swing.JFileChooser.APPROVE_OPTION) 
                    {
                        if (chooser.getSelectedFile() != null)
                        {
                            outStream = chooser.getSelectedFile().getName();
                        }
                    }
                }
                if (outStream != null)
                {
                    tabbedPane.save(outStream);
                    setDataModified(false);
                }
            }
        }
        catch (Exception e)
        {
            System.out.println("Exception writing to "+outStream);
        }
    }
            
    private void exit()
    {
        save();
        System.exit(0);
    }
                   
    private void setDataModified(boolean b)
    {
        dataModified = b;
        updateHierarchyStreamInfo();
    }
    
    private void updateHierarchyStreamInfo()
    {
        String rNull = propertiesFile.getProperty("read_from_null_file_prompt");
        String r = propertiesFile.getProperty("read_from_file_prompt");
        String wNull = propertiesFile.getProperty("write_to_null_file_prompt");
        String w = propertiesFile.getProperty("write_to_file_prompt");

        inStreamLabel.setText((inStream == null) ? rNull : r + ((dataModified) ? "*" : "") + inStream);
        outStreamLabel.setText((outStream == null) ? wNull : w + outStream);
    }
    
    private void initStreamInfo()
    {
        updateHierarchyStreamInfo();
        javax.swing.JPanel p = new javax.swing.JPanel();
        p.add(inStreamLabel,java.awt.BorderLayout.NORTH);
        p.add(outStreamLabel,java.awt.BorderLayout.SOUTH);
        getContentPane().add(p,java.awt.BorderLayout.NORTH);
    }
    
    public static void main(String args[]) 
    {
        demo.addWindowListener(
            new java.awt.event.WindowAdapter() 
            {
                public void windowClosing(java.awt.event.WindowEvent ev) 
                {
                    if (dataModified) demo.save();
                    System.exit(0);
                }
            }
        );
        try
        {
            propertiesFile.load(new java.io.FileInputStream("InheritanceDemo.properties"));
        }
        catch (Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }
        demo.initMenus();
        demo.setJMenuBar(menuBar);
        demo.configure();
        
        // if there is a data file, show whatever is set as the initial view,
        // otherwise show the builder
        if (args.length < 4) 
        {
            System.out.println(propertiesFile.getProperty("too_few_args1"));
            System.out.println(propertiesFile.getProperty("too_few_args2"));
            System.exit(1); 
        }
        else
        {
            inStream = args[0];
            shapeHierarchyFilename = args[1];
            colorHierarchyFilename = args[2];
            edgeHierarchyFilename = args[3];
        }
        demo.initStreamInfo();
        demo.init();
        demo.setVisible(true);
    }
}
