public class JTableViewer extends javax.swing.JScrollPane 
{
    private DataStore store = null;
    private javax.swing.JTable table = null;
    private javax.swing.table.DefaultTableModel tableModel = null;
    private java.util.Properties propertiesFile = new java.util.Properties();
    private java.util.Properties cshapePropertiesFile = new java.util.Properties();
    private String shapeDescriptions[] = new String[8];      
    private String colorDescriptions[] = new String[7];      
    private String edgeDescriptions[] = new String[2];
          
    private static boolean dataModified = false;
    
    public JTableViewer(DataStore ds)
    {
        super ();
        store = ds;
        try
        {
            propertiesFile.load(new java.io.FileInputStream("JTableViewer.properties"));
            cshapePropertiesFile.load(new java.io.FileInputStream("CShape.properties"));
        }
        catch (Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }
        init();
    }
    
    private void init()
    {
        String columnHeads[] = { propertiesFile.getProperty("column_0_head"),
                                propertiesFile.getProperty("column_1_head"),
                                propertiesFile.getProperty("column_2_head"),
                                propertiesFile.getProperty("column_3_head"),
                                propertiesFile.getProperty("column_4_head"),
                                propertiesFile.getProperty("column_5_head") };
                                
        loadShapeProperties(cshapePropertiesFile);
        loadColorProperties(cshapePropertiesFile);
        loadEdgeProperties(cshapePropertiesFile);
                                 
        //create a 5-column table model with column names and no initial data, assign a table UI to this model
        String heads[] = {columnHeads[0],columnHeads[1],columnHeads[2],columnHeads[3],
                          columnHeads[4],columnHeads[5]
        }; 
        tableModel = new javax.swing.table.DefaultTableModel(heads,0);
        table = new javax.swing.JTable(tableModel);

        //adjust column widths
        javax.swing.table.TableColumnModel tableColumnModel = table.getColumnModel();
        for (int i = 0; i < 6; i++)
        {
            javax.swing.table.TableColumn column = tableColumnModel.getColumn(i); 
            column.setMinWidth((new java.lang.Integer(propertiesFile.getProperty("column_"+i+"_width"))).intValue());
        }
        table.setShowGrid(true);
        table.setGridColor(java.awt.Color.black);
        
        setViewportView(table);
        fillTable();
    }

    private void fillTable()
    {
        CShape cshapes[] = store.getNodes();
        for (int i = 0; i < cshapes.length; i++)
        {
            String objectData[] = { cshapes[i].getKey(),
                                    cshapes[i].getIdString(),
                                    shapeDescriptions[cshapes[i].getShape()],
                                    colorDescriptions[cshapes[i].getColor()],
                                    edgeDescriptions[cshapes[i].getEdge()],
                                    cshapes[i].getDescription()
            }; 
            tableModel.addRow(objectData);
        }
    }
    
    private void loadShapeProperties(java.util.Properties p)
    {
        shapeDescriptions[0] = p.getProperty("circle_property_description");
        shapeDescriptions[1] = p.getProperty("square_property_description");
        shapeDescriptions[2] = p.getProperty("triangle_property_description");
        shapeDescriptions[3] = p.getProperty("diamond_property_description");
        shapeDescriptions[4] = p.getProperty("parrallelogram_property_description");
        shapeDescriptions[5] = p.getProperty("pentagon_property_description");
        shapeDescriptions[6] = p.getProperty("inverted_triangle_property_description");
        shapeDescriptions[7] = p.getProperty("trapezoid_property_description");
    }

    private void loadColorProperties(java.util.Properties p)
    {
        colorDescriptions[0] = p.getProperty("blue_property_description");
        colorDescriptions[1] = p.getProperty("cyan_property_description");
        colorDescriptions[2] = p.getProperty("green_property_description");
        colorDescriptions[3] = p.getProperty("magenta_property_description");
        colorDescriptions[4] = p.getProperty("orange_property_description");
        colorDescriptions[5] = p.getProperty("pink_property_description");
        colorDescriptions[6] = p.getProperty("red_property_description");
    }

    private void loadEdgeProperties(java.util.Properties p)
    {
        edgeDescriptions[0] = p.getProperty("thin_property_description");
        edgeDescriptions[1] = p.getProperty("thick_property_description");
    }  
      
    private void cleanStart()
    {
        setVisible(false);
        setDataModified(false);
    }
       
    private void setDataModified(boolean b)
    {
        dataModified = b;
    }
    
    public boolean isModified()
    {
        return dataModified;
    }
}
