public class JTreeViewer extends javax.swing.JTabbedPane 
implements java.awt.event.MouseListener, 
           java.awt.event.KeyListener, 
           javax.swing.event.ListSelectionListener,
           javax.swing.event.ChangeListener,
           javax.swing.event.TreeSelectionListener,
           javax.swing.event.TreeModelListener
{
    private DataStore store = null;
    
    private javax.swing.JTree shapeTree = null;
    private javax.swing.JTree colorTree = null;
    private javax.swing.JTree edgeTree = null;
    private javax.swing.tree.DefaultTreeModel shapeHierarchy = null;
    private javax.swing.tree.DefaultTreeModel colorHierarchy = null;
    private javax.swing.tree.DefaultTreeModel edgeHierarchy = null;
    
    private ShapePropertyEditor shapePropertyEditor = null;
    private ColorPropertyEditor colorPropertyEditor = null;
    private EdgePropertyEditor edgePropertyEditor = null;
    
    private String[] nodeStrings = {"1","2","3","4","5","6","7"};
    private javax.swing.JList nodeList;
    private javax.swing.DefaultListSelectionModel nodeListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
        
    private java.util.Properties propertiesFile = new java.util.Properties();
    private static boolean dataModified = false;
    private String inStream = null;
    private static String shapeHierarchyFilename = null;
    private static String colorHierarchyFilename = null;
    private static String edgeHierarchyFilename = null;
        
    private int os;
    private static int WINDOWS = 0;
    private static int MAC = 1;    
    
    private int SHAPE_CHANGED = 0;
    private int COLOR_CHANGED = 1;
    private int EDGE_CHANGED = 2;
    
    public JTreeViewer(String datastore, String shapeHierarchy,
            String colorHierarchy, String edgeHierarchy)
    {
        super ();
        inStream = datastore;
        shapeHierarchyFilename = shapeHierarchy;
        colorHierarchyFilename = colorHierarchy;
        edgeHierarchyFilename = edgeHierarchy;
        
        store = new DataStore();
        store.load(datastore);
        init();
    }
    
    private void init()
    {
        try
        {
            propertiesFile.load(new java.io.FileInputStream("JTreeViewer.properties"));
        }
        catch (Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }

        String osName = System.getProperty("os.name");
        os = (osName.startsWith("Mac")) ? MAC : WINDOWS;

        shapeHierarchy = loadHierarchy(shapeHierarchyFilename);
        shapeTree = new javax.swing.JTree(shapeHierarchy);
        shapeTree.setCellRenderer(new CShapeTreeCellRenderer());
        shapeTree.setCellEditor(new CShapeTreeCellTextEditor());
        shapeTree.setEditable(true);
        shapeTree.setRowHeight(50);
        shapeTree.setSelectionRow(1);
        shapeTree.setVisible(true);
        
        colorHierarchy = loadHierarchy(colorHierarchyFilename);
        colorTree = new javax.swing.JTree(colorHierarchy);
        colorTree.setCellRenderer(new CShapeTreeCellRenderer());
        colorTree.setCellEditor(new CShapeTreeCellTextEditor());
        colorTree.setEditable(true);
        colorTree.setRowHeight(50);
        colorTree.setSelectionRow(1);
        colorTree.setVisible(true);

        edgeHierarchy = loadHierarchy(edgeHierarchyFilename);
        edgeTree = new javax.swing.JTree(edgeHierarchy);
        edgeTree.setCellRenderer(new CShapeTreeCellRenderer());
        edgeTree.setCellEditor(new CShapeTreeCellTextEditor());
        edgeTree.setEditable(true);
        edgeTree.setRowHeight(50);
        edgeTree.setSelectionRow(1);
        edgeTree.setVisible(true);
 
        expandTree(shapeTree);
        expandTree(colorTree);
        expandTree(edgeTree);
        
        nodeList = new javax.swing.JList(store.getNodes());
        nodeListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        nodeList.setSelectionModel(nodeListSelectionModel);
        nodeListSelectionModel.addSelectionInterval(0,0);
        nodeList.setFixedCellHeight(50);
        nodeList.setCellRenderer(new NodeRenderer());
        nodeList.setVisible(true);

        addTab(propertiesFile.getProperty("nodes_list_tab_title"), nodeList);
        addTab(propertiesFile.getProperty("shape_hierarchy_tab_title"),shapeTree);
        addTab(propertiesFile.getProperty("color_hierarchy_tab_title"),colorTree);
        addTab(propertiesFile.getProperty("edge_hierarchy_tab_title"),edgeTree);
        setVisible(true);

        shapePropertyEditor = new ShapePropertyEditor(
            propertiesFile.getProperty("shape_editor_title"),propertiesFile);
        shapePropertyEditor.addChangeListener(this);
        
        colorPropertyEditor = new ColorPropertyEditor(
            propertiesFile.getProperty("color_editor_title"),propertiesFile);
        colorPropertyEditor.addChangeListener(this);

        edgePropertyEditor = new EdgePropertyEditor(
            propertiesFile.getProperty("edge_editor_title"),propertiesFile);
        edgePropertyEditor.addChangeListener(this);
 
        shapeTree.addMouseListener(this);                
        shapeTree.addKeyListener(this);
        shapeTree.addTreeSelectionListener(this);
        shapeHierarchy.addTreeModelListener(this);

        colorTree.addMouseListener(this);                
        colorTree.addKeyListener(this);    
        colorTree.addTreeSelectionListener(this);
        colorHierarchy.addTreeModelListener(this);

        edgeTree.addMouseListener(this);                
        edgeTree.addKeyListener(this);    
        edgeTree.addTreeSelectionListener(this);
        edgeHierarchy.addTreeModelListener(this);

        nodeList.addListSelectionListener(this);
        addChangeListener(this);  
    }

    private void expandTree(javax.swing.JTree tree)
    {
        //expand entire tree, kludge, but could find a better way
        int currentRowCount = 0;
        int previousRowCount = -1;
        while (previousRowCount != currentRowCount)
        {
            previousRowCount = tree.getRowCount();
            for (int r = 0; r < previousRowCount; r++) tree.expandRow(r);
            currentRowCount = tree.getRowCount();
        }
    }

    public void mouseClicked(java.awt.event.MouseEvent me){}
    public void mousePressed(java.awt.event.MouseEvent me){}
    public void mouseReleased(java.awt.event.MouseEvent me){}
    public void mouseEntered(java.awt.event.MouseEvent me){}
    public void mouseExited(java.awt.event.MouseEvent me){}    
    public void keyTyped(java.awt.event.KeyEvent ke){}    
    public void keyPressed(java.awt.event.KeyEvent ke){}
    public void keyReleased(java.awt.event.KeyEvent ke){}
    
    private void invalidateCShapes()
    {
        paintComponents(getGraphics());
        if (shapePropertyEditor.isVisible())
        {
            shapePropertyEditor.paintComponents(shapePropertyEditor.getGraphics());
        }
        if (colorPropertyEditor.isVisible())
        {
            colorPropertyEditor.paintComponents(colorPropertyEditor.getGraphics());
        }
        if (edgePropertyEditor.isVisible())
        {
            edgePropertyEditor.paintComponents(edgePropertyEditor.getGraphics());
        }
    }
    
    public void setVisible(boolean b)
    {
        super.setVisible(b);
        
        if (!b)
        {
            shapePropertyEditor.setVisible(false);
            colorPropertyEditor.setVisible(false);
            edgePropertyEditor.setVisible(false);
        }
    }
    
    private void cleanStart()
    {
        setVisible(false);
        
        shapeTree = null;
    	colorTree = null;
    	edgeTree = null;
    	shapeHierarchy = null;
   	colorHierarchy = null;
    	edgeHierarchy = null;
    	shapePropertyEditor = null;
    	colorPropertyEditor = null;
    	edgePropertyEditor = null;
        
        setDataModified(false);
    }
       
    // make the editor frame associated with the current hierarchy tab visible and hide the others
    private void showEditor(int editor)
    {
        shapePropertyEditor.setVisible(editor == 1);
        colorPropertyEditor.setVisible(editor == 2);
        edgePropertyEditor.setVisible(editor == 3);
    }
    
    private void updateEditors(CShape target)
    {
        shapePropertyEditor.setShape(target.getShape());
        shapePropertyEditor.setColor(target.getColor());
        shapePropertyEditor.setEdge(target.getEdge());
        shapePropertyEditor.paintComponents(shapePropertyEditor.getGraphics());
        
        colorPropertyEditor.setShape(target.getShape());
        colorPropertyEditor.setColor(target.getColor());
        colorPropertyEditor.setEdge(target.getEdge());
        colorPropertyEditor.paintComponents(colorPropertyEditor.getGraphics());

        edgePropertyEditor.setShape(target.getShape());
        edgePropertyEditor.setColor(target.getColor());
        edgePropertyEditor.setEdge(target.getEdge());
        edgePropertyEditor.paintComponents(edgePropertyEditor.getGraphics());
    }
    
    private void ApplySelections(javax.swing.JTree tree, int type, int value)
    {
        // this code gets the last selection.  Maybe we want to do something else
        javax.swing.tree.TreePath path = tree.getLeadSelectionPath();
        if (path == null) return; // nothing selected
        javax.swing.tree.DefaultMutableTreeNode selectedNode = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);

        // propagate properties to children of this parent
        java.util.Enumeration e = selectedNode.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();

            CShape cshape = (CShape)nextNode.getUserObject();
            switch (type)
            {
                case 0:
                    cshape.setShape(value);
                    break;
                case 1:
                    cshape.setColor(value);
                    break;
                case 2:
                    cshape.setEdge(value);
                    break;
                default:
                    break;
            }
        }
        setDataModified(true);
    }    
    
    // a different tab was selected or the Apply button in an editor was pressed
    public void stateChanged(javax.swing.event.ChangeEvent ce)
    {
        Object source = ce.getSource();
        if (source == this)
        {
            showEditor(getSelectedIndex());
            paintComponents(getGraphics());
            return;
        }
        
        if ((source != shapePropertyEditor) &&
            (source != colorPropertyEditor) &&
            (source != edgePropertyEditor)) return;
            
        if (source == shapePropertyEditor) 
            ApplySelections(shapeTree,SHAPE_CHANGED,((ShapePropertyEditor)source).getShape());
        if (source == colorPropertyEditor) 
            ApplySelections(colorTree,COLOR_CHANGED,((ColorPropertyEditor)source).getColor());
        if (source == edgePropertyEditor) 
            ApplySelections(edgeTree,EDGE_CHANGED,((EdgePropertyEditor)source).getEdge());
        setDataModified(true);
        
        invalidateCShapes();  
        repaintFix();
    }

    // a different list node was selected, update the three hierarchies, and their editors    
    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
            int index = nodeList.getSelectedIndex();
            CShape target = store.getNode(index);
            updateEditors(target);    
        
        if (getSelectedIndex() == 0)  // ignore this event if a side-effect of a tree selection
        {
            UpdateSelectionInTrees(shapeTree, shapeHierarchy, target);
            UpdateSelectionInTrees(colorTree, colorHierarchy, target);
            UpdateSelectionInTrees(edgeTree, edgeHierarchy, target);
        }
   }
    
    // a different tree node was selected, update the node list, the other hierarchies, and the editors    
    public void valueChanged(javax.swing.event.TreeSelectionEvent tse)
    {
        javax.swing.tree.TreePath path;
        javax.swing.tree.DefaultMutableTreeNode selectedNode;
        CShape target;

        path = tse.getNewLeadSelectionPath();
        if (path == null) return; // nothing selected
        selectedNode = (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
        target = (CShape)selectedNode.getUserObject();
        
        if (getSelectedIndex() != 0)  // ignore this event if a side-effect of a node selection
        {
            UpdateSelectionInTrees(shapeTree, shapeHierarchy, target);
            UpdateSelectionInTrees(colorTree, colorHierarchy, target);
            UpdateSelectionInTrees(edgeTree, edgeHierarchy, target);
        
            // set selection in list of nodes to the one selected in the tree
            for (int i = 0; i < nodeStrings.length; i++)
            {
                CShape cshape = store.getNode(i);
                if (cshape == target) 
                {
                    nodeListSelectionModel.addSelectionInterval(0,i);
                    break;
                }
            }
        }
        
        invalidateCShapes();
        repaintFix();
    }
    
    // find a target CShape in a tree model and select that node in a tree
    private void UpdateSelectionInTrees(javax.swing.JTree tree,
                                        javax.swing.tree.DefaultTreeModel model,
                                        CShape target)
    {
        javax.swing.tree.DefaultMutableTreeNode root = 
            (javax.swing.tree.DefaultMutableTreeNode)model.getRoot();
        java.util.Enumeration e = root.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode node = 
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();
            CShape cshape = (CShape)node.getUserObject();
            if (cshape == target)
            {
                javax.swing.tree.TreePath p = new javax.swing.tree.TreePath(model.getPathToRoot(node));
                tree.setSelectionPath(p);
            }
        }
    }
    
    // node text was edited
    public void treeNodesChanged(javax.swing.event.TreeModelEvent tme) 
    {
        setDataModified(true);
    }
    public void treeNodesInserted(javax.swing.event.TreeModelEvent tme) {}
    public void treeNodesRemoved(javax.swing.event.TreeModelEvent tme) {}
    public void treeStructureChanged(javax.swing.event.TreeModelEvent tme) {}

    private void setDataModified(boolean b)
    {
        dataModified = b;
    }
    
    private void repaintFix()
    {
        if (os == WINDOWS) 
        {
            invalidate();
        }
    }
    
    public boolean isModified()
    {
        return dataModified;
    }
    
    public void showEditor()
    {
        showEditor(getSelectedIndex());
    }
    
    public void paintComponents()
    {
        invalidateCShapes();
    }
    
    public void save(String file)
    {
        store.save(file);
    }
    
    private javax.swing.tree.DefaultTreeModel loadHierarchy(String file)
    {
        javax.swing.tree.DefaultTreeModel hierarchy = null;
        java.util.Properties properties = new java.util.Properties();
        try
        {
            properties.load(new java.io.FileInputStream(file));
        }
        catch (Exception e)
        {
            System.out.println(e);
            return hierarchy;
        }
        // working variables
        String rootKey = properties.getProperty("root");
        java.util.Vector keysVector = new java.util.Vector();
        java.util.Vector nodesVector = new java.util.Vector();
        javax.swing.tree.DefaultMutableTreeNode childNode;
        javax.swing.tree.DefaultMutableTreeNode parentNode;
        CShape cshape;
        String link;
        String childKey;
        String parentKey;
        int j = 1;
        
        //initialize hierarchy
        javax.swing.tree.DefaultMutableTreeNode root =
            new javax.swing.tree.DefaultMutableTreeNode();
        CShape rootCShape = new CShape();
        rootCShape.setIdString("-1");
        rootCShape.setKey(rootKey);
        root.setUserObject(rootCShape);
        hierarchy = new javax.swing.tree.DefaultTreeModel(root);
        
        // create parallel tree node and key lists 
        CShape cshapes[] = store.getNodes();
        for (int i=0; i < cshapes.length; i++)
        {
            keysVector.addElement(cshapes[i].getKey());
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                new javax.swing.tree.DefaultMutableTreeNode();
            nextNode.setUserObject(cshapes[i]);
            nodesVector.addElement(nextNode);
        }
        keysVector.addElement(rootKey);
        nodesVector.addElement(root);

        // For each link, extract the child and parent keys.
        // For each key, if it is new, create a tree node for it and assign as the user object
        // what has already been loaded.  If the key is a duplicate, just use the already constructed node
        while ((link = properties.getProperty("link_"+j)) != null)
        {
            childKey = link.substring(0,link.indexOf("_"));
            parentKey = link.substring(link.indexOf("_")+1);
            
            childNode = (javax.swing.tree.DefaultMutableTreeNode)nodesVector.elementAt(
                keysVector.indexOf(childKey));
            parentNode = (javax.swing.tree.DefaultMutableTreeNode)nodesVector.elementAt(
                keysVector.indexOf(parentKey));
            hierarchy.insertNodeInto(childNode,parentNode,0);
            j++;
        }
        return hierarchy;
    }
}
