public class ShapePropertyEditor extends javax.swing.JFrame 
implements javax.swing.event.ListSelectionListener,
           java.awt.event.ActionListener
{
    private String[] shapeStrings = {CShape.CIRCLE_STRING, CShape.SQUARE_STRING, 
        CShape.TRIANGLE_STRING,CShape.DIAMOND_STRING, 
        CShape.PARRALLELOGRAM_STRING, CShape.PENTAGON_STRING, 
        CShape.INVERTED_TRIANGLE_STRING,
        CShape.TRAPEZOID_STRING};
    private javax.swing.JList shapeList;
    private javax.swing.DefaultListSelectionModel shapeListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
    private ShapePropertyEditorRenderer shapeRenderer = null;
    private int minWidth, minHeight, boundsX, boundsY;
    private javax.swing.JButton applyButton = new javax.swing.JButton();
    private java.util.Vector listeners = new java.util.Vector();
 
    public ShapePropertyEditor(String title, java.util.Properties propertiesFile)
    {
        super(title);
 
        minWidth = new java.lang.Integer(propertiesFile.getProperty("editor_frame_width")).intValue();
        minHeight = new java.lang.Integer(propertiesFile.getProperty("editor_frame_height")).intValue();
        boundsX = new java.lang.Integer(propertiesFile.getProperty("editor_frame_boundsX")).intValue();
        boundsY = new java.lang.Integer(propertiesFile.getProperty("editor_frame_boundsY")).intValue();
        applyButton.setText(propertiesFile.getProperty("editor_apply_button_text"));
        
        setSize(minWidth,minHeight);
        shapeRenderer = new ShapePropertyEditorRenderer();
                
        shapeList = new javax.swing.JList(shapeStrings);
        shapeListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        shapeList.setSelectionModel(shapeListSelectionModel);
        shapeList.addListSelectionListener(this);
        shapeListSelectionModel.addSelectionInterval(0,0);
        shapeList.setFixedCellHeight(50);
        shapeList.setCellRenderer(shapeRenderer);
        shapeList.setVisible(true);

        applyButton.addActionListener(this);
        applyButton.setVisible(true);
        getContentPane().add(shapeList);
        getContentPane().add(applyButton,java.awt.BorderLayout.SOUTH);
    }
            
    public void addChangeListener(javax.swing.event.ChangeListener cl)
    {
        listeners.add(cl);
    }
    
    public void removeChangeListener(javax.swing.event.ChangeListener cl)
    {
        listeners.remove(cl);
    }
    
    protected void fireStateChanged(javax.swing.event.ChangeEvent ce)
    {
        for (int i = listeners.size()-1; i >= 0; i--)
        {
            javax.swing.event.ChangeListener cl = (javax.swing.event.ChangeListener)listeners.elementAt(i);
            cl.stateChanged(ce);
        }
    }

    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        fireStateChanged(new javax.swing.event.ChangeEvent(this));
    }

    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
    }

    public void setBounds(java.awt.Rectangle r)
    {
        r.width = minWidth;
        r.height = minHeight;
        super.setBounds(r);
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        super.setBounds(boundsX, boundsY, minWidth, minHeight);
    }
    
    public void setShape (int s)
    {
        shapeListSelectionModel.addSelectionInterval(0,s);
    }

    public int getShape()
    {
        return shapeList.getSelectedIndex();
    }    

    public void setColor (int c)
    {
        shapeRenderer.setColor(c);
    }

    public void setEdge (int e)
    {
        shapeRenderer.setEdge(e);
    }
}