public class TableDemo extends javax.swing.JFrame 
implements java.awt.event.ActionListener
{
    private static JTableViewer table = null;
    
    private String fileMenuName;
    private String[] fileMenuItems;
    private char[] fileMenuShortCuts;
    private javax.swing.JMenu fileMenu;
    private static javax.swing.JMenuBar menuBar;
    
    private static String inStream = null;
    private static javax.swing.JLabel inStreamLabel = new javax.swing.JLabel();
    
    private final static TableDemo demo = new TableDemo();
    private static java.util.Properties propertiesFile = new java.util.Properties();
    private DataStore store = new DataStore();
    
    public TableDemo()
    {
        super ("Table Demo");
    }
    
    private void initMenus ()
    {
        fileMenuName = propertiesFile.getProperty("file_menu_name");
        fileMenuItems = new String[] {
            propertiesFile.getProperty("file_menu_item_1"),
            propertiesFile.getProperty("file_menu_item_2")
        };
        
        fileMenuShortCuts = new char[] {
            propertiesFile.getProperty("file_menu_shortcut_1").charAt(0),
            propertiesFile.getProperty("file_menu_shortcut_2").charAt(0)
        };
        
        fileMenu = new javax.swing.JMenu(fileMenuName);
        menuBar = new javax.swing.JMenuBar();

        //mnemonics and accelerators for each menu
        for (int i=0; i < fileMenuItems.length; i++)
        {
            javax.swing.JMenuItem menuItem = new javax.swing.JMenuItem(fileMenuItems[i], fileMenuShortCuts[i]);
            menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(fileMenuShortCuts[i],
            java.awt.Event.CTRL_MASK, false));
            menuItem.addActionListener(this);
            fileMenu.add(menuItem);
        }
        menuBar.add(fileMenu);
    }
    
    private void init()
    {
        configure();
        if (inStream != null)
        {
            store.load(inStream);
            table = new JTableViewer(store);
            table.setVisible(true);
            getContentPane().add(table);
        }
    }
    
    private void configure()
    {
        setSize(
            new java.lang.Integer(propertiesFile.getProperty("application_frame_width")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("application_frame_height")).intValue());
    }

    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        String command = ae.getActionCommand();
        if (propertiesFile.getProperty("file_menu_item_1").equals(command)) open();
        if (propertiesFile.getProperty("file_menu_item_2").equals(command)) exit(); 
    }
    
    private void cleanStart()
    {
        if (table != null)
        {	
            table.setVisible(false);
            table = null;
        }
        getContentPane().removeAll();
        initStreamInfo();
        pack();
    }
        
    private void open()
    {
        javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
        chooser.setMultiSelectionEnabled(false);
        String extensions[] = {"store"};
        chooser.addChoosableFileFilter(new FileFilterExt(extensions, "Data (*.store)"));
        int option = chooser.showOpenDialog(this);
        if (option == javax.swing.JFileChooser.APPROVE_OPTION) 
        {
            if (chooser.getSelectedFile() != null)
            {
                inStream = chooser.getSelectedFile().getName();
                cleanStart();
                init();
                updateHierarchyStreamInfo();
                setVisible(true);
            }
        }
    }
        
    private void exit()
    {
        System.exit(0);
    }
                   
    private void updateHierarchyStreamInfo()
    {
        String rNull = propertiesFile.getProperty("read_from_null_file_prompt");
        String r = propertiesFile.getProperty("read_from_file_prompt");
        inStreamLabel.setText((inStream == null) ? rNull : r + inStream);
    }
    
    private void initStreamInfo()
    {
        updateHierarchyStreamInfo();
        javax.swing.JPanel p = new javax.swing.JPanel();
        p.add(inStreamLabel,java.awt.BorderLayout.NORTH);
        getContentPane().add(p,java.awt.BorderLayout.NORTH);
    }
    
    public static void main(String args[]) 
    {
        demo.addWindowListener(
            new java.awt.event.WindowAdapter() 
            {
                public void windowClosing(java.awt.event.WindowEvent ev) 
                {
                    System.exit(0);
                }
            }
        );
        try
        {
            propertiesFile.load(new java.io.FileInputStream("TableDemo.properties"));
        }
        catch (Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }
        demo.initMenus();
        demo.setJMenuBar(menuBar);
        demo.configure();
        
        // if there is a data file, load it
        if (args.length >= 1) 
        {
            inStream = args[0];
            demo.init();
        }
        demo.initStreamInfo();
        demo.setVisible(true);
    }
}
