public class ColorPropertyEditor extends javax.swing.JFrame 
implements javax.swing.event.ListSelectionListener,
           java.awt.event.ActionListener
{
    private String[] colorStrings = {CShape.BLUE_STRING, CShape.CYAN_STRING, 
        CShape.GREEN_STRING,CShape.MAGENTA_STRING, 
        CShape.ORANGE_STRING, CShape.PINK_STRING, 
        CShape.RED_STRING};
    private javax.swing.JList colorList;
    private javax.swing.DefaultListSelectionModel colorListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
    private ColorPropertyEditorRenderer colorRenderer = null;
    private int minWidth, minHeight, boundsX, boundsY;
    private javax.swing.JTree colorTree;
    private javax.swing.tree.DefaultTreeModel colorTreeModel = null;
    private javax.swing.tree.DefaultMutableTreeNode currentChild = null;
    private javax.swing.JFrame containingFrame = null;
    private javax.swing.JButton applyButton = new javax.swing.JButton();
    private InheritanceDemo demo;
 
    public ColorPropertyEditor(javax.swing.JTree tree, javax.swing.tree.DefaultTreeModel model,
                               String title, java.util.Properties propertiesFile,
                               javax.swing.JFrame frame, InheritanceDemo idemo)
    {
        super(title);
        minWidth = new java.lang.Integer(propertiesFile.getProperty("editor_frame_width")).intValue();
        minHeight = new java.lang.Integer(propertiesFile.getProperty("editor_frame_height")).intValue();
        boundsX = new java.lang.Integer(propertiesFile.getProperty("editor_frame_boundsX")).intValue();
        boundsY = new java.lang.Integer(propertiesFile.getProperty("editor_frame_boundsY")).intValue();
        applyButton.setText(propertiesFile.getProperty("editor_apply_button_text"));

        containingFrame = frame;
        demo = idemo;
        colorTree = tree;
        colorTreeModel = model;
        setSize(minWidth,minHeight);
        colorRenderer = new ColorPropertyEditorRenderer(propertiesFile);
                
        colorList = new javax.swing.JList(colorStrings);
        colorListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        colorList.setSelectionModel(colorListSelectionModel);
        colorList.addListSelectionListener(this);
        colorListSelectionModel.addSelectionInterval(0,0);
        colorList.setFixedCellHeight(50);
        colorList.setCellRenderer(colorRenderer);
        colorList.setVisible(true);

        applyButton.addActionListener(this);
        applyButton.setVisible(true);
        getContentPane().add(colorList);
        getContentPane().add(applyButton,java.awt.BorderLayout.SOUTH);
    }

    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        ApplySelections();
        demo.propertyChanged();
    }

    private void ApplySelections()
    {
        // this code gets the last selection.  Maybe we want to do something else
        javax.swing.tree.TreePath path = colorTree.getLeadSelectionPath();
        if (path == null) return; // nothing selected
        javax.swing.tree.DefaultMutableTreeNode selectedNode = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
        CShape cshape = (CShape)selectedNode.getUserObject();
        cshape.setColor(colorList.getSelectedIndex());

        PropagateProperties(selectedNode,colorList.getSelectedIndex());
        containingFrame.setVisible(true);
    }
             
    private void PropagateProperties(javax.swing.tree.DefaultMutableTreeNode parent, int color)
    {
        // propagate properties to children of this parent
        java.util.Enumeration e = parent.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();

            CShape cshape = (CShape)nextNode.getUserObject();
            cshape.setColor(color);
        }
    }    
        
    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
    }

    public void setBounds(java.awt.Rectangle r)
    {
        r.width = minWidth;
        r.height = minHeight;
        super.setBounds(r);
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        super.setBounds(boundsX, boundsY, minWidth, minHeight);
    }
    
    public void setColor (int s)
    {
        colorListSelectionModel.addSelectionInterval(0,s);
    }

    public int getColor()
    {
        return colorList.getSelectedIndex();
    }    

    public void setShape (int s)
    {
        colorRenderer.setShape(s);
    }

    public void setEdge (int e)
    {
        colorRenderer.setEdge(e);
    }
}