public class EdgePropertyEditor extends javax.swing.JFrame 
implements javax.swing.event.ListSelectionListener, 
           java.awt.event.ActionListener
{
    private String[] edgeStrings = {CShape.THIN_STRING, CShape.THICK_STRING};
    private static javax.swing.JTabbedPane tabbedPane = new javax.swing.JTabbedPane();
    private javax.swing.JList edgeList;
    private javax.swing.DefaultListSelectionModel edgeListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
    private EdgePropertyEditorRenderer edgeRenderer = null;
    private int minWidth, minHeight, boundsX, boundsY;
    private javax.swing.JTree edgeTree;
    private javax.swing.tree.DefaultTreeModel edgeTreeModel = null;
    private javax.swing.tree.DefaultMutableTreeNode currentChild = null;
    private javax.swing.JFrame containingFrame = null;
    private javax.swing.JButton applyButton = new javax.swing.JButton();
    private InheritanceDemo demo;
 
    public EdgePropertyEditor(javax.swing.JTree tree, javax.swing.tree.DefaultTreeModel model,
                              String title, java.util.Properties propertiesFile,
                              javax.swing.JFrame frame, InheritanceDemo idemo)
    {
        super(title);

        minWidth = new java.lang.Integer(propertiesFile.getProperty("editor_frame_width")).intValue();
        minHeight = new java.lang.Integer(propertiesFile.getProperty("editor_frame_height")).intValue();
        boundsX = new java.lang.Integer(propertiesFile.getProperty("editor_frame_boundsX")).intValue();
        boundsY = new java.lang.Integer(propertiesFile.getProperty("editor_frame_boundsY")).intValue();
        applyButton.setText(propertiesFile.getProperty("editor_apply_button_text"));

        edgeTree = tree;
        edgeTreeModel = model;
        containingFrame = frame;
        demo = idemo;
        setSize(minWidth,minHeight);
        edgeRenderer = new EdgePropertyEditorRenderer(propertiesFile);
                        
        edgeList = new javax.swing.JList(edgeStrings);
        edgeListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        edgeList.setSelectionModel(edgeListSelectionModel);
        edgeList.addListSelectionListener(this);
        edgeListSelectionModel.addSelectionInterval(0,0);
        edgeList.setFixedCellHeight(50);
        edgeList.setCellRenderer(edgeRenderer);
        edgeList.setVisible(true);

        applyButton.addActionListener(this);
        applyButton.setVisible(true);
        getContentPane().add(edgeList);
        getContentPane().add(applyButton,java.awt.BorderLayout.SOUTH);
    }
    
    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        ApplySelections();
        demo.propertyChanged();
    }


    private void ApplySelections()
    {
        // this code gets the last selection.  Maybe we want to do something else
        javax.swing.tree.TreePath path = edgeTree.getLeadSelectionPath();
        if (path == null) return; // nothing selected
        javax.swing.tree.DefaultMutableTreeNode selectedNode = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
        CShape cshape = (CShape)selectedNode.getUserObject();
        cshape.setEdge(edgeList.getSelectedIndex());

        PropagateProperties(selectedNode,edgeList.getSelectedIndex());
        containingFrame.setVisible(true);
    }
             
    private void PropagateProperties(javax.swing.tree.DefaultMutableTreeNode parent, int edge)
    {
        // propagate properties to children
        java.util.Enumeration e = parent.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();

            CShape cshape = (CShape)nextNode.getUserObject();
            cshape.setEdge(edge);
        }
    }    
        
    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
    }

    public void setBounds(java.awt.Rectangle r)
    {
        r.width = minWidth;
        r.height = minHeight;
        super.setBounds(r);
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        super.setBounds(boundsX, boundsY, minWidth, minHeight);
    }
    
    public void setEdge (int s)
    {
        edgeListSelectionModel.addSelectionInterval(0,s);
    }

    public int getEdge()
    {
        return edgeList.getSelectedIndex();
    }    

    public void setShape (int s)
    {
        edgeRenderer.setShape(s);
    }

    public void setColor (int c)
    {
        edgeRenderer.setColor(c);
    }
}