public class InheritanceDemo extends javax.swing.JFrame 
implements java.awt.event.MouseListener, 
           java.awt.event.KeyListener, 
           javax.swing.event.ListSelectionListener,
           java.awt.event.ActionListener,
           javax.swing.event.ChangeListener,
           javax.swing.event.TreeSelectionListener,
           javax.swing.event.TreeModelListener
{
    private static javax.swing.JTabbedPane tabbedPane = null;

    private javax.swing.JTree shapeTree = null;
    private javax.swing.JTree colorTree = null;
    private javax.swing.JTree edgeTree = null;
    
    private javax.swing.tree.DefaultTreeModel shapeHierarchy = null;
    private javax.swing.tree.DefaultTreeModel colorHierarchy = null;
    private javax.swing.tree.DefaultTreeModel edgeHierarchy = null;
    
    private ShapePropertyEditor shapePropertyEditor = null;
    private ColorPropertyEditor colorPropertyEditor = null;
    private EdgePropertyEditor edgePropertyEditor = null;
    
    private String[] nodeStrings = {"1","2","3","4","5","6","7"};
    private javax.swing.JList nodeList;
    private javax.swing.DefaultListSelectionModel nodeListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
        
    private String fileMenuName;
    private String[] fileMenuItems;
    private char[] fileMenuShortCuts;
    private String editMenuName;
    private String[] editMenuItems;
    private char[] editMenuShortCuts;
    private String nodeMenuName;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenu editMenu;
    private javax.swing.JMenu nodeMenu;
    private javax.swing.JCheckBoxMenuItem menuItemNodeIdVisible;
    private javax.swing.JCheckBoxMenuItem menuItemNodeDescriptionVisible;
    private javax.swing.JCheckBoxMenuItem menuItemPropertyDescriptionVisible;
    private static javax.swing.JMenuBar menuBar;
    
    private static javax.swing.JLabel inStreamLabel = new javax.swing.JLabel();
    private static javax.swing.JLabel outStreamLabel = new javax.swing.JLabel();
    private static boolean dataModified = false;
    
    private final static InheritanceDemo demo = new InheritanceDemo();
    private HierarchyStore hstore = null;
    private static java.util.Properties propertiesFile = new java.util.Properties();
    private java.util.Properties datasetPropertiesFile = new java.util.Properties();
    private static String inStream = null;
    private static String outStream = null;
        
    public InheritanceDemo()
    {
        super ("Inheritance Demo");
    }
    
    private void initMenus ()
    {
        fileMenuName = propertiesFile.getProperty("file_menu_name");
        fileMenuItems = new String[] {
            propertiesFile.getProperty("file_menu_item_1"),
            propertiesFile.getProperty("file_menu_item_2"),
            propertiesFile.getProperty("file_menu_item_3")
        };
        
        fileMenuShortCuts = new char[] {
            propertiesFile.getProperty("file_menu_shortcut_1").charAt(0),
            propertiesFile.getProperty("file_menu_shortcut_2").charAt(0),
            propertiesFile.getProperty("file_menu_shortcut_3").charAt(0)
        };
        
        editMenuName = propertiesFile.getProperty("edit_menu_name");
        editMenuItems = new String[] {
            propertiesFile.getProperty("edit_menu_item_1"),
        };
        editMenuShortCuts = new char[] {
            propertiesFile.getProperty("edit_menu_shortcut_1").charAt(0)
        };
        nodeMenuName = propertiesFile.getProperty("node_menu_name");

        fileMenu = new javax.swing.JMenu(fileMenuName);
        editMenu = new javax.swing.JMenu(editMenuName);
        nodeMenu = new javax.swing.JMenu(nodeMenuName);    
        menuBar = new javax.swing.JMenuBar();

        //mnemonics and accelerators for each menu
        for (int i=0; i < fileMenuItems.length; i++)
        {
            javax.swing.JMenuItem menuItem = new javax.swing.JMenuItem(fileMenuItems[i], fileMenuShortCuts[i]);
            menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(fileMenuShortCuts[i],
            java.awt.Event.CTRL_MASK, false));
            menuItem.addActionListener(this);
            fileMenu.add(menuItem);
        }
        menuBar.add(fileMenu);
        for (int i=0; i < editMenuItems.length; i++)
        {
            javax.swing.JMenuItem menuItem = new javax.swing.JMenuItem(editMenuItems[i], editMenuShortCuts[i]);
            menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(editMenuShortCuts[i],
            java.awt.Event.CTRL_MASK, false));
            menuItem.addActionListener(this);
            editMenu.add(menuItem);
        }
        menuBar.add(editMenu);

        menuItemNodeIdVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_1"));
        menuItemNodeIdVisible.addActionListener(this);
        menuItemNodeIdVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_node_id_visible_state")).booleanValue());
        
        menuItemNodeDescriptionVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_2"));
        menuItemNodeDescriptionVisible.addActionListener(this);
        menuItemNodeDescriptionVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_node_id_description_state")).booleanValue());
        
        menuItemPropertyDescriptionVisible = 
            new javax.swing.JCheckBoxMenuItem(propertiesFile.getProperty("node_menu_item_3"));
        menuItemPropertyDescriptionVisible.addActionListener(this);
        menuItemPropertyDescriptionVisible.setState(
            new java.lang.Boolean(propertiesFile.getProperty("node_menu_property_description_visible_state")).booleanValue());
        
        nodeMenu.add(menuItemNodeIdVisible);
        nodeMenu.add(menuItemNodeDescriptionVisible);
        nodeMenu.add(menuItemPropertyDescriptionVisible);
        menuBar.add(nodeMenu);

        //disable certain items
        //file-save
        javax.swing.JMenuItem grayItem = fileMenu.getItem(1);
        grayItem.setEnabled(false);
    }
    
    private void init ()
    {
        try
        {
            datasetPropertiesFile.load(new java.io.FileInputStream(inStream));
            hstore = new HierarchyStore(datasetPropertiesFile);
        }
        catch (Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }
        setDataModified(false);

        //load some configuration values
        CShape cshape = new CShape();

        cshape.setShowId(menuItemNodeIdVisible.getState());
        cshape.setShowNodeDescription(menuItemNodeDescriptionVisible.getState());
        cshape.setShowPropertyDescription(menuItemPropertyDescriptionVisible.getState());

        cshape.setIdFont(
            propertiesFile.getProperty("id_font_name"),
            new java.lang.Integer(propertiesFile.getProperty("id_font_style")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("id_font_size")).intValue());
        cshape.setDescriptionFont(
            propertiesFile.getProperty("description_font_name"),
            new java.lang.Integer(propertiesFile.getProperty("description_font_style")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("description_font_size")).intValue());
        setSize(
            new java.lang.Integer(propertiesFile.getProperty("application_frame_width")).intValue(),
            new java.lang.Integer(propertiesFile.getProperty("application_frame_height")).intValue());
            
        tabbedPane = new javax.swing.JTabbedPane();
        shapeHierarchy = hstore.getShapeHierarchy();
        shapeTree = new javax.swing.JTree(shapeHierarchy);
        shapeTree.setCellRenderer(new CShapeTreeCellRenderer());
        shapeTree.setCellEditor(new CShapeTreeCellTextEditor());
        shapeTree.setEditable(true);
        shapeTree.setRowHeight(50);
        shapeTree.setSelectionRow(1);
        shapeTree.setVisible(true);
        shapePropertyEditor = new ShapePropertyEditor(shapeTree,shapeHierarchy,
            propertiesFile.getProperty("shape_editor_title"),propertiesFile,this,demo);
        
        colorHierarchy = hstore.getColorHierarchy();
        colorTree = new javax.swing.JTree(colorHierarchy);
        colorTree.setCellRenderer(new CShapeTreeCellRenderer());
        colorTree.setCellEditor(new CShapeTreeCellTextEditor());
        colorTree.setEditable(true);
        colorTree.setRowHeight(50);
        colorTree.setSelectionRow(1);
        colorTree.setVisible(true);
        colorPropertyEditor = new ColorPropertyEditor(colorTree,colorHierarchy,
            propertiesFile.getProperty("color_editor_title"),propertiesFile,this,demo);

        edgeHierarchy = hstore.getEdgeHierarchy();
        edgeTree = new javax.swing.JTree(edgeHierarchy);
        edgeTree.setCellRenderer(new CShapeTreeCellRenderer());
        edgeTree.setCellEditor(new CShapeTreeCellTextEditor());
        edgeTree.setEditable(true);
        edgeTree.setRowHeight(50);
        edgeTree.setSelectionRow(1);
        edgeTree.setVisible(true);
        edgePropertyEditor = new EdgePropertyEditor(edgeTree,edgeHierarchy,
            propertiesFile.getProperty("edge_editor_title"),propertiesFile,this,demo);
 
        ExpandTree(shapeTree);
        ExpandTree(colorTree);
        ExpandTree(edgeTree);
        
        nodeList = new javax.swing.JList(hstore.getNodes());
        nodeListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        nodeList.setSelectionModel(nodeListSelectionModel);
        nodeListSelectionModel.addSelectionInterval(0,0);
        nodeList.setFixedCellHeight(50);
        nodeList.setCellRenderer(new NodeRenderer());
        nodeList.setVisible(true);

/*      support for editing list as a combo box if needed, needs debugging
  
        javax.swing.JComboBox nodeComboBox = new javax.swing.JComboBox();
        javax.swing.DefaultComboBoxModel nodeComboBoxModel = new javax.swing.DefaultComboBoxModel(hstore.getNodes());
        nodeComboBox.setModel(nodeComboBoxModel);
        nodeComboBox.setSelectedIndex(0);
        nodeComboBox.setRenderer(new NodeRenderer());
        nodeComboBox.setEditor(new CShapeComboBoxEditor());
        nodeComboBox.setEditable(true);
        nodeComboBox.setVisible(true);
        tabbedPane.addTab(propertiesFile.getProperty("nodes_list_tab_title"), nodeComboBox);
*/

        tabbedPane.addTab(propertiesFile.getProperty("nodes_list_tab_title"), nodeList);
        tabbedPane.addTab(propertiesFile.getProperty("shape_hierarchy_tab_title"),shapeTree);
        tabbedPane.addTab(propertiesFile.getProperty("color_hierarchy_tab_title"),colorTree);
        tabbedPane.addTab(propertiesFile.getProperty("edge_hierarchy_tab_title"),edgeTree);
        tabbedPane.setVisible(true);

        shapeTree.addMouseListener(this);                
        shapeTree.addKeyListener(this);
        shapeTree.addTreeSelectionListener(this);
        shapeHierarchy.addTreeModelListener(this);

        colorTree.addMouseListener(this);                
        colorTree.addKeyListener(this);    
        colorTree.addTreeSelectionListener(this);
        colorHierarchy.addTreeModelListener(this);

        edgeTree.addMouseListener(this);                
        edgeTree.addKeyListener(this);    
        edgeTree.addTreeSelectionListener(this);
        edgeHierarchy.addTreeModelListener(this);

        nodeList.addListSelectionListener(this);
        tabbedPane.addChangeListener(this);  

        getContentPane().add(tabbedPane);
    }

    private void ExpandTree(javax.swing.JTree tree)
    {
        //expand entire tree, kludge, but could find a better way
        int currentRowCount = 0;
        int previousRowCount = -1;
        while (previousRowCount != currentRowCount)
        {
            previousRowCount = tree.getRowCount();
            for (int r = 0; r < previousRowCount; r++) tree.expandRow(r);
            currentRowCount = tree.getRowCount();
        }
    }

    public void mouseClicked(java.awt.event.MouseEvent me){}
    public void mousePressed(java.awt.event.MouseEvent me){}
    public void mouseReleased(java.awt.event.MouseEvent me){}
    public void mouseEntered(java.awt.event.MouseEvent me){}
    public void mouseExited(java.awt.event.MouseEvent me){}    
    public void keyTyped(java.awt.event.KeyEvent ke){}    
    public void keyPressed(java.awt.event.KeyEvent ke){}
    public void keyReleased(java.awt.event.KeyEvent ke){}
    
    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        String command = ae.getActionCommand();
        if (propertiesFile.getProperty("file_menu_item_1").equals(command)) Open(); 
        if (propertiesFile.getProperty("file_menu_item_2").equals(command)) Save(); 
        if (propertiesFile.getProperty("file_menu_item_3").equals(command)) Exit();
        if (propertiesFile.getProperty("edit_menu_item_1").equals(command)) 
        {
            showEditor(tabbedPane.getSelectedIndex());   
        }
        if (propertiesFile.getProperty("node_menu_item_1").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowId(menuItemNodeIdVisible.getState());
        }
        if (propertiesFile.getProperty("node_menu_item_2").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowNodeDescription(menuItemNodeDescriptionVisible.getState());
        }
        if (propertiesFile.getProperty("node_menu_item_3").equals(command)) 
        {
            CShape cshape = new CShape();
            cshape.setShowPropertyDescription(menuItemPropertyDescriptionVisible.getState());
        }
        invalidateCShapes();
    }
    
    private void invalidateCShapes()
    {
        tabbedPane.paintComponents(tabbedPane.getGraphics());
        if (shapePropertyEditor.isVisible())
        {
            shapePropertyEditor.paintComponents(shapePropertyEditor.getGraphics());
        }
        if (colorPropertyEditor.isVisible())
        {
            colorPropertyEditor.paintComponents(colorPropertyEditor.getGraphics());
        }
        if (edgePropertyEditor.isVisible())
        {
            edgePropertyEditor.paintComponents(edgePropertyEditor.getGraphics());
        }
    }
    
    private void cleanStart()
    {
        tabbedPane.setVisible(false);
        shapePropertyEditor.setVisible(false);
        colorPropertyEditor.setVisible(false);
        edgePropertyEditor.setVisible(false);
        
        shapeTree = null;
    	colorTree = null;
    	edgeTree = null;
    	shapeHierarchy = null;
   	colorHierarchy = null;
    	edgeHierarchy = null;
    	shapePropertyEditor = null;
    	colorPropertyEditor = null;
    	edgePropertyEditor = null;
        
        setDataModified(false);
        updateHierarchyStreamInfo();
        getContentPane().removeAll();
        pack();
    }

    private void Open()
    {
        javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
        chooser.setMultiSelectionEnabled(false);
        String extensions[] = {"properties"};
        chooser.addChoosableFileFilter(new FileFilterXML(extensions, "Hierarchy (*.properties)"));
        int option = chooser.showOpenDialog(this);
        if (option == javax.swing.JFileChooser.APPROVE_OPTION) 
        {
            if (chooser.getSelectedFile() != null)
            {
                inStream = chooser.getSelectedFile().getName();
                cleanStart();
                demo.init();
                setDataModified(false);
                pack();
                updateHierarchyStreamInfo();
                setVisible(true);
            }
        }
    }
        
    private void Save()
    {
        try
        {
            if (dataModified)
            {
                if (outStream == null)
                {
                    javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
                    chooser.setMultiSelectionEnabled(false);
                    chooser.setFileSelectionMode(javax.swing.JFileChooser.FILES_ONLY);
                    String extensions[] = {"properties"};
                    chooser.addChoosableFileFilter(new FileFilterXML(extensions, "Hierarchies (*.properties)"));
                    int option = chooser.showSaveDialog(this);
                    if (option == javax.swing.JFileChooser.APPROVE_OPTION) 
                    {
                        if (chooser.getSelectedFile() != null)
                        {
                            outStream = chooser.getSelectedFile().getName();
                        }
                    }
                }
                if (outStream != null)
                {
                    java.util.Properties outputPropertiesFile = new java.util.Properties();
                    hstore.Save(outputPropertiesFile);
                    outputPropertiesFile.store(new java.io.FileOutputStream(outStream),"");
                    setDataModified(false);
                }
            }
        }
        catch (Exception e)
        {
            System.out.println("Exception writing to "+outStream);
        }
    }
            
    private void Exit()
    {
        Save();
        System.exit(0);
    }
       
    // make the editor frame associated with the current hierarchy tab visible and hide the others
    private void showEditor(int editor)
    {
        shapePropertyEditor.setVisible(editor == 1);
        colorPropertyEditor.setVisible(editor == 2);
        edgePropertyEditor.setVisible(editor == 3);
//        setVisible(true);
    }
    
    private void updateEditors(CShape target)
    {
        shapePropertyEditor.setShape(target.getShape());
        shapePropertyEditor.setColor(target.getColor());
        shapePropertyEditor.setEdge(target.getEdge());
        
        colorPropertyEditor.setShape(target.getShape());
        colorPropertyEditor.setColor(target.getColor());
        colorPropertyEditor.setEdge(target.getEdge());

        edgePropertyEditor.setShape(target.getShape());
        edgePropertyEditor.setColor(target.getColor());
        edgePropertyEditor.setEdge(target.getEdge());
    }
    
    // a different list node was selected, update the three hierarchies, and their editors    
    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
            int index = nodeList.getSelectedIndex();
            CShape target = hstore.getNode(index);
            updateEditors(target);    
        
        if (tabbedPane.getSelectedIndex() == 0)  // ignore this event if a side-effect of a tree selection
        {
            UpdateSelectionInTrees(shapeTree, shapeHierarchy, target);
            UpdateSelectionInTrees(colorTree, colorHierarchy, target);
            UpdateSelectionInTrees(edgeTree, edgeHierarchy, target);
        }
   }
    
    // a different tree node was selected, update the node list, the other hierarchies, and the editors    
    public void valueChanged(javax.swing.event.TreeSelectionEvent tse)
    {
            javax.swing.tree.TreePath path;
            javax.swing.tree.DefaultMutableTreeNode selectedNode;
            CShape target;

            path = tse.getNewLeadSelectionPath();
            if (path == null) return; // nothing selected
            selectedNode = (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
            target = (CShape)selectedNode.getUserObject();
            updateEditors(target);
            invalidateCShapes();
        
        if (tabbedPane.getSelectedIndex() != 0)  // ignore this event if a side-effect of a node selection
        {
            UpdateSelectionInTrees(shapeTree, shapeHierarchy, target);
            UpdateSelectionInTrees(colorTree, colorHierarchy, target);
            UpdateSelectionInTrees(edgeTree, edgeHierarchy, target);
        
            // set selection in list of nodes to the one selected in the tree
            for (int i = 0; i < nodeStrings.length; i++)
            {
                CShape cshape = hstore.getNode(i);
                if (cshape == target) 
                {
                    nodeListSelectionModel.addSelectionInterval(0,i);
                    break;
                }
            }
            showEditor(tabbedPane.getSelectedIndex());
        }
    }
    
    public void stateChanged(javax.swing.event.ChangeEvent ce)
    {
        // different tab selected so show the appropriate editor, or none
        showEditor(tabbedPane.getSelectedIndex());
        
        // could have just applied a change, should really be handled by changeProperty event handler
        int index = nodeList.getSelectedIndex();
        CShape target = hstore.getNode(index);
        updateEditors(target);
    }
            
    // find a target CShape in a tree model and select that node in a tree
    private void UpdateSelectionInTrees(javax.swing.JTree tree,
                                        javax.swing.tree.DefaultTreeModel model,
                                        CShape target)
    {
        javax.swing.tree.DefaultMutableTreeNode root = 
            (javax.swing.tree.DefaultMutableTreeNode)model.getRoot();
        java.util.Enumeration e = root.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode node = 
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();
            CShape cshape = (CShape)node.getUserObject();
            if (cshape == target)
            {
                javax.swing.tree.TreePath p = new javax.swing.tree.TreePath(model.getPathToRoot(node));
                tree.setSelectionPath(p);
            }
        }
    }
    
    public void treeNodesChanged(javax.swing.event.TreeModelEvent tme) 
    {
        setDataModified(true);
    }
    public void treeNodesInserted(javax.swing.event.TreeModelEvent tme) {}
    public void treeNodesRemoved(javax.swing.event.TreeModelEvent tme) {}
    public void treeStructureChanged(javax.swing.event.TreeModelEvent tme) {}

    public void propertyChanged()
    {            
        invalidateCShapes();  //we would add this if frame wasn't already being set visible
        setDataModified(true);
    }

    private void setDataModified(boolean b)
    {
        dataModified = b;
        updateHierarchyStreamInfo();
        //file-save-menu
        javax.swing.JMenuItem grayItem = fileMenu.getItem(1);
        grayItem.setEnabled(dataModified);
    }    

    private void updateHierarchyStreamInfo()
    {
        String rNull = propertiesFile.getProperty("read_from_null_file_prompt");
        String r = propertiesFile.getProperty("read_from_file_prompt");
        String wNull = propertiesFile.getProperty("write_to_null_file_prompt");
        String w = propertiesFile.getProperty("write_to_file_prompt");

        inStreamLabel.setText((inStream == null) ? rNull : r + ((dataModified) ? "*" : "") + inStream);
        outStreamLabel.setText((outStream == null) ? wNull : w + outStream);
    }
    
    private void initStreamInfo()
    {
        updateHierarchyStreamInfo();
        javax.swing.JPanel p = new javax.swing.JPanel();
        p.add(inStreamLabel,java.awt.BorderLayout.NORTH);
        p.add(outStreamLabel,java.awt.BorderLayout.SOUTH);
        getContentPane().add(p,java.awt.BorderLayout.NORTH);
    }
    
    public static void main(String args[]) 
    {
        demo.addWindowListener(
            new java.awt.event.WindowAdapter() 
            {
                public void windowClosing(java.awt.event.WindowEvent ev) 
                {
                    if (dataModified) demo.Save();
                    System.exit(0);
                }
            }
        );
        try
        {
            propertiesFile.load(new java.io.FileInputStream("inheritance.properties"));
        }
        catch (Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }
        demo.initMenus();
        demo.setJMenuBar(menuBar);
        if (args.length >= 1) 
        {
            inStream = args[0];
            demo.init();
            if (args.length >= 2) outStream = args[1];
        }
        demo.initStreamInfo();
        demo.setVisible(true);
    }
}
