public class ShapePropertyEditor extends javax.swing.JFrame 
implements javax.swing.event.ListSelectionListener,
           java.awt.event.ActionListener
{
    private String[] shapeStrings = {CShape.CIRCLE_STRING, CShape.SQUARE_STRING, 
        CShape.TRIANGLE_STRING,CShape.DIAMOND_STRING, 
        CShape.PARRALLELOGRAM_STRING, CShape.PENTAGON_STRING, 
        CShape.INVERTED_TRIANGLE_STRING,
        CShape.TRAPEZOID_STRING};
    private javax.swing.JList shapeList;
    private javax.swing.DefaultListSelectionModel shapeListSelectionModel = 
        new javax.swing.DefaultListSelectionModel();
    private ShapePropertyEditorRenderer shapeRenderer = null;
    private int minWidth, minHeight, boundsX, boundsY;
    private javax.swing.JTree shapeTree;
    private javax.swing.tree.DefaultTreeModel shapeTreeModel = null;
    private javax.swing.tree.DefaultMutableTreeNode currentChild = null;
    private javax.swing.JFrame containingFrame = null;
    private javax.swing.JButton applyButton = new javax.swing.JButton();
    private InheritanceDemo demo;
 
    public ShapePropertyEditor(javax.swing.JTree tree, javax.swing.tree.DefaultTreeModel model, 
                               String title, java.util.Properties propertiesFile,
                               javax.swing.JFrame frame, InheritanceDemo idemo)
    {
        super(title);
 
        minWidth = new java.lang.Integer(propertiesFile.getProperty("editor_frame_width")).intValue();
        minHeight = new java.lang.Integer(propertiesFile.getProperty("editor_frame_height")).intValue();
        boundsX = new java.lang.Integer(propertiesFile.getProperty("editor_frame_boundsX")).intValue();
        boundsY = new java.lang.Integer(propertiesFile.getProperty("editor_frame_boundsY")).intValue();
        applyButton.setText(propertiesFile.getProperty("editor_apply_button_text"));
        
        shapeTree = tree;
        shapeTreeModel = model;
        containingFrame = frame;
        demo = idemo;
        setSize(minWidth,minHeight);
        shapeRenderer = new ShapePropertyEditorRenderer(propertiesFile);
                
        shapeList = new javax.swing.JList(shapeStrings);
        shapeListSelectionModel.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        shapeList.setSelectionModel(shapeListSelectionModel);
        shapeList.addListSelectionListener(this);
        shapeListSelectionModel.addSelectionInterval(0,0);
        shapeList.setFixedCellHeight(50);
        shapeList.setCellRenderer(shapeRenderer);
        shapeList.setVisible(true);

        applyButton.addActionListener(this);
        applyButton.setVisible(true);
        getContentPane().add(shapeList);
        getContentPane().add(applyButton,java.awt.BorderLayout.SOUTH);
    }
    
    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        ApplySelections();
        demo.propertyChanged();
    }

    private void ApplySelections()
    {
        // this code gets the last selection.  Maybe we want to do something else
        javax.swing.tree.TreePath path = shapeTree.getLeadSelectionPath();
        if (path == null) return; // nothing selected
        javax.swing.tree.DefaultMutableTreeNode selectedNode = 
            (javax.swing.tree.DefaultMutableTreeNode)path.getPathComponent(path.getPathCount()-1);
        CShape cshape = (CShape)selectedNode.getUserObject();
        cshape.setShape(shapeList.getSelectedIndex());

        PropagateProperties(selectedNode,shapeList.getSelectedIndex());
        containingFrame.setVisible(true);
    }
             
    private void PropagateProperties(javax.swing.tree.DefaultMutableTreeNode parent, int shape)
    {
        // propagate properties to children
        java.util.Enumeration e = parent.preorderEnumeration();
        while (e.hasMoreElements())
        {
            javax.swing.tree.DefaultMutableTreeNode nextNode =
                (javax.swing.tree.DefaultMutableTreeNode)e.nextElement();

            CShape nextShape = (CShape)nextNode.getUserObject();
            nextShape.setShape(shape);
        }
    }    
        
    public void valueChanged(javax.swing.event.ListSelectionEvent lse)
    {
    }

    public void setBounds(java.awt.Rectangle r)
    {
        r.width = minWidth;
        r.height = minHeight;
        super.setBounds(r);
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        super.setBounds(boundsX, boundsY, minWidth, minHeight);
    }
    
    public void setShape (int s)
    {
        shapeListSelectionModel.addSelectionInterval(0,s);
    }

    public int getShape()
    {
        return shapeList.getSelectedIndex();
    }    

    public void setColor (int c)
    {
        shapeRenderer.setColor(c);
    }

    public void setEdge (int e)
    {
        shapeRenderer.setEdge(e);
    }
}