public class ShapePropertyEditorRenderer extends javax.swing.JPanel implements javax.swing.ListCellRenderer
{
    private int selectionIndex = 0;
    private boolean isElementSelected = false;
    private java.awt.image.BufferedImage bimg;
    private int fillColor = CShape.DEFAULT_COLOR;
    private int edge = CShape.DEFAULT_EDGE;
    private java.util.Properties propertiesFile = new java.util.Properties();
    private String shapeDescriptions[] = new String[8];
      
    public ShapePropertyEditorRenderer(java.util.Properties p)
    {
        setBackground(java.awt.Color.white);
        propertiesFile = p;
        shapeDescriptions[0] = p.getProperty("circle_property_description");
        shapeDescriptions[1] = p.getProperty("square_property_description");
        shapeDescriptions[2] = p.getProperty("triangle_property_description");
        shapeDescriptions[3] = p.getProperty("diamond_property_description");
        shapeDescriptions[4] = p.getProperty("parrallelogram_property_description");
        shapeDescriptions[5] = p.getProperty("pentagon_property_description");
        shapeDescriptions[6] = p.getProperty("inverted_triangle_property_description");
        shapeDescriptions[7] = p.getProperty("trapezoid_property_description");
    }
    
    public java.awt.Component getListCellRendererComponent(
    javax.swing.JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
    {
        selectionIndex = index;
        isElementSelected = isSelected;
        if (isSelected) setBackground(java.awt.Color.gray);
        return this;
    }
    
    public void drawShape(int w, int h, java.awt.Graphics2D g2) 
    {
        CShape cshape = new CShape("0",selectionIndex,fillColor,edge,shapeDescriptions[selectionIndex]);
        if (isElementSelected) 
        {
            setBackground(java.awt.Color.gray);
        }
        else
        {
            setBackground(java.awt.Color.white);
        }
        cshape.draw(g2,5.0f,5.0f,40.0f,40.0f);        
        setBackground(java.awt.Color.white);
    }

    public java.awt.Graphics2D createGraphics2D(int w, int h) {
        java.awt.Graphics2D g2 = null;
        if (bimg == null || bimg.getWidth() != w || bimg.getHeight() != h) {
            bimg = (java.awt.image.BufferedImage) createImage(w, h);
        } 
        g2 = bimg.createGraphics();
        g2.setBackground(getBackground());
        g2.clearRect(0, 0, w, h);
        g2.setRenderingHint(java.awt.RenderingHints.KEY_ANTIALIASING,
                            java.awt.RenderingHints.VALUE_ANTIALIAS_ON);
        return g2;
    }

    public void paint(java.awt.Graphics g) 
    {
        java.awt.Dimension d = getSize();
        java.awt.Graphics2D g2 = createGraphics2D(d.width, d.height);
        drawShape(d.width, d.height, g2);
        g2.dispose();
        g.drawImage(bimg, 0, 0, this);
    }

    public void setColor (int index)
    {
        fillColor = index;
    }
    
    public void setEdge (int index)
    {
        edge = index;
    }
}
    
    
    