package edu.mit.is.okapi.servlets;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
//import eay.JCrypt;

/**
 * Okapi MIT Login Servlet.
 *
 * Processes login
 *
 * @author OKI Dev team
 * @version $Revision: 1.3 $ / $Date: 2002/05/07 22:24:56 $
 */

public class LoginServlet extends HttpServlet {
    /**Initialize global variables*/
    public void init(ServletConfig config) throws ServletException
    {
	super.init(config);
    }


    /**Process the HTTP Get request*/
    public void doGet(HttpServletRequest request,
		      HttpServletResponse response)
	throws ServletException, IOException
    {

	javax.servlet.http.HttpSession session =  request.getSession();

	/*
	   The user may be in one of several states when this method is called.

	   Initial access should be with the GET parameter of the URL
	   which was requested but required authentication, or with no
	   parameters.

	   A test is done to be sure they support cookies.

	   if they accept cookies, they are sent to the SSL port for
	   authentication.

	   Authentication may be via certificate, or by a form submission.
	   The form submission uses the POST method.
	*/


	// There may be:
	//                certificate + ssl
	//                    log user in
	//                ssl no cert
	//                    give form
	//                no ssl, but cookie
	//                    redirect to ssl
	//                no ssl no cookie
	//                   was cookie test attempted?
	//                      yes: error need cookies
	//                      no: test cookies

	boolean isSecure = (request.getServerPort() == 443);
	String emailFromCert = (String)
	    request.getAttribute("SSL_CLIENT_S_DN_Email");

	if (isSecure)
	{
	    // is there a certificate?
	    if (!"nodefault".equals(emailFromCert))
	    {
		/*
LOGIN BY CERTIFICATE AUTHENTICATION
		*/
		// yes there is
		long time = (new java.util.Date()).getTime()/1000;

		String okIPCookie = null;
		Cookie[] cookies = request.getCookies();
		for (int i = 0; i < cookies.length; i++)
		{
		    Cookie c = cookies[i];
		    String name = c.getName();
		    if (name.equals("ok_iptest"))
		    {
			okIPCookie = c.getValue();
		    }
		}

		String ignoreIP = "N";
		if (!request.getRemoteAddr().toString().equals(
							       okIPCookie)) {
		    System.err.println("IP was "+okIPCookie+" is "+
				       request.getRemoteAddr());
		    ignoreIP = "Y";
		}

		// log user in by giving cookie and setting it in the database
		int rand = (int) Math.round(Math.random()*100000+0.5);
		int rand1 = (int) Math.round(Math.random()*1000+0.5);
		int rand2 = (int) Math.round(Math.random()*100000+0.5);
		String ssl_cookie= emailFromCert+"/"
		    +request.getRemoteAddr()+"/N/"
		    +(new java.util.Date()).getTime()/1000+"/"+rand+rand1;
		String cookie= emailFromCert+"/"
		    +request.getRemoteAddr()+"/"+ignoreIP+"/"
		    +time+"/"+rand2;

		Cookie ssl_c =  new Cookie("ok_authen_ssl", ssl_cookie);
		ssl_c.setPath("/");
		ssl_c.setSecure(true);
		Cookie c = new Cookie("ok_authen", cookie);
		c.setPath("/");
		response.addCookie(c);
		response.addCookie(ssl_c);

		// put cookie in the database
		try {
		  /*
REPLACE WITH
     OKI API
		    if (db == null) {
			okapi.coreobjects.OkapiSite.giveDBtoTool(
					 "okapi.servlets.okapi.LoginServlet");
			// db should now be set via callback to setDB
			// note, we never return the db to the site.
		    }

		    String query = "insert into AUTHENTICATION_COOKIE_TABLE "+
		"(Cookie, SslCookie, LastAccess, IP, IgnoreIP, UserKey)"+
			" values(?, ?, ?, ?, ?, ?)";
		    String[] queryargs = {"ok_authen="+cookie,
					  "ok_authen_ssl="+ssl_cookie,
					  ""+time,
					  ""+request.getRemoteAddr(),
					  ignoreIP,
					  emailFromCert};
		    try {
			int resultCount = db.executeUpdate(query, queryargs);
		    } catch(Exception se ) {
			System.err.println("Error: "+se);
			se.printStackTrace();
		    }//end catch
		  */
		} catch(Exception se ) {
		    System.err.println("Error: "+se);
		    se.printStackTrace();
		}//end catch

		// they have the cookie
		// we have the cookie

		// now send them where they wanted to go.
		String h = "";
		String u = "";

		try
		{
		    h = request.getParameter("h");
		    u = request.getParameter("u");
		}
		catch(Exception e)
		{
		}

		System.err.println("Ok, giving cookies and redirecting");
		response.sendRedirect("http"+
				      ("1".equals(h) ? "" :"s")+
				      "://"+
				      request.getServerName()+
				      java.net.URLDecoder.decode(u));
	    }
	    else // they don't have a certificate.  Try login via password
	    {
		/*
LOGIN BY USERNAME/PASSWORD STEP 1: give form
		*/

		String u = "";
		String h = "";
		try
		{
		    u = request.getParameter("u");
		    h = request.getParameter("h");
		}
		catch(Exception e)
		{
		}

		// give form
		response.setContentType("text/html");
		printForm(response.getWriter(), u, h, "", "");
	    }
	}
	else
	{
	    // not secure
	    // is there a parameter cookie_test?
	    String cookieTest = null;

	    try
	    {
		cookieTest = request.getParameter("cookie_test");
	    }
	    catch(Exception e)
	    {
	    }

	    if (cookieTest != null)
	    {
		/*
INITIAL PHASE STEP 2: verify cookies are being accepted
		*/

		// we tried to give a cookie, is it there?
		String okCookie = null;
		Cookie[] cookies = request.getCookies();
		for (int i = 0; i < cookies.length; i++)
		{
		    Cookie c = cookies[i];
		    String name = c.getName();
		    if (name.equals("ok_test"))
		    {
			okCookie = c.getValue();
		    }
		}
		if ("test".equals(okCookie))
		{
		    /*
MOVE TO SSL GET PHASE (either certificate or username password)
		     */
		    // cookies work
		    // redirect, noting that we were initially insecure
		    response.sendRedirect("https://"+
					  request.getServerName()+
					  request.getRequestURI()+
					  "?"+request.getQueryString()
					  + ((request.getQueryString() == null)? "" : "&")
					  +"h=1");
		}
		else
		{
		    /*
INITIAL PHASE STEP 2 FAILURE
		     */

		    // Error page, need cookies
		    String u = "";
		    try
		    {
			u = request.getParameter("u");
		    }
		    catch(Exception e)
		    {
		    }

		    PrintWriter out = response.getWriter();
		    response.setContentType("text/html");
		    out.println("<html><head><title>cookies required</title></head>");
		    out.println("<body>");
		    out.println("<p>You must enable cookies.");
		    out.println("<p><form method=\"get\"><input type=\"submit\" value=\"continue\">");
		    out.println("<input type=\"hidden\" name=\"u\" value=\""
				+u+"\">");
		    out.println("</body></html>");
		}
	    }
	    else
	    {
		/*
INITIAL PHASE STEP 1: try cookie.
		 */

		Cookie c = new Cookie("ok_test", "test");
		c.setPath("");
		response.addCookie(c);

		Cookie c2 = new Cookie("ok_iptest", request.getRemoteAddr());
		c2.setPath("");
		response.addCookie(c2);

		response.sendRedirect("http://"+
				      request.getServerName()+
				      request.getRequestURI()+
				      "?"+request.getQueryString()+
				      "&cookie_test=1");
	    }
	}
    }

    /**Process the HTTP Post request with username/password */
    public void doPost(HttpServletRequest request,
		       HttpServletResponse response)
	throws ServletException, IOException
    {
	//	System.err.println("Post");

	javax.servlet.http.HttpSession session =  request.getSession();

	java.util.Hashtable postValues
	    = HttpUtils.parsePostData(request.getContentLength(),
				      request.getInputStream());

	String username = ((String []) postValues.get("username"))[0];
	username = username.replace(' ', '_');
	username = username.toLowerCase();
	String usernameSuffix = "";
	if (postValues.get("suffix") != null)
	    usernameSuffix = ((String []) postValues.get("suffix"))[0];
	String password = ((String []) postValues.get("password"))[0];

	// ignore any extra @
	if (!"".equals(usernameSuffix) && (username.indexOf("@") != -1)) {
	    username = username.substring(0,username.indexOf("@"));
	}

	String u = ((String[]) postValues.get("u"))[0];
	String h = ((String[]) postValues.get("h"))[0];

	String visitor = (postValues.get("visitor") != null) ?
	    ((String[]) postValues.get("visitor"))[0] : "";


	//System.err.println("h:'"+h+"'");
	//System.err.println("u:'"+u+"'");
	System.err.println("username:'"+username+"'");
	System.err.println("usernameSuffix:'"+usernameSuffix+"'");
	System.err.println("visitor:'"+visitor+"'");
	System.err.println("keys:"
	+java.util.Arrays.asList(postValues.keySet().toArray()));

	String userName = username+usernameSuffix;

	// Want username, password, optional url
	// OR 'visitor'  username='visitor', password='visitor', optional url

	if ("visitor".equals(username) && "visitor".equals(password)
	    && "visitor".equals(visitor)) {
	    Cookie c = new Cookie("ok_authen", "visitor");
	    c.setPath("/");
	    response.addCookie(c);
	    // now send them where they wanted to go.
	    response.sendRedirect("http"+
				  ("1".equals(h) ? "" :"s")+
				  "://"+
				  request.getServerName()+u);
	} else {
	    // look up username in password db
	    // select name, password_encoded, force_change
	    //                from USER_PASSWORD where name = ?
	    // 'username'

	    try {
		  /*
REPLACE WITH
     OKI API

		if (db == null) {
		    okapi.coreobjects.OkapiSite.giveDBtoTool(
			     "okapi.servlets.okapi.LoginServlet");
		    // db should now be set via callback to setDB
		    // note, we never return the db to the site.
		}

		String query="SELECT password_encoded, force_change, password_hint"+
			" FROM USER_PASSWORD"+
			" where name = ?";

		String[] queryargs = {userName};
		java.util.Hashtable loginHT
		    = db.getFirstRowAsHashtable(query, queryargs);

		// if user is not in passwd db
		// (nothing returned)
		if (loginHT.size() == 0) {
		    //	  error: use certificate only
		    response.setContentType("text/html");
		    System.err.println(System.currentTimeMillis()/1000
				   + " LOGIN FAILURE - UNKNOWN USERNAME - USER '"+username+"'");
		    printForm(response.getWriter(), u, h, "",
			  "<h2 class=\"error\">Username not known</h2>\n"+
			  "<p>Please try again below.  Visit our <a href=\"/help.html\">help page</a> if you continue to have difficulty logging in. </p>");
		    return;
		} else {
		    if (postValues.get("hint") != null) {
			// give hint
			String hint = (String) loginHT.get("PASSWORD_HINT");
			response.setContentType("text/html");
			if ("".equals(hint)) {
			    printForm(response.getWriter(), u, h, username,
			      "<h2 class=\"error\">Password Hint</h2>\n"+
			      "<p>You have no password hint.</p>");
			} else {
			    printForm(response.getWriter(), u, h, username,
			      "<h2 class=\"error\">Password Hint</h2>\n"+
			      "<p>Your password hint is '"+hint+"'</p>");
			}
			return;
		    } else {
			// login

			String passwordEncoded = (String) loginHT.get("PASSWORD_ENCODED");
			int forceChange
			    = ((java.math.BigDecimal) loginHT.get("FORCE_CHANGE")).intValue();

			System.err.println("has password");
			// compare password with password_encoded

			String salt = passwordEncoded.substring(0,2);
			String tryPassword = JCrypt.crypt(salt, password);

			if (passwordEncoded.equals(tryPassword)) {
			    if (forceChange == 1) { // force change
				// forward to newpass
				response.sendRedirect("../changePassword/?force=1");
			    }

			    // OK

			    / *
PASSWORD ACCEPTED
			    * /

			    long time = (new java.util.Date()).getTime()/1000;


			    // log user in by giving cookie and setting it in the database
			    int rand = (int) Math.round(Math.random()*100000+0.5);
			    int rand1 = (int) Math.round(Math.random()*1000+0.5);
			    int rand2 = (int) Math.round(Math.random()*100000+0.5);

			    // look at IP addr from cookie
			    String okIPCookie = null;
			    Cookie[] cookies = request.getCookies();
			    for (int i = 0; i < cookies.length; i++)
			    {
				Cookie c = cookies[i];
				String name = c.getName();
				if (name.equals("ok_iptest"))
				{
				    okIPCookie = c.getValue();
				}
			    }

			    String ignoreIP = "N";
			    if (!request.getRemoteAddr().toString().equals(
									   okIPCookie)) {
				System.err.println("IP was "+okIPCookie+" is "+
						   request.getRemoteAddr());
				ignoreIP = "Y";
			    }

			    String ssl_cookie= userName+"/"
				+request.getRemoteAddr()+"/N/"
				+(new java.util.Date()).getTime()/1000+"/"+rand+rand1;
			    String cookie= userName+"/"
				+request.getRemoteAddr()+"/"+ignoreIP+"/"
				+time+"/"+rand2;

			    Cookie ssl_c =  new Cookie("ok_authen_ssl", ssl_cookie);
			    ssl_c.setPath("/");
			    ssl_c.setSecure(true);
			    Cookie c = new Cookie("ok_authen", cookie);
			    c.setPath("/");
			    response.addCookie(c);
			    response.addCookie(ssl_c);

			    // put cookie in the database
			    try {
				if (db == null) {
				    okapi.coreobjects.OkapiSite.giveDBtoTool(
									     "okapi.servlets.okapi.LoginServlet");
				// db should now be set via callback to setDB
				// note, we never return the db to the site.
				}
				
				query = "insert into AUTHENTICATION_COOKIE_TABLE "+
				    "(Cookie, SslCookie, LastAccess, IP, IgnoreIP, UserKey)"+
				    " values(?, ?, ?, ?, ?, ?)";
				queryargs = new String[] {"ok_authen="+cookie,
							  "ok_authen_ssl="+ssl_cookie,
							  ""+time,
							  ""+request.getRemoteAddr(),
							  ignoreIP,
							  userName};
				try {
				    int resultCount = db.executeUpdate(query, queryargs);
				} catch(Exception se ) {
				    System.err.println("Error: "+se);
				    se.printStackTrace();
			    }//end catch
			    } catch(Exception se ) {
				System.err.println("Error: "+se);
				se.printStackTrace();
			    }//end catch

			    // they have the cookie
			    // we have the cookie
			    


			    // now send them where they wanted to go.
			    System.err.println("Ok, giving cookies and redirecting");
			    System.err.print("Ignore ->");
			    response.sendRedirect("http"+
						  ("1".equals(h) ? "" :"s")+
						  "://"+
						  request.getServerName()+u);
			}
		    }
		}

	    // if password doesn't match
	    //	add error message to page, return to form
		System.err.println(System.currentTimeMillis()/1000
				   + " LOGIN FAILURE - BAD PASSWORD - USER '"+username+"'");
		PrintWriter out = response.getWriter();
		response.setContentType("text/html");
		printForm(response.getWriter(), u, h, username,
			  "<h2 class=\"error\">Password Incorrect</h2>\n"+
			  "<p>Please try again below.  Visit our <a href=\"/help.html\">help page</a> if you continue to have difficulty logging in. </p>");
		  */
	    } catch(Exception e)
	    {
	    }
	}
    }

    void printForm(PrintWriter out, String u, String h, String username, String error) {
	String head = 
"<html>\n"+
"<head>\n"+
"<title>stellar log in</title>\n"+
"\n"+
"<script language=\"JavaScript\">\n"+
"<!--\n"+
"function MM_swapImgRestore() { //v3.0\n"+
"var i,x,a=document.MM_sr; for(i=0;a&&i<a.length&&(x=a[i])&&x.oSrc;i++) x.src=x.oSrc;\n"+
"}\n"+
"\n"+
"function MM_preloadImages() { //v3.0\n"+
"var d=document; if(d.images){ if(!d.MM_p) d.MM_p=new Array();\n"+
"var i,j=d.MM_p.length,a=MM_preloadImages.arguments; for(i=0; i<a.length; i++)\n"+
"if (a[i].indexOf(\"#\")!=0){ d.MM_p[j]=new Image; d.MM_p[j++].src=a[i];}}\n"+
"}\n"+
"\n"+
"function MM_findObj(n, d) { //v4.01\n"+
"var p,i,x;  if(!d) d=document; if((p=n.indexOf(\"?\"))>0&&parent.frames.length) {\n"+
"d=parent.frames[n.substring(p+1)].document; n=n.substring(0,p);}\n"+
"if(!(x=d[n])&&d.all) x=d.all[n]; for (i=0;!x&&i<d.forms.length;i++) x=d.forms[i][n];\n"+
"for(i=0;!x&&d.layers&&i<d.layers.length;i++) x=MM_findObj(n,d.layers[i].document);\n"+
"if(!x && d.getElementById) x=d.getElementById(n); return x;\n"+
"}\n"+
"\n"+
"function MM_swapImage() { //v3.0\n"+
"var i,j=0,x,a=MM_swapImage.arguments; document.MM_sr=new Array; for(i=0;i<(a.length-2);i+=3)\n"+
"if ((x=MM_findObj(a[i]))!=null){document.MM_sr[j++]=x; if(!x.oSrc) x.oSrc=x.src; x.src=a[i+2];}\n"+
"}\n"+
"//-->\n"+
"</script>\n"+
"<link rel=\"stylesheet\" href=\"/stellarInfo.css\" type=\"text/css\">\n"+
"</head>\n"+
"\n"+
"<body bgcolor=\"#FFFFFF\" text=\"#000000\" leftmargin=\"0\" topmargin=\"0\" marginwidth=\"0\" marginheight=\"0\" onLoad=\"MM_preloadImages('/images/navbar/helpOV.gif','/images/navbar/infoOV.gif','/images/navbar/newsOV.gif','/images/navbar/starOV.gif')\">\n"+
"<table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n"+
"<tr>\n"+
"\n"+
"<!-- top nav bar -->\n"+
"<td background=\"/images/navbar/navBG.gif\" height=\"50\"> \n"+
"<table width=\"100%\" border=\"0\">\n"+
"<tr>\n"+
"<td><a href=\"/index.html\" onMouseOut=\"MM_swapImgRestore()\" onMouseOver=\"MM_swapImage('star','','/images/navbar/starOV.gif',1)\"><img name=\"star\" border=\"0\" src=\"/images/navbar/star.gif\" width=\"170\" height=\"50\"></a></td>\n"+
"<td>\n"+
"<div align=\"right\"><a href=\"/news.html\" onMouseOut=\"MM_swapImgRestore()\" onMouseOver=\"MM_swapImage('news','','/images/navbar/newsOV.gif',1)\"><img name=\"news\" border=\"0\" src=\"/images/navbar/news.gif\" width=\"70\" height=\"50\"></a><a href=\"/info.html\" onMouseOut=\"MM_swapImgRestore()\" onMouseOver=\"MM_swapImage('info','','/images/navbar/infoOV.gif',1)\"><img name=\"info\" border=\"0\" src=\"/images/navbar/info.gif\" width=\"70\" height=\"50\"></a><a href=\"/help.html\" onMouseOut=\"MM_swapImgRestore()\" onMouseOver=\"MM_swapImage('help','','/images/navbar/helpOV.gif',1)\"><img name=\"help\" border=\"0\" src=\"/images/navbar/help.gif\" width=\"70\" height=\"50\"></a></div>\n"+
"</td>\n"+
"</tr>\n"+
"</table>\n"+
"</td>\n"+
"</tr>\n"+
"<tr>\n"+
"<td>\n"+
"<table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"25\">\n"+
"<tr> \n"+
"\n"+
"<!-- If there some extra links for this section, add them here -->\n"+
"<td width=\"100\" valign=\"top\"><!-- #BeginEditable \"xtraNav\" --> <!-- #EndEditable --></td>\n"+
"<!-- the main content for the page -->\n"+
"<td valign=\"top\" width=\"100%\"><!-- #BeginEditable \"content\" -->\n"+
"<p>\n"+
"<form action=\"\" method=\"post\">\n"+
"<p>\n"+
"<input type=\"hidden\" name=\"u\" value=\""+u+"\">\n"+
"<input type=\"hidden\" name=\"h\" value=\""+h+"\">\n"+
"</p>\n"+
error+
"<h2>Login for Students outside of MIT</h2>\n"+
"<ul>\n"+
"<p>Username: \n"+
"<input type=\"text\" name=\"username\" value=\""+username+"\">\n"+
"  <i>&nbsp;&nbsp;Hint: Your username includes your institution, e.g. @nus.edu.sg<br>\n"+
"</i>Password: \n"+
"<input type=\"password\" name=\"password\">\n"+
"<input type=\"submit\" name=\"login\" value=\"login\">\n"+
"<input type=\"submit\" name=\"hint\" value=\"Retrieve your password hint\">\n"+
"<p><a href=\"../changePassword/\">Change password</a>\n"+
"<p>\n"+
"</ul>\n"+
"</form>\n"+
"\n"+
"<form action=\"\" method=\"post\">\n"+
"<h2>Visitor Access</h2>\n"+
"<ul>\n"+
"<p>Visitors may not have access to all files.<br>\n"+
"<input type=\"submit\" value=\"login as visitor\">\n"+
"<input type=\"hidden\" name=\"u\" value=\""+u+"\">\n"+
"<input type=\"hidden\" name=\"h\" value=\""+h+"\">\n"+
"<input type=\"hidden\" name=\"suffix\" value=\"visitor\">\n"+
"<input type=\"hidden\" name=\"username\" value=\"visitor\">\n"+
"<input type=\"hidden\" name=\"password\" value=\"visitor\">\n"+
"<input type=\"hidden\" name=\"visitor\" value=\"visitor\">\n"+
"</p>\n"+
"</ul>\n"+
"</form>\n"+
"<h2>MIT Access</h2>\n"+
"<ul>\n"+
"<p>MIT people should  use their <a href=\"http://web.mit.edu/is/help/cert\">MIT personal certificate</a> for authentication.\n"+
"</ul>\n"+
"<!-- #EndEditable --></td>\n"+
"</tr>\n"+
"</table>\n"+
"</td>\n"+
"</tr>\n"+
"<tr>\n"+
"\n"+
"<!-- navbar for screen readers -->\n"+
"<td align=\"right\" valign=\"bottom\">\n"+
"<p class=\"whisper\"><a href=\"/index.html\" class=\"white\"> home</a> - <a href=\"/news.html\" class=\"white\">news</a> - <a href=\"/info.html\" class=\"white\">info</a> - <a href=\"/help.html\" class=\"white\">help</a></p>\n"+
"</td>\n"+
"</tr>\n"+
"</table>\n"+
"</body>\n"+
"<!-- #EndTemplate -->\n"+
"</html>\n"+
"";

	out.println(head);
    }
    /**Clean up resources*/
    public void destroy() {
    }
}
