package edu.mit.is.service.authentication.impl.authenkrb4;
/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including software, documents, or other related items (the
   "Software"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM)  API Implementation License. By
   obtaining, using and/or copying this Software, you agree that you have
   read, understand, and will comply with the following terms and
   conditions of the MIT OKI(TM) API Implementation License:

   Permission to use, copy, modify, and distribute this Software and its
   documentation, with or without modification, for any purpose and
   without fee or royalty is hereby granted, provided that you include
   the following on ALL copies of the Software or portions thereof,
   including modifications or derivatives, that you make:

    *  The full text of the MIT OKI(TM) API Implementation License in a
       location viewable to users of the redistributed or derivative
       work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed or
       derivative Software:
       "Copyright (c) 2002 Massachusetts Institute of Technology
       All Rights Reserved."

    *  Notice of any changes or modifications to the MIT OKI(TM)
       Software, including the date the changes were made. Any modified
       software must be distributed in such as manner as to avoid any
       confusion with the original MIT OKI(TM) Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Software without
   specific, written prior permission. Title to copyright in the Software
   and any associated documentation will at all times remain with the
   copyright holders.

   The export of software employing encryption technology may require a
   specific license from the United States Government. It is the
   responsibility of any person or organization contemplating export to
   obtain such a license before exporting this Software.

*/

/*
 * $Source: /cvs/oki/tech/contrib/mit/impl/src/edu/mit/is/service/authentication/impl/authenkrb4/Authenticate.java,v $
 */

/**
 * Class containing methods used in Kerberos 4 implementation.n 
 *
 * Licensed under the {@link org.okip.service.ApiImplementationLicenseMIT MIT OKI(TM) API Implementation License}.
 *
 * @version $Name:  $ / $Revision: 1.5 $ / $Date: 2002/07/03 19:04:38 $
*/
public class Authenticate {
  private static String crealm = "ATHENA.MIT.EDU";
  private static String srealm = "ATHENA.MIT.EDU";
  private static String kdc    = "KERBEROS.MIT.EDU";

  private static String sinstFrom(String alias)
  throws Exception {
    java.net.InetAddress aliasAddr = java.net.InetAddress.getByName(alias);
    java.net.InetAddress realAddr  =
      java.net.InetAddress.getByName(aliasAddr.getHostAddress());
    String               realHost  = realAddr.getHostName().toLowerCase();
    int                  index     = realHost.indexOf(".");

    if (-1 != index) {
      realHost = realHost.substring(0, index);
    }

    return realHost;
  }

  /**
   * Method getUserCredentials
   *
   *
   * @param clientName
   * @param clientInst
   * @param clientPassWord
   *
   * @return krb4.lib.Krb4Creds
   *
   * @throws Throwable
   *
   */
  public static krb4.lib.Krb4Creds getUserCredentials(String clientName,
          String clientInst, char[] clientPassWord)
  throws Throwable {
    byte[]             key    =
      krb4.lib.crypto.des.string_to_key_bytes(new String(clientPassWord));

    krb4.lib.Krb4ASReq as_req = new krb4.lib.Krb4ASReq(clientName,
                                  clientInst, crealm, srealm);

    as_req.send(kdc);

    krb4.lib.Krb4ASRep as_rep = as_req.getKrb4ASRep();

    return as_rep.getCreds(key);
  }

  /**
   * Method getServiceTicket
   *
   *
   * @param clientCredentials
   * @param service
   * @param host
   * @param clientName
   * @param clientInst
   *
   * @return byte[]
   *
   * @throws Throwable
   *
   */
  public static byte[] getServiceTicket(krb4.lib.Krb4Creds clientCredentials,
                                        String service, String host,
                                        String clientName, String clientInst)
  throws Throwable {
    if (null != clientCredentials) {
      krb4.lib.Krb4TGSReq tgs_req = new krb4.lib.Krb4TGSReq(clientCredentials,
                                      clientName, clientInst, crealm,
                                      service, sinstFrom(host), srealm);

      tgs_req.send(kdc);

      krb4.lib.Krb4TGSRep tgs_rep = tgs_req.getKrb4TGSRep();
      krb4.lib.Krb4Creds  creds   = tgs_rep.getCreds(clientCredentials);
      krb4.lib.Krb4APReq  ap_req  = new krb4.lib.Krb4APReq(creds, clientName,
                                      clientInst, crealm, false);

      return ap_req.encode();
    }

    return null;
  }

  /**
   * Method isValidClient
   *
   *
   * @param clientTicket
   * @param clientAddress
   * @param srvtab
   *
   * @return String
   *
   * @throws Throwable
   *
   */
  public static String isValidClient(byte[] clientTicket,
                                     byte[] clientAddress, byte[] srvtab)
  throws Throwable {
    krb4.lib.Krb4Encode ref        = new krb4.lib.Krb4Encode(srvtab);
    String              service    = ref.getString();
    String              instance   = ref.getString();
    String              realm      = ref.getString();
    byte                keyVersion = ref.getByte();
    byte[]              servKey    = ref.getBytes(8);
    krb4.lib.Krb4APReq  ap_req     = new krb4.lib.Krb4APReq(clientTicket);

    ap_req.authenticate(servKey, service, instance, realm, clientAddress);

    return new String(ap_req.getSessionKey());
  }
}
