package edu.mit.is.service.authentication.impl.authenkrb5;
/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including software, documents, or other related items (the
   "Software"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM)  API Implementation License. By
   obtaining, using and/or copying this Software, you agree that you have
   read, understand, and will comply with the following terms and
   conditions of the MIT OKI(TM) API Implementation License:

   Permission to use, copy, modify, and distribute this Software and its
   documentation, with or without modification, for any purpose and
   without fee or royalty is hereby granted, provided that you include
   the following on ALL copies of the Software or portions thereof,
   including modifications or derivatives, that you make:

    *  The full text of the MIT OKI(TM) API Implementation License in a
       location viewable to users of the redistributed or derivative
       work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed or
       derivative Software:
       "Copyright (c) 2002 Massachusetts Institute of Technology
       All Rights Reserved."

    *  Notice of any changes or modifications to the MIT OKI(TM)
       Software, including the date the changes were made. Any modified
       software must be distributed in such as manner as to avoid any
       confusion with the original MIT OKI(TM) Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Software without
   specific, written prior permission. Title to copyright in the Software
   and any associated documentation will at all times remain with the
   copyright holders.

   The export of software employing encryption technology may require a
   specific license from the United States Government. It is the
   responsibility of any person or organization contemplating export to
   obtain such a license before exporting this Software.
*/

/*
 * $Source: /cvs/oki/tech/contrib/mit/impl/src/edu/mit/is/service/authentication/impl/authenkrb5/Factory.java,v $
 */

/**
 * Implementation of Kerberos 5 authentication for OKI
 *
 * Licensed under the {@link org.okip.service.ApiImplementationLicenseMIT MIT OKI(TM) API Implementation License}.
 *
 * @version $Revision: 1.10 $ / $Date: 2002/07/03 19:09:00 $
*/

public class Factory
implements org.okip.service.authentication.api.Factory {
  private static final org.okip.service.authentication.api.AuthenticationException UNIMPLEMENTED =
    new org.okip.service.authentication.api.AuthenticationException("Unimplemented method");

  private static final org.okip.service.shared.api.Type KERBEROS5 =
    new edu.mit.is.service.authentication.impl.authenkrb5.Krb5Type();

  private org.okip.service.shared.api.Agent             owner     = null;

  /**
   * Return owner of this Factory.
   *
   *
   * @return org.okip.service.shared.api.Agent Owner
   *
   */
  public org.okip.service.shared.api.Agent getOwner() {
    return this.owner;
  }

  /**
   * Set the owner of this Factory.
   *
   *
   * @param Agent Owner
   *
   */
  public void setOwner(org.okip.service.shared.api.Agent owner) {
    this.owner = owner;
  }

  private java.util.Map properties = null;

  /**
   * Get properties of this Factory.
   *
   *
   * @return java.util.Map properties
   *
   */
  public java.util.Map getProperties() {
    return this.properties;
  }

  /**
   * Set the properties of this Factory.
   *
   *
   * @param properties
   *
   */
  public void setProperties(java.util.Map properties) {
    this.properties = properties;
  }

  private org.okip.service.shared.api.Type defaultAuthenType = KERBEROS5;

  /**
   * Returns the types of Authentication that this agent currently has.
   *
   * @return org.okip.service.shared.api.Type
   *
   */
  public org.okip.service.shared.api.Type getDefaultAuthenType() {
    return this.defaultAuthenType;
  }

  /**
   * Sets the default authentication type to be used.
   *
   * @param authenType
   *
   */
  public void setDefaultAuthenType(
          org.okip.service.shared.api.Type defaultAuthenType) {
  }

  /**
   * Return Boolean true if the Agent is authenticated for the
   * specified authentication type.
   *
   *
   * @param agent
   * @param authenType
   *
   * @return Boolean true iff the agent is authenticated under the authenType.
   *
   * @throws org.okip.service.authentication.api.AuthenticationException
   *
   */
  public Boolean isAuthenticated(org.okip.service.shared.api.Agent agent,
                                 org.okip.service.shared.api.Type authenType)
  throws org.okip.service.authentication.api.AuthenticationException {
    if ((null != authenType) && (authenType.equals(KERBEROS5))) {
      return isAuthenticated (agent);
    }
    throw UNIMPLEMENTED;
  }

  /**
   * Method isAuthenticated for default authenType
   *
   *
   * @param agent
   *
   * @return Boolean true iff the agent is authenticated under the
   * default authenType.
   *
   * @throws org.okip.service.authentication.api.AuthenticationException
   *
   */
  public Boolean isAuthenticated(org.okip.service.shared.api.Agent agent)
  throws org.okip.service.authentication.api.AuthenticationException {
	if (null != agent)
	{
      org.okip.service.shared.api.Agent test =
        agent.getProxy
          (edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent.class);
      if(null != test) {
        edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent aa
          = (edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent) test;
        javax.security.auth.Subject subject = aa.getSubject();
        if(null != subject) {
          java.util.Set creds = subject.getPrivateCredentials ();
          if (null != creds) {
            java.util.Iterator iterator = creds.iterator();
            while (iterator.hasNext()) {
              Object cred = iterator.next();
              if (cred instanceof javax.security.auth.kerberos.KerberosTicket) {
                javax.security.auth.kerberos.KerberosTicket ticket =
                  (javax.security.auth.kerberos.KerberosTicket) cred;
                if (ticket.isCurrent()) {
                  return new Boolean (true);
                }
                if(null != subject.getPrivateCredentials()) {
                  subject.getPrivateCredentials().clear();
                }
              }
            }
          }
        }
        return new Boolean(false);
      }
    }
    throw UNIMPLEMENTED;
  }

  /**
   * Returns an Agent that may or may not be anonymous.
   *
   * @return org.okip.service.shared.api.Agent
   *
   * @throws org.okip.service.authentication.api.AuthenticationException
   *
   */
  public org.okip.service.shared.api.Agent getAgent()
  throws org.okip.service.authentication.api.AuthenticationException {
    return getAgentInternal(null);
  }

  /**
   * Returns the types of Authentication that are available
   * to the current implementation.
   *
   * @return org.okip.service.shared.api.Type[]
   *
   * @throws org.okip.service.authentication.api.AuthenticationException
   *
   */
  public org.okip.service.shared.api.Type[] getAuthenTypes()
  throws org.okip.service.authentication.api.AuthenticationException {
    org.okip.service.shared.api.Type[] types =
      new org.okip.service.shared.api.Type[1];

    types[0] = KERBEROS5;

    return types;
  }

  /**
   * Returns the types of Authentication that this agent currently has.
   *
   * @return org.okip.service.shared.api.Type
   *
   * @throws org.okip.service.authentication.api.AuthenticationException
   *
   */
  public org.okip.service.shared.api.Type[] getAuthenTypes(
          org.okip.service.shared.api.Agent agent)
  throws org.okip.service.authentication.api.AuthenticationException {
	if (null != agent) {
      org.okip.service.shared.api.Agent test =
        agent.getProxy
          (edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent.class);
      if (null != test) {
        edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent aa
          = (edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent) test;
        org.okip.service.shared.api.Type[] types  =
          new org.okip.service.shared.api.Type[1];
        types[0] = aa.getAuthenType ();
        return types;
      }
      return null;
    }
    throw UNIMPLEMENTED;
  }

  /**
   * Return the agent authenticated under specified authentication type.
   *
   *
   * @param agent
   * @param type
   *
   * @return org.okip.service.shared.api.Agent
   *
   * @throws org.okip.service.authentication.api.AuthenticationException
   *
   */
  public org.okip.service.shared.api.Agent getAgent(
          org.okip.service.shared.api.Agent agent,
          org.okip.service.shared.api.Type type)
  throws org.okip.service.authentication.api.AuthenticationException {
	if ((null != agent) && (null != type) && (type.equals(KERBEROS5))) {
      org.okip.service.shared.api.Agent test =
        agent.getProxy
          (edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent.class);
      if (null != test) {
        if (isAuthenticated(test).booleanValue()) {
          return (test);
        }
      }
      return getAgentInternal(agent);
    }
    throw UNIMPLEMENTED;
  }

  /**
   * Destroy all authentications for the agent.
   *
   * @param agent
   *
   * @throws org.okip.service.authentication.api.AuthenticationException
   *
   */
  public void destroyAuthen(org.okip.service.shared.api.Agent agent)
  throws org.okip.service.authentication.api.AuthenticationException {
    destroyAuthen (agent, KERBEROS5);
    return;
  }

  /**
   * Destroy agent's authentication of specified type.
   *
   *
   * @param agent
   * @param type
   *
   * @throws org.okip.service.authentication.api.AuthenticationException
   *
   */
  public void destroyAuthen(org.okip.service.shared.api.Agent agent,
                            org.okip.service.shared.api.Type type)
  throws org.okip.service.authentication.api.AuthenticationException {
    if ((null!= agent) && (null != type) && (type.equals(KERBEROS5))) {
      org.okip.service.shared.api.Agent test =
        agent.getProxy
          (edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent.class);
      if (null != test)
      {
        edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent aa =
          (edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent) test;
        javax.security.auth.Subject subject = aa.getSubject();
        if(null != subject) {
          if(null != subject.getPrivateCredentials()) {
            subject.getPrivateCredentials().clear() ;
          }
          if(null != subject.getPublicCredentials()) {
            subject.getPublicCredentials().clear() ;
          }
        }
      }
      return;
    }
    throw UNIMPLEMENTED;
  }

  /**
   * Destroy agent's authentication for each of the array of specified types.
   *
   *
   * @param agent
   * @param types
   *
   * @throws org.okip.service.authentication.api.AuthenticationException
   *
   */
  public void destroyAuthen(org.okip.service.shared.api.Agent agent,
                            org.okip.service.shared.api.Type[] types)
  throws org.okip.service.authentication.api.AuthenticationException {
    if (null != types) {
      for (int i = 0; types.length != i; i++) {
        if (types[i].equals(KERBEROS5)) {
          destroyAuthen (agent, KERBEROS5);
          return;
        }
      }
    }
    throw UNIMPLEMENTED;
  }

  private org.okip.service.shared.api.Agent getAgentInternal(
          org.okip.service.shared.api.Agent agent)
  throws org.okip.service.authentication.api.AuthenticationException {
    try {
      if ((null==agent) || ! isAuthenticated(agent).booleanValue())
      {
        javax.security.auth.login.LoginContext lc =
          new javax.security.auth.login.LoginContext("AuthenKrb5", new com.sun.security.auth.callback.DialogCallbackHandler());

        lc.login();
        javax.security.auth.Subject subject = lc.getSubject();
        lc = new javax.security.auth.login.LoginContext("MitidServer", subject);
        lc.login();
        subject = lc.getSubject();
        return
          new edu.mit.is.service.authentication.impl.authenkrb5.AuthenticatedAgent(
			  agent, subject);
      }
      return agent;
    }
    catch (Throwable ex) {
      throw new org.okip.service.authentication.api.AuthenticationException(
        ex.getMessage());
    }
  }
}
