package edu.mit.is.service.dbc;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.
*/

/*
 * $Source: /cvs/oki/tech/contrib/mit/impl/src/edu/mit/is/service/dbc/Blob.java,v $
 */

/**
 * The mapping in the OKI DBC API programming language for the SQL
 * BLOB value. An SQL BLOB is a built-in type that stores a Binary
 * Large Object as a column value in a row of a database table. By
 * default drivers implement a Blob object using an SQL locator(BLOB),
 * which means that a Blob object contains a logical pointer to the
 * SQL BLOB data rather than the data itself. A Blob object is valid
 * for the duration of the transaction in which it was created.
 * <p>
 * Methods in the interfaces ResultSet, CallableStatement, and
 * PreparedStatement, such as getBlob and setBlob allow a programmer
 * to access an SQL BLOB value. The Blob interface provides methods
 * for getting the length of an SQL BLOB (Binary Large Object) value,
 * for materializing a BLOB value on the client, and for determining
 * the position of a pattern of bytes within a BLOB value. In
 * addition, this interface has methods for updating a BLOB value.
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.7 $ / $Date: 2002/07/29 18:45:55 $
 */
public class Blob
implements org.okip.service.dbc.api.Blob, java.io.Serializable {

  /* methods which are in Blob in 1.4 but not in this interface:
     getBinaryStream
     setBinaryStream
   */
  private static final String                                GETBYTES        =
    "getBytes";
  private static final String                                POSITION        =
    "position";
  private static final Long                                  INVALIDPOSITION =
    new Long(-1);

  private static final org.okip.service.dbc.api.DbcException LOCALBLOBGETBYTESFAILED =
    new org.okip.service.dbc.api.DbcException(
      org.okip.service.dbc.api.DbcException.BLOBGETBYTESFAILED);

  private static final org.okip.service.dbc.api.DbcException LOCALBLOBPOSITIONFAILED =
    new org.okip.service.dbc.api.DbcException(
      org.okip.service.dbc.api.DbcException.BLOBPOSITIONFAILED);

  private edu.mit.is.service.dbc.ResultSet                   rows            =
    null;
  private int                                                row             =
    0;
  private int                                                col             =
    0;
  private Long                                               pos             =
    INVALIDPOSITION;
  private Integer                                            len             =
    new Integer(0);
  private byte[]                                             pattern         =
    null;
  private Long                                               start           =
    INVALIDPOSITION;

  private long                                               length          =
    -1;

  protected Blob(edu.mit.is.service.dbc.ResultSet rows, long length, int row,
                 int col) {
    this.rows   = rows;
    this.row    = row;
    this.col    = col;
    this.length = length;
  }

  private byte[] getBytesRemotely(edu.mit.is.util.romi.client.Client client)
  throws org.okip.service.dbc.api.DbcException {
    Object[] args = new Object[2];

    args[0] = this.pos;
    args[1] = this.len;

    Class[] argstypes = new Class[2];

    argstypes[0] = args[0].getClass();
    argstypes[1] = args[1].getClass();

    try {
      return (byte[]) client.invoke(this, GETBYTES, argstypes, args);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    return null;
  }

  /**
   * Method getBytesLocally
   *
   *
   * @return byte[]
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public byte[] getBytesLocally()
  throws org.okip.service.dbc.api.DbcException {
    java.sql.ResultSet rs = null;

    try {
      rs = rows.getResultSet();

      int rsType = rs.getType();

      if (rs.TYPE_FORWARD_ONLY == rsType) {
        if (rs.next()) {
          return getBytesFromResultSet(rs);
        }
      }

      else if (rs.TYPE_SCROLL_INSENSITIVE == rsType) {
        rs.setFetchDirection(rs.FETCH_FORWARD);

        if (rs.first()) {
          return getBytesFromResultSet(rs);
        }
      }
      else if (rs.TYPE_SCROLL_SENSITIVE == rsType) {
        rs.setFetchDirection(rs.FETCH_FORWARD);

        boolean isRowAvailable = rs.first();

        if (rs.first()) {
          return getBytesFromResultSet(rs);
        }
      }

      java.sql.Statement statement = rs.getStatement();

      rs.close();
      statement.close();
    }
    catch (Exception ex) {
      if (null != rs) {
        try {
          rs.close();
        }
        catch (java.sql.SQLException e) {}
      }

      String message = LOCALBLOBGETBYTESFAILED.getMessage();

      message += ex.getMessage();

      throw new org.okip.service.dbc.api.DbcException(message);
    }

    return null;
  }

  private byte[] getBytesFromResultSet(java.sql.ResultSet rs)
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    int     cursor         = 0;
    boolean isRowAvailable = true;

    while (isRowAvailable) {
      if (cursor == this.row) {
        java.sql.Blob blob = (java.sql.Blob) rs.getObject(this.col + 1);

        return blob.getBytes(this.pos.longValue(), this.len.intValue());
      }

      cursor++;

      isRowAvailable = rs.next();
    }

    return null;
  }

  private Long positionRemotely(edu.mit.is.util.romi.client.Client client)
  throws org.okip.service.dbc.api.DbcException {
    Object[] args = new Object[2];

    args[0] = this.pattern;
    args[1] = this.start;

    Class[] argstypes = new Class[2];

    argstypes[0] = args[0].getClass();
    argstypes[1] = args[1].getClass();

    try {
      return (Long) client.invoke(this, POSITION, argstypes, args);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    return null;
  }

  private Long positionLocally()
  throws org.okip.service.dbc.api.DbcException {
    java.sql.ResultSet rs = null;

    try {
      rs = rows.getResultSet();

      int rsType = rs.getType();

      if (rs.TYPE_FORWARD_ONLY == rsType) {
        if (rs.next()) {
          return positionFromResultSet(rs);
        }
      }
      else if (rs.TYPE_SCROLL_INSENSITIVE == rsType) {
        rs.setFetchDirection(rs.FETCH_FORWARD);

        if (rs.first()) {
          return positionFromResultSet(rs);
        }
      }
      else if (rs.TYPE_SCROLL_SENSITIVE == rsType) {
        rs.setFetchDirection(rs.FETCH_FORWARD);

        boolean isRowAvailable = rs.first();

        if (rs.first()) {
          return positionFromResultSet(rs);
        }
      }

      java.sql.Statement statement = rs.getStatement();

      rs.close();
      statement.close();
    }
    catch (Exception ex) {
      if (null != rs) {
        try {
          rs.close();
        }
        catch (java.sql.SQLException ex1) {
          ex1.printStackTrace();
        }
      }

      String message = LOCALBLOBPOSITIONFAILED.getMessage();

      message += ex.getMessage();

      throw new org.okip.service.dbc.api.DbcException(message);
    }

    return INVALIDPOSITION;
  }

  private Long positionFromResultSet(java.sql.ResultSet rs)
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    int     cursor         = 0;
    boolean isRowAvailable = true;

    while (isRowAvailable) {
      if (cursor == this.row) {
        java.sql.Blob blob = (java.sql.Blob) rs.getObject(this.col + 1);

        return new Long(blob.position(this.pattern, this.start.longValue()));
      }

      cursor++;

      isRowAvailable = rs.next();
    }

    return INVALIDPOSITION;
  }

  /**
   * Retrieves all or part of the BLOB value that
   * this Blob object represents, as an array of bytes.
   *
   * @param pos
   * @param len
   *
   * @return byte[]
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public byte[] getBytes(long pos, int len)
  throws org.okip.service.dbc.api.DbcException {
    byte[] bytes = null;

    this.pos = new Long(pos);
    this.len = new Integer(len);

    try {
      edu.mit.is.service.dbc.Factory     factory = this.rows.getFactory();
      edu.mit.is.util.romi.client.Client client  =
        (edu.mit.is.util.romi.client.Client) factory.getClient();

      if (edu.mit.is.util.romi.client.Client.isValid(client)) {
        bytes = getBytesRemotely(client);
      }
      else {
        bytes = getBytesLocally();
      }

      return bytes;
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    return null;
  }

  /**
   * Returns the number of bytes in the BLOB value
   * designated by this Blob object.
   *
   * @return long
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public long length()
  throws org.okip.service.dbc.api.DbcException {
    return this.length;
  }

  /**
   * Retrieves the byte position in the BLOB value
   * designated by this Blob object at which pattern
   * begins.
   *
   * @param pattern
   * @param start
   *
   * @return long
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public long position(org.okip.service.dbc.api.Blob pattern, long start)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the byte position at which the
   * specified byte array pattern begins within
   * the BLOB value that this Blob object represents.
   *
   * @param pattern
   * @param start
   *
   * @return long
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public long position(byte[] pattern, long start)
  throws org.okip.service.dbc.api.DbcException {
    this.pattern = pattern;
    this.start   = new Long(start);

    Long thePosition = INVALIDPOSITION;

    try {
      edu.mit.is.service.dbc.Factory     factory = this.rows.getFactory();
      edu.mit.is.util.romi.client.Client client  =
        (edu.mit.is.util.romi.client.Client) factory.getClient();

      if (edu.mit.is.util.romi.client.Client.isValid(client)) {
        thePosition = positionRemotely(client);
      }
      else {
        thePosition = positionLocally();
      }
    }
    catch (Exception ex) {}

    return thePosition.longValue();
  }

  /**
   * Writes the given array of bytes to the BLOB
   * value that this Blob object represents,
   * starting at position pos, and returns the
   * number of bytes written.
   *
   * @param pos
   * @param bytes
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int setBytes(long pos, byte[] bytes)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Writes all or part of the given byte array
   * to the BLOB value that this Blob object
   * represents and returns the number of bytes
   * written.
   *
   * @param pos
   * @param bytes
   * @param offset
   * @param len
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int setBytes(long pos, byte[] bytes, int offset, int len)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Truncates the BLOB value that this Blob
   * object represents to be len bytes in length.
   *
   * @param len
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void truncate(long len)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

}
