package edu.mit.is.service.dbc;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/contrib/mit/impl/src/edu/mit/is/service/dbc/Clob.java,v $
 */

/**
 * The mapping in the OKI DBC API programming language for the SQL
 * CLOB type. An SQL CLOB is a built-in type that stores a Character
 * Large Object as a column value in a row of a database table. By
 * default drivers implement a Clob object using an SQL locator(CLOB),
 * which means that a Clob object contains a logical pointer to the
 * SQL CLOB data rather than the data itself. A Clob object is valid
 * for the duration of the transaction in which it was created.
 * <p>
 * The Clob interface provides methods for getting the length of an
 * SQL CLOB (Character Large Object) value, for materializing a CLOB
 * value on the client, and for searching for a substring or CLOB
 * object within a CLOB value.  Methods in the interfaces ResultSet,
 * CallableStatement, and PreparedStatement, such as getClob and
 * setClob allow a programmer to access an SQL CLOB value. In
 * addition, this interface has methods for updating a CLOB value.
 *
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.5 $ / $Date: 2002/07/29 18:45:56 $
 */
public class Clob
implements org.okip.service.dbc.api.Clob, java.io.Serializable {

  /* methods which are in Clob in 1.4 but not in this interface:
     getAsciiStream
     getCharacterStream
     setAsciiStream
     setCharacterStream
   */

  private static final String                                GETSUBSTRING    =
    "getSubString";
  private static final String                                POSITION        =
    "position";
  private static final Long                                  INVALIDPOSITION =
    new Long(-1);

  private static final org.okip.service.dbc.api.DbcException LOCALCLOBGETSUBSTRINGFAILED =
    new org.okip.service.dbc.api.DbcException(
      org.okip.service.dbc.api.DbcException.CLOBGETSUBSTRINGFAILED);

  private static final org.okip.service.dbc.api.DbcException LOCALCLOBPOSITIONFAILED =
    new org.okip.service.dbc.api.DbcException(
      org.okip.service.dbc.api.DbcException.BLOBPOSITIONFAILED);

  private edu.mit.is.service.dbc.ResultSet                   rows            =
    null;
  private int                                                row             =
    0;
  private int                                                col             =
    0;
  private Long                                               pos             =
    INVALIDPOSITION;
  private Integer                                            len             =
    new Integer(0);
  private String                                             searchstr       =
    null;
  private Long                                               start           =
    INVALIDPOSITION;

  private long                                               length          =
    0;

  protected Clob(edu.mit.is.service.dbc.ResultSet rows, long length, int row,
                 int col) {
    this.rows   = rows;
    this.row    = row;
    this.col    = col;
    this.length = length;
  }

  private String getSubStringRemotely(
          edu.mit.is.util.romi.client.Client client)
  throws org.okip.service.dbc.api.DbcException,
         org.okip.util.romi.client.api.ClientException {
    Object[] args = new Object[2];

    args[0] = this.pos;
    args[1] = this.len;

    Class[] argstypes = new Class[2];

    argstypes[0] = args[0].getClass();
    argstypes[1] = args[1].getClass();

    return (String) client.invoke(this, GETSUBSTRING, argstypes, args);
  }

  /**
   * Method getSubStringLocally
   *
   *
   * @return String
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public String getSubStringLocally()
  throws org.okip.service.dbc.api.DbcException {
    java.sql.ResultSet rs = null;

    try {
      rs = rows.getResultSet();

      int rsType = rs.getType();

      if (rs.TYPE_FORWARD_ONLY == rsType) {
        if (rs.next()) {
          return getSubStringFromResultSet(rs);
        }
      }
      else if (rs.TYPE_SCROLL_INSENSITIVE == rsType) {
        rs.setFetchDirection(rs.FETCH_FORWARD);

        if (rs.first()) {
          return getSubStringFromResultSet(rs);
        }
      }
      else if (rs.TYPE_SCROLL_SENSITIVE == rsType) {
        rs.setFetchDirection(rs.FETCH_FORWARD);

        boolean isRowAvailable = rs.first();

        if (rs.first()) {
          return getSubStringFromResultSet(rs);
        }
      }

      java.sql.Statement statement = rs.getStatement();

      rs.close();
      statement.close();
    }
    catch (Exception ex) {
      if (null != rs) {
        try {
          rs.close();
        }
        catch (java.sql.SQLException ex2) {}
      }

      String message = LOCALCLOBGETSUBSTRINGFAILED.getMessage();

      message += ex.getMessage();

      throw new org.okip.service.dbc.api.DbcException(message);
    }

    return null;
  }

  private String getSubStringFromResultSet(java.sql.ResultSet rs)
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    int     cursor         = 0;
    boolean isRowAvailable = true;

    while (isRowAvailable) {
      if (cursor == this.row) {
        java.sql.Clob clob = (java.sql.Clob) rs.getObject(this.col + 1);

        return clob.getSubString(this.pos.longValue(), this.len.intValue());
      }

      cursor++;

      isRowAvailable = rs.next();
    }

    return null;
  }

  private Long positionRemotely(edu.mit.is.util.romi.client.Client client)
  throws org.okip.service.dbc.api.DbcException,
         org.okip.util.romi.client.api.ClientException {
    Object[] args = new Object[2];

    args[0] = this.searchstr;
    args[1] = this.start;

    Class[] argstypes = new Class[2];

    argstypes[0] = args[0].getClass();
    argstypes[1] = args[1].getClass();

    return (Long) client.invoke(this, POSITION, argstypes, args);
  }

  private Long positionLocally()
  throws org.okip.service.dbc.api.DbcException {
    java.sql.ResultSet rs = null;

    try {
      rs = rows.getResultSet();

      int rsType = rs.getType();

      if (rs.TYPE_FORWARD_ONLY == rsType) {
        if (rs.next()) {
          return positionFromResultSet(rs);
        }
      }
      else if (rs.TYPE_SCROLL_INSENSITIVE == rsType) {
        rs.setFetchDirection(rs.FETCH_FORWARD);

        if (rs.first()) {
          return positionFromResultSet(rs);
        }
      }
      else if (rs.TYPE_SCROLL_SENSITIVE == rsType) {
        rs.setFetchDirection(rs.FETCH_FORWARD);

        boolean isRowAvailable = rs.first();

        if (rs.first()) {
          return positionFromResultSet(rs);
        }
      }

      java.sql.Statement statement = rs.getStatement();

      rs.close();
      statement.close();
    }
    catch (Exception ex) {
      if (null != rs) {
        try {
          rs.close();
        }
        catch (java.sql.SQLException ex2) {}
      }

      String message = LOCALCLOBPOSITIONFAILED.getMessage();

      message += ex.getMessage();

      throw new org.okip.service.dbc.api.DbcException(message);
    }

    return INVALIDPOSITION;
  }

  private Long positionFromResultSet(java.sql.ResultSet rs)
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    int     cursor         = 0;
    boolean isRowAvailable = true;

    while (isRowAvailable) {
      if (cursor == this.row) {
        java.sql.Clob clob = (java.sql.Clob) rs.getObject(this.col + 1);

        return new Long(clob.position(this.searchstr,
                                      this.start.longValue()));
      }

      cursor++;

      isRowAvailable = rs.next();
    }

    return INVALIDPOSITION;
  }

  /**
   * Retrieves a copy of the specified substring
   * in the CLOB value designated by this Clob object.
   *
   * @param pos
   * @param length
   *
   * @return String
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public String getSubString(long pos, int length)
  throws org.okip.service.dbc.api.DbcException {
    this.pos = new Long(pos);
    this.len = len;

    String                         substr  = null;

    edu.mit.is.service.dbc.Factory factory = this.rows.getFactory();

    try {
      edu.mit.is.util.romi.client.Client client =
        (edu.mit.is.util.romi.client.Client) factory.getClient();

      if (edu.mit.is.util.romi.client.Client.isValid(client)) {
        substr = getSubStringRemotely(client);
      }
      else {
        substr = getSubStringLocally();
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    return substr;
  }

  /**
   * Retrieves the number of characters in the
   * CLOB value designated by this Clob object.
   *
   * @return long
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public long length()
  throws org.okip.service.dbc.api.DbcException {
    return this.length;
  }

  /**
   * Retrieves the character position at which
   * the specified Clob object searchstr appears
   * in this Clob object.
   *
   * @param searchstr
   * @param start
   *
   * @return long
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public long position(org.okip.service.dbc.api.Clob searchstr, long start)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the character position at which
   * the specified substring searchstr appears
   * in the SQL CLOB value represented by this
   * Clob object.
   *
   * @param searchstr
   * @param start
   *
   * @return long
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public long position(String searchstr, long start)
  throws org.okip.service.dbc.api.DbcException {
    Long thePosition = INVALIDPOSITION;

    this.searchstr = searchstr;
    this.start     = new Long(start);

    edu.mit.is.service.dbc.Factory factory = this.rows.getFactory();

    try {
      edu.mit.is.util.romi.client.Client client =
        (edu.mit.is.util.romi.client.Client) factory.getClient();

      if (edu.mit.is.util.romi.client.Client.isValid(client)) {
        thePosition = positionRemotely(client);
      }
      else {
        thePosition = positionLocally();
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    return thePosition.longValue();
  }

  /**
   * Writes the given Java String to the
   * CLOB value that this Clob object designates
   * at the position pos.
   *
   * @param pos
   * @param str
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int setString(long pos, String str)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Writes len characters of str, starting at
   * character offset, to the CLOB value that
   * this Clob represents.
   *
   * @param pos
   * @param str
   * @param offset
   * @param len
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int setString(long pos, String str, int offset, int len)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Truncates the CLOB value that this Clob
   * designates to have a length of len characters.
   *
   * @param len
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void truncate(long len)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

}
