package edu.mit.is.service.dbc;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/contrib/mit/impl/src/edu/mit/is/service/dbc/Factory.java,v $
 */

/**
 * Factory for DBC API.
 * <p>
 * This Factory is the equivalent of javax.sql.DataSource.
 * It contains the getConnection methods from DataSource.
 * The name Factory is used here instead of DataSource for
 * the convenience of the OKI FactoryManager.
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.8 $ / $Date: 2002/07/29 18:45:56 $
 */
public class Factory
implements org.okip.service.dbc.api.Factory, java.io.Serializable {

  /**
   * Method getConnection
   *
   * @param username
   * @param password
   *
   * @return Connection
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Connection getConnection(String username,
          String password)
  throws org.okip.service.dbc.api.DbcException {
    return edu.mit.is.service.dbc.Connection.create(username, password);

  }

  /**
   * Method getConnection
   *
   * @param database
   * @param username
   * @param password
   *
   * @return Connection
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Connection getConnection(String database,
          String username, String password)
  throws org.okip.service.dbc.api.DbcException {
    return edu.mit.is.service.dbc.Connection.create(database, username,
            password);
  }

  /**
   * Method getConnection
   *
   * @param databaseURL
   * @param driverName
   * @param username
   * @param password
   *
   * @return Connection
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Connection getConnection(String databaseURL,
          String driverName, String username, String password)
  throws org.okip.service.dbc.api.DbcException {
    return edu.mit.is.service.dbc.Connection.create(databaseURL, driverName,
            username, password, this);
  }

  /**
   * This method uses the Dbc properties:
   * Username, Password, Database (or the
   * combination of DatabaseURL and DriverName)
   * from the Dbc Factory properties. These
   * properties are either passed in from the
   * application using the Dbc Factory or come
   * from the Dbc Factory.properties configuration
   * file if the application doesn't specify
   * them.
   *
   * @return Connection
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Connection getConnection()
  throws org.okip.service.dbc.api.DbcException {
    try {
      java.util.Properties property =
        getPropertiesFromPRB(this.getClass().getName());

      if (null != property) {
        String databaseURL = (String) property.get("databaseURL");
        String driverName  = (String) property.get("driverName");
        String username    = (String) property.get("username");
        String password    = (String) property.get("password");
        String maxRows_str = (String) property.get("maxRows");

        return edu.mit.is.service.dbc.Connection.create(databaseURL,
                driverName, username, password, new Integer(maxRows_str),
                this);
      }
    }
    catch (org.okip.service.shared.api.Exception e) {
      e.printStackTrace();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    return null;
  }

  /**
   * Method getClient
   *
   *
   * @return org.okip.util.romi.client.api.Client
   *
   * @throws Exception
   *
   */
  public org.okip.util.romi.client.api.Client getClient()
  throws Exception {
    String                            className = this.getClass().getName();
    org.okip.service.shared.api.Agent agent     = this.getOwner();

    if (null != agent) {
      if (null == agent.getProxy(edu.mit.is.util.romi.RomiAgent.class)) {
        java.util.Properties property = getPropertiesFromPRB(className);

        if (null != property) {
          String romiApiPath  = (String) property.get("RomiClientAPI");
          String romiImplPath = (String) property.get("RomiClientIMPL");
          String host         = (String) property.get("host");
          String port         = (String) property.get("port");
          String server       = (String) property.get("server");

          if ((null != romiApiPath) &&!romiApiPath.trim().equals("")) {
            if ((null != romiImplPath) &&!romiImplPath.trim().equals("")) {
              org.okip.util.romi.client.api.Factory romiFactory =
                (org.okip.util.romi.client.api.Factory) org.okip.service.shared.api.FactoryManager.getFactory(
                  romiApiPath, romiImplPath, agent);

              if (null != romiFactory) {
                org.okip.util.romi.client.api.Client client =
                  romiFactory.createClient(host,
                                           (new Integer(port)).intValue(),
                                           server);

                if (null != client) {
                  edu.mit.is.util.romi.RomiAgent romiAgent =
                    (edu.mit.is.util.romi.RomiAgent) (agent.getProxy(
                      edu.mit.is.util.romi.RomiAgent.class));

                  romiAgent.setClient(client);

                  return client;
                }
              }
            }
          }
        }
      }

      if (null != agent.getProxy(edu.mit.is.util.romi.RomiAgent.class)) {
        edu.mit.is.util.romi.RomiAgent       romiAgent =
          (edu.mit.is.util.romi.RomiAgent) (agent.getProxy(
            edu.mit.is.util.romi.RomiAgent.class));
        org.okip.util.romi.client.api.Client test      =
          romiAgent.getClient();

        return romiAgent.getClient();
      }
    }

    return null;
  }

  private java.util.Properties getPropertiesFromPRB(String implPackageName)
  throws org.okip.service.shared.api.Exception {
    java.util.Properties properties = null;

    try {
      java.util.PropertyResourceBundle prb =
        (java.util.PropertyResourceBundle) java.util.ResourceBundle.getBundle(
          new String(implPackageName));

      if (null != prb) {
        properties = new java.util.Properties();

        java.util.Enumeration keys = prb.getKeys();

        while (keys.hasMoreElements()) {
          String key = (String) (keys.nextElement());

          properties.setProperty(key, (String) prb.handleGetObject(key));
        }
      }
    }
    catch (Exception ex) {}

    return properties;
  }

  private org.okip.service.shared.api.Agent owner = null;

  /**
   * Return owner of this Factory.
   *
   *
   * @return org.okip.service.shared.api.Agent Owner
   *
   */
  public org.okip.service.shared.api.Agent getOwner() {
    return this.owner;
  }

  /**
   * Set the owner of this Factory.
   *
   *
   *
   * @param owner
   *
   */
  public void setOwner(org.okip.service.shared.api.Agent owner) {
    this.owner = owner;
  }

}
