package edu.mit.is.service.dbc;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/contrib/mit/impl/src/edu/mit/is/service/dbc/PreparedStatement.java,v $
 */

/**
 * An object that represents a precompiled SQL statement.
 *
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.7 $ / $Date: 2002/07/31 15:31:10 $
 */
public class PreparedStatement
extends Statement
implements org.okip.service.dbc.api.PreparedStatement, java.io.Serializable {

  private String                            sql         = null;
  private edu.mit.is.service.dbc.Connection con         = null;
  private static final Object[]             NOARGS      = new Object[0];
  private static final Class[]              NOARGSTYPES = new Class[0];

  /**
   * Constructor PreparedStatement
   *
   *
   */
  public PreparedStatement() {}

  /**
   * Constructor PreparedStatement
   *
   *
   * @param con
   * @param sql
   *
   */
  public PreparedStatement(edu.mit.is.service.dbc.Connection con,
                           String sql) {
    this.con = con;
    this.sql = sql;
  }

  /* methods which are in PreparedStatement in 1.4 but not in this interface:
     setAsciiStream
     setBinaryStream
     setCharacterStream
   */

  /**
   * Adds a set of parameters to this
   * PreparedStatement object's batch
   * of commands.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void addBatch()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Clears the current parameter
   * values immediately.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void clearParameters()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Executes the SQL statement in this
   * PreparedStatement object, which may
   * be any kind of SQL statement.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean execute()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Executes the SQL query in this
   * PreparedStatement object and returns
   * the ResultSet object generated by
   * the query.
   *
   * @return ResultSet
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.ResultSet executeQuery()
  throws org.okip.service.dbc.api.DbcException {
    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {

          edu.mit.is.service.dbc.ResultSet result =
            (edu.mit.is.service.dbc.ResultSet) client.invoke(this,
              "executeQuery", NOARGSTYPES, NOARGS);

          return result;
        }
        else {
          java.sql.Connection conn = con.getConnection();

          if (null != conn) {
            java.sql.PreparedStatement ps =
              (java.sql.PreparedStatement) conn.prepareStatement(this.sql);

            if (null != ps) {
              java.sql.ResultSet               rs     = ps.executeQuery(sql);
              edu.mit.is.service.dbc.ResultSet dbc_rs =
                new edu.mit.is.service.dbc.ResultSet(con, rs, sql);

              rs.close();
              ps.close();

              return dbc_rs;
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Executes the SQL statement in this
   * PreparedStatement object, which must
   * be an SQL INSERT, UPDATE or DELETE
   * statement; or an SQL statement that
   * returns nothing, such as a DDL statement.
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int executeUpdate()
  throws org.okip.service.dbc.api.DbcException {
    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {

          Integer result = (Integer) client.invoke(this, "executeUpdate",
                                                   NOARGSTYPES, NOARGS);

          return result.intValue();
        }
        else {
          java.sql.Connection conn = con.getConnection();

          if (null != conn) {
            java.sql.PreparedStatement ps =
              (java.sql.PreparedStatement) conn.prepareStatement(this.sql);

            if (null != ps) {
              int result = ps.executeUpdate();

              ps.close();

              return result;
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Retrieves a java.sql.ResultSetMetaData object
   * that contains information about the
   * columns of the ResultSet object that
   * will be returned when this
   * PreparedStatement object is executed.
   *
   * @return ResultSetMetaData
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.ResultSetMetaData getMetaData()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the number, types and properties
   * of this PreparedStatement object's parameters.
   *
   * @return ParameterMetaData
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.ParameterMetaData getParameterMetaData()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter
   * to the given Array object.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setArray(int parameterIndex, org.okip.service.dbc.api.Array x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given java.math.BigDecimal value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setBigDecimal(int parameterIndex, java.math.BigDecimal x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter
   * to the given Blob object.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setBlob(int parameterIndex, org.okip.service.dbc.api.Blob x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Java boolean value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setBoolean(int parameterIndex, boolean x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Java byte value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setByte(int parameterIndex, byte x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Java array of bytes.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setBytes(int parameterIndex, byte[] x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Clob object.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setClob(int parameterIndex, org.okip.service.dbc.api.Clob x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given java.sql.Date value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setDate(int parameterIndex, java.sql.Date x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given java.sql.Date value,
   * using the given Calendar object.
   *
   *
   * @param parameterIndex
   * @param x
   * @param cal
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setDate(int parameterIndex, java.sql.Date x,
                      java.util.Calendar cal)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Java double value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setDouble(int parameterIndex, double x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Java float value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setFloat(int parameterIndex, float x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Java int value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setInt(int parameterIndex, int x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Java long value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setLong(int parameterIndex, long x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter
   * to SQL NULL.
   *
   *
   * @param parameterIndex
   * @param sqlType
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setNull(int parameterIndex, int sqlType)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter
   * to SQL NULL.
   *
   *
   * @param paramIndex
   * @param sqlType
   * @param typeName
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setNull(int paramIndex, int sqlType, String typeName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the value of the designated
   * parameter using the given object.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setObject(int parameterIndex, Object x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the value of the designated
   * parameter with the given object.
   *
   *
   * @param parameterIndex
   * @param x
   * @param targetSqlType
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setObject(int parameterIndex, Object x, int targetSqlType)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the value of the designated
   * parameter with the given object.
   *
   *
   * @param parameterIndex
   * @param x
   * @param targetSqlType
   * @param scale
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setObject(int parameterIndex, Object x, int targetSqlType,
                        int scale)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given REF(<structured-type>) value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setRef(int parameterIndex, java.sql.Ref x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Java short value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setShort(int parameterIndex, short x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given Java String value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setString(int parameterIndex, String x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given java.sql.Time value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setTime(int parameterIndex, java.sql.Time x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given java.sql.Time value,
   * using the given Calendar object.
   *
   *
   * @param parameterIndex
   * @param x
   * @param cal
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setTime(int parameterIndex, java.sql.Time x,
                      java.util.Calendar cal)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given java.sql.Timestamp value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setTimestamp(int parameterIndex, java.sql.Timestamp x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given java.sql.Timestamp value,
   * using the given java.util.Calendar object.
   *
   *
   * @param parameterIndex
   * @param x
   * @param cal
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setTimestamp(int parameterIndex, java.sql.Timestamp x,
                           java.util.Calendar cal)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Sets the designated parameter to
   * the given java.net.URL value.
   *
   *
   * @param parameterIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setURL(int parameterIndex, java.net.URL x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

}
