package edu.mit.is.service.dbc;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/contrib/mit/impl/src/edu/mit/is/service/dbc/ResultSet.java,v $
 */

/**
 *A table of data representing a database result set, which is usually
 *generated by executing a statement that queries the database.
 * <p>
 * @see(java.sql.ResultSet)
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.10 $ / $Date: 2002/08/06 14:36:58 $ */
public class ResultSet
implements org.okip.service.dbc.api.ResultSet, java.io.Serializable {

  /*
    methods in 1.4 but not in this interface
    getAsciiStream()
    getBinaryStream()
    getCharacterStream()
    updateAsciiStream()
    updateBinaryStream()
    updateCharacterStream()
*/
  private edu.mit.is.service.dbc.Connection        con                  =
    null;

  private String                                   sql                  =
    null;
  private Object[]                                 objs                 =
    null;
  private int[]                                    targetSqlTypes       =
    null;
  private int[]                                    scales               =
    null;

  private Object[]                                 rows                 =
    null;
  private int                                      index                = 0;
  private int                                      offset               = 0;
  private int                                      currentRow           = 0;

  private String                                   catalog              =
    null;
  private String                                   schemaPattern        =
    null;
  private String                                   procedureNamePattern =
    null;
  private String                                   columnNamePattern    =
    null;
  private String                                   procedureName        =
    null;
  private String                                   tableNamePattern     =
    null;
  private String[]                                 tableTypes           =
    null;

  private static final Object[]                    NOARGS               =
    new Object[0];
  private static final Class[]                     NOARGSTYPES          =
    new Class[0];
  private static final String                      GETROWS              =
    "getRows";

  private edu.mit.is.service.dbc.ResultSetMetaData rowsMetaData         =
    null;

  /**
   * constructor
   *
   */
  public ResultSet() {}

  /**
   * Constructor ResultSet
   *
   *
   * @param con
   * @param sql
   *
   */
  public ResultSet(edu.mit.is.service.dbc.Connection con, String sql) {
    this.con = con;
    this.sql = sql;
  }

  /**
   * Constructor ResultSet
   *
   *
   * @param con
   * @param rs
   * @param sql
   *
   */
  public ResultSet(edu.mit.is.service.dbc.Connection con,
                   java.sql.ResultSet rs, String sql) {
    this.con = con;
    this.sql = sql;

    createDbcResultSet(rs);
  }

  /**
   * Constructor ResultSet
   *
   *
   * @param rowsMetaData
   * @param rows
   *
   */
  public ResultSet(edu.mit.is.service.dbc.ResultSetMetaData rowsMetaData,
                   Object[] rows) {
    this.rowsMetaData = rowsMetaData;
    this.rows         = rows;
  }

  /**
   * Method setConnection
   *
   *
   * @param conn
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setConnection(edu.mit.is.service.dbc.Connection conn)
  throws org.okip.service.dbc.api.DbcException {
    this.con = conn;
  }

  /**
   * Method setSQL
   *
   *
   * @param sql
   *
   */
  public void setSQL(String sql) {
    this.sql = sql;
  }

  /**
   * Method getSQL
   *
   *
   * @return String
   *
   */
  public String getSQL() {
    return this.sql;
  }

  private void createDbcResultSet(java.sql.ResultSet rs) {
    try {
      java.sql.ResultSetMetaData rsMeta = rs.getMetaData();

      this.rowsMetaData =
        new edu.mit.is.service.dbc.ResultSetMetaData(rsMeta);
      this.rows         = createRows(rs);
    }
    catch (java.sql.SQLException ex) {
      ex.printStackTrace();
    }
    catch (org.okip.service.dbc.api.DbcException ex1) {
      ex1.printStackTrace();
    }
  }

  private Object[] createRows(java.sql.ResultSet rs)
  throws org.okip.service.dbc.api.DbcException {
    try {
      int rsType = rs.getType();

      if (rs.TYPE_FORWARD_ONLY == rsType) {
        if (rs.next()) {
          return createCells(rs);
        }
      }
      else if (rs.TYPE_SCROLL_INSENSITIVE == rsType) {
        try {
          rs.setFetchDirection(rs.FETCH_FORWARD);
        }
        catch (Exception ex1) {}

        if (rs.first()) {
          return createCells(rs);
        }
      }
      else if (rs.TYPE_SCROLL_SENSITIVE == rsType) {
        try {
          rs.setFetchDirection(rs.FETCH_FORWARD);
        }
        catch (Exception ex2) {}

        boolean isRowAvailable = rs.first();

        if (rs.first()) {
          return createCells(rs);
        }
      }
    }
    catch (java.sql.SQLException ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(
        org.okip.service.dbc.api.DbcException.LOCALGETROWSFAILED);
    }

    return null;
  }

  private Object[] createCells(java.sql.ResultSet rs)
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    Object[] rows           = new Object[(this.con.getMaxRows()).intValue()];
    int      cursor         = 0;
    int      rowsAdded      = 0;
    boolean  isRowAvailable = true;

    while (isRowAvailable) {
      if (cursor >= this.offset) {
        Object[] columns = new Object[this.rowsMetaData.getColumnCount()];

        for (int i = 0; columns.length != i; i++) {
          columns[i] = createCell(rs.getObject(i + 1), rowsAdded, i);
        }

        rows[rowsAdded] = columns;

        rowsAdded++;
      }

      if ((this.con.getMaxRows()).intValue() == rowsAdded) {
        break;
      }

      cursor++;

      isRowAvailable = rs.next();
    }

    if (0 == rowsAdded) {
      return null;
    }

    if ((this.con.getMaxRows()).intValue() > rowsAdded) {
      Object[] newRows = new Object[rowsAdded];

      for (int j = 0; rowsAdded != j; j++) {
        newRows[j] = rows[j];
      }

      rows = newRows;
    }

    return rows;
  }

  private Object createCell(Object obj, int row, int col)
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    Object cell = obj;

    if (obj instanceof java.sql.Blob) {
      cell = makeBlob((java.sql.Blob) obj, row, col);
    }
    else if (obj instanceof java.sql.Clob) {
      cell = makeClob((java.sql.Clob) obj, row, col);
    }

    return obj;
  }

  private org.okip.service.dbc.api.Blob makeBlob(java.sql.Blob blob, int row,
          int col)
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    return new edu.mit.is.service.dbc.Blob(
      (edu.mit.is.service.dbc.ResultSet) this.clone(), blob.length(), row,
      col);
  }

  private org.okip.service.dbc.api.Clob makeClob(java.sql.Clob clob, int row,
          int col)
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    return new edu.mit.is.service.dbc.Clob(
      (edu.mit.is.service.dbc.ResultSet) this.clone(), clob.length(), row,
      col);
  }

  /**
   * Method clone
   *
   *
   * @return Object
   *
   */
  public Object clone() {
    try {
      return new edu.mit.is.service.dbc.ResultSet(this.rowsMetaData, rows);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    return null;
  }

  protected java.sql.ResultSet getResultSet()
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    if ((null != tableNamePattern) || (null != tableTypes)) {
      return getTables();
    }
    else if (null != this.sql) {
      return getQuery();
    }
    else if ((null != this.procedureName)
             || (null != this.columnNamePattern)) {
      return getProcedureColumns();
    }
    else {
      return getProcedures();
    }
  }

  private java.sql.ResultSet getTables()
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    java.sql.DatabaseMetaData meta =
      (java.sql.DatabaseMetaData) con.getMetaData();

    return meta.getTables(this.catalog, this.schemaPattern,
                          this.tableNamePattern, this.tableTypes);
  }

  private java.sql.ResultSet getQuery()
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    java.sql.ResultSet rs = null;

    if (null == objs) {
      java.sql.Connection conn      = con.getConnection();
      java.sql.Statement  statement =
        (java.sql.Statement) conn.createStatement();

      rs = statement.executeQuery(sql);
    }
    else {
      java.sql.PreparedStatement statement =
        (java.sql.PreparedStatement) con.prepareStatement(sql);

      for (int i = 0; objs.length != i; i++) {
        if (null != this.scales) {
          statement.setObject(i + 1, objs[i], targetSqlTypes[i], scales[i]);
        }
        else if (null != targetSqlTypes) {
          statement.setObject(i + 1, objs[i], targetSqlTypes[i]);
        }
        else {
          statement.setObject(i + 1, objs[i]);
        }
      }

      rs = statement.executeQuery();
    }

    return rs;
  }

  private java.sql.ResultSet getProcedureColumns()
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    java.sql.DatabaseMetaData meta =
      (java.sql.DatabaseMetaData) con.getMetaData();

    return meta.getProcedureColumns(this.catalog, this.schemaPattern,
                                    this.procedureName,
                                    this.columnNamePattern);
  }

  private java.sql.ResultSet getProcedures()
  throws org.okip.service.dbc.api.DbcException, java.sql.SQLException {
    java.sql.DatabaseMetaData meta =
      (java.sql.DatabaseMetaData) con.getMetaData();

    return meta.getProcedures(this.catalog, this.schemaPattern,
                              this.procedureNamePattern);
  }

  /**
   * Method getFactory
   *
   *
   * @return edu.mit.is.service.dbc.Factory
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public edu.mit.is.service.dbc.Factory getFactory()
  throws org.okip.service.dbc.api.DbcException {
    return con.getFactory();
  }

  /**
   * Method getRows
   *
   *
   *
   * @return Object[]
   * @throws Exception
   *
   */
  public Object[] getRows()
  throws Exception {
    edu.mit.is.service.dbc.Factory     factory = this.getFactory();
    edu.mit.is.util.romi.client.Client client  =
      (edu.mit.is.util.romi.client.Client) factory.getClient();

    if (edu.mit.is.util.romi.client.Client.isValid(client)) {
      return getRowsRemotely(client);
    }
    else {
      return getRowsLocally();
    }
  }

  private Object[] getRowsRemotely(edu.mit.is.util.romi.client.Client client)
  throws Exception {
    return (Object[]) client.invoke(this, GETROWS, NOARGSTYPES, NOARGS);
  }

  /**
   * Method getRowsLocally
   *
   *
   *
   * @return Object[]
   * @throws Exception
   *
   */
  public Object[] getRowsLocally()
  throws Exception {
    java.sql.ResultSet         rs     = getResultSet();
    java.sql.ResultSetMetaData rsMeta = rs.getMetaData();

    this.rowsMetaData = new edu.mit.is.service.dbc.ResultSetMetaData(rsMeta);
    this.rows         = createRows(rs);

    java.sql.Statement statement = rs.getStatement();

    rs.close();
    statement.close();

    return this.rows;
  }

  /**
   * Moves the cursor to the given row
   * number in this ResultSet object.
   *
   * @param row
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean absolute(int row)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Moves the cursor to the end of this
   * ResultSet object, just after the
   * last row.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void afterLast()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Moves the cursor to the front of this
   * ResultSet object, just before the
   * first row.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void beforeFirst()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Cancels the updates made to the
   * current row in this ResultSet
   * object.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void cancelRowUpdates()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Clears all warnings reported on
   * this ResultSet object.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void clearWarnings()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Releases this ResultSet object's database
   * and JDBC resources immediately instead of
   * waiting for this to happen when it is
   * automatically closed.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void close()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Deletes the current row from this ResultSet
   * object and from the underlying database.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void deleteRow()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Maps the given ResultSet column name
   * to its ResultSet column index.
   *
   * @param columnName
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int findColumn(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    java.util.List columnNames_list = this.rowsMetaData.getColumnNames();

    for (int i = 0; i < columnNames_list.size(); i++) {
      String a_column = (String) columnNames_list.get(i);

      if (a_column.equalsIgnoreCase(columnName)) {
        return (i + 1);
      }
    }

    throw new org.okip.service.dbc.api.DbcException("ColumnName "
            + columnName + " is not valid");
  }

  /**
   * Moves the cursor to the first row
   * in this ResultSet object.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean first()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as an Array object
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return Array
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Array getArray(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as an Array object
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return Array
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Array getArray(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.math.BigDecimal
   * with full precision.
   *
   * @param columnIndex
   *
   * @return BigDecimal
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.math.BigDecimal getBigDecimal(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.math.BigDecimal
   * with full precision.
   *
   * @param columnName
   *
   * @return java.math.BigDecimal
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.math.BigDecimal getBigDecimal(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a Blob object
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return Blob
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Blob getBlob(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a Blob object
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return Blob
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Blob getBlob(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a boolean
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean getBoolean(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a boolean
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean getBoolean(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a byte
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return byte
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public byte getByte(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a byte
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return byte
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public byte getByte(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a byte array
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return byte[]
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public byte[] getBytes(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a byte array
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return byte[]
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public byte[] getBytes(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a Clob object
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return Clob
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Clob getClob(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a Clob object
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return Clob
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Clob getClob(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the concurrency mode of
   * this ResultSet object.
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getConcurrency()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the name of the SQL cursor
   * used by this ResultSet object.
   *
   * @return String
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public String getCursorName()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Date object
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return Date
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Date getDate(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Date object
   * in the Java programming language.
   *
   * @param columnIndex
   * @param calendar
   *
   * @return Date
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Date getDate(int columnIndex, java.util.Calendar calendar)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Date object
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return Date
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Date getDate(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Date object
   * in the Java programming language.
   *
   * @param columnName
   * @param calendar
   *
   * @return Date
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Date getDate(String columnName, java.util.Calendar calendar)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a double
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return double
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public double getDouble(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a double
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return double
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public double getDouble(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the fetch direction
   * for this ResultSet object.
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getFetchDirection()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the fetch size for
   * this ResultSet object.
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getFetchSize()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a float
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return float
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public float getFloat(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    Object row  = rows[currentRow];
    Object cell = ((Object[]) row)[columnIndex - 1];

    return ((Float) cell).floatValue();
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a float
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return float
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public float getFloat(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    return getFloat(findColumn(columnName));
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as an int
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getInt(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    Object row  = rows[currentRow];
    Object cell = ((Object[]) row)[columnIndex - 1];

    return ((Integer) cell).intValue();
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as an int
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getInt(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    return getInt(findColumn(columnName));
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a long
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return long
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public long getLong(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    Object row  = rows[currentRow];
    Object cell = ((Object[]) row)[columnIndex - 1];

    return ((Long) cell).longValue();
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a long
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return long
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public long getLong(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    return getLong(findColumn(columnName));
  }

  /**
   * Retrieves the number, types and
   * properties of this ResultSet object's
   * columns.
   *
   * @return ResultSetMetaData
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.ResultSetMetaData getMetaData()
  throws org.okip.service.dbc.api.DbcException {
    return this.rowsMetaData;
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as an Object
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return Object
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public Object getObject(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    Object row  = rows[currentRow];
    Object cell = ((Object[]) row)[columnIndex - 1];

    return cell;
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as an Object
   * in the Java programming language.
   *
   * @param columnIndex
   * @param map
   *
   * @return Object
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public Object getObject(int columnIndex, java.util.Map map)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as an Object
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return Object
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public Object getObject(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    return getObject(findColumn(columnName));
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as an Object
   * in the Java programming language.
   *
   *
   * @param colName
   * @param map
   *
   * @return Object
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public Object getObject(String colName, java.util.Map map)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Ref object
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return Ref
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Ref getRef(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Ref object
   * in the Java programming language.
   *
   *
   * @param colName
   *
   * @return Ref
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Ref getRef(String colName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the current row number.
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getRow()
  throws org.okip.service.dbc.api.DbcException {
    return currentRow;
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a short
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return short
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public short getShort(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    Object row  = rows[currentRow];
    Object cell = ((Object[]) row)[columnIndex - 1];

    return ((Short) cell).shortValue();
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a short
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return short
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public short getShort(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the Statement object
   * that produced this ResultSet object.
   *
   * @return Statement
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Statement getStatement()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a String
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return String
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public String getString(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    Object row  = rows[index - 1];
    Object cell = ((Object[]) row)[columnIndex - 1];

    return (String) cell;
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a String
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return String
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public String getString(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    return getString(findColumn(columnName));
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Time object
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return Time
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Time getTime(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    Object row  = rows[currentRow];
    Object cell = ((Object[]) row)[columnIndex - 1];

    return (java.sql.Time) cell;
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Time object
   * in the Java programming language.
   *
   * @param columnIndex
   * @param calendar
   *
   * @return Time
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Time getTime(int columnIndex, java.util.Calendar calendar)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Time object
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return Time
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Time getTime(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    return getTime(findColumn(columnName));
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Time object
   * in the Java programming language.
   *
   * @param columnName
   * @param calendar
   *
   * @return Time
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Time getTime(String columnName, java.util.Calendar calendar)
  throws org.okip.service.dbc.api.DbcException {
    return getTime(findColumn(columnName), calendar);
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Timestamp object
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return Timestamp
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Timestamp getTimestamp(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    Object row  = rows[currentRow];
    Object cell = ((Object[]) row)[columnIndex - 1];

    return (java.sql.Timestamp) cell;
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Timestamp object
   * in the Java programming language.
   *
   * @param columnIndex
   * @param calendar
   *
   * @return Timestamp
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Timestamp getTimestamp(int columnIndex,
                                         java.util.Calendar calendar)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Timestamp object
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return Timestamp
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Timestamp getTimestamp(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    return getTimestamp(findColumn(columnName));
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.sql.Timestamp object
   * in the Java programming language.
   *
   * @param columnName
   * @param calendar
   *
   * @return Timestamp
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.Timestamp getTimestamp(String columnName,
                                         java.util.Calendar calendar)
  throws org.okip.service.dbc.api.DbcException {
    return getTimestamp(findColumn(columnName), calendar);
  }

  /**
   * Retrieves the type of this
   * ResultSet object.
   *
   * @return int
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getType()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.net.URL object
   * in the Java programming language.
   *
   * @param columnIndex
   *
   * @return URL
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.net.URL getURL(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the value of the designated
   * column in the current row of this
   * ResultSet object as a java.net.URL object
   * in the Java programming language.
   *
   * @param columnName
   *
   * @return URL
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.net.URL getURL(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves the first warning reported
   * by calls on this ResultSet object.
   *
   * @return SQLWarning
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.SQLWarning getWarnings()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Inserts the contents of the insert
   * row into this ResultSet object and
   * into the database.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void insertRow()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves whether the cursor is
   * after the last row
   * in this ResultSet object.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean isAfterLast()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves whether the cursor is
   * before the first row
   * in this ResultSet object.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean isBeforeFirst()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves whether the cursor is
   * on the first row
   * of this ResultSet object.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean isFirst()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves whether the cursor is
   * on the last row
   * of this ResultSet object.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean isLast()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Moves the cursor to the last row
   * in this ResultSet object.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean last()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Moves the cursor to the remembered
   * cursor position, usually the
   * current row.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void moveToCurrentRow()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Moves the cursor to the insert row.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void moveToInsertRow()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Moves the cursor down one row
   * from its current position.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean next()
  throws org.okip.service.dbc.api.DbcException {
    if (null != rows) {
      if (index < rows.length) {
        currentRow = index;

        index++;

        return true;
      }
      else {
        this.offset += index;
        index       = 1;    //used to be 0
        currentRow  = 0;
        this.rows   = null;

        try {
          this.rows = getRows();
        }
        catch (Exception ex) {}

        return (null != this.rows);
      }
    }

    return false;
  }

  /**
   * Moves the cursor to the previous row
   * in this ResultSet object.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean previous()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Refreshes the current row with its
   * most recent value in the database.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void refreshRow()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Moves the cursor a relative number of rows,
   * either positive or negative.
   *
   * @param rows
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean relative(int rows)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves whether a row has been deleted.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean rowDeleted()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves whether the current row
   * has had an insertion.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean rowInserted()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Retrieves whether the current row
   * has been updated.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean rowUpdated()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Gives a hint as to the direction in
   * which the rows in this ResultSet
   * object will be processed.
   *
   * @param direction
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setFetchDirection(int direction)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Gives the JDBC driver a hint as to
   * the number of rows that should be
   * fetched from the database when more
   * rows are needed for this ResultSet
   * object.
   *
   * @param rows
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setFetchSize(int rows)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with Array value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateArray(int columnIndex, org.okip.service.dbc.api.Array x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateArray(String columnName, org.okip.service.dbc.api.Array x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.math.BigDecimal value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateBigDecimal(int columnIndex, java.math.BigDecimal x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.sql.BigDecimal value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateBigDecimal(String columnName, java.math.BigDecimal x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a Blob value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateBlob(int columnIndex, org.okip.service.dbc.api.Blob x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a Blob value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateBlob(String columnName, org.okip.service.dbc.api.Blob x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a boolean value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateBoolean(int columnIndex, boolean x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a boolean value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateBoolean(String columnName, boolean x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a byte value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateByte(int columnIndex, byte x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a byte value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateByte(String columnName, byte x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a byte array value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateBytes(int columnIndex, byte[] x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a byte array value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateBytes(String columnName, byte[] x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a Clob value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateClob(int columnIndex, org.okip.service.dbc.api.Clob x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a Clob value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateClob(String columnName, org.okip.service.dbc.api.Clob x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.sql.Date value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateDate(int columnIndex, java.sql.Date x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.sql.Date value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateDate(String columnName, java.sql.Date x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a double value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateDouble(int columnIndex, double x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a double value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateDouble(String columnName, double x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a float value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateFloat(int columnIndex, float x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a float value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateFloat(String columnName, float x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a int value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateInt(int columnIndex, int x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a int value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateInt(String columnName, int x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a long value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateLong(int columnIndex, long x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a long value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateLong(String columnName, long x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a null value.
   *
   * @param columnIndex
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateNull(int columnIndex)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a null value.
   *
   * @param columnName
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateNull(String columnName)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a Object value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateObject(int columnIndex, Object x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a Object value.
   *
   * @param columnIndex
   * @param x
   * @param scale
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateObject(int columnIndex, Object x, int scale)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a Object value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateObject(String columnName, Object x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a Object value.
   *
   * @param columnName
   * @param x
   * @param scale
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateObject(String columnName, Object x, int scale)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.sql.Ref value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateRef(int columnIndex, java.sql.Ref x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.sql.Ref value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateRef(String columnName, java.sql.Ref x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the underlying database with
   * the new contents of the current row
   * of this ResultSet object.
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateRow()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a short value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateShort(int columnIndex, short x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a short value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateShort(String columnName, short x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a String value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateString(int columnIndex, String x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a String value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateString(String columnName, String x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.sql.Time value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateTime(int columnIndex, java.sql.Time x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.sql.Time value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateTime(String columnName, java.sql.Time x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.sql.Timestamp value.
   *
   * @param columnIndex
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateTimestamp(int columnIndex, java.sql.Timestamp x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Updates the designated column
   * with a java.sql.Timestamp value.
   *
   * @param columnName
   * @param x
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void updateTimestamp(String columnName, java.sql.Timestamp x)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

  /**
   * Reports whether the last column read
   * had a value of SQL NULL.
   *
   * @return boolean
   *
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean wasNull()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED");
  }

}
