package edu.mit.is.service.dbc;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/contrib/mit/impl/src/edu/mit/is/service/dbc/Statement.java,v $
 */

/**
 * The object used for executing a static SQL statement and returning the results it produces.
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.10 $ / $Date: 2002/08/06 14:36:58 $
 */
public class Statement
implements org.okip.service.dbc.api.Statement, java.io.Serializable {

  //private edu.mit.is.service.dbc.Factory factory = null;

  private edu.mit.is.service.dbc.Connection con = null;

  /**
   * constructor
   *
   */

  public Statement() {}

  /**
   * Constructor Statement
   *
   *
   * @param con
   *
   */
  public Statement(edu.mit.is.service.dbc.Connection con) {
    this.con = con;
  }

  /**
   * Adds the given SQL command to
   * the current list of commmands
   * for this Statement object.
   *
   * @param sql
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void addBatch(String sql)
  throws org.okip.service.dbc.api.DbcException {}

  /**
   * Cancels this Statement object
   * if both the DBMS and driver support
   * aborting an SQL statement.
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void cancel()
  throws org.okip.service.dbc.api.DbcException {}

  /**
   * Empties this Statement object's
   * current list of SQL commands.
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void clearBatch()
  throws org.okip.service.dbc.api.DbcException {}

  /**
   * Clears all the warnings reported on this Statement object.
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void clearWarnings()
  throws org.okip.service.dbc.api.DbcException {}

  /**
  * Releases this Statement object's
  * database and JDBC resources immediately
  * instead of waiting for this to happen
  * when it is automatically closed.
  *
  * @throws org.okip.service.dbc.api.DbcException
  *
  */
  public void close()
  throws org.okip.service.dbc.api.DbcException {}

  /**
   * Executes the given SQL statement,
   * which may return multiple results.
   *
   * @param sql
   *
   * @return boolean
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean execute(String sql)
  throws org.okip.service.dbc.api.DbcException {
    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {
          Class[]  argsTypes = new Class[1];
          Object[] args      = new Object[1];

          argsTypes[0] = sql.getClass();
          args[0]      = sql;

          Boolean result = (Boolean) client.invoke(this, "execute",
                                                   argsTypes, args);

          if (null != result) {
            return result.booleanValue();
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              "client.invoke() returns null");
          }
        }
        else {
          java.sql.Connection conn =
            (java.sql.Connection) con.getConnection();

          if (null != conn) {
            java.sql.Statement statement = conn.createStatement();

            if (null != statement) {
              return statement.execute(sql);
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Executes the given SQL statement,
   * which may return multiple results, and
   * signals the driver that any
   * auto-generated keys should be made
   * available for retrieval.
   *
   * @param sql
   * @param autoGeneratedKeys
   *
   * @return boolean
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean execute(String sql, int autoGeneratedKeys)
  throws org.okip.service.dbc.api.DbcException {
    return executeRemotely(sql,
                           new Integer(autoGeneratedKeys)).booleanValue();
  }

  protected Boolean executeRemotely(String sql, Integer autoGeneratedKeys)
  throws org.okip.service.dbc.api.DbcException {
    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {
          Class[]  argsTypes = new Class[2];
          Object[] args      = new Object[2];

          argsTypes[0] = sql.getClass();
          argsTypes[1] = autoGeneratedKeys.getClass();
          args[0]      = sql;
          args[1]      = autoGeneratedKeys;

          Boolean result = (Boolean) client.invoke(this, "executeRemotely",
                                                   argsTypes, args);

          if (null != result) {
            return result;
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              "client.invoke() returns null");
          }
        }
        else {
          java.sql.Connection conn =
            (java.sql.Connection) con.getConnection();

          if (null != conn) {
            java.sql.Statement statement = conn.createStatement();

            if (null != statement) {
              return new Boolean(
                statement.execute(sql, autoGeneratedKeys.intValue()));
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Executes the given SQL statement,
   * which may return multiple results, and
   * signals the driver that the
   * auto-generated keys indicated
   * in the given array should be
   * made available for retrieval.
   *
   * @param sql
   * @param columnIndexes
   *
   * @return boolean
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean execute(String sql, int[] columnIndexes)
  throws org.okip.service.dbc.api.DbcException {
    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {
          Class[]  argsTypes = new Class[2];
          Object[] args      = new Object[2];

          argsTypes[0] = sql.getClass();
          argsTypes[1] = columnIndexes.getClass();
          args[0]      = sql;
          args[1]      = columnIndexes;

          Boolean result = (Boolean) client.invoke(this, "execute",
                                                   argsTypes, args);

          if (null != result) {
            return result.booleanValue();
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              "client.invoke() returns null");
          }
        }
        else {
          java.sql.Connection conn =
            (java.sql.Connection) con.getConnection();

          if (null != conn) {
            java.sql.Statement statement = conn.createStatement();

            if (null != statement) {
              return statement.execute(sql, columnIndexes);
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Executes the given SQL statement,
   * which may return multiple results, and
   * signals the driver that the
   * auto-generated keys indicated
   * in the given array should be
   * made available for retrieval.
   *
   * @param sql
   * @param columnNames
   *
   * @return boolean
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean execute(String sql, String[] columnNames)
  throws org.okip.service.dbc.api.DbcException {
    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {
          Class[]  argsTypes = new Class[2];
          Object[] args      = new Object[2];

          argsTypes[0] = sql.getClass();
          argsTypes[1] = columnNames.getClass();
          args[0]      = sql;
          args[1]      = columnNames;

          Boolean result = (Boolean) client.invoke(this, "execute",
                                                   argsTypes, args);

          if (null != result) {
            return result.booleanValue();
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              "client.invoke() returns null");
          }
        }
        else {
          java.sql.Connection conn =
            (java.sql.Connection) con.getConnection();

          if (null != conn) {
            java.sql.Statement statement = conn.createStatement();

            if (null != statement) {
              return statement.execute(sql, columnNames);
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Submits a batch of commands to the
   * database for execution and if all
   * commands execute successfully,
   * returns an array of update counts.
   *
   * @return int[]
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int[] executeBatch()
  throws org.okip.service.dbc.api.DbcException {
    return null;
  }

  /**
   * Executes the given SQL statement,
   * which returns a single ResultSet object.
   *
   * @param sql
   *
   * @return ResultSet
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.ResultSet executeQuery(String sql)
  throws org.okip.service.dbc.api.DbcException {
    try {
      edu.mit.is.util.romi.client.Client client =
        (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

      if (edu.mit.is.util.romi.client.Client.isValid(client)) {
        return executeQueryRemotely(client, sql);
      }
      else {
        return executeQueryLocally(sql);
      }
    }
    catch (Exception ex1) {
      ex1.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex1.getMessage());
    }
  }

  /**
   * Method executeQueryRemotely
   *
   *
   * @param client
   * @param sql
   *
   * @return org.okip.service.dbc.api.ResultSet
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.ResultSet executeQueryRemotely(
          edu.mit.is.util.romi.client.Client client, String sql)
  throws org.okip.service.dbc.api.DbcException {
    Class[]  argsTypes = new Class[1];
    Object[] args      = new Object[1];

    argsTypes[0] = sql.getClass();
    args[0]      = sql;

    try {
      edu.mit.is.service.dbc.ResultSet result =
        (edu.mit.is.service.dbc.ResultSet) client.invoke(this,
          "executeQuery", argsTypes, args);

      result.setConnection(this.con);

      return result;
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Method executeQueryLocally
   *
   *
   * @param sql
   *
   * @return org.okip.service.dbc.api.ResultSet
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.ResultSet executeQueryLocally(String sql)
  throws org.okip.service.dbc.api.DbcException {
    try {
      java.sql.Connection conn = con.getConnection();

      if (null != conn) {
        java.sql.Statement statement =
          (java.sql.Statement) conn.createStatement();

        if (null != statement) {
          java.sql.ResultSet               rs     =
            statement.executeQuery(sql);
          edu.mit.is.service.dbc.ResultSet dbc_rs =
            new edu.mit.is.service.dbc.ResultSet(con, rs, sql);

          rs.close();
          statement.close();

          return dbc_rs;
        }
        else {
          throw new org.okip.service.dbc.api.DbcException(
            org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Executes the given SQL statement,
   * which may be an INSERT, UPDATE, or
   * DELETE statement or an SQL statement
   * that returns nothing, such as an SQL DDL
   * statement.
   *
   * @param sql
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int executeUpdate(String sql)
  throws org.okip.service.dbc.api.DbcException {

    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {
          Class[]  argsTypes = new Class[1];
          Object[] args      = new Object[1];

          argsTypes[0] = sql.getClass();
          args[0]      = sql;

          Integer result = (Integer) client.invoke(this, "executeUpdate",
                                                   argsTypes, args);

          return result.intValue();
        }
        else {
          java.sql.Connection conn = con.getConnection();

          if (null != conn) {
            java.sql.Statement statement =
              (java.sql.Statement) conn.createStatement();

            if (null != statement) {
              int result = statement.executeUpdate(sql);

              statement.close();

              return result;
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Executes the given SQL statement and
   * signals the driver with the given flag
   * about whether the auto-generated keys
   * produced by this Statement object
   * should be made available for retrieval.
   *
   * @param sql
   * @param autoGeneratedKeys
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int executeUpdate(String sql, int autoGeneratedKeys)
  throws org.okip.service.dbc.api.DbcException {
    return executeUpdateRemotely(sql,
                                 new Integer(autoGeneratedKeys)).intValue();
  }

  protected Integer executeUpdateRemotely(String sql,
                                          Integer autoGeneratedKeys)
  throws org.okip.service.dbc.api.DbcException {
    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {
          Class[]  argsTypes = new Class[2];
          Object[] args      = new Object[2];

          argsTypes[0] = sql.getClass();
          argsTypes[1] = autoGeneratedKeys.getClass();
          args[0]      = sql;
          args[1]      = autoGeneratedKeys;

          Integer result = (Integer) client.invoke(this,
                                                   "executeUpdateRemotely",
                                                   argsTypes, args);

          if (null != result) {
            return result;
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              "client.invoke() returns null");
          }
        }
        else {
          java.sql.Connection conn =
            (java.sql.Connection) con.getConnection();

          if (null != conn) {
            java.sql.Statement statement = conn.createStatement();

            if (null != statement) {
              return new Integer(
                statement.executeUpdate(sql, autoGeneratedKeys.intValue()));
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Executes the given SQL statement and signals
   * the driver that the auto-generated keys
   * indicated in the given array should be
   * made available for retrieval.
   *
   * @param sql
   * @param columnIndexes
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int executeUpdate(String sql, int[] columnIndexes)
  throws org.okip.service.dbc.api.DbcException {

    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {
          Class[]  argsTypes = new Class[2];
          Object[] args      = new Object[2];

          argsTypes[0] = sql.getClass();
          argsTypes[1] = columnIndexes.getClass();
          args[0]      = sql;
          args[1]      = columnIndexes;

          Integer result = (Integer) client.invoke(this, "executeUpdate",
                                                   argsTypes, args);

          return result.intValue();
        }
        else {
          java.sql.Connection conn = con.getConnection();

          if (null != conn) {
            java.sql.Statement statement =
              (java.sql.Statement) conn.createStatement();

            if (null != statement) {
              int result = statement.executeUpdate(sql, columnIndexes);

              statement.close();

              return result;
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Executes the given SQL statement and signals
   * the driver that the auto-generated keys
   * indicated in the given array should be
   * made available for retrieval.
   *
   * @param sql
   * @param columnNames
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int executeUpdate(String sql, String[] columnNames)
  throws org.okip.service.dbc.api.DbcException {

    try {
      if (null != con.getFactory()) {
        edu.mit.is.util.romi.client.Client client =
          (edu.mit.is.util.romi.client.Client) con.getFactory().getClient();

        if (edu.mit.is.util.romi.client.Client.isValid(client)) {
          Class[]  argsTypes = new Class[2];
          Object[] args      = new Object[2];

          argsTypes[0] = sql.getClass();
          argsTypes[1] = columnNames.getClass();
          args[0]      = sql;
          args[1]      = columnNames;

          Integer result = (Integer) client.invoke(this, "executeUpdate",
                                                   argsTypes, args);

          return result.intValue();
        }
        else {
          java.sql.Connection conn = con.getConnection();

          if (null != conn) {
            java.sql.Statement statement =
              (java.sql.Statement) conn.createStatement();

            if (null != statement) {
              int result = statement.executeUpdate(sql, columnNames);

              statement.close();

              return result;
            }
            else {
              throw new org.okip.service.dbc.api.DbcException(
                org.okip.service.dbc.api.DbcException.INVALIDSTATEMENT);
            }
          }
          else {
            throw new org.okip.service.dbc.api.DbcException(
              org.okip.service.dbc.api.DbcException.INVALIDCONNECTION);
          }
        }
      }
      else {
        throw new org.okip.service.dbc.api.DbcException(
          org.okip.service.dbc.api.DbcException.INVALIDFACTORY);
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();

      throw new org.okip.service.dbc.api.DbcException(ex.getMessage());
    }
  }

  /**
   * Retrieves the Connection object that
   * produced this Statement object.
   *
   * @return Connection
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.Connection getConnection()
  throws org.okip.service.dbc.api.DbcException {
    return this.con;
  }

  /**
   * Retrieves the direction for fetching rows
   * from database tables that is the
   * default for result sets generated
   * from this Statement object.
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getFetchDirection()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the number of result set rows
   * that is the default fetch size for
   * ResultSet  objects generated
   * from this Statement object.
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getFetchSize()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves any auto-generated keys created
   * as a result of executing this Statement object.
   *
   * @return ResultSet
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.ResultSet getGeneratedKeys()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the maximum number of bytes
   * that can be returned for character and
   * binary column values in a ResultSet object
   * produced by this Statement object.
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getMaxFieldSize()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the maximum number of rows
   * that a ResultSet object produced by
   * this Statement object can contain.
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getMaxRows()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Moves to this Statement object's next result,
   * returns true if it is a ResultSet  object,
   * and implicitly closes any current ResultSet
   * object(s) obtained with the method getResultSet.
   *
   * @return boolean
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean getMoreResults()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Moves to this Statement object's next result,
   * deals with any current ResultSet object(s)
   * according to the instructions specified
   * by the given flag, and returns true
   * if the next result is a ResultSet object.
   *
   * @param current
   *
   * @return boolean
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public boolean getMoreResults(int current)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the number of seconds the driver
   * will wait for a Statement object to execute.
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getQueryTimeout()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the current result as a ResultSet object.
   *
   * @return ResultSet
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public org.okip.service.dbc.api.ResultSet getResultSet()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the result set concurrency for
   * ResultSet objects generated by this
   * Statement object.
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getResultSetConcurrency()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the result set holdability for
   * ResultSet objects generated by this
   * Statement object.
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getResultSetHoldability()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the result set type for
   * ResultSet objects generated by this
   * Statement object.
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getResultSetType()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the current result as an update count;
   * if the result is a ResultSet object or there are
   * no more results, -1 is returned.
   *
   * @return int
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public int getUpdateCount()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Retrieves the first warning reported
   * by calls on this Statement object.
   *
   * @return SQLWarning
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public java.sql.SQLWarning getWarnings()
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Sets the SQL cursor name to the given String,
   * which will be used by subsequent Statement
   * object execute methods.
   *
   * @param name
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setCursorName(String name)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Sets escape processing on or off.
   *
   * @param enable
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setEscapeProcessing(boolean enable)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Gives the driver a hint as to the direction
   * in which rows will be processed in ResultSet
   * objects created using this Statement object.
   *
   * @param direction
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setFetchDirection(int direction)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Gives the JDBC driver a hint as to the number
   * of rows that should be fetched from the
   * database when more rows are needed.
   *
   * @param rows
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setFetchSize(int rows)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Sets the limit for the maximum number of bytes
   * in a ResultSet column storing character or
   * binary values to the given number of bytes.
   *
   * @param max
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setMaxFieldSize(int max)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Sets the limit for the maximum number of rows
   * that any ResultSet object can contain
   * to the given number.
   *
   * @param max
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setMaxRows(int max)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

  /**
   * Sets the number of seconds the driver
   * will wait for a Statement object to
   * execute to the given number of seconds.
   *
   * @param seconds
   *
   * @throws org.okip.service.dbc.api.DbcException
   *
   */
  public void setQueryTimeout(int seconds)
  throws org.okip.service.dbc.api.DbcException {
    throw new org.okip.service.dbc.api.DbcException("NOT IMPLEMENTED! ");
  }

}
