package edu.mit.is.util.romi.server;

public class Factory
implements org.okip.util.romi.server.api.Factory {

  private org.okip.service.shared.api.Agent owner = null;

  /**
   * Method getOwner
   *
   * @return org.okip.service.shared.api.Agent
   *
   */
  public org.okip.service.shared.api.Agent getOwner() {
    return this.owner;
  }

  /**
   * Method setOwner
   *
   * @param org.okip.service.shared.api.Agent
   *
   */
  public void setOwner(org.okip.service.shared.api.Agent owner) {
    this.owner = owner;
  }

  /**
   * Method validate
   *
   * @param props
   *
   * @throws org.okip.util.romi.server.ServerException
   *
   */
  private final void validate(final java.util.Map props) throws
  org.okip.util.romi.server.api.ServerException {

      final boolean valid =
            (props != null) && (props.get("srvtab") != null) &&
             new java.io.File((String)props.get("srvtab")).exists() &&
             (props.get("policyProp") != null)&& (props.get("policy") != null) &&
             (props.get("server") != null) && (props.get("location") != null) ;

            // Enforce these parameters???
            //&& (props.get("host") != null); // && (props.get("port") != null)

      if ( !valid )
        throw new org.okip.util.romi.server.api.ServerException("Invalid parameters for server setup");
  }

  private final String getExportedName(final java.util.Map props) {
      final String host = (String)props.get("host");
      final String port = (String)props.get("port");
      final String server = (String) props.get("server");

      return ("//" + ( host == null ? "localhost" : host ) + ( port == null ? "" : ":" + port) + "/" + server).trim();
  }

  /**
   * Method setup
   *
   *
   * @param props
   *
   * @throws org.okip.util.romi.server.ServerException
   *
   */
  public void setup(java.util.Map props)
  throws org.okip.util.romi.server.api.ServerException {
     validate(props);
     java.io.FileInputStream in        =    null;
    try {


      in  = new java.io.FileInputStream((String) props.get("srvtab"));

      byte[]                  srvtab    = new byte[0];
      int                     length    = 0;
      int                     available = 0;

      while (0 != in.available()) {
        available = in.available();

        byte[] newsrvtab = new byte[length + available];

        System.arraycopy(srvtab, 0, newsrvtab, 0, length);
        in.read(newsrvtab, length, available);

        srvtab = newsrvtab;
        length += available;
      }

      in.close();

      if (null == System.getSecurityManager()) {
        System.setSecurityManager(new java.rmi.RMISecurityManager());
      }

      java.util.Properties props2 = new java.util.Properties();

      props2.put(props.get("policyProp"), props.get("policy"));

      java.rmi.activation.ActivationGroupDesc.CommandEnvironment ace   = null;
      java.rmi.activation.ActivationGroupDesc                    group =
        new java.rmi.activation.ActivationGroupDesc(props2, ace);

      java.rmi.activation.ActivationGroupID                      agi   =
        java.rmi.activation.ActivationGroup.getSystem().registerGroup(group);

      java.rmi.activation.ActivationGroup.createGroup(agi, group, 0);

      java.rmi.MarshalledObject          data =
        new java.rmi.MarshalledObject(srvtab);

      String server = (String) props.get("server");
      String exportedName = getExportedName(props);

      System.out.println("DEBUG: ExportedName = " + exportedName);
      System.out.flush();

      java.rmi.activation.ActivationDesc desc =
      new java.rmi.activation.ActivationDesc(server,
                                               (String) props.get("location"),
                                               data);
      //System.out.println("DEBUG: desc =  " + desc);
      //System.out.flush();

      java.rmi.Remote                    mri  =
        java.rmi.activation.Activatable.register(desc);

      //System.out.println("DEBUG: mri =  " + mri);
      //System.out.flush();

      java.rmi.Naming.rebind(exportedName, mri);

      System.out.println("Exported Server as " + exportedName);
      System.out.flush();
    }
    catch(Throwable ex) {
      throw new org.okip.util.romi.server.api.ServerException(ex.getMessage());
    } finally {
        if ( in != null ) {
            try {
                in.close();
            } catch(Exception ignored) {
                // . . .
            } finally {
                in = null;
            }
        }
    }
  }

}
