// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb4.lib;

import java.io.*;

public class Krb4KDCErr {
	public byte pvno;
	public byte msgType;
	boolean littleEndian;
	public String cname;
	public String cinst;
	public String crealm;
	public long timestamp;
	public int error_code;
	public String error_text;

	public Krb4KDCErr(
		String new_cname,
		String new_cinst,
		String new_crealm,
		long new_timestamp,
		int new_error_code,
		String new_error_text
	) {
		pvno = (byte)Krb4.KRB_PROT_VERSION;
		msgType = (byte)Krb4.AUTH_MSG_ERR_REPLY;
		littleEndian = true;
		cname = new_cname;
		cinst = new_cinst;
		crealm = new_crealm;
		timestamp = new_timestamp;
		error_code = new_error_code;
		error_text = new_error_text;
	}

	public byte[] encode() throws Krb4Exception, IOException {
		ByteArrayOutputStream o = new ByteArrayOutputStream();
		o.write(pvno);
		byte temp = (byte)((msgType << 1) +
			(littleEndian ? 1 : 0));
		o.write(temp);
		o.write(Krb4Encode.toBytes(cname));
		o.write(Krb4Encode.toBytes(cinst));
		o.write(Krb4Encode.toBytes(crealm));
		o.write(Krb4Encode.toBytes((int)(timestamp / 1000L)));
		o.write(Krb4Encode.toBytes(error_code));
		o.write(Krb4Encode.toBytes(error_text));
		return o.toByteArray();
	}

	public Krb4KDCErr(byte[] data) throws Krb4Exception {
		Krb4Encode ref = new Krb4Encode(data);
		pvno = ref.getByte();
		if (pvno != Krb4.KRB_PROT_VERSION)
			throw new Krb4Exception(Krb4.RD_AP_VERSION);
		byte temp = ref.getByte();
		msgType = (byte)(temp >>> 1);
		if (msgType != Krb4.AUTH_MSG_ERR_REPLY)
			throw new Krb4Exception(Krb4.RD_AP_MSG_TYPE);
		littleEndian = (temp & 1) == 1;
		ref.setByteOrder(littleEndian);
		cname = ref.getString();
		cinst = ref.getString();
		crealm = ref.getString();
		timestamp = (long)ref.getInt() * 1000L;
		error_code = ref.getInt();
		error_text = ref.getString();
	}
}
