package org.okip.service.authentication.api;
/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/authentication/api/Factory.java,v $
 */

/**
 * Factory for Authentication.
 *
 * <p>
 * The Authentication API does not contain information that is expected
 * to be part of the implementation.
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.10 $ / $Date: 2002/07/03 18:53:51 $
 */
public interface Factory
extends org.okip.service.shared.api.Factory {
  /**
   * Returns an Agent that may or may not be anonymous.
   *
   * @return org.okip.service.shared.api.Agent
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  org.okip.service.shared.api.Agent getAgent()
  throws org.okip.service.shared.api.Exception;

  /**
   * Returns the types of Authentication that are available
   * to the current implementation.
   *
   * @return org.okip.service.shared.api.Type[]
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  org.okip.service.shared.api.Type[] getAuthenTypes()
  throws org.okip.service.shared.api.Exception;

  /**
   * Returns the types of Authentication that this agent currently has.
   *
   * @return org.okip.service.shared.api.Type
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  org.okip.service.shared.api.Type getDefaultAuthenType()
  throws org.okip.service.shared.api.Exception;

  /**
   * Sets the default authentication type to be used.
   *
   * @param authenType
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  void setDefaultAuthenType(org.okip.service.shared.api.Type authenType)
  throws org.okip.service.shared.api.Exception;

  /**
   * Return array of possible authentication types.
   *
   * @param agent
   *
   * @return org.okip.service.shared.api.Type[]
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  org.okip.service.shared.api.Type[] getAuthenTypes(
    org.okip.service.shared.api.Agent agent)
  throws org.okip.service.shared.api.Exception;

  /**
   * Return Boolean true if the Agent is authenticated for the
   * specified authentication type.
   *
   *
   * @param agent
   * @param authenType
   *
   * @return Boolean true iff the agent is authenticated under the authenType.
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  Boolean isAuthenticated(org.okip.service.shared.api.Agent agent,
                          org.okip.service.shared.api.Type authenType)
  throws org.okip.service.shared.api.Exception;

  /**
   * Method isAuthenticated for default authenType
   *
   *
   * @param agent
   *
   * @return Boolean true iff the agent is authenticated under the
   * default authenType.
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  Boolean isAuthenticated(org.okip.service.shared.api.Agent agent)
  throws org.okip.service.shared.api.Exception;

  /**
   * Return the agent authenticated under specified authentication type.
   *
   *
   * @param agent
   * @param type
   *
   * @return org.okip.service.shared.api.Agent
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  org.okip.service.shared.api.Agent getAgent(
    org.okip.service.shared.api.Agent agent,
    org.okip.service.shared.api.Type type)
  throws org.okip.service.shared.api.Exception;

  /**
   * Destroy all authentications for the agent.
   *
   * @param agent
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  void destroyAuthen(org.okip.service.shared.api.Agent agent)
  throws org.okip.service.shared.api.Exception;

  /**
   * Destroy agent's authentication of specified type.
   *
   *
   * @param agent
   * @param type
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  void destroyAuthen(org.okip.service.shared.api.Agent agent,
                     org.okip.service.shared.api.Type type)
  throws org.okip.service.shared.api.Exception;

  /**
   * Destroy agent's authentication for each of the array of specified types.
   *
   *
   * @param agent
   * @param types
   *
   * @throws org.okip.service.shared.api.Exception
   *
   */
  void destroyAuthen(org.okip.service.shared.api.Agent agent,
                     org.okip.service.shared.api.Type[] types)
  throws org.okip.service.shared.api.Exception;

}
