package org.okip.service.dbc.api;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/dbc/api/Array.java,v $
 */

/**
 * The mapping in the OKI DBC API for the SQL type ARRAY. By default,
 * an Array value is a transaction-duration reference to an SQL ARRAY
 * value. By default, an Array object is implemented using an SQL
 * LOCATOR(array) internally, which means that an Array object
 * contains a logical pointer to the data in the SQL ARRAY value
 * rather than containing the ARRAY value's data.
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.7 $ / $Date: 2002/07/29 18:42:42 $
 */
public interface Array
extends java.io.Serializable {

  /**
   * Retrieves the contents of the SQL ARRAY
   * value designated by this Array object
   * in the form of an array in the Java
   * programming language.
   *
   * @return Object
   *
   * @throws DbcException
   *
   */
  Object getArray()
  throws DbcException;

  /**
   * Retrieves a slice of the SQL ARRAY value
   * designated by this Array object,
   * beginning with the specified index and
   * containing up to count successive elements
   * of the SQL array.
   *
   * @param index
   * @param count
   *
   * @return Object
   *
   * @throws DbcException
   *
   */
  Object getArray(long index, int count)
  throws DbcException;

  /**
   * Retreives a slice of the SQL ARRAY value
   * designated by this Array object,
   * beginning with the specified index and
   * containing up to count successive elements
   * of the SQL array.
   *
   *
   * @param index
   * @param count
   * @param map
   *
   * @return Object
   *
   * @throws DbcException
   *
   */
  Object getArray(long index, int count, java.util.Map map)
  throws DbcException;

  /**
   * Retrieves the contents of the SQL ARRAY
   * value designated by this Array object.
   *
   * @param map
   *
   * @return Object
   *
   * @throws DbcException
   *
   */
  Object getArray(java.util.Map map)
  throws DbcException;

  /**
   * Retrieves the JDBC type of the elements
   * in the array designated by this Array
   * object.
   *
   * @return int
   *
   * @throws DbcException
   *
   */
  int getBaseType()
  throws DbcException;

  /**
   * Retrieves the SQL type name of the elements
   * in the array designated by this Array object.
   *
   * @return String
   *
   * @throws DbcException
   *
   */
  String getBaseTypeName()
  throws DbcException;

  /**
   * Retrieves a result set that contains the
   * elements of the SQL ARRAY value designated
   * by this Array object.
   *
   * @return ResultSet
   *
   * @throws DbcException
   *
   */
  ResultSet getResultSet()
  throws DbcException;

  /**
   * Retrieves a result set holding the elements
   * of the subarray that starts at index and
   * contains up to count successive elements.
   *
   * @param index
   * @param count
   *
   * @return ResultSet
   *
   * @throws DbcException
   *
   */
  ResultSet getResultSet(long index, int count)
  throws DbcException;

  /**
   * Retrieves a result set holding the elements
   * of the subarray that starts at index and
   * contains up to count successive elements.
   *
   * @param index
   * @param count
   * @param map
   *
   * @return ResultSet
   *
   * @throws DbcException
   *
   */
  ResultSet getResultSet(long index, int count, java.util.Map map)
  throws DbcException;

  /**
   * Retrieves a result set that contains the
   * elements of the SQL ARRAY value designated
   * by this Array object.
   *
   * @param map
   *
   * @return ResultSet
   *
   * @throws DbcException
   *
   */
  ResultSet getResultSet(java.util.Map map)
  throws DbcException;
}
