package org.okip.service.dbc.api;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/dbc/api/CallableStatement.java,v $
 */

/**
 * The interface used to execute SQL stored procedures.
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.7 $ / $Date: 2002/07/29 18:42:42 $
 */
public interface CallableStatement
extends PreparedStatement {

  /* methods in 1.4 but not in this API:
     setAsciiStream
     setBinaryStream
     setCharacterStream
   */

  /**
   * Retrieves the value of the designated JDBC ARRAY
   * parameter as an Array object in the Java
   * programming language.
   *
   *
   * @param parameterIndex
   *
   * @return array
   *
   * @throws DbcException
   *
   */
  Array getArray(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC ARRAY parameter as
   * an Array object in the Java programming language.
   *
   *
   * @param parameterName
   *
   * @return array
   *
   * @throws DbcException
   *
   */
  Array getArray(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC NUMERIC
   * parameter as a java.math.BigDecimal object with as
   * many digits to the right of the decimal point as the
   * value contains.
   *
   *
   * @param parameterIndex
   *
   * @return BigDecimal
   *
   * @throws DbcException
   *
   */
  java.math.BigDecimal getBigDecimal(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC NUMERIC parameter
   * as a java.math.BigDecimal object with as many
   * digits to the right of the decimal point as the
   * value contains.
   *
   *
   * @param parameterName
   *
   * @return BigDecimal
   *
   * @throws DbcException
   *
   */
  java.math.BigDecimal getBigDecimal(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC BLOB
   * parameter as a Blob object in the Java programming
   * language.
   *
   *
   * @param parameterIndex
   *
   * @return blob
   *
   * @throws DbcException
   *
   */
  Blob getBlob(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC BLOB parameter
   * as a Blob object in the Java programming language.
   *
   *
   * @param parameterName
   *
   * @return blob
   *
   * @throws DbcException
   *
   */
  Blob getBlob(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC BIT
   * parameter as a boolean in the Java programming
   * language.
   *
   *
   * @param parameterIndex
   *
   * @return boolean
   *
   * @throws DbcException
   *
   */
  boolean getBoolean(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC BIT parameter as
   * a boolean in the Java programming language.
   *
   *
   * @param parameterName
   *
   * @return boolean
   *
   * @throws DbcException
   *
   */
  boolean getBoolean(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * TINYINT parameter as a byte in the Java programming
   * language.
   *
   *
   * @param parameterIndex
   *
   * @return byte
   *
   * @throws DbcException
   *
   */
  byte getByte(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC TINYINT parameter
   * as a byte in the Java programming language.
   *
   *
   * @param parameterName
   *
   * @return byte
   *
   * @throws DbcException
   *
   */
  byte getByte(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * BINARY or VARBINARY parameter as an array of
   * byte values in the Java programming language.
   *
   *
   * @param parameterIndex
   *
   * @return bytes
   *
   * @throws DbcException
   *
   */
  byte[] getBytes(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC BINARY or
   * VARBINARY parameter as an array of byte
   * values in the Java programming language.
   *
   *
   * @param parameterName
   *
   * @return bytes
   *
   * @throws DbcException
   *
   */
  byte[] getBytes(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * CLOB parameter as a Clob object in the Java
   * programming language.
   *
   *
   * @param parameterIndex
   *
   * @return clob
   *
   * @throws DbcException
   *
   */
  Clob getClob(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC CLOB parameter
   * as a Clob object in the Java programming
   * language.
   *
   *
   * @param parameterName
   *
   * @return clob
   *
   * @throws DbcException
   *
   */
  Clob getClob(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * DATE parameter as a java.sql.Date object.
   *
   *
   * @param parameterIndex
   *
   * @return date
   *
   * @throws DbcException
   *
   */
  java.sql.Date getDate(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * DATE parameter as a java.sql.Date object,
   * using the given java.util.Calendar object
   * to construct the date.
   *
   *
   * @param parameterIndex
   * @param cal
   *
   * @return date
   *
   * @throws DbcException
   *
   */
  java.sql.Date getDate(int parameterIndex, java.util.Calendar cal)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC DATE parameter
   * as a java.sql.Date object.
   *
   *
   * @param parameterName
   *
   * @return date
   *
   * @throws DbcException
   *
   */
  java.sql.Date getDate(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC DATE parameter
   * as a java.sql.Date object, using the given
   * java.util.Calendar object to construct the date.
   *
   *
   * @param parameterName
   * @param cal
   *
   * @return date
   *
   * @throws DbcException
   *
   */
  java.sql.Date getDate(String parameterName, java.util.Calendar cal)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * DOUBLE parameter as a double in the Java
   * programming language.
   *
   *
   * @param parameterIndex
   *
   * @return double
   *
   * @throws DbcException
   *
   */
  double getDouble(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC DOUBLE
   * parameter as a double in the Java programming
   * language.
   *
   *
   * @param parameterName
   *
   * @return double
   *
   * @throws DbcException
   *
   */
  double getDouble(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * FLOAT parameter as a float in the Java
   * programming language.
   *
   *
   * @param parameterIndex
   *
   * @return float
   *
   * @throws DbcException
   *
   */
  float getFloat(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC FLOAT parameter
   * as a float in the Java programming language.
   *
   *
   * @param parameterName
   *
   * @return float
   *
   * @throws DbcException
   *
   */
  float getFloat(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * INTEGER parameter as an int in the Java
   * programming language.
   *
   *
   * @param parameterIndex
   *
   * @return int
   *
   * @throws DbcException
   *
   */
  int getInt(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC INTEGER
   * parameter as an int in the Java programming
   * language.
   *
   *
   * @param parameterName
   *
   * @return int
   *
   * @throws DbcException
   *
   */
  int getInt(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * BIGINT parameter as a long in the Java
   * programming language.
   *
   *
   * @param parameterIndex
   *
   * @return long
   *
   * @throws DbcException
   *
   */
  long getLong(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC BIGINT
   * parameter as a long in the Java
   * programming language.
   *
   *
   * @param parameterName
   *
   * @return long
   *
   * @throws DbcException
   *
   */
  long getLong(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated parameter
   * as an Object in the Java programming language.
   *
   *
   * @param parameterIndex
   *
   * @return Object
   *
   * @throws DbcException
   *
   */
  Object getObject(int parameterIndex)
  throws DbcException;

  /**
   * Returns an object representing the value of
   * OUT parameter i and uses map for the custom
   * mapping of the parameter value.
   *
   *
   * @param parameterIndex
   * @param map
   *
   * @return object
   *
   * @throws DbcException
   *
   */
  Object getObject(int parameterIndex, java.util.Map map)
  throws DbcException;

  /**
   * Retrieves the value of a parameter as an
   * Object in the Java programming language.
   *
   *
   * @param parameterName
   *
   * @return Object
   *
   * @throws DbcException
   *
   */
  Object getObject(String parameterName)
  throws DbcException;

  /**
   * Returns an object representing the value of
   * OUT parameter i and uses map for the custom
   * mapping of the parameter value.
   *
   *
   * @param parameterName
   * @param map
   *
   * @return Object
   *
   * @throws DbcException
   *
   */
  Object getObject(String parameterName, java.util.Map map)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * REF(<structured-type>) parameter as a Ref
   * object in the Java programming language.
   *
   *
   * @param parameterIndex
   *
   * @return ref
   *
   * @throws DbcException
   *
   */
  java.sql.Ref getRef(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC REF
   * (<structured-type>) parameter as a
   * Ref object in the Java programming
   * language.
   *
   *
   * @param parameterName
   *
   * @return ref
   *
   * @throws DbcException
   *
   */
  java.sql.Ref getRef(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * SMALLINT parameter as a short in the Java
   * programming language.
   *
   *
   * @param parameterIndex
   *
   * @return short
   *
   * @throws DbcException
   *
   */
  short getShort(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC SMALLINT
   * parameter as a short in the Java programming
   * language.
   *
   *
   * @param parameterName
   *
   * @return short
   *
   * @throws DbcException
   *
   */
  short getShort(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * CHAR, VARCHAR, or LONGVARCHAR parameter as
   * a String in the Java programming language.
   *
   *
   * @param parameterIndex
   *
   * @return String
   *
   * @throws DbcException
   *
   */
  String getString(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC CHAR, VARCHAR,
   * or LONGVARCHAR parameter as a String in the
   * Java programming language.
   *
   *
   * @param parameterName
   *
   * @return String
   *
   * @throws DbcException
   *
   */
  String getString(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of the designated
   * JDBC TIME parameter as a java.sql.Time
   * object.
   *
   *
   * @param parameterIndex
   *
   * @return Time
   *
   * @throws DbcException
   *
   */
  java.sql.Time getTime(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * TIME parameter as a java.sql.Time object,
   * using the given java.util.Calendar object
   * to construct the time.
   *
   *
   * @param parameterIndex
   * @param cal
   *
   * @return Time
   *
   * @throws DbcException
   *
   */
  java.sql.Time getTime(int parameterIndex, java.util.Calendar cal)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC TIME parameter
   * as a java.sql.Time object.
   *
   *
   * @param parameterName
   *
   * @return Time
   *
   * @throws DbcException
   *
   */
  java.sql.Time getTime(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC TIME parameter
   * as a java.sql.Time object, using the given
   * java.util.Calendar object to construct the time.
   *
   *
   * @param parameterName
   * @param cal
   *
   * @return Time
   *
   * @throws DbcException
   *
   */
  java.sql.Time getTime(String parameterName, java.util.Calendar cal)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * TIMESTAMP parameter as a java.sql.Timestamp
   * object.
   *
   *
   * @param parameterIndex
   *
   * @return Timestamp
   *
   * @throws DbcException
   *
   */
  java.sql.Timestamp getTimestamp(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * TIMESTAMP parameter as a java.sql.Timestamp
   * object, using the given java.util.Calendar
   * object to construct the Timestamp object.
   *
   *
   * @param parameterIndex
   * @param cal
   *
   * @return Timestamp
   *
   * @throws DbcException
   *
   */
  java.sql.Timestamp getTimestamp(int parameterIndex, java.util.Calendar cal)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC TIMESTAMP
   * parameter as a java.sql.Timestamp object.
   *
   *
   * @param parameterName
   *
   * @return Timestamp
   *
   * @throws DbcException
   *
   */
  java.sql.Timestamp getTimestamp(String parameterName)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC TIMESTAMP
   * parameter as a java.sql.Timestamp object,
   * using the given java.util.Calendar object
   * to construct the Timestamp object.
   *
   *
   * @param parameterName
   * @param cal
   *
   * @return Timestamp
   *
   * @throws DbcException
   *
   */
  java.sql.Timestamp getTimestamp(String parameterName,
                                  java.util.Calendar cal)
  throws DbcException;

  /**
   * Retrieves the value of the designated JDBC
   * DATALINK parameter as a java.net.URL object.
   *
   *
   * @param parameterIndex
   *
   * @return URL
   *
   * @throws DbcException
   *
   */
  java.net.URL getURL(int parameterIndex)
  throws DbcException;

  /**
   * Retrieves the value of a JDBC DATALINK
   * parameter as a java.net.URL object.
   *
   *
   * @param parameterName
   *
   * @return URL
   *
   * @throws DbcException
   *
   */
  java.net.URL getURL(String parameterName)
  throws DbcException;

  /**
   * Registers the OUT parameter in ordinal position
   * parameterIndex to the JDBC type sqlType.
   *
   *
   * @param parameterIndex
   * @param sqlType
   *
   * @throws DbcException
   *
   */
  void registerOutParameter(int parameterIndex, int sqlType)
  throws DbcException;

  /**
   * Registers the parameter in ordinal position
   * parameterIndex to be of JDBC type sqlType.
   *
   *
   * @param parameterIndex
   * @param sqlType
   * @param scale
   *
   * @throws DbcException
   *
   */
  void registerOutParameter(int parameterIndex, int sqlType, int scale)
  throws DbcException;

  /**
   * Registers the designated output parameter.
   *
   *
   * @param parameterIndex
   * @param sqlType
   * @param typeName
   *
   * @throws DbcException
   *
   */
  void registerOutParameter(int parameterIndex, int sqlType, String typeName)
  throws DbcException;

  /**
   * Registers the OUT parameter named
   * parameterName to the JDBC type sqlType.
   *
   *
   * @param parameterName
   * @param sqlType
   *
   * @throws DbcException
   *
   */
  void registerOutParameter(String parameterName, int sqlType)
  throws DbcException;

  /**
   * Registers the parameter named parameterName
   * to be of JDBC type sqlType.
   *
   *
   * @param parameterName
   * @param sqlType
   * @param scale
   *
   * @throws DbcException
   *
   */
  void registerOutParameter(String parameterName, int sqlType, int scale)
  throws DbcException;

  /**
   * Registers the designated output parameter.
   *
   *
   * @param parameterName
   * @param sqlType
   * @param typeName
   *
   * @throws DbcException
   *
   */
  void registerOutParameter(String parameterName, int sqlType,
                            String typeName)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given java.math.BigDecimal value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setBigDecimal(String parameterName, java.math.BigDecimal x)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given Java boolean value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setBoolean(String parameterName, boolean x)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given Java byte value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setByte(String parameterName, byte x)
  throws DbcException;

  /**
   * Sets the designated parameter to the given
   * Java array of bytes.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setBytes(String parameterName, byte[] x)
  throws DbcException;

  /**
   * Sets the designated parameter to the given
   * java.sql.Date value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setDate(String parameterName, java.sql.Date x)
  throws DbcException;

  /**
   * Sets the designated parameter to the given
   * java.sql.Date value, using the given
   * java.util.Calendar object.
   *
   *
   * @param parameterName
   * @param x
   * @param cal
   *
   * @throws DbcException
   *
   */
  void setDate(String parameterName, java.sql.Date x, java.util.Calendar cal)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given Java double value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setDouble(String parameterName, double x)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given Java float value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setFloat(String parameterName, float x)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given Java int value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setInt(String parameterName, int x)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given Java long value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setLong(String parameterName, long x)
  throws DbcException;

  /**
   * Sets the designated parameter to SQL NULL.
   *
   *
   * @param parameterName
   * @param sqlType
   *
   * @throws DbcException
   *
   */
  void setNull(String parameterName, int sqlType)
  throws DbcException;

  /**
   * Sets the designated parameter to SQL NULL.
   *
   *
   * @param parameterName
   * @param sqlType
   * @param typeName
   *
   * @throws DbcException
   *
   */
  void setNull(String parameterName, int sqlType, String typeName)
  throws DbcException;

  /**
   * Sets the value of the designated parameter
   * with the given object.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setObject(String parameterName, Object x)
  throws DbcException;

  /**
   * Sets the value of the designated parameter
   * with the given object.
   *
   *
   * @param parameterName
   * @param x
   * @param targetSqlType
   *
   * @throws DbcException
   *
   */
  void setObject(String parameterName, Object x, int targetSqlType)
  throws DbcException;

  /**
   * Sets the value of the designated parameter
   * with the given object.
   *
   *
   * @param parameterName
   * @param x
   * @param targetSqlType
   * @param scale
   *
   * @throws DbcException
   *
   */
  void setObject(String parameterName, Object x, int targetSqlType, int scale)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given Java short value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setShort(String parameterName, short x)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given Java String value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setString(String parameterName, String x)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given java.sql.Time value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setTime(String parameterName, java.sql.Time x)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given java.sql.Time value, using the
   * given java.util.Calendar object.
   *
   *
   * @param parameterName
   * @param x
   * @param cal
   *
   * @throws DbcException
   *
   */
  void setTime(String parameterName, java.sql.Time x, java.util.Calendar cal)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given java.sql.Timestamp value.
   *
   *
   * @param parameterName
   * @param x
   *
   * @throws DbcException
   *
   */
  void setTimestamp(String parameterName, java.sql.Timestamp x)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given java.sql.Timestamp value, using
   * the given java.util.Calendar object.
   *
   *
   * @param parameterName
   * @param x
   * @param cal
   *
   * @throws DbcException
   *
   */
  void setTimestamp(String parameterName, java.sql.Timestamp x,
                    java.util.Calendar cal)
  throws DbcException;

  /**
   * Sets the designated parameter to the
   * given java.net.URL object.
   *
   *
   * @param parameterName
   * @param val
   *
   * @throws DbcException
   *
   */
  void setURL(String parameterName, java.net.URL val)
  throws DbcException;

  /**
   * Retrieves whether the last OUT parameter
   * read had the value of SQL NULL.
   *
   * @return boolean
   *
   * @throws DbcException
   *
   */
  boolean wasNull()
  throws DbcException;

}
