/*
don't reap in middle of transaction if possible
keep pools in static var, per vm
steal oldest connection if none available
connection objects re-get delegate if it's stolen
*/

package org.okip.service.dbc.impl;

import org.okip.service.dbc.api.DbcException;


public class Factory implements org.okip.service.dbc.api.Factory,
  java.io.Serializable {

  private final String DRIVER_CLASS_PROP = "org.okip.service.dbc.impl.driverName";
  private final String DATABASE_URL_PROP = "org.okip.service.dbc.impl.databaseURL";
  private final String USERNAME_PROP = "org.okip.service.dbc.impl.username";
  private final String PASSWORD_PROP = "org.okip.service.dbc.impl.password";

  private final String POOL_PROP = "org.okip.service.dbc.impl.useConnectionPool";
  private final String POOL_MIN_PROP = "org.okip.service.dbc.impl.poolMin";
  private final String POOL_MAX_PROP = "org.okip.service.dbc.impl.poolMax";

  private static java.util.Properties properties;
  private org.okip.service.shared.api.Agent agent;

  // Put all the props/init logic here.
  static {
    try {
      java.util.PropertyResourceBundle prb  =
        (java.util.PropertyResourceBundle) java.util.ResourceBundle.getBundle("org.okip.service.dbc.impl.Factory");
      if ( prb == null )
        throw new DbcException("Cannot locate factory configuration properties.");
      properties = prbToprops( prb );
    } catch (Exception ex ) {
      System.err.println("Exception initializing factory! Cannot locate or load factory configuration properties. Exception: " + ex );
      ex.printStackTrace( System.err );
    }
    
  };

  public org.okip.service.dbc.api.Connection getConnection( String databaseURL, String driverName, String username, String password) throws DbcException {

    if ( driverName != null ) {
      try {
        Class c = Class.forName( driverName );
      } catch ( ClassNotFoundException cnfe ) {
        throw ( chainException( "Cannot load database driver class '" + driverName + "'", cnfe));
      }
    }

    try {
      String doPool = properties.getProperty(POOL_PROP, "0");
      boolean usePool;

      usePool = doPool.equals("1");
      int min = java.lang.Integer.valueOf(properties.getProperty(POOL_MIN_PROP,"4")).intValue();
      
      int max = java.lang.Integer.valueOf(properties.getProperty(POOL_MAX_PROP,"10")).intValue();

if ( usePool )
      return new org.okip.service.dbc.impl.PooledConnectionImpl( databaseURL, driverName, username, password, min, max);
			//else
      return new org.okip.service.dbc.impl.ConnectionImpl( databaseURL, driverName, username, password);
    } catch ( Exception ex ) {
      throw( chainException("TODO: message here(1)", ex));
    }
  }

  public org.okip.service.dbc.api.Connection getConnection(String username, String password) throws DbcException {
    String driverName = properties.getProperty(DRIVER_CLASS_PROP);

    // Check for mandatory properties
    String databaseURL = null;
    if ( (databaseURL = properties.getProperty(DATABASE_URL_PROP)) == null)
      throw new DbcException("Mandatory Property '" + DATABASE_URL_PROP + "' undefined!");
    return getConnection( databaseURL, driverName, username, password);
  }

  public org.okip.service.dbc.api.Connection getConnection( String databaseURL,
      String username, String password) throws DbcException {

    // Check for mandatory properties

    String driverName = properties.getProperty(DRIVER_CLASS_PROP);
    return getConnection( databaseURL, driverName, username, password);
  }

  public  org.okip.service.dbc.api.Connection getConnection()
  throws DbcException {

    // Check for mandatory properties
    String username;
    if ( (username = properties.getProperty(USERNAME_PROP)) == null)
      throw new DbcException("Mandatory Property '" + USERNAME_PROP + "' undefined!");

    String password = null;
    if ( (password = properties.getProperty(PASSWORD_PROP)) == null)
      throw new DbcException("Mandatory Property '" + PASSWORD_PROP + "' undefined!");
    return getConnection(username, password);
  }

  private static final java.util.Properties prbToprops(
    final java.util.PropertyResourceBundle prb) {
    java.util.Properties p = new java.util.Properties();
    java.util.Enumeration keys = prb.getKeys();

    while( keys.hasMoreElements() ) {
      String key = ( String ) keys.nextElement();
      p.setProperty( key, (String)prb.handleGetObject(key) );
    }
    return p;
  }

  private static DbcException chainException( final String message, final Throwable t )
  throws DbcException {
    DbcException e = new DbcException( message);
    e.initCause( t ); // ignore returned Throwable!
    return e;
  }

  public org.okip.service.shared.api.Agent getOwner() {
    return agent;
  }

  public void setOwner(org.okip.service.shared.api.Agent agent) {
    this.agent = agent;
  }

}
