package org.okip.service.dbc.impl;

import org.okip.service.dbc.api.DbcException;

// TODO: maybe use weak refs

public class PooledConnectionImpl
extends ConnectionImpl {

  private static java.util.Hashtable pools;
  private static Mutex mutex;

  private int poolMin;
  private int poolMax;
  private int poolTimeout = 1000;
  private long poolLastUse = 0;

  private Reaper myReaper;

  /* this is only here as a placeholder for logic that will
     be needed for ROMI */

  private static boolean DEBUG = false;
  private static final String CLASSNAME = PooledConnectionImpl.class.getName();

  static {
    //System.out.println("init");
    DEBUG = "true".equals( System.getProperty("org.okip.service.dbc.impl.PooledConnectionImpl.DEBUG") );
    pools = new java.util.Hashtable();
    mutex = new Mutex();
  }

  public PooledConnectionImpl(final String databaseURL, final String driverName,
			final String username, final String password, final int poolMin,
			final int poolMax) throws DbcException {
		super(databaseURL, driverName, username, password);

    this.poolMin = poolMin;
    this.poolMax = poolMax;

    try {
      if (!isRemote) {
            if (driverName != null)
                            Class.forName( driverName );
            setDelegate();
      }
    } catch ( ClassNotFoundException cnfe ) {
      chainException( "Cannot load database driver class '" + driverName
                      + "'", cnfe);
    } catch ( Exception ex ) {
      chainException("", ex);
    }
  }

  private java.sql.Connection getConnectionFromPool() {
    mutex.lock();
    ResourcePool pool=null;
    String key = databaseURL + username + password;

    if (pools.containsKey(key)) {
      pool = (ResourcePool) pools.get(key);
    } else {

      Class params[] = new Class[1];

      java.util.Properties props = new java.util.Properties();
      props.setProperty("databaseURL", databaseURL);
      props.setProperty("driverName", driverName);
      props.setProperty("username", username);
      props.setProperty("password", password);

      try {
        //params[0] = Class.forName("java.util.Properties");
        params[0] = java.util.Properties.class;
	pool = new ResourcePool(this.getClass().getMethod("createResource", params),
				props,
				poolMin,
				poolMax);
      } catch (Exception e) {}
      pools.put(key, pool);
    }
    mutex.unlock();
    return (java.sql.Connection) pool.getResource();
  }

  protected void setDelegate() {
			try {
        delegate = getConnectionFromPool();
        myReaper = new Reaper(this, poolTimeout);
        myReaper.start();
			} catch ( Exception ex ) {
				chainException("", ex);
			}
  }

  private void releaseConnectionToPool(Object res) {
    ResourcePool pool=null;
    String key = databaseURL + username + password;

    pool = (ResourcePool) pools.get(key);
    try {
      pool.releaseResource(res);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void releaseDelegate() {
    releaseConnectionToPool(delegate);
    delegate = null;
    myReaper.interrupt();
    try {
      myReaper.join();
    } catch (Exception e) {}
  }

  public static Object createResource(java.util.Properties props) {
    Object res = null;

    String databaseURL = props.getProperty("databaseURL");
    String driverName = props.getProperty("driverName");
    String username = props.getProperty("username");
    String password = props.getProperty("password");

    try {
      if ( (driverName != null) && !driverName.equals(""))
					Class.forName( driverName );
      res = java.sql.DriverManager.getConnection( databaseURL, username,
						  password );
    } catch ( ClassNotFoundException cnfe ) {
      chainException( "Cannot load database driver class '" + driverName
                      + "'", cnfe);
    } catch ( Exception ex ) {
      chainException("", ex);
    }
    return res;
  }


  protected void validateDelegate () {
    if (!isRemote && delegate == null) {
      setDelegate();
    }
    poolLastUse = System.currentTimeMillis();
  }

  public void close() throws DbcException {
    try {
      if (!isRemote && delegate!=null) {
         releaseDelegate();
      }
    //} catch( java.sql.SQLException e ) {
    } catch( Exception e ) {
      throw chainException("",e);
    }
  }

  protected boolean reap () {
    if (delegate == null)
      return true;
    if (System.currentTimeMillis()-poolLastUse >= poolTimeout) {
      releaseDelegate();
      return true;
    }
    return false;
  }
}

class Mutex {
  boolean locked=false;
  java.util.Vector fifo = new  java.util.Vector();

  public synchronized boolean lock()
  {
    if (locked) {
      boolean myTurn = false;
      Thread me = Thread.currentThread ();

      fifo.add (me);

      while (locked && !myTurn) {
	try {
	  wait();
	} catch (InterruptedException e) {}
	if (fifo.firstElement() == me)
	  myTurn = true;
      }

      fifo.remove(me);
    }
    locked=true;
    return true;
  }

  public synchronized void unlock()
  {
    locked = false;
    notifyAll();
  }
}

class Reaper extends Thread {
  int timeout;
  PooledConnectionImpl myConn;

  public Reaper(PooledConnectionImpl conn, int timeout)
  {
    myConn = conn;
    this.timeout = timeout;
  }

  public void run() {
    while (true) {
      System.out.println("Reaper: awake");

      if (myConn.reap())
	break;

      try {
	System.out.println("Reaper: sleeping");
	Thread.sleep(timeout);
      } catch (InterruptedException e) {}
    }
  }
}
