package org.okip.service.dbc.impl;

import java.lang.reflect.*;
import java.util.*;

public class ResourcePool implements java.io.Serializable {

  private Method createMethod;
  private Properties resourceProperties;

  private java.util.Vector freeList;
  private java.util.Vector usedList;
  Vector queue = new Vector();

  private int minResources;
  private int maxResources;
  private int timeout;
  private Thread reaper;

  public ResourcePool (final Method createMethod,
		       final Properties resourceProperties,
		       final int min,
		       final int max) {

    this.freeList = new java.util.Vector ();
    this.usedList = new java.util.Vector ();
    this.queue = new Vector();

    this.createMethod = createMethod;
    this.resourceProperties = resourceProperties;
    this.minResources = min;
    this.maxResources = max;

    /*
      reaper = new Reaper(this, timeout);
      reaper.start();
    */
    ensureMinimum();
  }

  public synchronized String toString() {
    String s = "Pool Status:\n";
    java.util.Iterator resources;
      
    try {
      resources = freeList.iterator();
      s = s + "  Free List:";
      while (resources.hasNext()) {
        Object o = resources.next();
	s = s + " " + o.hashCode();
      }
      s = s + "\n";

      resources = usedList.iterator();
      s = s + "  Used List:";
      while (resources.hasNext()) {
        Object o = resources.next();
	s = s + " " + o.hashCode();
      }
      s = s + "";
    } catch (Exception e) {}
    return s;
  }
  
  private Object createResource() {
    Object res = null;
    Object args[] = new Object[1];

    args[0] = resourceProperties;	
    try {
      res = createMethod.invoke(null, args);
    } catch (Exception e) {}
    return res;
  }
  
  private synchronized void ensureMinimum () {
    System.out.println("pool.ensureMinimum");
    Object res;
    while (freeList.size() + usedList.size() < minResources) {
      res = createResource();
      freeList.add(res);
    }
    System.out.println(this);
  }

  protected synchronized Object getResource () {
    Object res = null;

    if (freeList.isEmpty() || !queue.isEmpty()) {
      if (usedList.size() < maxResources) {
	res = createResource();
      } else {
	System.out.println("must wait");
	boolean myTurn = false;
	Thread me = Thread.currentThread ();

	queue.add (me);
      
	while (!myTurn) {      
	  try {
	    wait();
	  } catch (InterruptedException e) {}
	  if ((queue.firstElement() == me) && (!freeList.isEmpty()))
	    myTurn = true;
	}
      
	queue.remove(me);
	System.out.println("done wait");
	res = freeList.remove(0);
      }
    } else {
      res = freeList.remove(0);
    }
    if (res!=null) {
      usedList.add(res);
    }
    System.out.println(this);
    return res;
  }

  public synchronized void releaseResource (Object res) 
    throws Exception {
    System.out.println("pool.releaseResource");

    if (!usedList.contains(res))
      throw new Exception ("Attempt to release unused resource");
    
    usedList.remove(res);
    freeList.add(res);
    
    notifyAll();
    System.out.println(this);
  }

  /*
    protected synchronized boolean reap () {
    System.out.println("Reaper: reap");
    java.util.Iterator resources = resourceList.iterator();
    try {
      while (resources.hasNext()) {
	Poolable res = (Poolable) resources.next();
	System.out.println("Reaper: examining resource " + res.hashCode());
	if (res.getDelegate() != null && res.isExpired()) {
	  System.out.println("Reaper: collecting resource");
	  releaseResource(res);
	}
      }
    } catch (Exception e) {
      return true;
    }
    return false;
  }
  */
  
}

