package org.okip.service.dbc.test;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Assert;

public class DbcTest
extends TestCase {
	private static org.okip.service.dbc.api.Factory f;
	private static org.okip.service.dbc.api.Connection c;
	private static org.okip.service.dbc.api.Statement s;
	private org.okip.service.dbc.api.ResultSet rs;

	private java.util.List list = new java.util.ArrayList( 101 );

	private static String[] jdbcEscapes = {
 		"drop table xyz",
 	"create table xyz as select SYSDATE x from dual",
	// Time/Date literals
		"INSERT INTO xyz values ( {d '1998-12-25'} )",
		"select * from  xyz",
	// insert a DBMS date object with the value,
	"SELECT {t '12:05:30'} from dual",         // select a DBMS time object with the value,
	"SELECT {ts '1998-12-25 12:05:30.9876'} from dual",  // select a DBMS timestamp object with the value,
	// Time/Date functions,
	"SELECT NOW() FROM dual",
	"SELECT {fn LCASE('AAA')} FROM dual",
	"INSERT {fn CURDATE()} INTO xyz",
	"SELECT {fn CURTIME()} FROM dual",
	"SELECT {fn NOW()} FROM dual",
	"SELECT {fn MONTHNAME({fn NOW()})} FROM dual",  // Note that Extended SQL can nest,
	"SELECT {fn DAYNAME({fn NOW()})} FROM dual",
	"SELECT {fn DAYOFMONTH({fn NOW()})} FROM dual",
	"SELECT {fn DAYOFWEEK({fn NOW()})} FROM dual",
	"SELECT {fn DAYOFYEAR({fn NOW()})} FROM dual",
	"SELECT {fn HOUR({fn NOW()})} FROM dual",
	"SELECT {fn MINUTE({fn NOW()})} FROM dual",
	"SELECT {fn SECOND({fn NOW()})} FROM dual",
	"SELECT {fn MONTH({fn NOW()})} FROM dual",
	"SELECT {fn QUARTER({fn NOW()})} FROM dual",
	"SELECT {fn WEEK({fn NOW()})} FROM dual",
	"SELECT {fn YEAR({fn NOW()})} FROM dual",
	"SELECT {fn TIMESTAMPADD(day, 1, {fn NOW()})} FROM dual",
	"SELECT {fn TIMESTAMPDIFF(day, {fn NOW()}, { d '1998-12-25'})} FROM dual",
	// String functions,
	"SELECT {fn ASCII('a')} FROM dual",
	"SELECT {fn CHAR(41)} FROM dual",
	"SELECT {fn CONCAT('This_string_', 'should_be_all_connected.')} FROM dual",
	"SELECT {fn DIFFERENCE('a1111', 'b1111')} FROM dual",
	"SELECT {fn INSERT('Sentence that is good if it  a verb.', 29, 0, 'has, ')} FROM dual",
	//"SELECT LCASE('AAA') FROM dual",
	//"SELECT {fn LCASE('AAA')} FROM dual",
	"SELECT {fn LEFT('123456789', 3)} FROM dual",
	"SELECT {fn LENGTH('1234567890')} FROM dual",
	"SELECT {fn LOCATE('345', '123456')} FROM dual",
	"SELECT {fn LTRIM('   123')} FROM dual",
	"SELECT {fn REPEAT('boo ', 10)} FROM dual",
	"SELECT {fn REPLACE('aaabbbcccaaabbbccc', 'aaa', 'AAA')} FROM dual",
	"SELECT {fn RIGHT('1234567890', 3)} FROM dual",
	"SELECT {fn RTRIM('qweqweqwe   ')} FROM dual",
	"SELECT {fn SOUNDEX('foo bar')} FROM dual",
	"SELECT 'Two spaces between A and B: A' + {fn SPACE(2)} + 'B' FROM dual",
	"SELECT {fn SUBSTRING('1234567890', 2, 3)} FROM dual",
	"SELECT {fn UCASE('aaa')} FROM dual",
	// DBMS functions,
	"SELECT {fn DATABASE()} FROM dual",
	"SELECT {fn USER()} FROM dual",
	"SELECT {fn IFNULL(myColumn, 'if the column is null')} FROM dual",
	// Data Conversion functions,
	"SELECT {fn CONVERT(1, BIGINT)} FROM dual",
	"SELECT {fn CONVERT(1, VARCHAR)} FROM dual",
	"SELECT {fn CONVERT(1, CHAR)} FROM dual",
	"SELECT {fn CONVERT(1, DECIMAL)} FROM dual",
	"SELECT {fn CONVERT(1, DOUBLE)} FROM dual",
	"SELECT {fn CONVERT(1, FLOAT)} FROM dual",
	"SELECT {fn CONVERT(1, INTEGER)} FROM dual",
	"SELECT {fn CONVERT(1, REAL)} FROM dual",
	"SELECT {fn CONVERT(1, SMALLINT)} FROM dual",
	"SELECT {fn CONVERT(1, TINYINT)} FROM dual",
	"SELECT {fn CONVERT(1, BIT)} FROM dual",
	"SELECT {fn CONVERT(1, BINARY)} FROM dual",
	"SELECT {fn CONVERT(1, VARBINARY)} FROM dual",
	"SELECT {fn CONVERT(1, LONGVARBINARY)} FROM dual",
	"SELECT {fn CONVERT(1, LONGVARCHAR)} FROM dual",
	"SELECT {fn CONVERT({d '1997-02-28'}, DATE)} FROM dual",
	"SELECT {fn CONVERT({ts '1997-02-28 12:59:59'}, TIMESTAMP)} FROM dual",
	"SELECT {fn CONVERT({t '12:59:59'}, TIME)} FROM dual",
	// Math functions,
	"SELECT {fn ABS(-100)} FROM dual",
	"SELECT {fn ACOS(0.1)} FROM dual",
	"SELECT {fn ASIN(0.1)} FROM dual",
	"SELECT {fn ATAN(0.1)} FROM dual",
	"SELECT {fn ATAN2(1.1, 2.2)} FROM dual",
	"SELECT {fn CEILING(1.1)} FROM dual",
	"SELECT {fn FLOOR(1.1)} FROM dual",
	"SELECT {fn COS(45)} FROM dual",
	"SELECT {fn SIN(45)} FROM dual",
	"SELECT {fn TAN(45)} FROM dual",
	"SELECT {fn COT(45)} FROM dual",
	"SELECT {fn DEGREES(3)} FROM dual",
	"SELECT {fn EXP(1.1)} FROM dual",
	"SELECT {fn LOG(30)} FROM dual",
	"SELECT {fn LOG10(30)} FROM dual",
	"SELECT {fn MOD(23, 13)} FROM dual",
	"SELECT {fn PI()} FROM dual",
	"SELECT {fn POWER(3, 3)} FROM dual",
	"SELECT {fn RADIANS(45.0)} FROM dual",
	"SELECT {fn RAND()} FROM dual",
	"SELECT {fn ROUND(123.456789, 3)} FROM dual",
	"SELECT {fn SIGN(-70)} FROM dual",
	"SELECT {fn SQRT(12345.0)} FROM dual",
	"SELECT {fn TRUNCATE(123.456789, 3)} FROM dual"
};

	public DbcTest(final String name) {
			super(name);
	}

	public static void main (String[] args) {
		junit.textui.TestRunner.run (suite());
	}

	public static Test suite ( ) {
		TestSuite suite= new TestSuite("DBC API Test");
		suite.addTestSuite(DbcTest.class);
	    return suite;
	}

	public void testConnection() {
		System.out.println("testConnection() START");
		System.out.flush();

		try {
			f = (org.okip.service.dbc.api.Factory) org.okip.service.shared.api.FactoryManager.getDefaultFactory( "org.okip.service.dbc.api", null);
			System.out.println("f = " + f );
			System.out.flush();
			Assert.assertNotNull( f );
			//c = f.getConnection("jdbc:oracle:thin:@sruti.mit.edu:1521:srutis",null,"scott","tiger");
                        
                        //NOTE: Omit the domain name when I'm not networked!!
                        c = f.getConnection("jdbc:oracle:thin:@sruti:1521:srutis", null,"scott","tiger");
			Assert.assertNotNull( (Object)c );
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
		System.out.println("testConnection() END, c = " + c);
                System.out.println();
		System.out.flush();
	}

	public void testStatement() {
			System.out.println("testStatement() START");
			System.out.flush();

			Assert.assertNotNull( f );
			Assert.assertNotNull( c );
		try {
			s = c.createStatement();
			//Assert.assertNotNull( (Object)c );
			Assert.assertNotNull( s );
			System.out.println( s );
		} catch ( Exception ex ) {
					ex.printStackTrace();
		} finally {
			System.out.println("testStatement() END");
                        System.out.println();
			System.out.flush();
		}
	}

	public void testEscapes() {
				int x = jdbcEscapes.length;
				//java.sql.ResultSet rs = null;
				org.okip.service.dbc.api.ResultSet rs = null;
				//java.sql.PreparedStatement ps = null;
				org.okip.service.dbc.api.PreparedStatement ps = null;
				for ( int j = 0; j < x; ++j ) {
						String sql = jdbcEscapes[ j ];
						System.out.println ( "EXECUTING '" + jdbcEscapes[j] + "'" );
						try {
								ps = c.prepareStatement( sql );
								//System.out.println(  );
								//System.out.println( ps );
								rs = ps.executeQuery();
								String s = "rs is ";
								if ( ! (rs instanceof java.io.Serializable) )
									s += " not ";
								s += "an instance of java.io.Serializable ";
								System.out.println( s );
								s = " and rs is ";
								if ( ! (rs instanceof java.io.Externalizable) )
									s += " not ";
								s += "an instance of java.io.Externalizable ";
								System.out.println( s );

								//System.out.println(  );
								java.sql.ResultSetMetaData rsmd = rs.getMetaData();
								int ncols = rsmd.getColumnCount();
								//System.out.println ( ncols + " columns. Entering jdbc escapes loop . . ." );
								int k = 0;

								 // Iterate through the result and print the employee names
								//boolean x = rs.next();
								//System.out.println ( " rs.next() returns "  + x);
							while ( rs.next() ) {
									for ( k = 0; k < ncols; ++k )
											System.out.println ( (k + 1 ) + ". " + rs.getString (k + 1) + "\t" );
									System.out.println( );
						 }
					} catch ( Exception ex ) {
							System.out.println ( "EXCEPTION with '" + sql + "'");
							ex.printStackTrace(System.out);
					} finally {
							//disconnect(rs, ps, c);
			}
		}
	}


/*
	public void testpreparedStatement() {
				Assert.assertNotNull( f );
				Assert.assertNotNull( c );
				Assert.assertNotNull( s );
				String sql = "select sysdate, user from dual";
				org.okip.service.dbc.api.PreparedStatement ps = null;
				org.okip.service.dbc.api.ResultSet rs = null;

			try {
					ps = c.prepareStatement( sql );
					Assert.assertNotNull( ps );
					System.out.println( "ps = " + ps );
					rs = ps.executeQuery();
					System.out.println( "rs = " + rs );

					Assert.assertNotNull( rs );
					java.sql.ResultSetMetaData rsmd = rs.getMetaData();
					int ncols = rsmd.getColumnCount();
					int k = 0;

					while ( rs.next() ) {
							for ( k = 0; k < ncols; ++k )
									System.out.println ( (k + 1 ) + ". " + rs.getString (k + 1) + "\t" );
							System.out.println( );
					}
			} catch ( Exception ex ) {
					System.out.println ( "EXCEPTION with '" + sql + "'");
					ex.printStackTrace(System.out);
			}
	}
*/

public void testUpdate() {
    System.out.println("testUpdate() START");
    System.out.flush();

    Assert.assertNotNull( f );
    Assert.assertNotNull( c );
    Assert.assertNotNull( s );
    String updSQL1 = "CREATE TABLE test(name varCHAR(10) NOT NULL, updater VARCHAR(30) DEFAULT(USER) NOT NULL, last_update_time DATE DEFAULT(SYSDATE) NOT NULL)",
        testColVal = "tEsTiNg123",
           updSQL2 = "INSERT INTO TEST(name) VALUES('" + testColVal + "')",
           query1  = "SELECT name FROM test",
        currentSQL = "";
    try {
        setVerbose(true);
        //NOTE: The following 3 statements MUST be omiited. These are done by Setup() !!!!
        //currentSQL = updSQL1;
        //int x = s.executeUpdate( currentSQL );
        //System.out.println("s.executeUpdate('" + currentSQL + "' returned " + x);
        currentSQL = updSQL2;
        int y = s.executeUpdate( currentSQL );
        System.out.println("s.executeUpdate('" + currentSQL + "') returned " + y);
        //java.sql.ResultSet rs = s.executeQuery(sql);
        currentSQL = query1;
        org.okip.service.dbc.api.ResultSet rs = s.executeQuery(currentSQL);
        Assert.assertNotNull( rs );
        java.sql.ResultSetMetaData rsmd = rs.getMetaData();
        int ncols = rsmd.getColumnCount();
        int k = 0;
        String colVal = null;
        while ( rs.next() ) {
                        for ( k = 0; k < ncols; ++k ) {
                                System.out.println ( (k + 1 ) + ". " + (colVal = rs.getString( k + 1 )) + "\t" );
                        }
                        System.out.println( );
        }
        Assert.assertEquals("Incorrect result", colVal, testColVal );
        //CASCADING TESTS!!
        testResultSet(null);
    } catch ( Exception ex ) {
                    System.out.println ( "EXCEPTION with '" + currentSQL + "'");
                    ex.printStackTrace(System.out);
    } finally {
                System.out.println("testUpdate() END");
                System.out.println();
                System.out.flush();
                setVerbose(false);
    }
}

public void testResultSet(Object o) {
    String sql = null;
    try {
        System.out.println("testResultSet() START");
        System.out.flush();
 //CASCADING TESTS!!
        Assert.assertNotNull( f );
        Assert.assertNotNull( c );
        Assert.assertNotNull( s );
        //sql = "select sysdate, user from dual";
        sql = "select * from "  + ( (o != null) ? o :"test");
        doQuery(sql);
    } catch ( Exception ex ) {
            System.out.println ( "EXCEPTION with '" + sql + "'");
            ex.printStackTrace(System.out);
    } finally {
            System.out.println("testResultSet() END");
            System.out.println();
            System.out.flush();
    }
}

public void tearDown() {
    System.out.println(getClass().getName() + ": tearDown() START");
    String currentSQL = null;
    try {
        if ( (c != null) && (s != null)  ) {
            setVerbose(true);
            c.clearWarnings();
            currentSQL = "drop TABLE TEST";
            int x = s.executeUpdate( currentSQL );
            System.out.println("s.executeUpdate('" + currentSQL + "') returned " + x);
        }
    } catch ( Exception ex ) {
        System.out.println ( "EXCEPTION with '" + currentSQL + "'");
        ex.printStackTrace(System.out);
    } finally {
        System.out.println("tearDown() END");
        System.out.println();
        System.out.flush();
        setVerbose(false);
    }
}

public void setUp() {
    System.out.println("setUp() START");
    System.out.flush();
    setVerbose(true);

    String updSQL1 = "CREATE TABLE test(name varCHAR(10) NOT NULL, updater VARCHAR(30) DEFAULT(USER) NOT NULL , last_update_time DATE DEFAULT(SYSDATE) NOT NULL)",
        testColVal = "tEsTiNg123",
           updSQL2 = "INSERT INTO TEST VALUES('" + testColVal + "')",
           query1  = "SELECT name FROM test",
        currentSQL = "";
    try {
            if ( (c != null) && (s != null) ) {
                        currentSQL = updSQL1;
                        int x = s.executeUpdate( currentSQL );
                        System.out.println("s.executeUpdate('" + currentSQL + "') returned " + x);
            }
    } catch ( Exception ex ) {
            System.out.println ( "EXCEPTION with '" + currentSQL + "'");
            ex.printStackTrace(System.out);
    } finally {
        System.out.println("setUp() END");
        System.out.println();
        System.out.flush();
        setVerbose(false);
    }
}
    

private java.io.PrintWriter driverLogWriter = null;

private void setVerbose(boolean verbose) {
    if ( verbose ) {
        // Save current one
        if ( driverLogWriter != null ) 
			return;
        java.sql.DriverManager.setLogWriter( new java.io.PrintWriter( 
                               new java.io.BufferedWriter( new java.io.OutputStreamWriter( System.err ) ) ) );
        driverLogWriter = java.sql.DriverManager.getLogWriter();
	  System.out.println("driverLogWriter set to " + driverLogWriter);
    } else {
        return;
        // Restore saved one
				//java.sql.DriverManager.setLogWriter( null );
    }
}

private void setVerbose() {
		if ( driverLogWriter != null ) return;
    		java.sql.DriverManager.setLogWriter( new java.io.PrintWriter( 
                                    new java.io.BufferedWriter( new java.io.OutputStreamWriter( System.err ) ) ) );
	      driverLogWriter = java.sql.DriverManager.getLogWriter();
}

public void testQuery(final String sql) {
    doQuery(sql);
}

private void doQuery(final String sql) /* throws Exception */ {
    System.out.println();
    System.out.println("doQuery('" + sql + "') STARTS");
    System.out.flush();

    org.okip.service.dbc.api.Connection conn = null;
    org.okip.service.dbc.api.Statement stmt = null;
    org.okip.service.dbc.api.PreparedStatement pstmt = null;
    org.okip.service.dbc.api.ResultSet rs = null;
    int rows = 0;
    try {
        // Connect to the database
        conn = f.getConnection("scott","tiger");
        Assert.assertNotNull(conn);

        // It's faster when you don't commit automatically
        conn.setAutoCommit (false);

        stmt = conn.createStatement();
        rs = stmt.executeQuery(sql);
     
        // Iterate through the result and print the employee names
        java.sql.ResultSetMetaData rsmd = rs.getMetaData();
        int ncols = rsmd.getColumnCount();
        int k = 0,  l =  ncols - 1;

             // Display column heading
				for ( ; k++ <= l; ) {
					System.out.print( rsmd.getColumnName( k + 1) + " | " );
				}
				System.out.println( rsmd.getColumnName( k + 1) );

        while ( rs.next() ) {
                ++rows;
                System.out.print("#" + rows + ": ");
                for ( k = 0; k < ncols; ++k ) {
                    int max = ncols - 1;
                        System.out.print(
                                /* (k + 1 ) + ". " + */ rs.getString (k + 1)
                                + ( ((ncols > 1) &&  (k < max)  ) ? ", " : "" )
                                //+ "\t"  );
                        );
                }
                System.out.println( );
        }
    } catch ( Exception ex ) {
            System.out.println ( "EXCEPTION with '" + sql + "'");
            ex.printStackTrace(System.out);
    } finally {
        try {
            // Close all the resources
            if (rs != null)
              rs.close();
            if (stmt != null)
              stmt.close();
            if (pstmt != null)
              pstmt.close();
            if (conn != null)
              conn.close();
        } catch( Exception ignored ) {
		rs = null;
		stmt = null;
		pstmt = null;
		conn = null;
	}
    }
    System.out.println();
    System.out.println("*****************************************************");
    System.out.println("doQuery ENDS returning " + rows + " rows.");
    System.out.println("*****************************************************");
    System.out.println();
    System.out.flush();
}

public void testPreparedInsert() {
    System.out.println("testPreparedInsert() START");
    System.out.flush();
    String url = null;
    if ( true ) {
    try {
      String url1 = System.getProperty("JDBC_URL");
      if (url1 != null)
        url = url1;
    } catch (Exception e) {
      // If there is any security exception, ignore it
      // and use the default
    }
	org.okip.service.dbc.api.Connection conn = null;
	org.okip.service.dbc.api.Statement stmt = null;
	org.okip.service.dbc.api.PreparedStatement pstmt = null;
	org.okip.service.dbc.api.ResultSet rset = null;
    try {

    // Connect to the database
	    conn = f.getConnection("scott","tiger");
	    Assert.assertNotNull(conn);
	    //new ConnectionImpl(url, "scott", "tiger");
	    //  DriverManager.getConnection (url, "scott", "tiger");

	    // It's faster when you don't commit automatically
	    conn.setAutoCommit (false);

    	// Prepare a statement to cleanup the emp table
    	stmt = conn.createStatement ();
    	try
    	{
    	  boolean b = stmt.execute ("delete from EMP where EMPNO = 1500");
          System.out.println("stmt.execute(\"delete from EMP where EMPNO = 1500\") returned " + b);
          //conn.commit();
    	}
    	catch (Exception e) {
      		// Ignore an error here
      		e.printStackTrace();
    	}

    	try {
    		 boolean b = stmt.execute ("delete from EMP where EMPNO = 507");
                System.out.println("stmt.execute(\"delete from EMP where EMPNO = 507\") returned " + b);
                System.out.println("// ENSURE WE COMMIT HERE!!!");
                conn.commit();
    	} catch (/* java.sql.SQL */ Exception e) {
      	// Ignore an error here too
      		e.printStackTrace();
    	}

    	// Close the statement
    	stmt.close();
        stmt = null;
        testQuery("select * from emp where empno in ( 1500, 507 )");
    	// Prepare to insert new names in the EMP table
    	pstmt = conn.prepareStatement ("insert into EMP (EMPNO, ENAME) values (?, ?)");
	//System.out.println("pstmt from insert into EMP (EMPNO, ENAME) values (?, ?) was " + pstmt);
    	// Add LESLIE as employee number 1500
    	pstmt.setInt (1, 1500);          // The first ? is for EMPNO
    	pstmt.setString (2, "LESLIE");   // The second ? is for ENAME
    	// Do the insertion
    	boolean b = pstmt.execute ();
        System.out.println("pstmt.execute returned " + b);
        testQuery("select * from emp where empno in ( 1500, 507 )");
    	// Add MARSHA as employee number 507
    	pstmt.setInt (1, 507);           // The first ? is for EMPNO
    	pstmt.setString (2, "MARSHA");   // The second ? is for ENAME
    	// Do the insertion
    	b = pstmt.execute ();
        System.out.println("pstmt.execute returned " + b);
        testQuery("select * from emp where empno in ( 1500, 507 )");
         // ENSURE WE COMMIT HERE!!!
    	// Close the statement
        pstmt.close();
        // ENSURE WE COMMIT HERE!!!
        // NOTE: A commit() wasn't strictly necessary below for the OracleDriver. The close() seems to do the trick!
        // Was it because a commit was previously done on the Connection? Or because of the close() autosaving?
        conn.commit();
        System.out.println("// CLOSED the connecion");
    	  conn.close();
        testQuery("select * from emp where empno in ( 1500, 507 )");
        testResultSet("emp");
    } catch (Exception e) {
        e.printStackTrace();
   } finally {
          System.out.println("testPreparedInsert() END");
          System.out.println();
          System.out.flush();
   }
 }
       System.out.println("testPreparedInsert() END");
       System.out.flush();
}

};

/*
public void testStream() {
 	String url = "jdbc:oracle:oci8:@";
    try {
      String url1 = System.getProperty("JDBC_URL");
      if (url1 != null)
        url = url1;
    } catch (Exception e) {
      // If there is any security exception, ignore it
      // and use the default
    }
	org.okip.service.dbc.api.Connection conn = null;
	org.okip.service.dbc.api.Statement stmt = null;
	org.okip.service.dbc.api.PreparedStatement pstmt = null;
	org.okip.service.dbc.api.ResultSet rset = null;
    try
    {
	    // Connect to the database
	    conn = f.getConnection("scott","tiger");
	    Assert.assertNotNull(conn);
	    //new ConnectionImpl(url, "scott", "tiger");
	    //  DriverManager.getConnection (url, "scott", "tiger");

	    // It's faster when you don't commit automatically
	    conn.setAutoCommit (false);

	    // Create a Statement
 		stmt = conn.createStatement ();

	   // Create the example table
      	stmt.execute ("drop table streamexample");
      	    // Create the table
		    stmt.execute ("create table streamexample (NAME varchar2 (256), DATA long)");

		    // Let's insert some data into it.  We'll put the source code
		    // for this very test in the database.
		    //java.io.File file = new java.io.File("StreamExample.java");
		    java.io.File file = new java.io.File("DbcTest.java");
		    java.io.InputStream is = new java.io.FileInputStream("DbcTest.java");
		    //java.io.InputStream is = new java.io.FileInputStream("StreamExample.java");
		    pstmt = conn.prepareStatement ("insert into streamexample (data, name) values (?, ?)");
		    ////XXXX pstmt.setAsciiStream (1, is, (int)file.length ());
		    pstmt.setString (2, "StreamExample");
		    pstmt.execute ();

		    // Do a query to get the row with NAME 'StreamExample'
		    rset = stmt.executeQuery ("select DATA from streamexample where NAME='StreamExample'");

		    // Get the first row
		    if (rset.next ())
		    {
		      // Get the data as a Stream from Oracle to the client
		      ////XXXX java.io.InputStream gif_data = rset.getAsciiStream (1);
			  java.io.InputStream gif_data = null;
		      // Open a file to store the gif data
		      java.io.FileOutputStream os = new java.io.FileOutputStream ("example.out");

		      // Loop, reading from the gif stream and writing to the file
		      int c;
		      while ((c = gif_data.read ()) != -1)
		        os.write (c);

		      // Close the file
		      os.close ();
		    }

    //} catch (java.sql.SQLException e) {
    //  System.out.println("// An exception would be raised if the table did not exist");
    //  System.out.println("// We just ignore it");
    //  e.printStackTrace();
    } catch( Exception ex ) {
		ex.printStackTrace();
	} finally {
		try {
		    // Close all the resources
		    if (rset != null)
		      rset.close();

		    if (stmt != null)
		      stmt.close();

		    if (pstmt != null)
		      pstmt.close();

		    if (conn != null)
		      conn.close();
	} catch( Exception ignored ) {
		rset = null;
		stmt = null;
		pstmt = null;
		conn = null;
	}
  }
 }
*/
