package org.okip.service.filing.api;
/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/api/CabinetEntry.java,v $
 */

/**
 * A entry in a Cabinet, usually a ByteStore or another Cabinet.
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.5 $ / $Date: 2002/07/22 16:08:14 $
 */
public interface CabinetEntry
extends java.io.Serializable {

  /**
   * Tests whether this is a Cabinet.
   *
   *
   * @return boolean - true if this entry is a Cabinet.
   *
   */
  public boolean isCabinet()
  throws FilingException;

  /**
   * Tests whether this is a ByteStore.
   *
   *
   * @return boolean - true if this entry is a ByteStore.
   *
   */
  public boolean isByteStore()
  throws FilingException;

  /**
   * Returns the Cabinet in which this is an entry, or null if it has
   * no parent (for example is the root cabinet).
   *
   *
   * @return Cabinet - the parent Cabinet of this entry, or null if it has
   * no parent (e.g. is the root cabinet)
   *
   * @throws FilingIOException - if an IO error occurs.
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the parent Cabinet.
   */
  public Cabinet getParent()
  throws FilingException;

  /**
   * Return the name of this CabinetEntry in its parent Cabinet.
   *
   * @return name
   *
   */
  public String getName()
  throws FilingException;

  /**
   * Return the path, the sequence of names of CabinetEntries leading
   * to this CabinetEntry from its root Cabinet.
   *
   * @return path
   *
   */
  public String getPath()
  throws FilingException;

  /**
   * Tests whether the Factory Owner may read this CabinetEntry.
   *
   * @return <code>true</code> if and only if this CabinetEntry can be
   *          read by the Factory Owner; <code>false</code> otherwise
   *
   */
  public boolean canRead()
  throws FilingException;

  /**
   * Tests whether the Factory Owner may modify this CabinetEntry.
   *
   * @return <code>true</code> if and only if the Factory Owner is
   *          allowed to write to this CabinetEntry
   *          <code>false</code> otherwise.
   *
   */
  public boolean canWrite()
  throws FilingException;

}
