package org.okip.service.filing.api;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/api/CabinetFactory.java,v $
 */

/**
 * <p> An installation may have several implementations of CabinetFactory
 * available.
 * <p>
 * The application chooses a CabinetFactory implementation through the
 * Factory.  From this CabinetFactory a new Cabinet is created from
 * the factory using <code>createRootCabinet()</code>, or, to reuse an
 * existing cabinet, <code>getCabinet(id)</code> is used.
 * <p>
 * A CabinetFactory implementation has some of the characteristics of
 * an operating system filesystem implementation.  Different
 * filesystems support different maximum file lengths, some may be
 * journaled, some may support RAID.  Different CabinetFactories may
 * support file replication, version control, quotas, etc.
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.6 $ / $Date: 2002/07/22 16:08:14 $ */
public interface CabinetFactory
extends java.io.Serializable {

  /**
   * Return owner of this CabinetFactory.
   *
   *
   * @return org.okip.service.shared.api.Agent Owner
   *
   */
  org.okip.service.shared.api.Agent getOwner();

  /**
   * Set the owner of this CabinetFactory.
   *
   *
   *
   * @param Owner
   *
   */
  void setOwner(org.okip.service.shared.api.Agent Owner);

  /**
   * List all root Cabinets currently available in this CabinetFactory
   * implementation.
   *
   * @return list of Cabinets
   * @throws FilingException - if error occurs.
   *
   */
  public Cabinet[] listRoots()
  throws FilingException;

  /**
   * Construct ID from a String.
   * <p> Each LocallyUniqueIdentifer has a unique string
   * representation.  This method constructs a valid Filing API ID
   * from a string, if the string represents a valid ID string.
   *
   *
   * @param id
   *
   * @return ID object whose String representation is the supplied argument string
   *
   * @throws FilingException - if the string does not represent a valid ID
   */
  public ID idFromString(String id)
  throws FilingException;

  /**
   * Get Cabinet by ID.
   *
   * @param id
   *
   * @return Cabinet with given ID
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the Cabinet with this ID.
   * @throws FilingIOException - if an IO error occurs accessing
   * the Cabinet with this ID.
   * @throws NotFoundException - if there is no Cabinet with this ID.
   */
  public Cabinet getCabinet(ID id)
  throws FilingException;

  /**
   * Get ByteStore by ID.
   *
   * @param id
   *
   * @return ByteStore with given ID
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the ByteStore with this ID.
   * @throws FilingIOException - if an IO error occurs accessing
   * the ByteStore with this ID.
   * @throws NotFoundException - if there is no ByteStore with this ID.
   */
  public ByteStore getByteStore(ID id)
  throws FilingException;

  /**
   * Method createRootCabinet
   *
   * create new empty root cabinet in this implementation
   *
   * @return Cabinet
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to create a new root Cabinet.
   * @throws FilingIOException - if an IO error occurs creating
   * the new root Cabinet.
   *
   */
  public Cabinet createRootCabinet()
  throws FilingException;

  /**
   * Treat a jar (or zip) file in a ByteStore as a Cabinet.
   *
   * @param jar
   *
   * @return Cabinet usable to access jar/zip file contained in ByteStore
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the ByteStore.
   * @throws FilingIOException - if an IO error occurs accessing
   * the ByteStore.
   */
  public Cabinet getCabinetFromJar(ByteStore jar)
  throws FilingException;

  /**
   * Return all capability properties of this CabinetFactory.
   *
   * Capability properties may include:<br>
   * <ul>
   * <li>replication (yes/no)
   * <li>versioning (yes/no)
   * <li>exclusive control to OKI system (yes/no)
   * <li>supports quota (yes/no)
   * <li>readwrite (yes/no)
   * <li>space limit (yes/no)
   * <li>degree of replication (Integer)
   * <li>number of versions supported (Integer)
   * <li>space available (Long)
   * </ul>
   * @return java.util.Map
   *
   */
  public java.util.Map getProperties()
  throws FilingException;
}

