package org.okip.service.filing.api;



/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/api/Factory.java,v $
 */

/**
 * Factory for Filing.
 *
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.3 $ / $Date: 2002/04/10 18:38:42 $
 */
public interface Factory
extends org.okip.service.shared.api.Factory {

  /**
   * Return types of CabinetFactory implementations available in the
   * installation
   *
   *
   * @param agent
   * @return java.util.List containing all factories available
   * @throws FilingException - if error occurs.
   *
   */
  public java.util.List getAvailableCabinetFactories(
    org.okip.service.shared.api.Agent agent)
  throws FilingException;

  /**
   * Return all capability properties which may be available in a
   * CabinetFactory implementation available through this Factory.
   *
   * Capability properties may include:<br>
   * <ul>
   * <li>replication (yes/no)
   * <li>versioning (yes/no)
   * <li>exclusive control to OKI system (yes/no)
   * <li>supports quota (yes/no)
   * <li>readwrite (yes/no)
   * <li>space limit (yes/no)
   * <li>degree of replication (Integer)
   * <li>number of versions supported (Integer)
   * <li>space available (Long)
   * </ul>
   * <p>
   * Positive capabilities are returned as a List.
   * <p>
   * An application may choose an implementation from the available
   * CabinetFactory implementations which has the desired capabilities.
   *
   * @return java.util.List
   * */
  public java.util.List getAvailableProperties();

  /**
   * Return CabinetFactory meeting specified properties requirements, or null.
   *
   * The properties map is specified using keys of shared.api.Type, and
   * values of e.g. Boolean, Long, or Double.  The application prepares a
   * map of desired qualities, e.g.<br>
   * key= new Type("Filing", "MIT", "supportsQuota"),
   *       value= new Boolean(true)<br>
   * key= new Type("Filing", "MIT", "supportsReplication"),
   *       value= new Boolean(true)<br>
   * key= new Type("Filing", "MIT", "minimumReplications"),
   *       value= new Integer(2)<br>
   * <p>
   * The factory either returns an array of CabinetFactories
   * implementing the API and meeting the specifications, or null.
   *
   *
   * @param properties
   * @return CabinetFactory[]
   * @throws FilingException - if error occurs.
   *
   */
  public CabinetFactory[] getCabinetFactory(java.util.Map properties)
  throws FilingException;

  /**
   * Return CabinetFactory of the specified Type.
   * <p>
   * The factory either returns a
   *
   *
   * @param factoryType
   * @return CabinetFactory or null
   * @throws FilingException - if error occurs.
   *
   */
  public CabinetFactory getCabinetFactoryByType(
    org.okip.service.shared.api.Type factoryType)
  throws FilingException;
}
