package org.okip.service.filing.impl;



import org.okip.service.filing.api.*;


/**
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including software, documents, or other related items (the
   "Software"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI&#153; API Implementation License. By
   obtaining, using and/or copying this Software, you agree that you have
   read, understand, and will comply with the following terms and
   conditions of the MIT OKI&#153; API Implementation License:

   Permission to use, copy, modify, and distribute this Software and its
   documentation, with or without modification, for any purpose and
   without fee or royalty is hereby granted, provided that you include
   the following on ALL copies of the Software or portions thereof,
   including modifications or derivatives, that you make:

    *  The full text of the MIT OKI&#153; API Implementation License in a
       location viewable to users of the redistributed or derivative
       work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed or
       derivative Software:
       "Copyright (c) 2002 Massachusetts Institute of Technology
       All Rights Reserved."

    *  Notice of any changes or modifications to the MIT OKI&#153;
       Software, including the date the changes were made. Any modified
       software must be distributed in such as manner as to avoid any
       confusion with the original MIT OKI&#153; Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Software without
   specific, written prior permission. Title to copyright in the Software
   and any associated documentation will at all times remain with the
   copyright holders.

   The export of software employing encryption technology may require a
   specific license from the United States Government. It is the
   responsibility of any person or organization contemplating export to
   obtain such a license before exporting this Software.
*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/impl/Factory.java,v $
 */

/**
 * Factory for Filing.
 *
 * <p>
 * Licensed under the {@link org.okip.service.ApiImplementationLicenseMIT MIT OKI&#153; API Implementation License}.
 *
 * @version $Name:  $ / $Revision: 1.5 $ / $Date: 2002/07/08 15:53:41 $
 */
public class Factory
implements org.okip.service.filing.api.Factory {

  private static java.util.Properties properties               = null;
  private static java.util.Map        cabinetFactoryClassNames =
    new java.util.Hashtable();

  static {
    try {
      java.util.PropertyResourceBundle prb  =
        (java.util.PropertyResourceBundle) java.util.ResourceBundle.getBundle(
          "org.okip.service.filing.impl.CabinetFactories");
      java.util.Enumeration            enum = prb.getKeys();

      while (enum.hasMoreElements()) {
        Object o = enum.nextElement();
      }

      int numFactories = Integer.parseInt((String) prb.getObject("numberoffactories"));
      for(int i=0; numFactories != i; i ++) {
        Class                            factoryTypeClass   =
          Class.forName((String) prb.getObject(new String("factory" + Integer.toString(i) + "TypeClass")));

        org.okip.service.shared.api.Type cabinetFactoryType =
          (org.okip.service.shared.api.Type) factoryTypeClass.newInstance();

        cabinetFactoryClassNames.put(cabinetFactoryType,
                                     (String) prb.getObject("factory" + Integer.toString(i) + "Class"));
      }
    }
    catch (Exception e) {
      System.err.println("oops " + e);
      e.printStackTrace();
    }
  }

  /**
   * Return types of CabinetFactory implementations available in the
   * installation
   *
   *
   * @param agent
   * @return java.util.List containing all factories available
   * @throws FilingException - if error occurs.
   *
   */
  public java.util.List getAvailableCabinetFactories(
          org.okip.service.shared.api.Agent agent)
  throws FilingException {

    // get all available Factories
    return java.util.Arrays.asList(
      cabinetFactoryClassNames.keySet().toArray());
  }

  /**
   * Return all capability properties which may be available in a
   * CabinetFactory implementation available through this Factory.
   *
   * Capability properties may include:<br>
   * <ul>
   * <li>replication
   * <li>versioning
   * <li>exclusive control to OKI system
   * <li>supports quota
   * <li>readonly
   * <li>space limit
   * </ul>
   * <p>
   * An application may choose an implementation from the available
   * CabinetFactory implementations which has the desired capabilities.
   *
   * @return java.util.List
   * */
  public java.util.List getAvailableProperties() {
    java.util.List                             allProperties  =
      new java.util.ArrayList();

    java.util.Iterator                         iter           =
      cabinetFactoryClassNames.keySet().iterator();

    org.okip.service.filing.api.CabinetFactory cabinetFactory = null;

    while (iter.hasNext()) {
      String cabinetFactoryClassName =
        (String) cabinetFactoryClassNames.get(iter.next());

      try {
        Class factoryClass = Class.forName(cabinetFactoryClassName);

        cabinetFactory =
          (org.okip.service.filing.api.CabinetFactory) factoryClass.newInstance();

        java.util.Map      properties = cabinetFactory.getProperties();
        java.util.Iterator iter2      = properties.keySet().iterator();

        while (iter2.hasNext()) {
          Object o = iter2.next();

          if (properties.get(o).equals(Boolean.TRUE)) {
            allProperties.add(o);
          }
        }
      }
      catch (Exception e) {
        System.err.println("oops " + e);
        e.printStackTrace();
      }
    }

    return allProperties;
  }

  /**
   * Return CabinetFactory meeting specified properties requirements, or null.
   *
   * The properties map is specified using keys of shared.api.Type, and
   * values of e.g. Boolean, Long, or Double.  The application prepares a
   * map of desired qualities, e.g.<br>
   * key= new Type("Filing", "MIT", "supportsQuota"),
   *       value= new Boolean(true)<br>
   * key= new Type("Filing", "MIT", "supportsReplication"),
   *       value= new Boolean(true)<br>
   * key= new Type("Filing", "MIT", "minimumReplications"),
   *       value= new Integer(2)<br>
   * <p>
   * The factory either returns an array of CabinetFactories
   * implementing the API and meeting the specifications, or null.
   *
   *
   * @param desiredProperties
   * @return CabinetFactory[]
   * @throws FilingException - if error occurs.
   */
  public CabinetFactory[] getCabinetFactory(java.util.Map desiredProperties)
  throws FilingException {

    java.util.ArrayList returns = new java.util.ArrayList();

    // prepare list of required properties
    // remove all Boolean false desired properties, false is the default
    java.util.Map      requiredProperties     =
      new java.util.Hashtable(desiredProperties);
    java.util.Iterator requiredPropertiesIter =
      requiredProperties.keySet().iterator();

    while (requiredPropertiesIter.hasNext()) {
      Object key = requiredPropertiesIter.next();

      if ((requiredProperties.get(key) instanceof Boolean)
              && (requiredProperties.get(key).equals(Boolean.FALSE))) {
        requiredProperties.remove(key);
      }
    }

    // cabinetFactories contains a map of cabinetFactoryClassNames
    // and their Types
    java.util.Iterator iter = cabinetFactoryClassNames.keySet().iterator();

    // examine each cabinetFactory,
    // comparing its properties to the desired properties
    while (iter.hasNext()) {
      try {
        String                                     cabinetFactoryClassName =
          (String) cabinetFactoryClassNames.get(iter.next());

        Class                                      factoryClass            =
          Class.forName(cabinetFactoryClassName);

        org.okip.service.filing.api.CabinetFactory cabinetFactory          =
          (org.okip.service.filing.api.CabinetFactory) factoryClass.newInstance();

        java.util.Map                              thisCabinetFactoryProperties =
          cabinetFactory.getProperties();

        java.util.Map                              testProperties          =
          requiredProperties;

        java.util.Iterator                         piter                   =
          desiredProperties.keySet().iterator();
        boolean                                    isOK                    =
          true;

        // step through desired properties.  If key (Type) exists in
        // cabinetFactory's properties, the value must be:
        //    the same if Boolean
        //    greater than if Long or Double
        // all 'true' or numeric properties must be matched
        while (piter.hasNext()) {
          org.okip.service.shared.api.Type keyType =
            (org.okip.service.shared.api.Type) piter.next();
          java.util.Iterator               iter2   =
            thisCabinetFactoryProperties.keySet().iterator();

          if (thisCabinetFactoryProperties.get(keyType) != null) {
            if (desiredProperties.get(keyType) instanceof Boolean) {
              isOK &= desiredProperties.get(keyType).equals(
                thisCabinetFactoryProperties.get(keyType));
            }
            else if (desiredProperties.get(keyType) instanceof Long) {
              isOK &= (((Long) desiredProperties.get(keyType)).compareTo(
                thisCabinetFactoryProperties.get(keyType)) <= 0);
            }
            else if (desiredProperties.get(keyType) instanceof Double) {
              isOK &= (((Double) desiredProperties.get(keyType)).compareTo(
                thisCabinetFactoryProperties.get(keyType)) <= 0);
            }

            testProperties.remove(keyType);
          }
        }

        // if properties are OK, add to array
        if (isOK && (testProperties.size() == 0)) {
          cabinetFactory.setOwner(this.owner);
          returns.add(cabinetFactory);
        }
      }
      catch (Exception e) {
        System.err.println("oops " + e);
        e.printStackTrace();
      }
    }

    return (CabinetFactory[]) returns.toArray(new CabinetFactory[0]);
  }

  /**
   * Return CabinetFactory of the specified Type.
   * <p>
   * The factory either returns a
   *
   *
   * @param factoryType
   * @return CabinetFactory or null
   * @throws FilingException - if error occurs.
   *
   */
  public CabinetFactory getCabinetFactoryByType(
          org.okip.service.shared.api.Type factoryType)
  throws FilingException {

    try {
      String                                     cabinetFactoryClassName =
        (String) cabinetFactoryClassNames.get(factoryType);

      Class                                      factoryClass            =
        Class.forName(cabinetFactoryClassName);

      org.okip.service.filing.api.CabinetFactory cabinetFactory          =
        (org.okip.service.filing.api.CabinetFactory) factoryClass.newInstance();

      cabinetFactory.setOwner(this.owner);

      return cabinetFactory;
    }
    catch (Exception e) {
      System.err.println("oops " + e);
      e.printStackTrace();

      throw new FilingException("error loading cabinetFactory:"
                                + e.getMessage());
    }
  }

  private org.okip.service.shared.api.Agent owner = null;

  /**
   * Return owner of this Factory.
   *
   *
   * @return org.okip.service.shared.api.Agent Owner
   *
   */
  public org.okip.service.shared.api.Agent getOwner() {
    return this.owner;
  }

  /**
   * Set the owner of this Factory.
   *
   *
   *
   * @param owner
   *
   */
  public void setOwner(org.okip.service.shared.api.Agent owner) {
    this.owner = owner;
  }
}
