package org.okip.service.filing.impl.localfilesystem;



import org.okip.service.filing.api.*;


/**
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including software, documents, or other related items (the
   "Software"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI&#153; API Implementation License. By
   obtaining, using and/or copying this Software, you agree that you have
   read, understand, and will comply with the following terms and
   conditions of the MIT OKI&#153; API Implementation License:

   Permission to use, copy, modify, and distribute this Software and its
   documentation, with or without modification, for any purpose and
   without fee or royalty is hereby granted, provided that you include
   the following on ALL copies of the Software or portions thereof,
   including modifications or derivatives, that you make:

    *  The full text of the MIT OKI&#153; API Implementation License in a
       location viewable to users of the redistributed or derivative
       work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed or
       derivative Software:
       "Copyright (c) 2002 Massachusetts Institute of Technology
       All Rights Reserved."

    *  Notice of any changes or modifications to the MIT OKI&#153;
       Software, including the date the changes were made. Any modified
       software must be distributed in such as manner as to avoid any
       confusion with the original MIT OKI&#153; Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Software without
   specific, written prior permission. Title to copyright in the Software
   and any associated documentation will at all times remain with the
   copyright holders.

   The export of software employing encryption technology may require a
   specific license from the United States Government. It is the
   responsibility of any person or organization contemplating export to
   obtain such a license before exporting this Software.
*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/impl/localfilesystem/LfsOutputStream.java,v $
 */

/**
 * LocalFileSystem InputStream object
 * <p>
 * Is seekable.
 * </p>
 * <p>
 * </p>
 * <p>
 * Licensed under the {@link org.okip.service.ApiImplementationLicenseMIT MIT OKI&#153; API Implementation License}.
 *
 * @version $Name:  $ / $Revision: 1.3 $ / $Date: 2002/04/29 19:57:57 $
 */

class LfsOutputStream
implements OkiOutputStream {
  private java.io.File             file       = null;
  private java.io.FileOutputStream out        = null;

  protected LfsOutputStream(java.io.File file)
  throws FilingPermissionDeniedException, FilingIOException {
    this.file = file;

    if (!this.file.canRead()) {
        throw new FilingPermissionDeniedException("Read permission denied");
    }

    if (!this.file.canWrite()) {
        throw new FilingPermissionDeniedException("Write permission denied");
    }

    try {
        this.out = new java.io.FileOutputStream(file);
    }
    catch (Exception e) {
        throw new FilingIOException(e.getMessage());
    }
  }


  /**
   * Writes b.length bytes to this IO Object.
   */
  public void write(byte[] b)
  throws FilingIOException, FilingPermissionDeniedException {
    try {
      this.out.write(b);
    }
    catch (java.io.IOException e) {
      throw new FilingIOException(e.getMessage());
    }

  }

  /**
   * Writes len bytes from the specified byte array starting at
   * offset off to this IO Object.
   */
  public void write(byte[] b, int off, int len)
  throws FilingIOException, FilingPermissionDeniedException {
    try {
      this.out.write(b, off, len);
    }
    catch (java.io.IOException e) {
      throw new FilingIOException(e.getMessage());
    }

  }

  /**
   * Writes the specified byte to this IO Object.
   */
  public void write(int b)
  throws FilingIOException, FilingPermissionDeniedException {
    try {
      this.out.write(b);
    }
    catch (java.io.IOException e) {
      throw new FilingIOException(e.getMessage());
    }

  }

  /**
   * Flushes this IO Object and forces any buffered output bytes
   * to be written out to the stream.
   */
  public void flush()
  throws FilingIOException {

    // there is no flush on RandomAccessFile
  }

  /**
   * Closes this IO Object and releases any system resources
   * associated with the IO Object.
   */
  public void close()
  throws FilingIOException {
    try {
      this.out.close();
    }
    catch (java.io.IOException e) {
      throw new FilingIOException(e.getMessage());
    }
  }

  /**
   * Return OutputStream in native environment which can be used to
   * access this ByteStore.  In Java, this is a java.io.OutputStream
   * object.  From this one may obtain a Writer via OutputStreamReader.
   *
   * @return A java.io.OutputStream object which may be used to write to this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to write to this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   */
  public java.io.OutputStream getNativeOutputStream()
  throws FilingPermissionDeniedException, FilingIOException {
    try {
      this.out = new java.io.FileOutputStream(file);

      return out;
    }
    catch (Exception e) {
      throw new FilingIOException(e.getMessage());
    }
  }
}
