package org.okip.service.filing.impl.remotefilesystem;



import org.okip.service.filing.api.*;
import org.okip.service.shared.api.Type;


/**
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including software, documents, or other related items (the
   "Software"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI&#153; API Implementation License. By
   obtaining, using and/or copying this Software, you agree that you have
   read, understand, and will comply with the following terms and
   conditions of the MIT OKI&#153; API Implementation License:

   Permission to use, copy, modify, and distribute this Software and its
   documentation, with or without modification, for any purpose and
   without fee or royalty is hereby granted, provided that you include
   the following on ALL copies of the Software or portions thereof,
   including modifications or derivatives, that you make:

    *  The full text of the MIT OKI&#153; API Implementation License in a
       location viewable to users of the redistributed or derivative
       work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed or
       derivative Software:
       "Copyright (c) 2002 Massachusetts Institute of Technology
       All Rights Reserved."

    *  Notice of any changes or modifications to the MIT OKI&#153;
       Software, including the date the changes were made. Any modified
       software must be distributed in such as manner as to avoid any
       confusion with the original MIT OKI&#153; Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Software without
   specific, written prior permission. Title to copyright in the Software
   and any associated documentation will at all times remain with the
   copyright holders.

   The export of software employing encryption technology may require a
   specific license from the United States Government. It is the
   responsibility of any person or organization contemplating export to
   obtain such a license before exporting this Software.
*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/impl/remotefilesystem/CabinetFactory.java,v $
 */

/**
 * Remote file system CabinetFactory implementation.
 *
 * <p>
 * Licensed under the {@link org.okip.service.ApiImplementationLicenseMIT MIT OKI&#153; API Implementation License}.
 *
 * @version $Name:  $ / $Revision: 1.1 $ / $Date: 2002/07/22 16:31:55 $
 */

public class CabinetFactory
implements org.okip.service.filing.api.CabinetFactory {

  private org.okip.service.shared.api.Agent owner = null;

  /**
   * Return owner of this Factory.
   *
   *
   * @return org.okip.service.shared.api.Agent Owner
   *
   */
  public org.okip.service.shared.api.Agent getOwner() {
    return this.owner;
  }

  /**
   * Set the owner of this Factory.
   *
   *
   *
   * @param owner
   *
   */
  public void setOwner(org.okip.service.shared.api.Agent owner) {
    this.owner = owner;
  }

  /**
   * List all root Cabinets currently available in this CabinetFactory
   * implementation.
   *
   * @return list of Cabinets
   * @throws FilingException - if error occurs.
   *
   */
  public Cabinet[] listRoots()
  throws FilingException {
    Cabinet[] rootCabinets = new Cabinet[1];

    rootCabinets[0] = new RfsCabinet(this);

    return rootCabinets;
  }

  /**
   * Construct ID from a String.
   * <p> Each RemotelyUniqueIdentifer has a unique string
   * representation.  This method constructs a valid Filing API ID
   * from a string, if the string represents a valid ID string.
   *
   *
   * @param id
   *
   * @return ID object whose String representation is the supplied argument string
   *
   * @throws FilingException - if the string does not represent a valid ID
   */
  public ID idFromString(String id)
  throws FilingException {
    return new RfsID(id);
  }

  /**
   * Get Cabinet by ID.
   *
   * @param id
   *
   * @return Cabinet with given ID
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the Cabinet with this ID.
   * @throws FilingIOException - if an IO error occurs accessing
   * the Cabinet with this ID.
   * @throws NotFoundException - if there is no Cabinet with this ID.
   */
  public Cabinet getCabinet(ID id)
  throws FilingException {
    return new RfsCabinet(this, id.toString());
  }

  /**
   * Get ByteStore by ID.
   *
   * @param id
   *
   * @return ByteStore with given ID
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the ByteStore with this ID.
   * @throws FilingIOException - if an IO error occurs accessing
   * the ByteStore with this ID.
   * @throws NotFoundException - if there is no ByteStore with this ID.
   */
  public ByteStore getByteStore(ID id)
  throws FilingException {
    return new RfsByteStore(this, id.toString());
  }

  /**
   * Method createRootCabinet
   *
   * create new empty root cabinet in this implementation
   *
   * @return Cabinet
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to create a new root Cabinet.
   * @throws FilingIOException - if an IO error occurs creating
   * the new root Cabinet.
   *
   */
  public Cabinet createRootCabinet()
  throws FilingException {
    try {
      return new RfsCabinet(this);
    }
    catch (NotFoundException e) {
      return null;
    }
  }

  /**
   * Treat a jar (or zip) file in a ByteStore as a Cabinet.
   *
   * @param jar
   *
   * @return Cabinet usable to access jar/zip file contained in ByteStore
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the ByteStore.
   * @throws FilingIOException - if an IO error occurs accessing
   * the ByteStore.
   */
  public Cabinet getCabinetFromJar(ByteStore jar)
  throws FilingException {
      throw new UnsupportedOperationException();
  }

  /**
   * Return all capability properties of this CabinetFactory.
   *
   * Capability properties may include:<br>
   * <ul>
   * <li>replication (true/false)
   * <li>versioning (true/false)
   * <li>exclusive control to OKI system (true/false)
   * <li>supports quota (true/false)
   * <li>readwrite (true/false)
   * <li>space limit (true/false)
   * <li>degree of replication (Integer)
   * <li>number of versions supported (Integer)
   * <li>space available (Long)
   * </ul>
   * @return java.util.Map
   *
   */
  public java.util.Map getProperties()
  throws FilingException {
    java.util.Map properties = new java.util.Hashtable();

    // this implementation supports no special capabilities
    // these negative capabilities are given as examples,
    // negative is the default so these are not actually needed
    // readwrite is provided as a positive
    properties.put(
      new org.okip.service.filing.api.CapabilityType("MIT", "replication"),
      new Boolean(false));
    properties.put(
      new org.okip.service.filing.api.CapabilityType("MIT", "quota"),
      new Boolean(false));
    properties.put(
      new org.okip.service.filing.api.CapabilityType("MIT", "readwrite"),
      new Boolean(true));

    return properties;
  }
  
  /**
   * getClient - gets romi Client if one exists.
   *
   * The existance of a romi Client indicates that this CabinetFactory
   * and the objects it creates exist on a romi Server.
   *
   * @return org.okip.util.romi.client.api.Client
   *
   * @throws org.okip.util.romi.client.api.ClientException
   */
  public org.okip.util.romi.client.api.Client getClient()
  throws FilingException {
    return null;
  }
  
}
