package org.okip.service.filing.impl.remotefilesystem;

import org.okip.service.filing.api.*;


/**
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including software, documents, or other related items (the
   "Software"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI&#153; API Implementation License. By
   obtaining, using and/or copying this Software, you agree that you have
   read, understand, and will comply with the following terms and
   conditions of the MIT OKI&#153; API Implementation License:

   Permission to use, copy, modify, and distribute this Software and its
   documentation, with or without modification, for any purpose and
   without fee or royalty is hereby granted, provided that you include
   the following on ALL copies of the Software or portions thereof,
   including modifications or derivatives, that you make:

    *  The full text of the MIT OKI&#153; API Implementation License in a
       location viewable to users of the redistributed or derivative
       work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed or
       derivative Software:
       "Copyright (c) 2002 Massachusetts Institute of Technology
       All Rights Reserved."

    *  Notice of any changes or modifications to the MIT OKI&#153;
       Software, including the date the changes were made. Any modified
       software must be distributed in such as manner as to avoid any
       confusion with the original MIT OKI&#153; Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Software without
   specific, written prior permission. Title to copyright in the Software
   and any associated documentation will at all times remain with the
   copyright holders.

   The export of software employing encryption technology may require a
   specific license from the United States Government. It is the
   responsibility of any person or organization contemplating export to
   obtain such a license before exporting this Software.
*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/impl/remotefilesystem/RfsCabinet.java,v $
 */

/**
 * Remote file system Cabinet implementation.
 *
 * <p>
 * Licensed under the {@link org.okip.service.ApiImplementationLicenseMIT MIT OKI&#153; API Implementation License}.
 *
 * @version $Name:  $ / $Revision: 1.2 $ / $Date: 2002/08/04 04:27:04 $
 */

public class RfsCabinet
implements org.okip.service.filing.api.Cabinet {
  private transient java.io.File cabinetFile = null;
  private transient RfsID id                 = null;
  private CabinetFactory cabinetFactory      = null;
  private boolean        create              = false;
  private String         idStr               = null;
  
  // Constructor
  protected RfsCabinet(CabinetFactory cabinetFactory)
  throws FilingPermissionDeniedException, FilingIOException,
         NotFoundException {
    this.cabinetFactory = cabinetFactory;
  }

  // Constructor
  protected RfsCabinet(CabinetFactory cabinetFactory, String idStr)
  throws FilingPermissionDeniedException, FilingIOException,
         NotFoundException {
    this.cabinetFactory = cabinetFactory;
    this.idStr = idStr;
  }

  // Constructor
  protected RfsCabinet(CabinetFactory cabinetFactory, String idStr,
                       boolean create)
  throws FilingPermissionDeniedException, FilingIOException {
    this.cabinetFactory = cabinetFactory;
    this.create         = create;
    this.idStr          = idStr;
  }
  
  private void openCabinet()
  throws NotFoundException, FilingPermissionDeniedException, FilingIOException {
    try {
      if(null == this.cabinetFile) {
        if (create) {
          this.cabinetFile = new java.io.File(this.id.toString());
          if (!this.cabinetFile.mkdir()) {
            throw new FilingIOException(id.toString() + " could not be created");
          }
        }  
        else {
          if (!this.cabinetFile.isDirectory()) {
            throw new NotFoundException(id + " not found");
          }
        }

        if (!this.cabinetFile.canRead()) {
          throw new FilingPermissionDeniedException("cannot read " + id);
        }

        this.id = new RfsID(this.cabinetFile.getCanonicalPath());
      }
    }
    catch (Exception e) {
      throw new FilingIOException(e.getMessage());
    }
  }

  /**
   * Method getProperties
   *
   * Return properties of this cabinet.  The properties map is
   * specified using keys of org.okip.service.shared.api.Type, and values of
   * e.g. Boolean, Long, or Double.  The application prepares a map of
   * desired qualities, e.g.<br>
   * key= new Type("Filing", "MIT", "supportsQuota"),
   *    value= new Boolean(true)<br>
   * key= new Type("Filing", "MIT", "supportsReplication"),
   *    value= new Boolean(true)<br>
   * key= new Type("Filing", "MIT", "minimumReplications"),
   * value= new Integer(2)
   *
   *
   * @return java.util.Map of properties of this Cabinet and implementation
   *
   */
  // CHANGED PROTOTYPE BELOW!!
  //public java.util.Map getProperties() {
  public java.util.Map getProperties() throws FilingException {
    return this.cabinetFactory.getProperties();
  }

  /**
   * Get ID of this Cabinet
   *
   * @return ID
   *
   */
  public ID getID()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
					try {
						return
							(ID) this.cabinetFactory.getClient().invoke(this,
								"getID",
								argTypes, args);
					} catch (Exception e) {
						throw new FilingException(e.getMessage());
					}
    } else {
      return
        getIDLocally();
    } 
  }
  
  private ID getIDLocally()
  throws FilingException {
    openCabinet();
    return this.id;
  }

  /**
   * Returns the time that this Cabinet was last modified.
   *
   * @return  A <code>long</code> value representing the time the file was
   *          last modified, measured in milliseconds since the epoch
   *          (00:00:00 GMT, January 1, 1970)
   *
   * @throws FilingIOException - if an IO error occurs.
   */
  public long getLastModifiedTime()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					((Long) this.cabinetFactory.getClient().invoke(this,
						"getLastModifiedTime",
						argTypes, args)).longValue();
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        getLastModifiedTimeLocally();
    } 
  }
  
  private long getLastModifiedTimeLocally()
  // [GS] CHANGE: throws FilingIOException {
  throws FilingException {
    openCabinet();
    long lastModified = this.cabinetFile.lastModified();

    if (lastModified == 0) {
      throw new FilingIOException("invalid lastModified time");
    }
    else {
      return lastModified;
    }
  }

  /**
   * Sets the last-modified time of this Cabinet.
   *
   * <p> All platforms support file-modification times to the nearest second,
   * but some provide more precision.  The argument will be truncated to fit
   * the supported precision.  If the operation succeeds and no intervening
   * operations on the file take place, then the next invocation of the
   * <code>{@link #getLastModifiedTime}</code> method will return the (possibly
   * truncated) <code>time</code> argument that was passed to this method.
   *
   * @param  time  The new last-modified time, measured in milliseconds since
   *               the epoch (00:00:00 GMT, January 1, 1970)
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingException - If the argument is negative
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the lastModified time.
   * @throws FilingIOException - if an IO error occurs setting the
   * lastModified time.
   */
  public boolean setLastModifiedTime(long time)
  throws FilingException {
    Class[] argTypes = new Class[1];
    Object[] args = new Object[1];
    if(null != this.cabinetFactory.getClient()) {
      argTypes[0] = long.class;
      args[0] = new Long(time);
			try {
				return
					((Boolean) this.cabinetFactory.getClient().invoke(this,
						"setLastModifiedTime",
          argTypes, args)).booleanValue();
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
		} else {
      return
        setLastModifiedTimeLocally(time);
    } 
  }
  
  private boolean setLastModifiedTimeLocally(long time)
  throws FilingException, FilingPermissionDeniedException, FilingIOException {
    if (time < 0L) {
      throw new FilingException("IllegalArgumentException - negative time");
    }

    openCabinet();
    boolean success = this.cabinetFile.setLastModified(time);

    if (!success) {
      if (!this.cabinetFile.canWrite()) {
        throw new FilingPermissionDeniedException(
          "cannot set lastModifiedTime");
      }

      throw new FilingIOException("IO Error setting lastModifiedTime");
    }
    else {
      return true;
    }
  }
  /**
   * Returns the time that this Cabinet was last accessed.
   *
   * Not all implementations will record last access times accurately,
   * due to caching and for performance.  The value returned will be
   * at least the last modified time, the actual time when a read was
   * performed may be later.
   *
   * @return  A <code>long</code> value representing the time the file was
   *          last accessed, measured in milliseconds since the epoch
   *          (00:00:00 GMT, January 1, 1970)
   *
   * @throws FilingIOException - if an IO error occurs.  */
  public long getLastAccessedTime()
  // [GS] CHANGE: throws FilingIOException {
  throws FilingException {
    return this.getLastModifiedTime();
  }


  /**
   * Returns the time that this Cabinet was created.
   *
   * Not all implementations will record the time of creation
   * accurately.  The value returned will be at least the last
   * modified time, the actual creation time may be earlier.
   *
   * @return  A <code>long</code> value representing the time the file was
   *          created, measured in milliseconds since the epoch
   *          (00:00:00 GMT, January 1, 1970)
   *
   * @throws FilingIOException - if an IO error occurs.
   */
  public long getCreatedTime()
  // [GS] CHANGE: throws FilingIOException {
      throws FilingException {
				return this.getLastModifiedTime();
  }

  /**
   * Deletes this Cabinet.
   *
   * Cabinet must be empty, and the Owner of the Factory must have
   * sufficient permissions.
   *
   * @return  <code>true</code> if and only if this Cabinet is
   *          successfully deleted; <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to delete this Cabinet.
   * @throws FilingIOException - if an IO error occurs deleting the
   * lastModified time.
   * @throws FilingException - if the Cabinet isn't empty.
   */
  public boolean delete()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					((Boolean) this.cabinetFactory.getClient().invoke(this,
						"delete",
						argTypes, args)).booleanValue();
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        deleteLocally();
    } 
  }
  
  private boolean deleteLocally()
  throws FilingIOException, FilingPermissionDeniedException, FilingException {

    openCabinet();
    boolean success = this.cabinetFile.delete();

    if (!success) {
      if (!this.cabinetFile.canWrite()) {
        throw new FilingPermissionDeniedException("read only");
      }

      throw new FilingIOException("IO Error on deleting");
    }
    else {
      return true;
    }
  }

  /**
   * Return owner of this Cabinet.
   *
   * Owner may be used for quota and/or access control.
   *
   * @return owner
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to learn who owns this Cabinet.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if owners of Cabinets
   * are not implemented.
   */
  public org.okip.service.shared.api.Agent getOwner()
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "Cabinet Ownership not implemented");
  }

  /**
   * Set owner of this Cabinet.
   *
   * Owner may be used for quota and/or access control.
   *
   * @param owner
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to change ownership of this Cabinet.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if owners of Cabinets
   * are not implemented.
   */
  public void setOwner(org.okip.service.shared.api.Agent owner)
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "Cabinet Ownership not implemented");
  }

  /**
   * Tests whether the Factory Owner may read this Cabinet.
   *
   * @return <code>true</code> if and only if this Cabinet can be
   *          read by the Factory Owner; <code>false</code> otherwise
   *
   */
  public boolean canRead()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					((Boolean) this.cabinetFactory.getClient().invoke(this,
						"canRead",
						argTypes, args)).booleanValue();
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        canReadLocally();
    } 
  }
  
  private boolean canReadLocally()
  throws FilingException {
    openCabinet();
    return this.cabinetFile.canRead();
  }

  /**
   * Marks this Cabinet so that only read operations are allowed.
   * After invoking this method this Cabinet is guaranteed not to
   * change until it is either deleted or marked to allow write
   * access.
   *
   * Note that whether or not a read-only Cabinet may be deleted
   * depends upon the underlying system of the implementation.
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the Cabinet readOnly.
   * @throws FilingIOException - if an IO error occurs.
   */
  public boolean setReadOnly()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					((Boolean) this.cabinetFactory.getClient().invoke(this,
						"setReadOnly",
						argTypes, args)).booleanValue();
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        setReadOnlyLocally();
    } 
  }
  
  private boolean setReadOnlyLocally()
  // [GS] CHANGE: throws FilingPermissionDeniedException, FilingIOException {
  throws FilingException {
    openCabinet();
    boolean success = this.cabinetFile.setReadOnly();

    if (!success) {
      if (!this.cabinetFile.canWrite()) {
        throw new FilingPermissionDeniedException(
          "cannot set this ByteStore Read Only");
      }

      throw new FilingIOException("IO Error setting ReadOnly");
    }
    else {
      return true;
    }
  }

  /**
   * Tests whether the Factory Owner may modify this Cabinet.
   *
   * @return <code>true</code> if and only if the Factory Owner is
   *          allowed to write to this Cabinet
   *          <code>false</code> otherwise.
   *
   */
  public boolean canWrite()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					((Boolean) this.cabinetFactory.getClient().invoke(this,
						"canWrite",
						argTypes, args)).booleanValue();
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        canWriteLocally();
    } 
  }
  
  private boolean canWriteLocally()
  throws FilingException {
    openCabinet();
    return this.cabinetFile.canWrite();
  }

  /**
   * Marks this Cabinet so that write operations are allowed.
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the Cabinet writable
   * @throws FilingIOException - if an IO error occurs
   * @throws UnsupportedFilingOperationException - if it is not
   * possible to set writable in this implementation
   */
  public boolean setWritable()
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "Cannot set a Cabinet Writable");
  }

  // not in java.io.File in 1.0-1.4

  /**
   * Create new ByteStore and add it to this Cabinet under the given
   * name.
   *
   * The name must not include this Cabinet's separationCharacter.
   *
   * @param   name  The name to be used
   *
   * @return  The ByteStore created
   *
   * @throws NameCollisionException - if name is already in use by a
   * CabinetEntry.
   * @throws FilingIOException - if an IO error occurs.
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to create a ByteStore.
   */
  public ByteStore createByteStore(String name)
  // [GS] CHANGE: throws NameCollisionException, FilingIOException,
  throws FilingException {
    Class[] argTypes = new Class[1];
    Object[] args = new Object[1];
    if(null != this.cabinetFactory.getClient()) {
      argTypes[0] = String.class;
      args[0] = name;
			try {
				return
					(ByteStore) this.cabinetFactory.getClient().invoke(this,
						"createByteStore",
						argTypes, args);
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        createByteStoreLocally(name);
    } 
  }
  
  private ByteStore createByteStoreLocally(String name)
  // [GS] CHANGE: throws NameCollisionException, FilingIOException,
  throws FilingException {
    openCabinet();
    // generate ID from my ID plus the name
    String newId = this.id.toString() + java.io.File.separator + name;

    if (!this.canWrite()) {
      throw new FilingPermissionDeniedException(
        "cannot create ByteStore in this cabinet ");
    }

    // the id may already exist, check
    java.io.File testFile = new java.io.File(newId);

    if (testFile.exists()) {
      throw new NameCollisionException("already exists");
    }

    return new RfsByteStore(this.cabinetFactory, newId, true);
  }

  /**
   * Create new anonymous ByteStore in this cabinet, to be deleted
   * automatically on exit
   *
   * @return  The ByteStore created
   *
   * @throws FilingIOException - if an IO error occurs.
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to create a ByteStore.
   */
  public ByteStore createTempByteStore()
  throws FilingIOException, FilingPermissionDeniedException {
    return null;
  }

  /**
   * Create new Cabinet and add it to this Cabinet under the given
   * name.
   *
   * The name must not include this Cabinet's separationCharacter.
   *
   * @param   name  The name to be used
   *
   * @return  The Cabinet created
   *
   * @throws NameCollisionException - if name is already in use by a
   * CabinetEntry
   * @throws IllegalCharacterException - if name includes the
   * separationCharacter
   * @throws FilingIOException - if an IO error occurs.
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to create a Cabinet.
   */
  public Cabinet createCabinet(String name)
  throws FilingException {
    Class[] argTypes = new Class[1];
    Object[] args = new Object[1];
    if(null != this.cabinetFactory.getClient()) {
      argTypes[0] = String.class;
      args[0] = name;
			try {
				return
					(Cabinet) this.cabinetFactory.getClient().invoke(this,
						"createCabinet",
						argTypes, args);
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        createCabinetLocally(name);
    } 
  }
  
  private Cabinet createCabinetLocally(String name)
  // [GS] CHANGE: throws NameCollisionException, IllegalCharacterException,
         // [GS] CHANGE: FilingIOException, FilingPermissionDeniedException {
  throws FilingException {
    openCabinet();
    // generate ID from my ID plus the name
    String newId = this.id.toString() + java.io.File.separator + name;

    if (!this.canWrite()) {
      throw new FilingPermissionDeniedException(
        "cannot create Cabinet in this cabinet ");
    }

    // the id may already exist, check
    java.io.File testFile = new java.io.File(newId);

    if (testFile.exists()) {
      throw new NameCollisionException("already exists");
    }

    return new RfsCabinet(this.cabinetFactory, newId, true);
  }

  /**
   * Method createByteStore
   *
   *
   * create new anonymous ByteStore in this Cabinet by copying  contents,
   * mimeType, and owner of another ByteStore.
   *
   *
   * @param oldByteStore
   *
   * @return ByteStore
   *
   * @throws NameCollisionException - if name of the oldByteStore is
   * already in use by a CabinetEntry in this Cabinet
   * @throws FilingIOException - if an IO error occurs
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to create a ByteStore
   */
  public ByteStore createByteStore(ByteStore oldByteStore)
  // [GS] CHANGE: throws NameCollisionException, FilingIOException,
         // [GS] CHANGE: FilingPermissionDeniedException {
  throws FilingException {
    return createByteStore(oldByteStore.getName(), oldByteStore);
  }

  /**
   * Create new named ByteStore in this Cabinet by copying contents,
   * mimeType, and owner of another ByteStore.
   *
   * @param name
   * @param oldByteStore
   *
   * @return ByteStore
   *
   * @throws NameCollisionException - if name is already in use by a
   * CabinetEntry.
   * @throws FilingIOException - if an IO error occurs.
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to create a ByteStore.
   */
  public ByteStore createByteStore(String name, ByteStore oldByteStore)
  // [GS] CHANGE: throws NameCollisionException, FilingIOException,
  throws FilingException {
    ByteStore newByteStore = this.createByteStore(name);

    newByteStore.setMimeType(oldByteStore.getMimeType());

    try {
      newByteStore.setOwner(oldByteStore.getOwner());
    }
    catch (UnsupportedFilingOperationException e) {}

    return newByteStore;
  }

  /**
   * Add CabinetEntry, must be from same CabinetFactory.
   *
   *
   * @param entry
   * @param name
   *
   * @throws NameCollisionException - if name is already in use by a
   * CabinetEntry
   * @throws IllegalCharacterException - if name includes the
   * separationCharacter.
   * @throws FilingIOException - if an IO error occurs.
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to add a Cabinet.
   * @throws UnsupportedFilingOperationException - if the add/remove semantics
   * are not supportable in the implmentation.
   */
  public void add(CabinetEntry entry, String name)
  throws NameCollisionException, IllegalCharacterException,
         FilingIOException, FilingPermissionDeniedException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "add/remove cabinetEntry not implemented");
  }

  /**
   * Remove CabinetEntry.  Does not destroy CabinetEntry.
   *
   *
   * @param entry
   *
   * @throws FilingIOException - if an IO error occurs.
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to remove a Cabinet.
   * @throws UnsupportedFilingOperationException - if the add/remove semantics
   * are not supportable in the implmentation.
   */
  public void remove(CabinetEntry entry)
  throws FilingIOException, FilingPermissionDeniedException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "add/remove cabinetEntry not implemented");
  }

  /**
   * Get CabinetEntry from Cabinet by ID.
   *
   * @param id
   *
   * @return CabinetEntry which has given ID.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the CabinetEntry with this ID.
   * @throws FilingIOException - if an IO error occurs accessing
   * the CabinetEntry with this ID.
   * @throws NotFoundException - if there is no CabinetEntry with this
   * ID in this Cabinet.
   */
  public CabinetEntry getCabinetEntry(ID id)
  // [GS] CHANGE: throws FilingPermissionDeniedException, FilingIOException,
         // [GS] CHANGE: NotFoundException {
  throws FilingException {
    Class[] argTypes = new Class[1];
    Object[] args = new Object[1];
    if(null != this.cabinetFactory.getClient()) {
      argTypes[0] = ID.class;
      args[0] = id;
			try {
				return
					(CabinetEntry) this.cabinetFactory.getClient().invoke(this,
						"getCabinetEntry",
						argTypes, args);
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        getCabinetEntryLocally(id);
    } 
  }
  
  private CabinetEntry getCabinetEntryLocally(ID id)
  throws FilingPermissionDeniedException, FilingIOException,
         NotFoundException {
   openCabinet();
   return new RfsCabinet(this.cabinetFactory, id.toString());
  }

  /**
   * Get CabinetEntry by name.  Not all CabinetEntrys have names,
   * but if it has a name, it is unique within a Cabinet.
   *
   * @param name
   *
   * @return CabinetEntry which has given name
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the CabinetEntry with this name.
   * @throws FilingIOException - if an IO error occurs accessing
   * the CabinetEntry with this name.
   * @throws NotFoundException - if there is no CabinetEntry with this
   * name in this Cabinet.
   */
  public CabinetEntry getCabinetEntry(String name)
  // [GS] CHANGE: throws FilingPermissionDeniedException, FilingIOException,
         // [GS] CHANGE: NotFoundException {
  throws FilingException {
    Class[] argTypes = new Class[1];
    Object[] args = new Object[1];
    if(null != this.cabinetFactory.getClient()) {
      argTypes[0] = String.class;
      args[0] = name;
			try {
				return
					(CabinetEntry) this.cabinetFactory.getClient().invoke(this,
						"getCabinetEntry",
						argTypes, args);
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        getCabinetEntryLocally(name);
    } 
  }
  
  private CabinetEntry getCabinetEntryLocally(String name)
  // [GS] CHANGE: throws FilingPermissionDeniedException, FilingIOException,
        // [GS] CHANGE:  NotFoundException {
  throws FilingException {
    openCabinet();
    // generate ID from my ID plus the name
    String       newId    = this.id.toString() + java.io.File.separator
                            + name;

    java.io.File testFile = new java.io.File(newId);

    if (!testFile.exists()) {
      throw new NotFoundException("CabinetEntry " + name + "not found");
    }

    if (testFile.isDirectory()) {
      return new RfsCabinet(this.cabinetFactory, newId);
    }
    else if (testFile.isFile()) {
      return new RfsByteStore(this.cabinetFactory, newId);
    }

    throw new FilingIOException("CabinetEntry " + name + " not valid");
  }

  /**
   * Get an Iterator over all CabinetEntries in this Cabinet.
   *
   * @return java.util.Iterator
   *
   */
  public java.util.Iterator entries()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					(java.util.Iterator) this.cabinetFactory.getClient().invoke(this,
						"entries",
						argTypes, args);
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        entriesLocally();
    } 
  }
  
  private java.util.Iterator entriesLocally()
  throws FilingException {
    openCabinet();

    java.util.List entriesList = new java.util.ArrayList();

    java.io.File[] list        = this.cabinetFile.listFiles();

    for (int i = 0; i < list.length; i++) {
      try {
        String path = list[i].getCanonicalPath();

        if (list[i].isDirectory()) {
          entriesList.add(new RfsCabinet(this.cabinetFactory, path));
        }
        else if (list[i].isFile()) {
          entriesList.add(new RfsByteStore(this.cabinetFactory, path));
        }
      }
      catch (Exception e) {}
    }

    return entriesList.iterator();
  }

  /**
   * Return the root Cabinet of this cabinet.
   *
   *
   * @return root Cabinet
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the root Cabinet.
   * @throws FilingIOException - if an IO error occurs.
   */
  public Cabinet getRootCabinet()
  throws FilingPermissionDeniedException, FilingIOException {
    try {
      return new RfsCabinet(this.cabinetFactory, java.io.File.separator);
    }
    catch (FilingPermissionDeniedException e) {
      throw e;
    }
    catch (FilingIOException e) {
      throw e;
    }
    catch (Exception e) {
      return null;
    }
  }

  /**
   * Return true if this Cabinet is the root Cabinet.
   *
   *
   * @return true if and only if this Cabinet is the root Cabinet.
   *
   */
  public boolean isRootCabinet()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					((Boolean) this.cabinetFactory.getClient().invoke(this,
						"isRootCabinet",
						argTypes, args)).booleanValue();
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        isRootCabinetLocally();
    } 
  }
  
  private boolean isRootCabinetLocally()
  throws FilingException {
    try {
      openCabinet();
      java.io.File parentFile = this.cabinetFile.getParentFile();

      if (parentFile != null) {
        return false;
      }
      else {
        return true;
      }
    }
    catch (Exception e) {}

    return false;
  }

  /**
   * Set quota for Agent in this Cabinet.
   *
   * @param agent
   * @param quotaBytes
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set quotas
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if quotas are not implemented.
   */
  public void setQuota(org.okip.service.shared.api.Agent agent,
                       long quotaBytes)
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException("Quotas not implemented");
  }

  /**
   * Get quota for agent in this Cabinet.
   *
   * @param agent
   *
   * @return long
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to remove a Cabinet.
   * @throws UnsupportedFilingOperationException - if quotas are not implemented.
   */
  public long getQuota(org.okip.service.shared.api.Agent agent)
  throws FilingPermissionDeniedException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException("Quotas not implemented");
  }

  /**
   * Get quota used by the agent in this Cabinet.
   *
   * @param agent
   *
   * @return long
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to examine the quota of this Agent
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if quotas are not implemented.
   */
  public long getQuotaUsed(org.okip.service.shared.api.Agent agent)
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException("Quotas not implemented");
  }

  /**
   * Get space available in Cabinet, for bytes.
   *
   *
   * @return long, space available in Cabinet, in bytes.
   *
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if space restrictions are
   * not implemented.
   */
  public long getAvailableBytes()
  throws FilingIOException, UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException("Quotas not implemented");
  }

  /**
   * Get number of bytes used in this Cabinet.
   *
   *
   * @return long - space used in Cabinet, in bytes.
   *
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if space restrictions are
   * not implemented.
   */
  public long getUsedBytes()
  throws FilingIOException, UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "Space Restrictions not implemented");
  }

  /**
   * Compares this Cabinet to another Object.  If the object is not
   * a Cabinet, it throws a <code>ClassCastException</code>.
   *
   * @param o  The <code>Object</code> to be compared to Cabinet
   *
   * @return 0 if they are equal, less than 0 or greater than 0 if
   * they differ, depending on the implementation-dependent quality
   * used for the comparison.  If an error occurs, return less than 0.
   *
   * @see java.lang.Comparable
   */
  public int compareTo(Object o) {
    try {
      return compareTo((Cabinet) o);
    }
    catch (ClassCastException e) {
      throw e;
    }
    catch (Exception e) {
      return -1000;
    }
  }

  /**
   * Compares this this Cabinet to another Cabinet.
   *
   * @param cabinet  The <code>Cabinet</code> to be compared to this Cabinet
   *
   * @return 0 if they are equal, less than 0 or greater than 0 if
   * they differ, depending on the implementation-dependent quality
   * used for the comparison.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the Cabinet being compared to
   * @throws FilingIOException - if an IO error occurs reading the
   * Cabinet being compared to
   *
   * @see java.lang.Comparable
   */
  public int compareTo(Cabinet cabinet)
  // [GS] CHANGE: throws FilingPermissionDeniedException, FilingIOException {
  throws FilingException {
    Class[] argTypes = new Class[1];
    Object[] args = new Object[1];
    if(null != this.cabinetFactory.getClient()) {
      argTypes[0] = Cabinet.class;
      args[0] = cabinet;
			try {
				return
					((Integer) this.cabinetFactory.getClient().invoke(this,
						"compareTo",
						argTypes, args)).intValue();
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        compareToLocally(cabinet);
    } 
  }
  
  private int compareToLocally(Cabinet cabinet)
  // [GS] CHANGE: throws FilingPermissionDeniedException, FilingIOException {
  throws FilingException {
    openCabinet();
    if (cabinet instanceof RfsCabinet) {
      return this.id.toString().compareTo(cabinet.getID().toString());
    }
    else {
      return -1000;
    }
  }

  /**
   * Tests this Cabinet for equality with the given object.
   * Returns <code>true</code> if and only if the argument is not
   * <code>null</code> and is equal to this Cabinet.
   *
   * @param object
   *
   * @return  <code>true</code> if and only if the objects are the same;
   *          <code>false</code> otherwise, or if an error occurs
   */
  public boolean equals(Object object) {
    try {
      if (object instanceof RfsCabinet) {
        Class[] argTypes = new Class[1];
        Object[] args = new Object[1];
        if(null != this.cabinetFactory.getClient()) {
          argTypes[0] = RfsCabinet.class;
          args[0] = (java.io.Serializable)object;
					try {
						return
							((Boolean) this.cabinetFactory.getClient().invoke(this,
								"equals",
								argTypes, args)).booleanValue();
					} catch (Exception e) {
							throw new FilingIOException(e.getMessage());
					}
        } else {
          return
            equalsLocally((RfsCabinet)object);
        } 
      }
      else {
        return false;
      }
    }
    catch(Throwable ex) {}
    return false;
  }
  
  public boolean equalsLocally(RfsCabinet cabinet)
  throws FilingException {
    openCabinet();
    return this.id.equals(cabinet.getID());
  }

  /**
   * Computes a hash code for this Cabinet.
   *
   * @return  A hash code for this Cabinet
   */
  public int hashCode() {
    try {
      Class[] argTypes = new Class[0];
      Object[] args = new Object[0];
      if(null != this.cabinetFactory.getClient()) {
				try {
					return
						((Integer) this.cabinetFactory.getClient().invoke(this,
							"hashCode",
							argTypes, args)).intValue();
				} catch (Exception e) {
						throw new FilingIOException(e.getMessage());
				}
      } else {
        return
          hashCodeLocally();
      }
    }
    catch(Throwable ex) {}
    return 0; 
  }
  
  private int hashCodeLocally()
  throws FilingException {
    openCabinet();
    return this.cabinetFile.hashCode();
  }

  /**
   * Returns the string representation of this Cabinet.
   *
   * @return  The string form of this Cabinet
   */
  public String toString() {
    try {
      Class[] argTypes = new Class[0];
      Object[] args = new Object[0];
      if(null != this.cabinetFactory.getClient()) {
				try {
					return
						(String) this.cabinetFactory.getClient().invoke(this,
							"toString",
							argTypes, args);
				} catch (Exception e) {
						throw new FilingIOException(e.getMessage());
				}
      } else {
        return
          toStringLocally();
      }
    }
    catch(Throwable ex) {}
    return ""; 
  }
  
  private String toStringLocally()
  throws FilingException {
    openCabinet();
    return this.id.toString();
  }

  /**
   * Tests whether this is a Cabinet.
   *
   *
   * @return boolean - true if this entry is a Cabinet.
   *
   */
  public boolean isCabinet() {
    return true;
  }

  /**
   * Tests whether this is a ByteStore.
   *
   *
   * @return boolean - true if this entry is a ByteStore.
   *
   */
  public boolean isByteStore() {
    return false;
  }

  /**
   * Returns the Cabinet in which this is an entry, or null if it has
   * no parent (for example is the root cabinet).
   *
   *
   * @return Cabinet - the parent Cabinet of this entry, or null if it has
   * no parent (e.g. is the root cabinet)
   *
   * @throws FilingIOException - if an IO error occurs.
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the parent Cabinet.
   */
  public Cabinet getParent()
  // [GS] CHANGE: throws FilingIOException, FilingPermissionDeniedException {
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					(Cabinet) this.cabinetFactory.getClient().invoke(this,
						"getParent",
						argTypes, args);
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        getParentLocally();
    } 
  }
  
  private Cabinet getParentLocally()
  throws FilingIOException, FilingPermissionDeniedException {
    try {
      openCabinet();
      java.io.File parentFile = this.cabinetFile.getParentFile();

      if (parentFile != null) {
        String parentString = parentFile.getCanonicalPath();

        return new RfsCabinet(this.cabinetFactory, parentString);
      }
      else {
        return null;
      }
    }
    catch (Exception e) {
      throw new FilingIOException("Error getting parent:" + e.getMessage());
    }
  }

  /**
   * Return the name of this CabinetEntry in its parent Cabinet.
   *
   * @return name
   *
   */
  public String getName()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					(String) this.cabinetFactory.getClient().invoke(this,
						"getName",
						argTypes, args);
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        getNameLocally();
    } 
  }
  
  private String getNameLocally()
  throws FilingException {
    openCabinet();
    return this.cabinetFile.getName();
  }

  /**
   * Return the path, the sequence of names of CabinetEntries leading
   * to this CabinetEntry from its root Cabinet, separated by the
   * separatorCharactor.
   *
   * @return path
   *
   */
  public String getPath()
  throws FilingException {
    Class[] argTypes = new Class[0];
    Object[] args = new Object[0];
    if(null != this.cabinetFactory.getClient()) {
			try {
				return
					(String) this.cabinetFactory.getClient().invoke(this,
						"getPath",
						argTypes, args);
      } catch (Exception e) {
					throw new FilingIOException(e.getMessage());
			}
    } else {
      return
        getPathLocally();
    } 
  }
  
  private String getPathLocally()
  throws FilingException {
    openCabinet();
    return this.id.toString();
  }
}

