package org.okip.service.filing.impl.remotefilesystem;



import org.okip.service.filing.api.*;


/**
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including software, documents, or other related items (the
   "Software"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI&#153; API Implementation License. By
   obtaining, using and/or copying this Software, you agree that you have
   read, understand, and will comply with the following terms and
   conditions of the MIT OKI&#153; API Implementation License:

   Permission to use, copy, modify, and distribute this Software and its
   documentation, with or without modification, for any purpose and
   without fee or royalty is hereby granted, provided that you include
   the following on ALL copies of the Software or portions thereof,
   including modifications or derivatives, that you make:

    *  The full text of the MIT OKI&#153; API Implementation License in a
       location viewable to users of the redistributed or derivative
       work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed or
       derivative Software:
       "Copyright (c) 2002 Massachusetts Institute of Technology
       All Rights Reserved."

    *  Notice of any changes or modifications to the MIT OKI&#153;
       Software, including the date the changes were made. Any modified
       software must be distributed in such as manner as to avoid any
       confusion with the original MIT OKI&#153; Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Software without
   specific, written prior permission. Title to copyright in the Software
   and any associated documentation will at all times remain with the
   copyright holders.

   The export of software employing encryption technology may require a
   specific license from the United States Government. It is the
   responsibility of any person or organization contemplating export to
   obtain such a license before exporting this Software.
*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/impl/remotefilesystem/RfsInputStream.java,v $
 */

/**
 * RemoteFileSystem InputStream object
 * <p>
 * Is seekable.
 * </p>
 * <p>
 * </p>
 * <p>
 * Licensed under the {@link org.okip.service.ApiImplementationLicenseMIT MIT OKI&#153; API Implementation License}.
 *
 * @version $Name:  $ / $Revision: 1.1 $ / $Date: 2002/07/22 16:31:55 $
 */

class RfsInputStream
implements OkiInputStream {
  private transient java.io.File             file       = null;
  private transient java.io.FileInputStream  in         = null;

  protected RfsInputStream(java.io.File file)
  throws FilingPermissionDeniedException, FilingIOException {
    this.file = file;

    if (!this.file.canRead()) {
        throw new FilingPermissionDeniedException("Read permission denied");
    }

    try {
      this.in = new java.io.FileInputStream(file);
    }
    catch (Exception e) {
        throw new FilingIOException(e.getMessage());
    }
  }

  /**
   * Reads the next byte of data from the IO Object.
   */
  public int read()
  throws FilingIOException {
    try {
      return this.in.read();
    }
    catch (java.io.IOException e) {
      throw new FilingIOException(e.getMessage());
    }
  }

  /**
   * Reads up to len bytes of data from the IO Object into an
   * array of bytes.
   */
  public int read(byte[] b, int off, int len)
  throws FilingIOException {
    try {
      return this.in.read(b, off, len);
    }
    catch (java.io.IOException e) {
      throw new FilingIOException(e.getMessage());
    }

  }

  /**
   * Reads some number of bytes from the IO Object and stores
   * them into the buffer array b.
   */
  public int read(byte[] b)
  throws FilingIOException {
    try {
      return this.in.read(b);
    }
    catch (java.io.IOException e) {
      throw new FilingIOException(e.getMessage());
    }
  }

  /**
   * Skips over and discards n bytes of data from this IO Object.
   */
  public long skip(long n)
  throws FilingIOException {
      try {
	  return (long) this.in.skip(n);
      }
      catch (java.io.IOException e) {
	  throw new FilingIOException(e.getMessage());
      }
  }

  /**
   * Returns the number of bytes that can be read (or skipped over)
   * from this IO Object without blocking by the next caller of a
   * method for this IO Object.
   */
  public long available()
  throws FilingIOException {
    try {
      return this.in.available();
    }
    catch (java.io.IOException e) {
      throw new FilingIOException(e.getMessage());
    }

  }

  /**
   * Closes this IO Object and releases any system resources
   * associated with the IO Object.
   */
  public void close()
  throws FilingIOException {
    try {
      this.in.close();
    }
    catch (java.io.IOException e) {
      throw new FilingIOException(e.getMessage());
    }

  }

  /**
   * Return InputStream in native environment which can be used to
   * access this ByteStore.  In Java, this is a java.io.InputStream
   * object.  From this one may obtain a Reader via InputStreamReader.
   *
   * @return A java.io.InputStream object which may be used to access this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to read this ByteStore.
   * @throws FilingIOException - if an IO error occurs.  */
  public java.io.InputStream getNativeInputStream()
  throws FilingPermissionDeniedException, FilingIOException {
    try {
      this.in.close();

      return new java.io.FileInputStream(file);
    }
    catch (Exception e) {
      throw new FilingIOException(e.getMessage());
    }
  }


  /**
   * Marks the current position in this input stream. A subsequent call to
   * the <code>reset</code> method repositions this stream at the last marked
   * position so that subsequent reads re-read the same bytes.
   *
   * <p> The <code>readlimit</code> arguments tells this input stream to
   * allow that many bytes to be read before the mark position gets
   * invalidated.
   *
   * <p> The general contract of <code>mark</code> is that, if the method
   * <code>markSupported</code> returns <code>true</code>, the stream somehow
   * remembers all the bytes read after the call to <code>mark</code> and
   * stands ready to supply those same bytes again if and whenever the method
   * <code>reset</code> is called.  However, the stream is not required to
   * remember any data at all if more than <code>readlimit</code> bytes are
   * read from the stream before <code>reset</code> is called.
   *
   * @param   readlimit   the maximum limit of bytes that can be read before
   *                      the mark position becomes invalid.
 * @throws FilingIOException - if an IO error occurs setting the mark
 * @throws UnsupportedFilingOperationException - if mark is not supported in this CabinetFactory implementation.
   */
  public void mark(int readlimit)
      throws FilingIOException, UnsupportedFilingOperationException {
      throw new UnsupportedFilingOperationException("mark/reset not supported");
  }

  /**
   * Repositions this stream to the position at the time the
   * <code>mark</code> method was last called on this input stream.
   *
   * <p> The general contract of <code>reset</code> is:
   *
   * <p><ul>
   *
   * <li> If the method <code>markSupported</code> returns
   * <code>true</code>, then:
   *
   *     <ul><li> If the method <code>mark</code> has not been called since
   *     the stream was created, or the number of bytes read from the stream
   *     since <code>mark</code> was last called is larger than the argument
   *     to <code>mark</code> at that last call, then a
   *     <code>FilingIOException</code> might be thrown.
   *
   *     <li> If such an <code>IOException</code> is not thrown, then the
   *     stream is reset to a state such that all the bytes read since the
   *     most recent call to <code>mark</code> (or since the start of the
   *     file, if <code>mark</code> has not been called) will be resupplied
   *     to subsequent callers of the <code>read</code> method, followed by
   *     any bytes that otherwise would have been the next input data as of
   *     the time of the call to <code>reset</code>. </ul>
   *
   * <li> If the method <code>markSupported</code> returns
   * <code>false</code>, then:
   *
   *     <ul><li> The call to <code>reset</code> may throw an
   *     <code>IOException</code>.
   *
   *     <li> If an <code>FilingIOException</code> is not thrown,
   *     then the stream is reset to a fixed state that depends on
   *     the particular type of the input stream and how it was
   *     created. The bytes that will be supplied to subsequent
   *     callers of the <code>read</code> method depend on the
   *     particular type of the input stream. </ul></ul>
   *
   *
 * @throws FilingIOException - if an IO error occurs gettting the mark
 * @throws UnsupportedFilingOperationException - if mark is not supported in this CabinetFactory implementation.
 */
  public void reset()
      throws FilingIOException, UnsupportedFilingOperationException {
      throw new UnsupportedFilingOperationException("mark/reset not supported");
  }

  /**
   * Tests if this input stream supports the <code>mark</code> and
   * <code>reset</code> methods. The <code>markSupported</code> method of
   * <code>InputStream</code> returns <code>false</code>.
   *
   * @return  <code>true</code> if this true type supports the mark and reset
   *          method; <code>false</code> otherwise.
   */
    public boolean markSupported() {
	return false;
  };
}
