package org.okip.service.localid.impl;

public class Factory implements org.okip.service.localid.api.Factory,
  java.io.Serializable {

    public static final int MAX_TRIES = 20;
    private final String checkSQL = "{? = call localidentifier.EXISTS(?)}";
    private final String addSQL = "{? = call localidentifier.ADD(?)}";
    private static org.okip.service.dbc.api.Factory dbFactory;
    private java.rmi.server.UID id = null;
    private org.okip.service.shared.api.Agent owner;
    private org.okip.service.dbc.api.Connection connection;

    static {
        if ( dbFactory == null ) {
           try {
                 dbFactory = (org.okip.service.dbc.api.Factory) org.okip.service.shared.api.FactoryManager.getDefaultFactory("org.okip.service.dbc.api", null);
            } catch (Exception ex) {
                System.out.println("ERROR: Problem loading dbc factory");
                System.out.flush();
                ex.printStackTrace();
            }
      }
    };

    private void generate() throws org.okip.service.localid.api.LocalIdException {
        int tries = 0;
        boolean found = true;
        java.rmi.server.UID id = null;
        
        while( found  && (tries++ < MAX_TRIES) ) {
            //java.util.Random num = new java.util.Random( System.currentTimeMillis() );
            //short x = (short) num.nextInt();
            id = new java.rmi.server.UID( );
            found = found( id.toString().toUpperCase() );
        }
        if ( (tries <= MAX_TRIES) && !found ) {
                if ( add( id.toString().toUpperCase() ) )
                    this.id = id;
                else 
                    throw new org.okip.service.localid.api.LocalIdException("Failed to persist unique identifier after " + tries + " attempts!");
        }
         else
                throw new org.okip.service.localid.api.LocalIdException("Could not generate unique identifier after " + tries + " attempts!");
    }
    
    public org.okip.service.localid.api.LocalId getLocalId() throws org.okip.service.localid.api.LocalIdException {
            generate();
            return new LocalIdImpl( this.id );
    }
    
    /**
     * Return owner of this Factory.
     *
     *
     * @return org.okip.service.shared.api.Agent Owner
     *
     */
    public org.okip.service.shared.api.Agent getOwner() {
        return this.owner;
    }
    
    /**
     * Set the owner of this Factory.
     *
     *
     * @param Agent Owner
     *
     */
    public void setOwner(org.okip.service.shared.api.Agent owner) {
        this.owner = owner;
    }
   
    private final boolean refreshConnection() {
        try {
            if ( !isConnectionValid( this.connection ) )
                this.connection = this.dbFactory.getConnection();
                return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }
    
    private final Long execLocalIdFunc( final String SQL ) throws org.okip.service.localid.api.LocalIdException {

            org.okip.service.dbc.api.CallableStatement cs = null;

            try {
                    // Get the connection again, as needed. Note that the following test may not suffice.
                    if ( !isConnectionValid(connection) ) {
                        connection = this.dbFactory.getConnection();
                    }
                    System.out.println("dbFactory = " + this.dbFactory );
                    cs = connection.prepareCall( SQL );
                    cs.registerOutParameter (1, java.sql.Types.INTEGER);
                    boolean flag = cs.execute();
                    int id = cs.getInt( 1 );
                    return new Long( id );
            } catch ( org.okip.service.dbc.api.DbcException ex ) {
                          ex.printStackTrace();
                        throw new org.okip.service.localid.api.LocalIdException( ex.getLocalizedMessage() );
            } catch ( final Exception ex2 ) {
                        ex2.printStackTrace();
                        throw new org.okip.service.localid.api.LocalIdException( ex2.getLocalizedMessage() );
            } finally {
                try {
                        cs.close();
                        connection.close();
                } catch ( Exception ignored ) {
                                // don't care
                } finally {
                    cs = null;
                }
           }
    }

    private final Long execLocalIdFunc( final String idString, final String SQL ) 
    throws org.okip.service.localid.api.LocalIdException {

            org.okip.service.dbc.api.CallableStatement cs = null;

            try {
                    // Get the connection again, as needed. Note that the following test may not suffice.
                    if ( !isConnectionValid(connection) ) {
                        connection = this.dbFactory.getConnection();
                    }
                    cs = connection.prepareCall( SQL );
                    cs.registerOutParameter (1, java.sql.Types.INTEGER);
                    cs.setString(2, idString);
                    boolean flag = cs.execute();
                    int id = cs.getInt( 1 );
                    return new Long( id );
            } catch ( org.okip.service.dbc.api.DbcException ex ) {
                        ex.printStackTrace();
                        throw new org.okip.service.localid.api.LocalIdException( ex.getLocalizedMessage() );
        } catch ( final Exception ex2 ) {
                        ex2.printStackTrace();
                        throw new org.okip.service.localid.api.LocalIdException( ex2.getLocalizedMessage() );
        } finally {
                try {
                        cs.close();
                        connection.close();
                } catch ( Exception ignored ) {
                        // don't care
                } finally {
                        cs = null;
                        //connection = null;
                }
        }
    }
    
    private boolean isConnectionValid(org.okip.service.dbc.api.Connection connection) {
        if ( connection == null )
            return false;
        try {
            if ( connection.isClosed() ) 
                return false;
            org.okip.service.dbc.api.DatabaseMetaData dbmd = connection.getMetaData();
        } catch ( org.okip.service.dbc.api.DbcException ex ) {
            return false;
        }
        return true;
    }
    
    /** Encapsulates the existence check -- makes a DB call. */
    private boolean found( final String id ) throws org.okip.service.localid.api.LocalIdException {
        return execLocalIdFunc(id, checkSQL).longValue() == 1;
    }
    
    /** Encapsulates id persistence -- makes a DB call. */
    private boolean add( final String id ) throws org.okip.service.localid.api.LocalIdException {
        return execLocalIdFunc( id, addSQL ).longValue() == 1;
    }   
}
