package org.okip.util.hierarchy.impl;

/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.
*/

public class Factory implements org.okip.util.hierarchy.api.Factory
{
	private org.okip.service.dbc.api.Connection mConnection;
	private org.okip.service.shared.api.Agent mOwner;

	public org.okip.service.shared.api.Agent getOwner()
	{
		return mOwner;
	}

	public void setOwner( org.okip.service.shared.api.Agent qOwner )
	{
		mOwner = qOwner;
	}

	public Factory()
	{
		try
		{
		  org.okip.service.dbc.api.Factory myFactory =
			 (org.okip.service.dbc.api.Factory) org.okip.service.shared.api.FactoryManager.getFactory(
			    "org.okip.service.dbc.api",    //TODO Should read the from a Properties file
			    "edu.mit.is.service.dbc",
			     null
			  );

          mConnection = myFactory.getConnection();

		}
		catch( Exception ex )
		{
			ex.printStackTrace();
		}
	}

	public org.okip.util.hierarchy.api.Hierarchy createHierarchy( java.io.Serializable qKey, String qName, String qDescription )
		throws org.okip.util.hierarchy.api.HierarchyException
	{
		// validate args
		if( qKey == null ||
			qName == null ||
			qDescription == null )
		{
			throw new NullPointerException();
		}
		if( ! ( qKey instanceof String ) )
		{
			throw new IllegalArgumentException();
		}
		String key = (String)qKey;
		String name = qName;
		String desc = qDescription;
		if( key.length() > Hierarchy.KEY_LENGTH )
		{
			throw new IllegalArgumentException();
		}
		if( name.length() > Hierarchy.NAME_LENGTH )
		{
			throw new IllegalArgumentException();
		}
		if( desc.length() > Hierarchy.DESCRIPTION_LENGTH )
		{
			throw new IllegalArgumentException();
		}
		// make sure the key isn't already in the table

		// create the hierarchy
		try
		{

			org.okip.service.dbc.api.PreparedStatement ps = mConnection.prepareStatement( "insert into hierarchies (key, name, description) values (" +
		   	                                                     "'" + key + "'," +
		                                                         "'" + name + "'," +
		                                                         "'" + desc + "')" );
			if( ps.executeUpdate() != 1 )
			{
				throw new org.okip.util.hierarchy.api.HierarchyException();
			}
		}
		catch( org.okip.service.dbc.api.DbcException ex )
		{
			ex.printStackTrace();
			throw new org.okip.util.hierarchy.api.HierarchyException();
		}
		return getHierarchy( qKey );
	}

	public org.okip.util.hierarchy.api.Hierarchy getHierarchy( java.io.Serializable qKey )
	{
		if( qKey == null )
		{
			throw new NullPointerException();
		}
		if( ! ( qKey instanceof String ) )
		{
			throw new IllegalArgumentException();
		}
		String key = (String)qKey;
		// create the hierarchy
		org.okip.service.dbc.api.ResultSet rs = null;
		String rkey = new String("");
		String name = new String("");
		String desc = new String("");

		try
		{
			org.okip.service.dbc.api.PreparedStatement ps = mConnection.prepareStatement( "select * from hierarchies " +
		                                                         "where key = '" + key + "'" );

			rs = ps.executeQuery();
			if( rs.next() )
			{
				rkey = rs.getString( 1 );   //TODO:change this to getString(String) once it is implemented
				name = rs.getString( 2 );
				desc = rs.getString( 3 );
			}
			else
			{
				return null;
			}
		}
		catch( org.okip.service.dbc.api.DbcException ex )
		{
			ex.printStackTrace();
			return null;
		}
		return new Hierarchy( rkey, name, desc, mConnection );
	}

	public void removeHierarchy( java.io.Serializable qKey )
	{
		if( qKey == null )
		{
			throw new NullPointerException();
		}
		if( ! ( qKey instanceof String ) )
		{
			throw new IllegalArgumentException();
		}
		String key = (String)qKey;

		try
		{
			org.okip.service.dbc.api.PreparedStatement ps1 = mConnection.prepareStatement( "delete from hierarchies " +
		                                                          "where key = '" + key + "'" );
			ps1.executeUpdate();
			org.okip.service.dbc.api.PreparedStatement ps2 = mConnection.prepareStatement( "delete from nodes " +
		                                                          "where hierarchy = '" + key + "'" );
		    ps2.executeUpdate();
		}
		catch( org.okip.service.dbc.api.DbcException ex )
		{
			ex.printStackTrace();
		}
	}
}
